!+
!  Name:
!     MCH_PAR

!  Purpose:
!     Machine-dependent constants.

!  Language:
!     Starlink Fortran 77

!  Type of Module:
!     Global constants include file.

!  Machine:
!     Compaq Alpha

!  Description:
!     This file defines the machine dependent constants relevant for
!     Fortran floating point arithmetic.
!
!     In order to write certain numerical procedures, numbers such as the
!     machine precision and the smallest and largest allowed positive 
!     numbers allowed need to be known. Such important properties of
!     floating-point arithmetic may be characterised by a simplified 
!     _model_ containing a small number of (machine-dependent) parameters, 
!     which may be chosen to describe closely the behaviour of floating point
!     arithmetic.
!
!     The model adopted here is based on that developed by Brown (1981),
!     and implemented in the NAG library (chapter X02). It is characterized
!     five parameters, B, P, EMIN, EMAX and ROUNDS. The machine precision (EPS),
!     smallest (SMALL) and largest (LARGE) positive model numbers, and the safe
!     range (SAFE) parameter, can be characterized in terms of these quantities.
!     Both the primitive parameters, and selected dependent quantities are
!     given in this file, to avoid the need for additional computations.
!
!     The file also contains parameters relating to other aspects of the
!     computing environment.

!  Reference:
!     Brown W.S., 1981. ACM Trans. Math. Software 7, 445.
!     NAG Fortran Library ...

!  Authors:
!     PCTR: P.C.T. Rees (STARLINK)
!     CSJ:  C.S. Jeffery (St Andrews)
!     {enter_new_authors_here}

!  History:
!     24-JAN-1992 (PCTR):
!      3-AUG-1993 (CSJ):
!        Original version: Sun ELC
!      8-MAY-2001 (CSJ):
!        Digital Compaq Alpha
!     {enter_changes_here}

!  Note:
!     The parameters in this file are machine dependent. If the NAG library
!     is implemented, they may be verified using the NAG example program 
!     X02AJFE.

!  Bugs:
!     {note_any_bugs_here}

!-
!.

!  Model parameter B.
      INTEGER MCH__B
      PARAMETER ( MCH__B = 2 )

!  Model parameter P.
      INTEGER MCH__P
      PARAMETER ( MCH__P = 53 )  

!  Model parameter EMIN.
      INTEGER MCH__EMIN
      PARAMETER ( MCH__EMIN = -1021 )

!  Model parameter EMAX.
      INTEGER MCH__EMAX
      PARAMETER ( MCH__EMAX = 1024 )  

!  Model parameter ROUNDS.
      LOGICAL MCH__ROUNDS
      PARAMETER ( MCH__ROUNDS = .TRUE. )

!  Machine precision EPS.
      DOUBLE PRECISION MCH__EPS
      PARAMETER ( MCH__EPS = 1.110223024625157D-016 ) 

!  Smallest positive model number SMALL.
      DOUBLE PRECISION MCH__SMALL
      PARAMETER ( MCH__SMALL = 2.22507385850721D-308 )

!  Largest positive model number LARGE.
      DOUBLE PRECISION MCH__LARGE
      PARAMETER ( MCH__LARGE = 1.7976931348623D+308 )

!  Safe range parameter SAFE.
      DOUBLE PRECISION MCH__SAFE
      PARAMETER ( MCH__SAFE = 2.22507385850721D-308 ) 

!  Largest positive integer ILARGE.
      INTEGER MCH__ILARGE
      PARAMETER ( MCH__ILARGE = 2147483647 )

!  Precision in decimal digits DIGITS.
      INTEGER MCH__DIGITS
      PARAMETER ( MCH__DIGITS = 15 )
