PROGRAM FLUXFIT

  !+
  !
  !  Program:
  !     fluxfit
  !
  !  Purpose:
  !     given an observed flux distribution, and a grid of
  !     theroetical flux distributions on a parameters space
  !     { Teff, theta, Eb-v }, find the best fit solution for
  !     Teff, theta and Eb-v, with errors.
  !
  !  Language:
  !     Fortran 90
  ! 
  !  Reference:
  !
  !  Authors:
  !     C.S.Jeffery (CSJ) (Armagh Observatory)
  !
  !  History:
  !     27-May-1999: CSJ
  !        original version
  !     01-Jun-2001: CSJ
  !        upgrade for better compatability with SFIT and
  !        better treatment of photometry
  !        BUT -- must give photometry after IUE data....every time
  !-


  !  Global constants:
  USE XFIT_DIMS


  !  Global variables:
  USE SFIT_GLOBAL

  USE SFIT_MOD_TGH

  !  Global variables:
  !  USE TFIT_MODELS

! Subroutine interfaces

INTERFACE
  SUBROUTINE KEY_GET (KEYWRD, PARSTR, KPROMPT, KSEP, KUNIT, KECHO, KSTAT)
    CHARACTER, INTENT(OUT)          :: KEYWRD*(*)  ! (Returned).
    CHARACTER, INTENT(INOUT)        :: PARSTR*(*)  ! (Given and Returned).
    CHARACTER, INTENT(IN), OPTIONAL :: KPROMPT*(*) ! (Given).
    CHARACTER, INTENT(IN), OPTIONAL :: KSEP*(*)    ! (Given).
    INTEGER, INTENT(IN), OPTIONAL   :: KUNIT       ! (Given).
    INTEGER, INTENT(IN), OPTIONAL   :: KECHO       ! (Given).
    INTEGER, INTENT(OUT), OPTIONAL  :: KSTAT       ! (Returned).
  END SUBROUTINE KEY_GET
END INTERFACE

  !  Local variables:


  ! program identifier
  CHARACTER(LEN=5), PARAMETER :: prog__id = "ffit"
  CHARACTER(LEN=5), PARAMETER :: prog__vers = "2.1.3"
  CHARACTER(LEN=11), PARAMETER :: prog__date = "20-JAN-2006"
  CHARACTER(LEN=60), PARAMETER :: prog__url = "http://star.arm.ac.uk/~csj/software_store/ffit.html"

  !  Command line control
  INTEGER      :: NREAD      ! unit number for input channel
  INTEGER      :: NWRITE     ! unit number for output channel

  CHARACTER*80 :: COMMAND    ! holder for current command line
  CHARACTER*12 :: KEYWRD     ! holder for current command or keyword
  CHARACTER*12 :: WAVCOM     ! command for wavelength definition 
  CHARACTER*12 :: MTHCOM     ! command for method definition 
  CHARACTER*12 :: DATCOM     ! command for data definition 

  !  Observed fluxes
  !  INTEGER, PARAMETER :: NOMAX = 10000  ! maximum number of observed wavelengths
  CHARACTER :: OBSFOLDER*72      ! folder containing observed spectra
  CHARACTER :: OBSFILE*72        ! filename for observed spectrum
  CHARACTER :: OBSTITLE*40       ! title for observed spectrum
  INTEGER   :: NWOBS             ! number of wavelengths in observation
  REAL      :: WOBS(NOMAX)       ! observed vwavelengths
  REAL      :: FOBS(NOMAX)       ! observed fluxes
  REAL      :: SOBS(NOMAX)       ! weights or errors on fluxes
  INTEGER   :: READ_STATUS=0     ! read status for data file

  REAL      :: FWHM=5          ! Default fwhm for spectrophotometry 

  !  Ouptut Filenames
  CHARACTER(LEN=80) :: FITFILE    ! file for output of final fit
  CHARACTER(LEN=80) :: PARFILE    ! file for output of final solution



  !  Observed photometry
  INTEGER, PARAMETER :: NPHMAX = 10  ! maximum numer of wavebands
  INTEGER   :: NPHOBS             ! number of wavebands considered
  REAL      :: PHOTOBS_W(NPHMAX)  ! photometry waveband (central)
  REAL      :: PHOTOBS_B(NPHMAX)  ! photoemtry waveband (half-width)
  REAL      :: PHOTOBS_C(NPHMAX)  ! photometry conversion constants
  REAL      :: PHOTOBS_M(NPHMAX)  ! photometry in magnitudes
  REAL      :: PHOTOBS_F(NPHMAX)  ! photometry in fluxes
  REAL      :: PHOTOBS_S(NPHMAX)  ! photmetry errors (standard deviation)

  !  Photometric averaging
  REAL, DIMENSION(NPHMAX,NTMAX,NGMAX,NHMAX) :: TOTAL_FLUX
  REAL                                      :: TOTAL_WEIGHT
  REAL                                      :: RESPONSE

  !  Binned spectrum
  INTEGER   :: NWBIN             ! number of wavelengths in binned spectrum
  REAL      :: WBIN(NWMAX)       ! Wavelengths for binned spectrum
  REAL      :: FBIN(NWMAX)       ! observed fluxes binned onto model grid
  REAL      :: SBIN(NWMAX)       ! flux weights binned onto model grid

  !  Integrated spectrum
  REAL      :: FTOT_OBS          ! Total flux in observed spectrum
  REAL      :: STOT_OBS          ! Error in total flux
  REAL      :: FTOT_MOD          ! Total flux in model spectrum
  REAL      :: STOT_MOD          ! Error in total flux - redundant

  !  Fitted spectrum (on NWBIN/WBIN)
  REAL      :: FFIT(NWMAX)       ! fitted fluxes binned onto model grid
  REAL      :: SFIT(NWMAX)       ! error in fitted fluxes

  !  Loop counters
  INTEGER   :: NW                ! loop counter for wavelength
  INTEGER   :: NT                ! loop counter for temperature
  INTEGER   :: NG                ! loop counter for gravity
  INTEGER   :: NH                ! loop counter for abundance
  INTEGER   :: NC                ! loop counter for wavebands

  !  Fit method
  CHARACTER(LEN=10) :: METHOD

  !  Fit coefficients
  INTEGER, PARAMETER :: NPAR = 3 ! Maxumum number of parameters in fit           
  INTEGER   :: NP = NPAR         ! Number of parameters in chi-squared fit
  REAL, DIMENSION(NPAR)  :: PARS   ! Parameters passed to chi-squared solver
  REAL, DIMENSION(NPAR)  :: PARS0  ! Starting values saved...
  REAL, DIMENSION(NPAR)  :: PARS1  ! Last values saved...
  REAL, DIMENSION(NPAR)  :: DPARS  ! Relative differences
  REAL, DIMENSION(NPAR)  :: DERIVS ! Derivatives of flux wrt PARS
  REAL, DIMENSION(NPAR)  :: DELTA_P!  variational amounts
  !  REAL      :: TEFF              ! Effective temperature (K)
  !  REAL      :: THETA             ! Angular radius (radians)
  REAL      :: EBV               ! Extinction coefficient E(B-V)
  CHARACTER(LEN=25) :: CPARS(NPAR)    ! Descriptors for free parameters

  !  Chi-squared minimisation control variables.
  INTEGER   :: LISTA(NPAR)       ! Regression control array
  INTEGER   :: MFIT              ! Number of coefficients in fit

  !  Chi-squared minimisation variables.
  INTEGER, PARAMETER :: NCHM = NPAR ! Maxumum number of parameters in fit 
  REAL      :: ALPHA(NCHM,NCHM)  ! curvature array
  REAL      :: COVAR(NCHM,NCHM)  ! Array of covariances
  REAL      :: CHISQ             ! Chi-squared residual
  REAL      :: ALAMDA            ! chi-squared search radius
  REAL      :: DPAR              ! sum of changes to fit parameters
  REAL      :: DCONV = 1.0E-03   ! Convergence criterion

  !  Chi-squared surface search
  INTEGER, PARAMETER  :: NTGRID = 51
  INTEGER, PARAMETER  :: NEGRID = 11
  REAL, DIMENSION(NTGRID,NEGRID) :: CHSQ_TEST

  !  Amoeba control variables
  INTEGER, PARAMETER  :: NAMBD = NPAR + 1 ! Logical dimension of AMOEBA
  INTEGER   :: MP = NPAR + 1        ! number of AMOEBA corners (= NP + 1)
  REAL      :: AMB__TOL = 9.0E-06   ! goodnes of fit tolerance
  REAL      :: AMB__P (NAMBD,NCHM)  ! function parameters at AMOEBA corners (mp,np)
  REAL      :: AMB__X (NCHM)        ! function parameters passed to FUNK (np)
  REAL      :: AMB__Y (NCHM+1)      ! function values at vertices (np)
  INTEGER   :: AMB__ITER            ! number of AMOEBA iterations


  !  Photometric systems
  CHARACTER*10 :: PHOTSYS           ! label for photometry

  !  Dipso sp2 file header
  CHARACTER(LEN=70) :: SP2_HEADER(2)   


  !  External subprograms
  EXTERNAL FLUX_PRED


  !  Local constants:
  DOUBLE PRECISION, PARAMETER :: TAPM__PI = 3.141592653589793238462643383279503D+00

  !-


  !  Announce the programme

  CALL FFIT_ANNOUNCE

  !  Initialise variables

  CALL FFIT_STARTUP



  !  Select Group Command
  !
  !     Allowed group commands are:
  !
  !        models         - sets up model atmosphere grid
  !        wavelength     - adjusts model wavelength grid
  !        method         - select solution method
  !        parameters     - initialize parameters
  !        data           - input observational data
  !        solve          - pursue solution
  !        end            - finish gracefully

  DO 

    !  Get the next command

    CALL KEY_GET ( KEYWRD, COMMAND, 'ffit>', ';', NREAD, 0, IOSTAT )
    CALL STR_LCASE ( KEYWRD )



    !  Exit condition

    GROUP: IF ( KEYWRD(1:3) == 'end' ) THEN
      EXIT



      !  Model atmosphere grid

    ELSEIF (KEYWRD(1:6)=='models') THEN 

      !  decode grid type 
      CALL KEY_GET ( KEYWRD, COMMAND, 'ffit-model>', ';', NREAD, 0, IOSTAT )
      CALL KEY_PI  ( KEYWRD, MODMODE )


      !  grid title
      READ  (*,'(A)') MODTITLE

      !  read grid
      CALL SFIT_RDGRID ( MODMODE, &
           NWMAX,NTMAX,NGMAX,NHMAX, &
           NWMOD,NTMOD,NGMOD,NHMOD, &
           WMOD, TMOD, GMOD, HMOD, &
           FMOD, CMOD )

      !  smooth grid -- assume IUE SWP resolution ~ 2.5A for now
      !  https://archive.stsci.edu/iue/manual/newsips/node15.html#SECTION00431100000000000000
      FWHM = 7  
      CALL SFIT_SMGRID ( FWHM, &
           NWMAX,NTMAX,NGMAX,NHMAX, &
           NWMOD,NTMOD,NGMOD,NHMOD, &
           WMOD, TMOD, GMOD, HMOD, &
           FMOD, CMOD )

      !sp2_header(1) = 'test input model at smgrid'
      !sp2_header(2) = 'fmod(wmod,1,1,1) -- fluxes'
      !call sp2wr (wmod(1:nwmod),fmod(1:nwmod,1,1,1),nwmod,sp2_header,21,'test_ffit_fmod.sp2')



      !  Adjust wavelength ranges of model grid
    ELSEIF (KEYWRD(1:10)=='wavelength') THEN


      !  Identify wavelengths to take from models
      !
      !  Allowed commands are:
      !
      !    end | }        -  escape from wavelength definition
      !    iue            -  selects wavelengths: 1125 - 3350
      !    user  w1  w2   -  selects wavelengths given by user
      !    phot   n label -  defines photometric wavebands ...
      !
      !    e.g.   phot 4 uvby
      !           phot 7 user  - 7 colour user defined
      !           phot 5 UBVRI
      !           phot 6 uvbyRI  - composite systems are allowed

      ! defaults
      WMIN = 1125
      WMAX = 3350

      WAVCOM(1:3) = '   '
      DO WHILE ( WAVCOM(1:3)/='end' .AND. WAVCOM(1:1)/='}' ) 
        CALL KEY_GET ( WAVCOM, COMMAND, 'ffit-wave>', ';', NREAD, 0, IOSTAT )
        CALL STR_LCASE ( WAVCOM )

        write (*,*) 'wavelength* ', wavcom, command(1:30)

        !  iue
        IF (WAVCOM(1:3)=='iue') THEN 
          WMIN = 1125
          WMAX = 3350

          !  user
        ELSEIF (WAVCOM(1:4)=='user') THEN
          CALL KEY_PR ( COMMAND, WMIN )
          CALL KEY_PR ( COMMAND, WMAX )


          ! photometry
        ELSEIF (WAVCOM(1:4)=='phot') THEN
          CALL KEY_PI ( COMMAND, NPHOBS )
          CALL KEY_PC ( COMMAND, PHOTSYS )
          CALL PHOT_DEFINE

        ENDIF

      ENDDO


      ! add infrared wavelengths to model atmospheres
      CALL GRID_ADD_IR

      ! restrict model grid to match selected wavelength range
      CALL GRID_RESTRICT

      ! rebin visual wavelengths into user supplied wavebeands
      CALL GRID_PHOT_BAND 

      ! Set up wavelength grid for binned data
      WBIN = WMOD; NWBIN = NWMOD



      !  Define method and number of fit parameters

      !  amoeba
      !  range   w1  w2       - wavelength range to include in fit

      !  ... to be implemented or scrubbed
      !  mask n               - wavelength regions to mask (n regions)
      !  continuum            - automatic continuum adjustment
      !  crawler              - obsolete
      !  levenburg-marquardt  - not working
      !  chi-squared          - surface only

    ELSEIF (KEYWRD(1:6)=='method') THEN

      MTHCOM(1:3) = '   '
      DO WHILE ( MTHCOM(1:3)/='end' .AND. MTHCOM(1:1)/='}' ) 
        CALL KEY_GET ( MTHCOM, COMMAND, 'ffit-method>', ';', NREAD, 0, IOSTAT )
        CALL STR_LCASE ( MTHCOM )

        write (*,*) 'method* ', mthcom, command(1:30)

        ! define observed wavelength range
        IF (MTHCOM(1:5)=='range') THEN
          CALL KEY_PR ( COMMAND, WOBS1 )
          CALL KEY_PR ( COMMAND, WOBS2 )

          ! define mask intervals
        ELSEIF (MTHCOM(1:4)=='mask') THEN

          ! defines solution method
        ELSEIF ( MTHCOM(1:3)/='end' .AND. MTHCOM(1:1)/='}' ) THEN
          METHOD = MTHCOM(1:10)

        ENDIF

      ENDDO




      !  Initialize parameters 

    ELSEIF (KEYWRD(1:5)=='param') THEN

      CALL KEY_GET ( KEYWRD, COMMAND, 'ffit-param>', ';', NREAD, 0, IOSTAT )
      CALL KEY_PI  ( KEYWRD, NP )

      write (*,*) 'param* ', keywrd, command(1:30)

      ! cludged at present, eventually it should be 
      !
      ! Teff_1   value delta
      ! Ebv      value delta
      ! logg_2   value delta
      ! ... etc
      !                     and the code will sort out the mapping to PARS.

      DO N = 1,NP
        READ  (*,*) PARS(N),DELTA_P(N),CPARS(N)
      ENDDO

      WRITE (11,'(A50)')'Initial values for parameters and increments:'
      DO I = 1,NP
        IF (0.01 < ABS(PARS(I)) .AND. ABS(PARS(I)) < 100.) THEN
        WRITE (* ,'(A25,F8.3,'' +/- '',F8.3)') CPARS(I),PARS(I),DELTA_P(I)
        WRITE (11,'(A25,F8.3,'' +/- '',F8.3)') CPARS(I),PARS(I),DELTA_P(I)
        ELSE
        WRITE (* ,'(A25,0P,E12.3,'' +/- '',E12.3)') CPARS(I),PARS(I),DELTA_P(I)
        WRITE (11,'(A25,0P,E12.3,'' +/- '',E12.3)') CPARS(I),PARS(I),DELTA_P(I)
        ENDIF
      ENDDO
      WRITE (11,*)

      DO N = 1,NP
        LISTA(N) = N
      ENDDO
      PARS0 = PARS

      ! scale delta_p
      WHERE (PARS>0.) DELTA_P = DELTA_P / PARS


      !  Read in data defining observation

    ELSEIF (KEYWRD(1:4)=='data') THEN

      ! get the data command

      !  folder name     - name of folder containing observed spectrum
      !  iue    name     - name of file containing iue spectrum
      !  phot   m1 m2 m3 m4   - enter photometry in predefined system
      !  errors s1 s2 s3 s4   - errors on photoemtry in same units/system


      DATCOM(1:3) = '   '
      DO WHILE ( DATCOM(1:3)/='end' .AND. DATCOM(1:1)/='}' ) 
        CALL KEY_GET ( DATCOM, COMMAND, 'ffit-data>', ';', NREAD, 0, IOSTAT )
        CALL STR_LCASE ( DATCOM )

        write (*,*) 'data* ', datcom, command(1:len_trim(command))


        ! define folder containing data
        IF (DATCOM(1:6)=='folder') THEN
          CALL KEY_PC ( COMMAND, OBSFOLDER )
          OBSFOLDER = OBSFOLDER(1:INDEX(OBSFOLDER,' ')-1)



          ! Read in iue spectrum
        ELSEIF (DATCOM(1:3)=='iue') THEN
          CALL KEY_PC ( COMMAND, OBSFILE )
          OBSFILE = TRIM(OBSFOLDER)//'/'//OBSFILE(1:INDEX(OBSFILE,' ')-1)

          OPEN  (1,FILE=OBSFILE,STATUS='OLD')
          READ  (1,'(A)') OBSTITLE
          READ  (1,*)
          READ  (1,*) NWOBS
          READ  (1,*) (WOBS(NW),FOBS(NW),NW=1,NWOBS)
          CLOSE (1)

          ! default value for IUE errors  (5%) -- can get these from MXLO files if we try harder
          SOBS = 0.05 * ABS(FOBS)
          WHERE ( SOBS<1.E-30 ) SOBS = 1 

          ! Rebin IUE spectrum onto model grid
          CALL REBIN ( WOBS,FOBS,SOBS,NWOBS, WBIN,FBIN,SBIN,NWBIN, 1 )
          FMAX = MAXVAL ( FBIN )

          ! cludge
          SBIN = 0.05 * ABS(FBIN)
          WHERE ( SBIN<1.E-30 ) SBIN = 1 


          ! Read in spectrum in sp2 format
        ELSEIF (DATCOM(1:3)=='sp2') THEN
          CALL KEY_PC ( COMMAND, OBSFILE )
          OBSFILE = TRIM(OBSFOLDER)//'/'//OBSFILE(1:INDEX(OBSFILE,' ')-1)

          OPEN  (1,FILE=OBSFILE,STATUS='OLD')
          READ  (1,'(A)') OBSTITLE
          READ  (1,*)
          READ  (1,*) NWOBS
          READ  (1,*) (WOBS(NW),FOBS(NW),NW=1,NWOBS)
          CLOSE (1)

          ! default value for errors  (1%)
          SOBS = 0.01 * FOBS
          WHERE ( SOBS<1.E-30 ) SOBS = 1 

          ! Rebin spectrum onto model grid
          CALL REBIN ( WOBS,FOBS,SOBS,NWOBS, WBIN,FBIN,SBIN,NWBIN, 1 )


          ! Read in spectrum in xyz format (lambda, f(lambda), s(lambda))
        ELSEIF (DATCOM(1:3)=='xyz') THEN
          CALL KEY_PC ( COMMAND, OBSFILE )
          OBSFILE = TRIM(OBSFOLDER)//'/'//OBSFILE(1:INDEX(OBSFILE,' ')-1)

          OPEN  (1,FILE=OBSFILE,STATUS='OLD')
          I = 1 
          READ_STATUS = 0
          DO WHILE ( READ_STATUS == 0 )
            READ (1,*,IOSTAT=READ_STATUS) WOBS(I),FOBS(I),SOBS(I)
            I = I + 1
          ENDDO
          CLOSE (1)
          NWOBS = I - 2

          ! Rebin spectrum onto model grid
          CALL REBIN ( WOBS,FOBS,SOBS,NWOBS, WBIN,FBIN,SBIN,NWBIN, 1 )
          ! Problems with new model grid being finer than IUE grid
          CALL RESAMPLE ( WOBS,SOBS,NWOBS, WBIN,SBIN,NWBIN, 1 )


          !spare code for test input data
          !sp2_header(1) = 'test input data at phot'
          !sp2_header(2) = 'fobs(wobs) -- fluxes'
          !call sp2wr (wobs,fobs,nwobs,sp2_header,21,'test_errs_fobs.sp2')
          !sp2_header(2) = 'fbin(wbin) -- fluxes'
          !call sp2wr (wbin,fbin,nwbin,sp2_header,21,'test_errs_fbin.sp2')
          !sp2_header(2) = 'sobs(wobs) -- errors'
          !call sp2wr (wobs,sobs,nwobs,sp2_header,21,'test_errs_sobs.sp2')
          !sp2_header(2) = 'sbin(wbin) -- errors'
          !call sp2wr (wbin,sbin,nwbin,sp2_header,21,'test_errs_sbin.sp2')



          ! Change default error 
        ELSEIF (DATCOM(1:5)=='sigma') THEN
          CALL KEY_PR ( COMMAND, SIGMA )
          SOBS = SIGMA * FOBS
          WHERE ( SOBS<1.E-30 ) SOBS = 1 

          ! Rebin spectrum onto model grid
          CALL REBIN ( WOBS,FOBS,SOBS,NWOBS, WBIN,FBIN,SBIN,NWBIN, 1 )
          CALL RESAMPLE ( WOBS,SOBS,NWOBS, WBIN,SBIN,NWBIN, 1 )



          ! Read photometry and convert to fluxes
        ELSEIF (DATCOM(1:4)=='phot') THEN

          DO NC = 1,NPHOBS
            CALL KEY_PR ( COMMAND, PHOTOBS_M(NC) )
          ENDDO
          PHOTOBS_F = 10.**(0.4*(PHOTOBS_C-PHOTOBS_M))

          WHERE (PHOTOBS_M < 0.1) PHOTOBS_F = 0.0
          WHERE (PHOTOBS_M < 0.1) PHOTOBS_S = 1.0

          ! Transfer photometry to binned flux arrays
          DO NC = 1,NPHOBS
            FBIN(NWIUE+NC) = PHOTOBS_F(NC)
            SBIN(NWIUE+NC) = PHOTOBS_S(NC)
          ENDDO



          ! Read photometry errors and convert to fluxes
        ELSEIF (DATCOM(1:6)=='errors') THEN

          DO NC = 1,NPHOBS
            CALL KEY_PR ( COMMAND, PHOTOBS_S(NC) )
          ENDDO
          PHOTOBS_S = 10.**(0.4*(PHOTOBS_C-PHOTOBS_M+PHOTOBS_S)) - PHOTOBS_F
          WHERE (PHOTOBS_M < 0.1) PHOTOBS_S = 1.0

          ! Transfer photometry errors to binned flux arrays
          DO NC = 1,NPHOBS
            SBIN(NWIUE+NC) = PHOTOBS_S(NC)
          ENDDO

        ENDIF

      ENDDO



      ! Get some global data properties once it available.
      IF (NWOBS .GT. 0) THEN

        ! Find limits of valid data
        CALL LOCATE ( WBIN,NWBIN, WOBS1, NOBS1)
        CALL LOCATE ( WBIN,NWBIN, WOBS2, NOBS2)

        ! Compute flux integral
        CALL QD_TRAP_R ( WBIN,FBIN,SBIN,NOBS1,NOBS2, FTOT_OBS,STOT_OBS )

      ENDIF



      ! Introduce default errors if needed:
      IF (PHOTOBS_S(1).LT.1.E-20) THEN         

        ! default error: +/- 0.01
        PHOTOBS_S = PHOTOBS_F * (10.**(0.4*( 0.01 ))-1.)             

        ! transfer to binned flux arrays
        DO NC = 1,NPHOBS
          SBIN(NWIUE+NC) = PHOTOBS_S(NC)
        ENDDO
      ENDIF




      !  Carry out solution

    ELSEIF (KEYWRD(1:5)=='solve') THEN


      ! Map the chi-squared surface
      SELECT_METHOD: IF (METHOD(1:3)=='chi') THEN
        WRITE (*,*) 'Compute chi^2 over model atmosphere grid'
        CALL FFIT_CHI_SQUARED


      ! Solve using multidimensional downhill simplex method (AMOEBA)
      ELSEIF (METHOD(1:6)=='amoeba') THEN
        WRITE (*,*) 'You are using amoeba!'
        CALL FFIT_AMOEBA


      ! Solve using a generalized non-linear least-squares method (Levenburg-Marquardt)
      ELSEIF (METHOD(1:9)=='levenburg') THEN
        WRITE (*,*) 'Levenburg-Marqardt has not been tested for some time'
        CALL FFIT_LEVENBURG


      ENDIF SELECT_METHOD



      ! Recompute final fitting solution
      DO NW = 1,NWMOD
        CALL FLUX_PRED (WBIN(NW),PARS,FFIT(NW),DERIVS,3)
      ENDDO

 

      ! Write out the fitting function
      FITFILE = TRIM(OBSFILE(1:68))//'.fit'
      OPEN  (1,FILE=FITFILE,STATUS='UNKNOWN')
      WRITE (1,'(A)') TRIM(OBSTITLE)//': model fit'
      WRITE (1,*)
      WRITE (1,*) NWBIN
      WRITE (1,'(0P,F8.0,1P,3E12.3)') (WBIN(NW),FBIN(NW),SBIN(NW),FFIT(NW),NW=1,NWMOD)
      CLOSE (1)
      WRITE (*,'(2A)') 'Spectral energy distribution and fits written to ',FITFILE


      ! Write results to reporting file
      PARFILE = TRIM(OBSFILE)//'.ffit_pars'
      OPEN (1, FILE=PARFILE,STATUS='UNKNOWN')
      WRITE (1,'(A)') TRIM(OBSTITLE)//': fit parameters'
      WRITE (1,*)
      DO I = 1,NP
        IF ( 1.E-2 < ABS(PARS(I)) .AND. ABS(PARS(I)) < 1.E3 ) THEN 
          WRITE (1,'(A25,'': '',F7.3,4X,'' +/- '',F7.3)') CPARS(I),PARS(I),SQRT(COVAR(I,I))
        ELSE
          WRITE (1,'(A25,'': '',1PE11.3,'' +/- '',E11.3)') CPARS(I),PARS(I),SQRT(COVAR(I,I))
      ENDIF
      ENDDO
      WRITE (1,'(A25,'': '',1PE11.3,'' nu :'',I6)') 'chi^2 ', CHISQ/(NWOBS-NP),NWOBS-NP
      CLOSE (1)
      WRITE (*,'(2A)') 'Best fit parameters written to ',PARFILE


      ! Compute flux integral for the FIT (over same range as data)
      SFIT = 1.0
      CALL QD_TRAP_R ( WBIN,FFIT,SFIT,NOBS1,NOBS2, FTOT_FIT,STOT_FIT )

      ! Report final results.
      WRITE (*,'(4A)') ' Data                       F(obs)     s         ', &
                      ' Teff    sig    Ebv    sig     Theta      sig     ', &
                      '   Chi^2  ', &
                      '    F(fit)       F(fit)/F(obs)  ' 
      WRITE (*,'(A25,1P,2E11.2,2x,0P,2F7.3,2F7.3,1P,4E11.2,0P,F10.3)') &
           OBSFILE, FTOT_OBS,STOT_OBS, (PARS(I),SQRT(COVAR(I,I)),I=1,NP), CHISQ/(NWOBS-NP), & 
           FTOT_FIT, FTOT_FIT/FTOT_OBS 
      WRITE (11,'(4A)') ' Data                       F(obs)     s         ', &
                      ' Teff    sig    Ebv    sig     Theta      sig     ', &
                      '   Chi^2  ', &
                      '    F(fit)       F(fit)/F(obs)  ' 
      WRITE (11,'(A25,1P,2E11.2,2x,0P,2F7.3,2F7.3,1P,4E11.2,0P,F10.3)') &
           OBSFILE, FTOT_OBS,STOT_OBS, (PARS(I),SQRT(COVAR(I,I)),I=1,NP), CHISQ/(NWOBS-NP), & 
           FTOT_FIT, FTOT_FIT/FTOT_OBS 





      !  No more commands
    ENDIF GROUP


  ENDDO






  ! Define Internal Procedures

CONTAINS




  SUBROUTINE FFIT_ANNOUNCE

    WRITE (*,*)
    WRITE (*,*) ' FFFF   FFFF  III  TTTTT   '
    WRITE (*,*) ' F      F      I     T     '
    WRITE (*,*) ' FFFF   FFF    I     T     '
    WRITE (*,*) ' F      F      I     T     '
    WRITE (*,*) ' F      F     III    T     '
    WRITE (*,*)
    WRITE (*,*) ' Program: ',PROG__ID
    WRITE (*,*) ' Version: ',PROG__VERS
    WRITE (*,*) ' Date   : ',PROG__DATE
    WRITE (*,*) ' URL    : ',PROG__URL
    WRITE (*,*)
    WRITE (*,*) ' Principal Author: Simon Jeffery, Armagh Observatory'
    WRITE (*,*)
    !    WRITE (*,*) ' Contributing Authors'
    !    WRITE (*,*)
    WRITE (*,*) ' For HELP, see ', PROG__URL
    WRITE (*,*)

    IF ( PROG__VERS == "2.1" ) THEN
      WRITE (*,*)
    ENDIF

  END SUBROUTINE FFIT_ANNOUNCE


  ! Inityialise everything

  SUBROUTINE FFIT_STARTUP

    ! Open a log file
    OPEN (11,FILE="FFIT.LOG",STATUS="UNKNOWN")

    ! Set up default i/o channels
    NREAD = 0
    NWRITE = 0
    CALL STR_DEF (NREAD, NWRITE)
    KEYWRD(1:3) = '   '

    ! Initialize photometry
    OBSFOLDER(1:1) = '.'          ! default folder for data
    PHOTSYS = '    '
    NPHOBS  = 0 
    NWOBS   = 0
    PHOTOBS_S = 0.

  END SUBROUTINE FFIT_STARTUP



  !  Internal subroutine defines photmetric system to be used

  SUBROUTINE PHOT_DEFINE

    ! user defined photometric system
    IF (PHOTSYS(1:4)=='user') THEN
      IF (NPHOBS.GT.0) THEN
        NPHOBS = MIN(NPHOBS,NPHMAX)
        READ  (*,*) PHOTOBS_W(1:NPHOBS)
        READ  (*,*) PHOTOBS_B(1:NPHOBS)
        READ  (*,*) PHOTOBS_C(1:NPHOBS)
      ENDIF

      ! standard photometry (Stromgren/Johnson mix)
    ELSE
      DO NC=1,NPHOBS

        ! u
        IF(PHOTSYS(NC:NC)=='u') THEN
          PHOTOBS_W(NC) = 3500.; PHOTOBS_B(NC) = 340.; PHOTOBS_C(NC) = -19.85
          ! v
        ELSEIF(PHOTSYS(NC:NC)=='v') THEN
          PHOTOBS_W(NC) = 4100.; PHOTOBS_B(NC) = 200.; PHOTOBS_C(NC) = -20.19
          ! b
        ELSEIF(PHOTSYS(NC:NC)=='b') THEN
          PHOTOBS_W(NC) = 4700.; PHOTOBS_B(NC) = 160.; PHOTOBS_C(NC) = -20.62
          ! y
        ELSEIF(PHOTSYS(NC:NC)=='y') THEN
          PHOTOBS_W(NC) = 5500.; PHOTOBS_B(NC) = 240.; PHOTOBS_C(NC) = -21.11
          ! U
        ELSEIF(PHOTSYS(NC:NC)=='U') THEN
          PHOTOBS_W(NC) = 3600.; PHOTOBS_B(NC) = 700.; PHOTOBS_C(NC) = -20.90
          ! B
        ELSEIF(PHOTSYS(NC:NC)=='B') THEN
          PHOTOBS_W(NC) = 4400.; PHOTOBS_B(NC) = 1000.; PHOTOBS_C(NC) = -20.36
          ! V
        ELSEIF(PHOTSYS(NC:NC)=='V') THEN
          PHOTOBS_W(NC) = 5500.; PHOTOBS_B(NC) = 900.; PHOTOBS_C(NC) = -21.02
          ! R
        ELSEIF(PHOTSYS(NC:NC)=='R') THEN
          PHOTOBS_W(NC) = 7000.; PHOTOBS_B(NC) = 2200.; PHOTOBS_C(NC) = -21.89
          ! I
        ELSEIF(PHOTSYS(NC:NC)=='I') THEN
          PHOTOBS_W(NC) = 9000.; PHOTOBS_B(NC) = 2400.; PHOTOBS_C(NC) = -22.70
          ! J
        ELSEIF(PHOTSYS(NC:NC)=='J') THEN
          PHOTOBS_W(NC) = 12500.; PHOTOBS_B(NC) = 3800.; PHOTOBS_C(NC) = -23.67
          ! H
        ELSEIF(PHOTSYS(NC:NC)=='H') THEN
          PHOTOBS_W(NC) = 16500.; PHOTOBS_B(NC) = 4800.; PHOTOBS_C(NC) = -24.80
          ! K
        ELSEIF(PHOTSYS(NC:NC)=='K') THEN
          PHOTOBS_W(NC) = 22000.; PHOTOBS_B(NC) = 5500.; PHOTOBS_C(NC) = -26.02
          ! L
        ELSEIF(PHOTSYS(NC:NC)=='L') THEN
          PHOTOBS_W(NC) = 34000.; PHOTOBS_B(NC) = 7000.; PHOTOBS_C(NC) = -27.73
          ! M
        ELSEIF(PHOTSYS(NC:NC)=='M') THEN
          PHOTOBS_W(NC) = 48000.; PHOTOBS_B(NC) = 12000.; PHOTOBS_C(NC) = -29.14
          ! N
        ELSEIF(PHOTSYS(NC:NC)=='N') THEN
          PHOTOBS_W(NC) = 102000.; PHOTOBS_B(NC) = 57000.; PHOTOBS_C(NC) = -32.28
          ! [12]
        ELSEIF(PHOTSYS(NC:NC)=='T') THEN
          PHOTOBS_W(NC) = 120000.; PHOTOBS_B(NC) = 60000.; PHOTOBS_C(NC) = -33.07
        ENDIF

      ENDDO

    ENDIF

  END SUBROUTINE PHOT_DEFINE






  !  Internal subroutine supplements model wavelength grid in infrared

  SUBROUTINE GRID_ADD_IR



    !    Locate 18000 A and copy grid wavelengths to temporary store
    CALL LOCATE (WMOD,NWMOD, 17999.,J17)
    JIR = NWMOD + 1 - J17
    WOBS(1:JIR) = ALOG10 ( WMOD(J17:NWMOD) )

    !    Extend wavelength grid
    WMOD(J17+1) = 17000.
    DO NW = J17+2,J17+13
      WMOD(NW) = WMOD(NW-1) + 500.   ! 17500 (500) 23000 
    ENDDO
    DO NW = J17+14,J17+20
      WMOD(NW) = WMOD(NW-1) + 1000.  ! 24000 (1000) 30000
    ENDDO
    DO NW = J17+21,J17+28
      WMOD(NW) = WMOD(NW-1) + 5000.  ! 35000 (5000) 70000 
    ENDDO
    WMOD(J17+29) = 10.**WOBS(JIR-1)   ! 100000
    WMOD(J17+30) = 10.**WOBS(JIR)     ! 200000

    !    Loop over models
    DO NH=1,NHMOD
      DO NG=1,NGMOD
        DO NT=1,NTMOD
          !     Copy infrared fluxes to temporary store and take logs
          FOBS(1:JIR) = ALOG10 ( FMOD(J17:NWMOD,NT,NG,NH) )
          !        FOBS(1:JIR) = FMOD(J17:NWMOD,NT)
          !     Interpolate flux at new wavelength points
          DO NW = J17+1,J17+30
            CALL TABINT (JIR,WOBS,FOBS,3,ALOG10(WMOD(NW)),FL)
            FMOD(NW,NT,NG,NH) = 10.**FL
          ENDDO
        ENDDO
      ENDDO
    ENDDO

    !    Update wavelength counter
    NWMOD = J17 + 30

  END SUBROUTINE GRID_ADD_IR




  !  Restrict model grid to match IUE range and supplement with photometry

  SUBROUTINE GRID_RESTRICT

    ! identify start and end in model grid

    NW_MIN = NWMOD; NW_MAX = 0
    DO NW = 1,NWMOD
      IF (WMIN>WMOD(NW)) NW_MIN = NW
      IF (WMAX>WMOD(NW)) NW_MAX = NW + 1
    ENDDO

    ! number of data in iue range
    NWIUE = NW_MAX - NW_MIN + 1
    IF (WMAX>WMOD(NWMOD)) NWIUE = 0 


    ! shift all data to start of arrays
    DO NW = NW_MIN,NWMOD                       
      WMOD(NW+1-NW_MIN)       = WMOD(NW)
      FMOD(NW+1-NW_MIN,:,:,:) = FMOD(NW,:,:,:)
    ENDDO

    ! store number of wavelengths in range
    NWMOD = NWMOD - NW_MIN + 1


  END SUBROUTINE GRID_RESTRICT




  ! Visual and IR photometry: user supplied wavebands

  SUBROUTINE GRID_PHOT_BAND

    ! compute mean flux in each waveband weighted by filter response
    TOTAL_FLUX = 0.0
    DO NC = 1,NPHOBS                                
      TOTAL_WEIGHT = 0.0
      DO NW=2,NWMOD-1
        RESPONSE             = 0.5 * ( WMOD(NW+1) - WMOD(NW-1) ) & 
             * PHOT_RESPONSE ( WMOD(NW)/10.,PHOTSYS(NC:NC) )
        TOTAL_WEIGHT         = TOTAL_WEIGHT         + RESPONSE 
        TOTAL_FLUX(NC,:,:,:) = TOTAL_FLUX(NC,:,:,:) + RESPONSE * FMOD(NW,:,:,:)
      ENDDO
      TOTAL_FLUX(NC,:,:,:) =  TOTAL_FLUX(NC,:,:,:) / TOTAL_WEIGHT
    ENDDO

    ! Replace model fluxes with photometric means
    DO NC = 1,NPHOBS
      WMOD(NWIUE+NC)       = PHOTOBS_W(NC)
      FMOD(NWIUE+NC,:,:,:) = TOTAL_FLUX(NC,:,:,:)
    ENDDO

    ! final number of wavelengths in model
    NWMOD = NWIUE + NPHOBS                                  

  END SUBROUTINE GRID_PHOT_BAND





  SUBROUTINE FFIT_CHI_SQUARED


    WRITE (*,'(A5,10x,A8)')  'chi^2','E(B-V)'
    WRITE (*,'(A5,12F12.3)') 'T_eff',((J-1)*0.05,J=1,NEGRID)  
 
    !     Teff loop
    DO I = 1, NTMOD
      TEFF = TMOD(I)
      PARS(1) = TEFF

      !       EBV loop

       DO  J = 1,NEGRID
        EBV = 0.0 + (J-1)*0.05
        PARS(2) = EBV

        !         theta
        THETA = 1.; PARS(3) = THETA
        DO NW = NOBS1,NOBS2
          CALL FLUX_PRED (WBIN(NW),PARS,FFIT(NW),DERIVS,3)
        ENDDO
        CALL QD_TRAP_R ( WBIN,FFIT,SBIN,NOBS1,NOBS2, FTOT_MOD,STOT_MOD )
        THETA = SQRT ( FTOT_OBS / FTOT_MOD ); PARS(3) = THETA

        !         chi-squared
        CALL NR_CHISQ (WBIN,FBIN,SBIN,NWBIN, PARS,3, CHISQ,COVAR,NCHM, FLUX_PRED)
        CHSQ_TEST(I,J) = CHISQ
 
      ENDDO

      !       report 
      WRITE (*,'(F8.0,1P,12E12.3)') TEFF, (CHSQ_TEST(I,J),J=1,NEGRID)

    ENDDO


    !    Locate minimum
    IMIN = 1; JMIN = 1; CHISQ = 1.E30
    DO I = 1,NTMOD
      DO J = 1,NEGRID
        IF (CHSQ_TEST(I,J)<CHISQ) THEN
          IMIN = I; JMIN = J; CHISQ = CHSQ_TEST(I,J)
        ENDIF
      ENDDO
    ENDDO

    !    Recompute chi-squared for best position in grid
    TEFF = TMOD(IMIN)
    PARS(1) = TEFF
    EBV = 0.0 + (JMIN-1)*0.05
    PARS(2) = EBV

    !      theta
    THETA = 1.; PARS(3) = THETA
    DO NW = NOBS1,NOBS2
      CALL FLUX_PRED (WBIN(NW),PARS,FFIT(NW),DERIVS,3)
    ENDDO
    CALL QD_TRAP_R ( WBIN,FFIT,SBIN,NOBS1,NOBS2, FTOT_MOD,STOT_MOD )
    THETA = SQRT ( FTOT_OBS / FTOT_MOD ); PARS(3) = THETA

    !      chi-squared
    CALL NR_CHISQ (WBIN,FBIN,SBIN,NWBIN, PARS,NP, CHISQ,COVAR,NCHM, FLUX_PRED)

    !      invert covariance matrix to find standard errors
    DELTA_P(1:NP) = 1.
    CALL NR_GAUSSJ (COVAR,NCHM,NP,DELTA_P,1,1)


  END SUBROUTINE FFIT_CHI_SQUARED




  ! Solve using multidimensional downhill simplex method (AMOEBA)

  SUBROUTINE FFIT_AMOEBA

    !      Compute IUE flux integral
    CALL QD_TRAP_R ( WBIN,FBIN,SBIN,1,NWBIN, FTOT_OBS,STOT_OBS )

    !      re-evaluate starting theta (not independent variable)
    THETA = 1.; PARS(3) = THETA
    DO NW = NOBS1,NOBS2
      CALL FLUX_PRED (WBIN(NW),PARS,FFIT(NW),DERIVS,3)
    ENDDO
    CALL QD_TRAP_R ( WBIN,FFIT,SBIN,NOBS1,NOBS2, FTOT_MOD,STOT_MOD )
    THETA = SQRT ( FTOT_OBS / FTOT_MOD ); PARS(3) = THETA


    !      Exclude some data from IUE fit by increasing the standard deviations
    FBIN(1:NOBS1-1)=0.; SBIN(1:NOBS1-1) = 1.
    FBIN(NOBS2:NWIUE)=0.; SBIN(NOBS2:NWIUE) = 1.


    !      Set simplex vertices to starting guesses
    DO I=1,NCHM
      AMB__P(1:MP,I) = PARS(I) * (1+DELTA_P(I))
      AMB__P(I+1,I)  = PARS(I)
    ENDDO


    !      Evaluate function at corners
    DO I = 1,MP
      AMB__X(1:NP) = AMB__P(I,1:NP)
      COVAR(1,1) = -99.              ! do not evaluate covariance matrix
      CALL NR_CHISQ (WBIN,FBIN,SBIN,NWBIN,  AMB__X,NP, CHISQ,COVAR,NCHM, FLUX_PRED)
      AMB__Y(I) = CHISQ
    ENDDO


    !      Find solution
    CALL NR_AMBCHSQ (AMB__P, AMB__Y, MP, NP, NCHM, AMB__TOL, AMB__ITER, &
         FLUX_PRED, WBIN, FBIN, SBIN, NWBIN)
    DO I=1,NCHM
      PARS(I) = AMB__P(I+1,I)
    ENDDO


    !      Reevaluate final solution (to get covariances)
    COVAR(1,1) = 0.
    CALL NR_CHISQ (WBIN,FBIN,SBIN,NWBIN,  PARS,NP, CHISQ,COVAR,NCHM, FLUX_PRED)
    !      Invert covariance matrix to find standard errors
    AMB__X(1:NP) = 1.
    CALL NR_GAUSSJ (COVAR,NCHM,NP,AMB__X,1,1)


  END SUBROUTINE FFIT_AMOEBA





  !  Solve using a generalized non-linear least-squares method (Levenburg-Marquardt)

  SUBROUTINE FFIT_LEVENBURG


    !                            Initialize

    !      Compute initial estimate for theta (using trial Teff)
    DO NW = NOBS1,NOBS2
      CALL FLUX_PRED (WBIN(NW),PARS,FFIT(NW),DERIVS,3)
    ENDDO
    CALL QD_TRAP_R ( WBIN,FFIT,SBIN,NOBS1,NOBS2, FTOT_MOD,STOT_MOD )
    THETA = SQRT ( FTOT_OBS / FTOT_MOD ); PARS(3) = THETA
    WRITE (*,'(1X,F7.0,0P,F7.3,1P,E10.2)')   PARS

    !      Write out binned fluxes - for debugging
    OPEN  (1,FILE='FLUX',STATUS='UNKNOWN')
    WRITE (1,'(A)') OBSTITLE
    WRITE (1,*)
    WRITE (1,*) NWBIN
    WRITE (1,'(0P,F8.0,1P,2e12.3)') (WBIN(NW),THETA*THETA*FFIT(NW),SBIN(NW),NW=NOBS1,NOBS2)
    CLOSE (1)

    !      Initial call to chi-squared minimisation engine.
    DCONV  =  1.E-4
    ALAMDA = -0.01
    ALLAST = ALAMDA
    CALL NR_MRQMIN (WBIN,FBIN,SBIN,NWBIN, PARS,NP, LISTA,MFIT, &
         COVAR,ALPHA,NCHM, CHISQ, FLUX_PRED, ALAMDA)

    !      Report results.
    WRITE (*,'(1X,F7.0,0P,F7.3,1P,E10.2,1P,4E10.2)')  &
         PARS, CHISQ, 9.99, DCONV, ALLAST


    alamda = 1.e-4


    !                             Iterate


    FIND_TEFF: DO I = 1,20

      !        Find theta
      THETA = 1.; PARS(3) = THETA
      DO NW = NOBS1,NOBS2
        CALL FLUX_PRED (WBIN(NW),PARS,FFIT(NW),DERIVS,3)
      ENDDO
      CALL QD_TRAP_R ( WBIN,FFIT,SBIN,NOBS1,NOBS2, FTOT_MOD,STOT_MOD )
      THETA = SQRT ( FTOT_OBS / FTOT_MOD ); PARS(3) = THETA

      !        Next chi-squared minimization step
      !         ALAMDA = 1.E-4
      ALLAST = ALAMDA
      CALL NR_MRQMIN (WBIN,FBIN,SBIN,NWBIN, PARS,NP, LISTA,MFIT, &
           COVAR,ALPHA,NCHM, CHISQ, FLUX_PRED, ALAMDA)

      !  	 Test for convergence
      DPAR = 0.
      DPAR = DPAR + ABS(TEFF - PARS(1))/TEFF
      IF (MFIT>1) DPAR = DPAR + ABS(EBV-PARS(2))
      IF (MFIT>2) DPAR = DPAR + ABS(THETA-PARS(3))/THETA
      TEFF  = PARS(1)
      EBV   = PARS(2)
      THETA = PARS(3)

      !  	 Report results.
      WRITE (*,'(1X,F7.0,0P,F7.3,1P,E10.2,1P,4E10.2)')  &
           PARS, CHISQ, DPAR, DCONV, ALLAST

      IF (DPAR<DCONV .AND. DPAR>0.) EXIT 

    ENDDO  FIND_TEFF


    !      Final calls to find errors and save results
    CALL NR_MRQMIN (WBIN,FBIN,SBIN,NWBIN, PARS,NP, LISTA,MFIT, & 
         COVAR,ALPHA,NCHM, CHISQ, FLUX_PRED, 0.0)

    TEFF  = PARS(1); STEFF  = SQRT(COVAR(1,1))
    EBV   = PARS(2); SEBV   = SQRT(COVAR(2,2))
    THETA = PARS(3); STHETA = SQRT(STOT_OBS / FTOT_MOD)
    IF (MFIT>2) STHETA = SQRT(COVAR(3,3))


  END SUBROUTINE FFIT_LEVENBURG



END PROGRAM FLUXFIT

