PROGRAM STERNE_OS

!+

!  Program:
!     STERNE_OS

!  Version:
!     3.0.1

!  Language:
!     Approaching Fortran95

!  Purpose:
!     Calculate LTE models with line-blanketing for homogeneous
!     and inhomogeneous stellar atmospheres.


!  History (STERNE):
!     1.0       B.Wolf & D.Schoenberner, Berlin 1975
!     2.0       U.Heber & R.Moeller, Kiel       1990
!     2.1-2.3   C.S Jeffery, St. Andrews/Armagh 1990-1998
!     2.4       P.M.Harrison, Armagh            20-May-1998
!     2.4b      C.S.Jeffery, Armagh             24-Feb-1999
!     2.5       C.S.Jeffery, Armagh             04-Mar-1999
!     2.5b      C.S.Jeffery, Armagh             18-May-1999
!     2.5c      C.S.Jeffery, Armagh             20-Oct-2000
!     2.6       C.S.Jeffery, Armagh             13-Nov-2000
!     2.6a      C.S.Jeffery, Armagh             18-Feb-2002
!     2.6d      C.S.Jeffery, Armagh             26-May-2004
!
!  History (STERNE_OS):
!     3.0       N.T.Behara, Armagh              21-Nov-2005
!     3.0.1     C.S.Jeffery, Armagh             18-Jul-2007


!  Description:
!     This is a model atmosphere program originally written
!     to model RCrB and EHe star atmospheres. It will treat any early-type
!     star providing gravity is not too high (the equation of state breaks 
!     for white dwarf atmospheres), the effective temperature is not too 
!     high (we don't treat non-LTE) or low (neither do we consider molecules).
!     At high L/M ratios, one should be careful about winds, non-LTE and 
!     speherical geometry -- but this code does a pretty good job for low-mass
!     stars. Provision for stratified atmopsheres is present, but the
!     self-consistent calculation of diffusively stratified models is 
!     done by a later program.  
!
!
!  Version 1
!     A simple LTE  code written by Wolf & Schoenberner (Berlin) which
!     accounted for continuous opacity sources only, but which did include
!     extra data for carbon and nitrogen.
!     It was given to CCP7 and implemented on the Cray XMP and VAX/VMS
!     in about 1987.
!
!  Version 2
!     Subsequently Heber (Kiel) adapted the code to include He-line blocking
!     and some UV line-blanketing, and to allow some composition stratification.
!     Moeller (Kiel) computed opacity distribution
!     functions for He-rich mixtures, and these were incorporated in Version 2 of the code.
!     This was given to CCP7 in 1990, and implemented on the Cray XMP at RAL.
!
!  Version 2.1
!     This version represents a port to a Unix Sun SPARCstation. The
!     Cray code did not port easily, with very old Fortran giving problems.
!     Since the opacity library had already been ported as part of the QUB code SPECTRUM
!     the opacity sections were stripped out, and a very slightly modified form of the
!     the QUB library substituted - the library now known as OPK. A lot of work was left
!     to do integrating this library with the remaining STERNE code, and cleaning out other
!     dead code. The philosophy at this stage was to change as little as possible,
!     and only to fix code that was giving wrong answers. Hence a lot of improvements are
!     still possible.
!
!  Version 2.2
!     On the port to SOLARIS, version 2.1 broke down completely. It was not clear what
!     had happened - whether it was a compiler/arithmetic problem or some violation of
!     memory. Substantial sections of code were cleaned up looking for the bug, although
!     much remains to be done. The problem was traced to subroutine ALKOEF which was
!     giving bad values to array C when scattering dominated.
!     ALKOEF code was replace by code from PARABEL in the SPECTRUM package - I had already
!     solved the problem of poorly behaved transfer solutions in scattering layers.

!  Version 2.3
!     The stability of the Avrett-Loeser treatment of radiative transfer
!     was in some doubt, so it has been replaced by a Feautrier scheme. This
!     major change coincided with a port to OSF/1 and Linux. FLXODF has been
!     redically rewritten and now calls RTF_FEAUTRIER and RTF_PHI, where
!     the RTF library provides a number of radiative transfer utilities.
!
!  Version 2.4
!     A number of bug fixes have been incorporated to allow convergence over
!     a wide region of logg-Teff space. These have also been incorporated
!     into v2.3.
!     Most importantly however, the whole continuous opacity treatment
!     has been changed. In this version all b-f opacities are treated using
!     data for the Opacity Project.
!     Therefore version 2.4 does not yet replace version 2.3. Instead there
!     are now 2 codes with completely independent treatments of continuous
!     opacity.
!  2.4b
!     In version 2.4b the OP library has been isolated from the remainder
!     of the program, communication is entirely by subroutine calls.
!     Also, OPK is consistent between STERNE and SPECTRUM, removing a
!     conflict between the use of common blocks. Further work to isolate
!     subroutine libraries is probably needed in OPK, or else some
!     routines need to migrate from OPK to OP. Some bugs removed.
!
!  Main features:
!     Opacities:
!        Free-Free            -  Power Law approximations
!        Bound-free           -  Individual transition treatment using
!                                Opacity Project data
!        bound-bound          -  ATLAS6-type opacity distribution functions
!                                (from Kurucz & Peytremann linelists)
!     Convection              -  NOT TREATED
!     Radiative transfer      -  Avrett-Loeser (originally)
!                             -  Feautrier (since v2.3)
!     Temperature Correction  -  Lucy-Unsold
!
!  Version 2.5
!     We attempt to introduce "Opacity Sampling" in order to account for
!     the line opacity. Linelists are read in once the wavelength grid
!     is established, selecting lines within DNU of the prescribed
!     frequencies. This frequency grid is now completely flexible, both
!     the OP data and BB data are established at initialization.
!     The line opacity itself is required by a new subroutine FLXOS
!     which replaces FLXODF. The type of b-b opacity to use is selected
!     via the character variable 'KEY__OPBB' which may take one of the
!     following values.
!
!          'CONTINUUM': continuum only using subroutine FLXODF
!          'ODF': opacity distribution fucntion using FLXODF
!          'SAMPLING': opacity sampling using FLXOS
!  2.5b
!     We also find that the OP b-f opacities may not yet be trustworthy
!     so it is necessary to be able to switch between 'Kiel' and 'OP'
!     libraries. This will be done by a separate opacity keyword.
!
!         'KEY__OPBB' see above  will become       'KEY__OPBB'
!         continuous opacities will be given by 'KEY__OPBF'
!     allowed values are
!         'KIEL'      -  old ATLAS type opacities
!         'OP'        -  OP opacities
!     To be done:
!     Opacities:
!        EITHER Kurucz ATLAS9 - type odfs to be introduced
!        OR     b-b opacities from Opacity Project to be incorporated
!               possibly using an opacity sampling technique.
!     Code structure:
!        A complete rewrite is needed (with proper design methods) in order
!        to obtain a code which can be properly extended.
!  2.5c
!     Minor changes to 'inmod', 'outmod' to allow free-format reading of
!     datafiles, to 'inalpha' to preserve metal abundances from previous models
!     Removed 'band' since it only applied to restricted spectral synthesis
!     Hence parameter 'ifi' no longer required.
!
!     The line opacity given to FLXODF by LINOP was found to become
!     excessively high (10^23!!) for some low gravity models.
!     The fix is a cap on CAPODF of, currently, 10^6.  (21/9/00).
!     Some minor adjustments to the convergence criterion (TEMKOR) were also made.
!  2.6a -- 2.6d
!     Changes introduced to allow the use of ATLAS9 type ODFS using KEY__OPBB
!     with a value of OD9 (instead of ODF which refers to atlas6 type odf's).
!     Conversion for Linux  + Intel Fortran Compiler completed.
!     Improved handling of convergence criteria. Moved this within
!     subroutine TEMKOR.
!
!
!  Version 3.0
!     A full new version incorporating b-f x-sections from the Opacity 
!     Project and b-b opacity using opacity sampling.
!
!  Version 3.1
!     Moved to use the gfortran compiler  to allow more cross-platform
!     flexibity. This followed a decade of using the Intel fortran compiler 
!     on redhat linux boxes, including the 50-node cluster in Armagh.
!     A consequence of the compiler change was a that large number of 
!     issues were thrown up, which had to be resolved. At the same time, 
!     some code rationalization was implemented. There remain opportunities 
!     fore more code cleaning. 
!                                                     csj: 16.8.12
!

!  Principal References:
!
!  Version 2.6
!    C.S.Jeffery, V.M.Woolf, D.L.Pollacco, 2001. A&A 376, 497
!    Time-resolved spectral analysis of the pulsating helium star V652 Her
!  Version 3.0
!    Behara, N. T. , Jeffery, C. S., 2006. A&A, 451, 643-650
!    LTE model atmospheres with new opacities: I. Methods and General Properties
!
!-


! Global variables and constants
  USE stn_parameters
  USE stn_switches
  USE stn_composition
  USE stn_intgra
  USE stn_convergence

  USE lte_wlgrid
  USE lte_model
  USE op_peach
  USE op_global


! Switch off global typing
  IMPLICIT NONE    


!     Version information
  CHARACTER (6)   :: STN__VERSION = '3.4.0a'
  CHARACTER (10)  :: STN__DATE = '2020.02.13'
  CHARACTER (40)  :: STN__AUTHOR = 'N.T.Behara & C.S.Jeffery'
  CHARACTER (80)  :: STN__HELP = 'http://star.arm.ac.uk/%7Ecsj/software_store/guide/sterne.html'

  !     Command line control
  CHARACTER (100) :: COMMAND ! holder for current command line
  CHARACTER (12)  :: KEYWRD  ! holder for current command or keyword
  CHARACTER (LEN=16) :: PARCOM  ! command for parameter entry
  CHARACTER (LEN=2) :: CDBG  ! debugger keyword
  INTEGER :: NREAD, NWRITE

  !     Filenames and units
  CHARACTER (100) :: F__LINES = 'LINES'  !  Linelist
  CHARACTER (100) :: F__MODEL = 'MODEL'  !  Model atmosphere
  CHARACTER (100) :: F__ABUND = 'ABUNDS' !  Additional abundances
  INTEGER :: N__LINES
  INTEGER :: N__MODEL
  INTEGER :: N__ABUND = 20
  INTEGER :: F__ABUND_OK

  !  Simple counters etc.
  INTEGER :: I, J, COUNT

  !  CPU time monitoring
  REAL :: CPU0, CPU1, CPU2 ! counter


  !  Convergence checking (see stn_convergence)
  REAL :: ERR             ! Maximum error in flux
  REAL :: RMS             ! RMS temperature correction
  REAL :: EPS             ! Convergence criteria

  !  as above
  REAL, ALLOCATABLE, DIMENSION (:) :: T_NEW   ! New temperature (after correction) 
  REAL, ALLOCATABLE, DIMENSION (:) :: DT_0    ! for Temkor subroutine, saved correction
  REAL, ALLOCATABLE, DIMENSION (:) :: DELT    ! Last temperature correction
  REAL, ALLOCATABLE, DIMENSION (:) :: DELF    ! Eddington flux error (fractional)

 
  !  opacity switches (move to stn_switches ?)
  INTEGER :: newtemp_it = 0  ! tells subroutine linop that it has a new temperature
  CHARACTER(len=12) :: KEY_OPBF        ! Selects b-f opacity treatment: 'KIEL' or 'OP'
  CHARACTER(len=12) :: KEY_OPBB        ! Selects b-b opacity treatment: 'CONT', 'ODF'
  ! or 'SAMPLING'
  INTEGER :: LINE_COUNT


  ! Local composition and depth-dependent arrays (stn_composition, optical_depth?)
  INTEGER :: IZ  ! Local pointer for composition
  REAL    :: AB  ! Temporary value hoder for abundance
  REAL :: ALPNN(40), XX, YY
  REAL, ALLOCATABLE, DIMENSION (:) :: PPE, PP, CAP4, GRAD_MODEL, T_MODEL



!.

  CALL CPU_TIME (CPU0)
  CALL STN_STARTUP

!  Input.
!   Read model parameters and switches

!    effective temparature and gravity
  READ(*,*) TEFF, GLG
!    number of depth points
  READ(*,*) ND
  CALL STN_ALLOCATE

!    composition: H, He, C, N          ! (Li - B redundant)
  ALPHA = 0.
  READ(*,*) ALPMET(1:2),ALPMET(6:7)
!    composition: O, Mg, Al, Si, S, Ca, Fe
  READ(*,*) ALPMET(8), ALPMET(12), ALPMET(13), ALPMET(14), ALPMET(16), ALPMET(20), ALPMET(26)
  ALPHA(1:56,ND) = ALPMET(1:56)

  READ(*,*) LM, MAX_ITER  !! , IFO, KIMAX - last two variable thought redundant
!    grey model ?
  READ(*,*) IGREY
!    stratification parameters
  READ(*,*) IEL, TAUX0, ALPHAX0
  IF ( IEL>0 .AND. IEL<8) ALPHA(IEL,1) = ALPHAX0

!    opacity type switches
  READ(*,*) KEY_OPBF
  READ(*,*) KEY_OPBB
!    opacity sampling interval and microturbulence (BB_DVC, VMIC)
  READ(*,*) BB_DVC, VMIC, EP_0
!    number of opacity wavelengths and ref wavelentgh
  READ(*,*) N_OPWL
  ALLOCATE(OPWL(N_OPWL))
  READ(*,*) (OPWL(i), i=1,N_OPWL)
  READ(*,*) REFWL

  WRITE(*,*)
  WRITE(*,*) 'MODEL PARAMETERS'
  WRITE(*,*)
  WRITE(*,*) 'EFFECTIVE TEMPERATURE  :  TEFF ',TEFF
  WRITE(*,*) 'SURFACE GRAVITY        : LOG G ',GLG
  WRITE(*,*) 'MICROTURBULENT VELOCITY:  VMIC ',VMIC
  WRITE(*,*)
  WRITE(*,*) 'CONTINUOUS (BF) OPACITY:       ',KEY_OPBF
  WRITE(*,*) 'LINE (BB) OPACITY      :       ',KEY_OPBB
  WRITE(*,*) 'NUMBER OF DEPTH POINTS :       ',ND
  WRITE(*,*) 'NUMBER OF FREQUENCIES  : (???) ',N_OPWL
  IF (IGREY ==0) & 
  WRITE(*,*) 'STARTING MODEL         :       GREY '
  WRITE(*,*)

  

  !! Extra abundances -- currently uniform composition
  
  !! Check for existence of additional input file
  !! This is a temporary solution to add more opacities without disrupting the exisitng I/O
  !! So ABUNDS must use the same units as the main input file -- which is not what Spectrum uses.
  !! This should be improved in the next version
  OPEN (UNIT=N__ABUND,FILE=F__ABUND,IOSTAT=F__ABUND_OK,STATUS='OLD')
  IF (F__ABUND_OK == 0) THEN
     WRITE(*,*) 'ADDITIONAL ABUNDANCES :     ',F__ABUND
     DO WHILE (F__ABUND_OK == 0)
        READ (N__ABUND,*,IOSTAT=F__ABUND_OK) IZ, AB
        IF (F__ABUND_OK == 0) THEN
           ALPMET(IZ)   = AB
           ALPHA(IZ,ND) = AB
        ENDIF
     ENDDO
  ENDIF


  
  !! initialise values:
  DELT = 0.0
  T_NEW = 0.0

  IF (IGREY /= 0) THEN
     WRITE(*,*) 'Using converged model to start iteration'
     CALL INMOD (XX,YY,ALPNN,OP_MAXNA,T_MODEL,GRAD_MODEL,CAP4,PP,PPE,ND)

     ! Use Teff and/or log g from previous model
     TEFF = XX
     GLG = YY

     ! Use composition from previous model
     DO i=1,OP_MAXNA
        DO j =1,ND
           OP_ALPNN(i,j) = ALPNN(i)
        END DO
     END DO

     ! import pressure, temperature and radiation pressure from previous model
     DO  J = 1,ND      
        P_E(j) = 10.**PPE(j)             
        P_TOTAL(j) = 10.**PP(j)             
        KAPPA(j)=10.**CAP4(j)
        GRAD(j) = GRAD_MODEL(j)
        TEMP(j) = T_MODEL(j)
     ENDDO

     WRITE (*,*) 'Model read'

     !****************
  ELSE
     !! INALPHA: call subroutine to set up 
     !! composition data, if previous model was not supplied (if GREY==1)
     CALL STN_INALPHA
  END IF

  !! read in wavelength grid, wl, and allocate and initialise emergent fluxes 
  !! and line blocking factors to be used when calculating the emergent flux 

  CALL READ_WLGRID
  WRITE(*,*) 'WAVELENGTH GRID READ'

  CALL STN_STAND
  WRITE(*,*) 'OPTICAL DEPTH STRATIFICATION DEFINED'

  CALL OP_ALP

  CALL STN_STRATI
  WRITE(*,*) 'CHEMICAL STRATIFICATION DEFINED (IF ANY)'

  !! read in OP continuous opacities depending 
  !! on the choice of KEY_OPBF in the input file 
  !! read in OP continuous opacities depending 
  !! on the choice of KEY_OPBF in the input file 

  IF (KEY_OPBF(1:2) == 'OP') THEN
     CALL OP_BFIN 
     WRITE(*,*) 'OP BF OPACITIES READ' 
     CALL OP_FEIN
     WRITE(*,*) 'FE BF OPACITIES READ'   
  ELSE
     CALL OP_PEACHIN
     WRITE(*,*) 'PEACH BF OPACITY TABLES READ'
  END IF

  !! read in linelist for opacity sampling depending
  !! on the choice of KEY_OPBB in the input file

  IF (KEY_OPBB(1:3) == 'SAM') THEN
     CALL BB_IN (BB_DVC) 
  END IF

  CALL CPU_TIME(CPU1)
  WRITE(*,*) 'DATA PREP COMPLETE:',CPU1-CPU0,' s'
  WRITE(*,*)
  

!!!!   All relevant data has been read   !!!!
!!!!     Start computing                 !!!!

  !! Start temperature iteration    

  WRITE(*,*) 'START TEMPERATURE ITERATION'


  DO COUNT=1,MAX_ITER

     NEWTEMP_IT = NEWTEMP_IT + 1    ! iteration number for subroutine linop

     CALL OP_ATFILL (ND)
     
     !! Hydrostatic equilibrium 
     CALL STN_HYDRO (KEY_OPBF) 
     
     !! Compute ionization fractions
     CALL OP_IONFRN (ND)

     !! set line count to 0:
     LINE_COUNT = 0    

     !! Calculate emergent flux.
     WRITE(*,*) 'flx ',KEY_OPBB(1:3)
     IF (KEY_OPBB(1:3) == 'ODF') THEN
        CALL FLXODF (NEWTEMP_IT,KEY_OPBF)
     ELSE IF (KEY_OPBB(1:3) == 'SAM') THEN
        CALL FLXOS (VMIC,KEY_OPBF, LINE_COUNT)
    ELSE IF (KEY_OPBB(1:3) == 'CON') THEN
        CALL FLXCON (NEWTEMP_IT,KEY_OPBF)
     END IF

     WRITE(*,*) 'BB OPACITY: NO. LINES:', line_count

     !! Temperature correction.
     CALL STN_TEMKOR (T_NEW,DT_0,DELT,DELF,ERR,RMS)
 
     !! Test convergence
!!! also need to test for abundances...

      WRITE(*,*) 'Test convergence', COUNT, ERR

      IF (ERR <= EPS) THEN
         WRITE (*,*) 'Flux converged'
         EXIT
      END IF

      IF (RMS <= EPS/100.0) THEN
         WRITE (*,*) 'Temperature correction converged'
         EXIT
      END IF

!      IF (LM == IFO) THEN
!         WRITE(*,*) 'Maximum number of iterations reached'
!         EXIT
!      END IF

     !! Output current model

      CALL OUTPUT1(KEY_OPBF,DELT,1,T_NEW)
      CALL OUTMOD
      CALL OUTMOD2(KEY_OPBF,KEY_OPBB)

     !! apply temperature correction

      DO I = 1,ND
         TEMP(I) = T_NEW(i)
      END DO

     LM = LM+1

     CALL CPU_TIME(CPU2)
     WRITE(*,'(/,x,a,i4,a,i6,a/)') &
          'LAST ITERATION TOOK: ',INT(CPU2-CPU1),' S: TOTAL ELAPSED: ',INT(CPU2-CPU0),' S'
     CPU1=CPU2

  END DO
  !! end temperature iteration

  !! Output final model after convergence or final iteration
  !! including files for DIPSO and SPECTRUM

   CALL OUTPUT1 (KEY_OPBF,DELT,0,T_NEW)
   CALL OUTMOD
   CALL OUTMOD2 (KEY_OPBF,KEY_OPBB)
   CALL STERNE_QUB



CONTAINS

  SUBROUTINE STN_STARTUP

    !+
    !
    !     S T N _ S T A R T U P
    !
    !     This subroutine sets default values for flags, switches and variables
    !     and initialises the i/o streams for the command interpreter
    !
    !-

    INTEGER, DIMENSION(8) :: DATE_VALS

    !.

    WRITE (*,*)
    WRITE (*,*)
    WRITE (*,*) '  SSS  TTTTT  EEEEE  RRR   N   N  EEEEE          OOO    SSS   '
    WRITE (*,*) ' S       T    E      R  R  NN  N  E             O   O  S      '
    WRITE (*,*) '  SS     T    EEE    RRR   N N N  EEE           O   O   SS    '
    WRITE (*,*) '    S    T    E      R R   N  NN  E             O   O     S   '
    WRITE (*,*) ' SSS     T    EEEEE  R  R  N   N  EEEEE  _____   OOO   SSS    '
    WRITE (*,*)
    WRITE (*,*) ' Version ', STN__VERSION
    WRITE (*,*) ' Date    ', STN__DATE
    WRITE (*,*) ' Authors ', STN__AUTHOR
    WRITE (*,*) ' Help    ', STN__HELP
    WRITE (*,*)

    CALL DATE_AND_TIME ( VALUES = DATE_VALS )
    WRITE (*,'(A,I5,2I3)') '  Run Date ', DATE_VALS(1:3)
    WRITE (*,'(A,2(I2,'':''),I2)') '  Run Time  ', DATE_VALS(5:7)
    WRITE (*,*)

    !     Set up default i/o channels
    CALL STR_DEF (NREAD, NWRITE)
    COMMAND(1:20) = '                    '
    COMMAND(21:40) = '                    '
    COMMAND(41:60) = '                    '
    COMMAND(61:80) = '                    '
    COMMAND(1:11) = 'stn_startup'
    KEYWRD(1:3)   = '   '


    !! initialise counters:
    NIT = 0

    !  Convergence criterion for max(df/f)           
    EPS = 0.001


  END SUBROUTINE STN_STARTUP
  
  SUBROUTINE STN_ALLOCATE

    !+
    !
    !     S T N _ A L L O C A T E
    !
    !     This subroutine allocates arrays dependent on number of depth points
    !
    !-

    ALLOCATE(PPE(ND))
    ALLOCATE(PP(ND))
    ALLOCATE(CAP4(ND))
    ALLOCATE(GRAD_model(ND))
    ALLOCATE(T_model(ND))

    ALLOCATE(ALPHA(1:80,ND))
    ALLOCATE(ALPMET(1:80))
    ALPMET = 0.
    ALLOCATE(BETMET(1:80))
    ALLOCATE(EPSMET(1:80))
    ALLOCATE(tau(ND))
    ALLOCATE(alq(ND))
    ALLOCATE(temp(ND))
    ALLOCATE(t_new(ND))
    ALLOCATE(DELT(ND))
    ALLOCATE(DELF(ND))
    ALLOCATE(P_total(ND))
    ALLOCATE(P_e(ND))
    ALLOCATE(kappa(ND))
    ALLOCATE(kappa_nu(ND))
    ALLOCATE(kappa_bb(ND))
    ALLOCATE(sig(ND))
    ALLOCATE(GRAD(ND))
    ALLOCATE(DENS(ND))
    ALLOCATE(DIST(ND))
    ALLOCATE(XINT(ND))
    ALLOCATE(DT_0(ND))
    DT_0 = 0.0


    ALLOCATE(FLX(ND))
    ALLOCATE(TNS(ND))
    ALLOCATE(KAPP(ND))
    ALLOCATE(KAPJ(ND))
    ALLOCATE(KAPF(ND))
    ALLOCATE(B(ND))      

  END SUBROUTINE STN_ALLOCATE


END PROGRAM sterne_os

!!#######################################################################


