SUBROUTINE BB_IN (DVC)

  !+
  !
  !  Program.
  !     BB_IN
  !
  !  Purpose.
  !     Set up linelist for opacity sampling calculation  
  !
  !  Reference.
  !     (Sneden et al., 1976, ApJ 204, 281)
  !
  !  Method. 
  !     DNU defines the width in frequency units of the spectral region to
  !     be sampled. 
  !
  !     A linelist is read in; those lines which lie within the
  !     specified frequency intervals, and belong to allowed atom/ion
  !     species, are stored internally (at present). The start and end
  !     index of each frequency interval is also saved.
  !
  !  Language:
  !     Fortran 90
  !
  !  Input arguments:
  !     DVC                !  Interval (delta v / c) in which to search for lines
  !
  !  Authors:
  !     CSJ: C. S. Jeffery (Armagh Observatory)
  !
  !  History:
  !     03-MAR-1999 (CSJ): New program
  !     18-MAY-2004 (NTB): modified
  !-

  USE tapm_par
  USE tapp_par
  USE lte_wlgrid
  USE op_global
  USE bb_global

  IMPLICIT NONE


  !  Subroutine Arguments (Given). 
  REAL :: DVC     !  Interval (delta v / c) to search for lines

  !  External Programs Called.
  REAL, EXTERNAL :: BB_VOIGT          !  Local version of Voigt function

  !  Local variables.
  INTEGER :: I, J                 !  Frequency interval counter
  INTEGER :: IM1, IP1          !  I-1, I+1
  INTEGER :: KKK                !  Atom identifier
  INTEGER :: IBB               !  Saved line counter
  INTEGER :: JJ                !  Ion identifier

  REAL :: DNU      !  Frequency interval to search for lines
  REAL :: NUS, NUF !  Start and end frequencies for interval

  REAL :: LLID     !  atom/ion identifier
  REAL :: LLNU     !  line frequency
  REAL :: LLWL     !  line wavelength
  REAL :: LLWL2    !Angstroms
  REAL :: LLGF     !  gf - value
  REAL :: LLEX     !  lower state excitation potential
  REAL :: LLGLO    !  lower state statistical weight
  REAL :: LLEXHI   !  upper state excitation potential
  REAL :: LLGHI    !  upper state statistical weight
  REAL :: LLGAMR   !  gamma_r  ! radiative damping
  REAL :: LLGAMS   !  gamma_s  ! electron damping
  REAL :: LLGAMW   !  gamma_w  ! van der Waal's damping

  INTEGER :: NP                !  index for tabulated H function
  INTEGER :: NQ
  REAL :: A                 !  argument for H function
  REAL :: V

  INTEGER :: ID   ! switch for valid LLID, ID=0 is valid, ID=1 is not 
  INTEGER :: count ! counter for invalid lines

  !  Local constants.

  REAL, PARAMETER :: CMERG = TAPP__H*1.D7 * TAPP__C*1.D2  ! Conversion of energy (cm^-1) to (erg)

  REAL, PARAMETER ::  LOG4PI =2*TAPM__LG2 + TAPM__LGPI    ! log (4 pi)


  REAL :: LOG_GAMRC  ! Classical radiation damping
  REAL :: GAMRC = 5.9D-5 * 4*TAPM__PI * (TAPP__C*1.D2) / (1.D1)! Classical radiation damping

  INTEGER, PARAMETER :: LINES = 1210  ! Unit number for linelist

  !.

  LOG_GAMRC = LOG10 ( GAMRC )

  ALLOCATE(BB_NUS(LL))
  ALLOCATE(BB_NUF(LL))


  !  Convert wavelengths to frequency units
  DO I = 1,OP_NW
     OP_NU(I) = (TAPP__C * 1.e10) / OP_WL(I)
  ENDDO

  !  Store atomic masses of atoms in linelist
  DO I = 1,BB_MAXBA
     BB_AM(I) = 999                ! dummy value - makes lines sharp
  ENDDO
  DO I = 1,OP_MAXNA
     BB_AM(OP_ALPZ(I)) = OP_ALPAM(I)     ! value from OP common blocks
  ENDDO

  !  Open linelist file
  OPEN (UNIT=LINES,FILE='LINELIST',STATUS='OLD')

  !  Loop through the frequencies, collecting line data where required
  LLNU = 1.0
  IBB  = 1      
  DO I = 1,OP_NW
     DNU = -OP_NU(I) * DVC
     NUS = OP_NU(I) - DNU
     NUF = OP_NU(I) + DNU
     BB_NUS(I) = IBB
     BB_NUF(I) = IBB 

     !  Backspace linelist to find first line for each wavelength
     DO WHILE (LLNU < NUS)
        BACKSPACE (LINES) 
        BACKSPACE (LINES) 
        READ(LINES,'(1X,F10.0,F6.0,1X,F7.0,2(F12.0,2X,F4.0),3F7.0)') &
             &  LLWL, LLID, LLGF, LLEX, LLGLO, LLEXHI, LLGHI, &
             &  LLGAMR, LLGAMS, LLGAMW

        LLNU = (TAPP__C * 1.D09) / LLWL

     ENDDO

     !  Read the linelist (stored in wavelength order)
     DO WHILE (LLNU > NUF)
        READ(LINES,'(1X,F10.0,F6.0,1X,F7.0,2(F12.0,2X,F4.0),3F7.0)', &
             &  END=999) &
             &  LLWL, LLID, LLGF, LLEX, LLGLO, LLEXHI, LLGHI, &
             &  LLGAMR, LLGAMS, LLGAMW

        LLNU = (TAPP__C * 1.D09) / LLWL

        !       Store lines in given frequency intervals
        IF (NUS > LLNU.AND.LLNU > NUF) THEN

           !h only !           IF (INT(LLID) == 1) then
           BB_NUF(I)  = IBB 
           KKK        = INT(LLID)
           BB_KK(IBB) = KKK                                ! k
           JJ        = NINT((LLID-KKK) * 100) + 1
           BB_JJ(IBB) = JJ                                ! j
           BB_LN(IBB) = MAX( ABS(OP_NU(I)-LLNU), 1.D-10 )
           BB_LN(IBB) = LOG10 (BB_LN(IBB))                 ! log (nu-nu_L)
           BB_NU(IBB) = (OP_NU(I)-LLNU)*(OP_NU(I)-LLNU)     ! (nu-nu_L)^2 ! for Doppler approximation
           BB_EX(IBB) = LLEX * CMERG                      ! chi (erg)
           BB_GF(IBB) = LLGF * TAPM__LN10                 ! -> ln gf

           !         radiative damping approximation
           IF (LLGAMR.EQ.0) LLGAMR = LOG_GAMRC - LOG10(LLWL)
           BB_GAM(IBB) = LLGAMR - LOG4PI                  ! log (gamma/4pi)

           !! CSJ 31/3/20
           !! Something broke here when I wasn't looking.
           !! We are only including the radiative damping
           !! and not attempting the default Stark or VdW broadening. Why ?
           !! Possibly because they need the local electron and hydrogen density ...
           !! but I thought we addressed that in the OS paper.
           !! There is a section of code in 'spc_lines' which does deliver
           !! estimates for both.  .... 
           
           !!        electron damping approximation
           !!          EXRES = TAP_IONPT(KKK,JJ) - LLEXHI*CMERG 
           !!          IF ( EXRES.GE.0. ) THEN
           !!            ZEFF = SQRT(13.5*JJ*JJ/(EXRES))
           !!          ELSE
           !!            ZEFF = 5
           !!          ENDIF
           !!          LLGAMS = 1.E-16*EFFZ**5 

           !!        van der Waal's damping approximation -- to be done

           !!            write(*,*) IBB, BB_KK(IBB), BB_JJ(IBB), BB_EX(IBB), BB_GF(IBB)

           IBB       = IBB + 1

           !H only!          ENDIF

        ENDIF

     ENDDO

  ENDDO

  !  Report number of lines read and check array dimensions
999 CLOSE (UNIT=LINES)

  IF (I /= OP_NW) THEN          
     DO J=I,OP_NW
        BB_NUS(J) = IBB
        BB_NUF(J) = IBB - 1
     END DO
  END IF

!!$      write(1701,*) count

  WRITE (*,*) '   BB OPACITIES READ:  ',IBB,' LINES FROM LIST'
  IF (IBB .GT. BB_MAXNL) THEN
     WRITE (*,*) ' Too many lines in list'
     STOP
  ENDIF

!!$      close(1701)

  !  Set up tabulated Voigt function ln H(a,v), 
  !       on the intervals           log a = -7,..,.1, log v=-7,...,1 
  DO NQ = 0,BB_MAXNQ
     V = 10.**(BB_QRANGE*FLOAT(NQ)/FLOAT(BB_MAXNQ)+BB_QZERO)
     DO NP = 0,BB_MAXNP
        A = 10.**(BB_PRANGE*FLOAT(NP)/FLOAT(BB_MAXNP)+BB_PZERO)
        BB_HH(NP,NQ) = LOG ( BB_VOIGT ( A, V ) )
     ENDDO
  ENDDO

END SUBROUTINE BB_IN
