*+  CHR_RTOC - Encode a real value as a string.
      SUBROUTINE CHR_RTOC (RVALUE,CVALUE,NCHAR)
*    Description :
*     Encode a real number as a character string, using as concise a
*     format as possible, and return the number of characters used.
*     In the event of an error, `*'s are written to the string.
*    Invocation :
*     CALL CHR_RTOC (RVALUE, CVALUE, NCHAR)
*    Arguments :
*     RVALUE  =  REAL (Given)
*           Value to be encoded.
*     CVALUE  =  CHARACTER*(*) (Returned)
*           String into which the value is to be encoded.
*     NCHAR  =  INTEGER (Returned)
*           Field width used in encoding the value.
*    Method :
*     Base field width on string size, subject to the maximum
*     required to display any real number.
*     Write to string in G format.   Remove any blanks.
*     (Can then remove any trailing zeros in the decimal part,
*     and also any '+' signs.)
*    Deficiencies : Doesn't remove leading zero after '-' in
*     exponent. This has not been corrected in case it upsets people.
*    Authors :
*     Jack Giddings.      (UCL::JRG)
*     A C Davenhall.      (ROE::ACD)
*     A J Chipperfield.   (RAL::AJC)
*    History :
*     3-JAN-1983:  Original.                             (UCL::JRG)
*     2-OCT-1984:  Improve the documentation.            (ROE::ACD)
*     26-OCT-1984: Max. possible no. of decimal places   (ROE::ACD)
*                  inreased from 6 to 7.
*      1-Sep-1988: Use LEN instead of CHR_SIZE           (RAL::AJC)
*     15-Jun-1989: Improve speed and comments
*                  Rely on WRITE to clear buffer
*                  Remove + only from exponent when there is one
*                  Use string size of FIELD rather than multiple
*                  finding used length etc.              (RAL::AJC)
*    Type definitions :
      IMPLICIT NONE    ! Force Total Declarations.
*    Import :
      REAL
     :  RVALUE         ! Decoded value.
*    Export :
      CHARACTER
     :  CVALUE*(*)     ! String to be filled.
      INTEGER
     :  NCHAR          ! Field width.
*    External references :
      INTEGER
     :  CHR_LEN,       ! String length.
     :  INDEX,         ! Index of character in string.
     :  MIN,           ! Minimum value.
     :  MAX            ! Maximum value.
*    Local variables :
      INTEGER
     :  FIELD,         ! Field width.
     :  EXTRAS,        ! Extra characters needed.
     :  PREC,          ! Precision.
     :  SIZE,          ! Character count.
     :  CODE,          ! Local status.
     :  I1,            ! Loop start index.
     :  I2,            ! Loop end index.
     :  I              ! Loop index.
      CHARACTER
     :  FORMAT*12      ! Fortran 77 format string.
*-

*   Construct a suitable format string.
      SIZE = LEN(CVALUE)
      PREC = 7
      IF (RVALUE .LT. 0.0) THEN
         EXTRAS = 7
      ELSE
         EXTRAS = 6
      END IF
      FIELD = MIN(SIZE, PREC+EXTRAS)
      PREC = MAX(FIELD - EXTRAS, 0)
      WRITE (FORMAT, '(''(1PG'',I3,''.'',I3'')'')', IOSTAT=CODE) FIELD,
     :  PREC

      IF (CODE .EQ. 0) THEN
*      The format string was written OK. Use it to write the value
*      into the buffer.
         WRITE (CVALUE,FORMAT, IOSTAT=CODE) RVALUE

         IF (CODE .EQ. 0) THEN
*          Value is written into CVALUE(1:FIELD)
*          Assume the remainder of CVALUE is cleared by the WRITE.
!            CALL CHR_TERM(FIELD, CVALUE)

*
*          Handle trailing superfluous characters after decimal point.

            I1 = INDEX(CVALUE(1:FIELD), '.')

            IF (I1 .GT. 0) THEN
*            There is a decimal point.
*            Find index of E or D (assumed upper case from WRITE),
*            and set a pointer to the character before it.
               I2 = INDEX(CVALUE(1:FIELD), 'E')
               IF (I2 .EQ. 0) THEN
                  I2 = INDEX(CVALUE(1:FIELD), 'D')
                  IF (I2 .EQ. 0) THEN
                     I2 = FIELD
                  ELSE
*                  No exponent, set pointer to end of buffer
                     I2 = I2 - 1
                  END IF
               ELSE
                  I2 = I2 - 1
               END IF

*
*             Replace superfluous zeros after decimal point with spaces.
*             Work back from the E,D or end - ignore spaces.
               DO I = I2, (I1+1), -1
                  IF (CVALUE(I:I) .EQ. '0') THEN
                     CVALUE(I:I) = ' '
                  ELSEIF (CVALUE(I:I) .NE. ' ') THEN
                     GOTO 1
                  END IF
               END DO
 1             CONTINUE
            END IF

*
*         Replace superfluous leading zeros in exponent with space
            I2 = INDEX(CVALUE(1:FIELD), 'E')
            IF (I2 .LE. 0) THEN
               I2 = INDEX(CVALUE(1:FIELD), 'D')
            END IF
            IF (I2 .GT. 0) THEN
*            There is an exponent
               IF (I2 .LT. FIELD) THEN
*               Replace leading zeros and plus sign with space,
*               ignoring spaces.
*               (should really ignore minus also)
                  DO I = I2+1, FIELD
                     IF ((CVALUE(I:I) .EQ. '0') .OR.
     :                   (CVALUE(I:I) .EQ. '+')) THEN
                        CVALUE(I:I) = ' '
                     ELSEIF (CVALUE(I:I) .NE. ' ') THEN
                        GOTO 2
                     END IF
                  END DO
 2                CONTINUE
               ENDIF
            ENDIF


*         Remove all spaces from the string and find the used length.
            CALL CHR_RMBLK(CVALUE)
            NCHAR = CHR_LEN(CVALUE)

*
*          Remove terminal E, D or .
             DOWHILE (INDEX('ED.', CVALUE(NCHAR:NCHAR)).GT.0)
                CVALUE(NCHAR:NCHAR) = ' '
                NCHAR = NCHAR-1
             ENDDO

         ELSE
*         Failed to write value - Write *s into buffer.
            CALL CHR_FILL('*', CVALUE(1:FIELD))
            NCHAR = FIELD
         END IF

      ELSE
*      Failed to write format string - Write *s into buffer.
         CALL CHR_FILL('*', CVALUE(1:FIELD))
         NCHAR = FIELD
      END IF

      END

