MODULE LTE_STRUCTURES

  !+
  !
  !  Module:
  !     LTE_STRUCTURES
  !
  !  Purpose:
  !=    Provides data structure for model atmosphere
  !
  !  Language:
  !     Fortran 95
  !
  !  Authors:
  !     C.S.Jeffery (CSJ) (Armagh Observatory)
  !     N.T.Behara (NTB) (Armagh Observatory)
  !
  !  History:
  !     06-MAR-2006: CSJ
  !
  !  Use:
  !     USE LTE_STRUCTURES
  !
  !-


  !     Model data structure
  TYPE MODEL_ATMOS

     ! metadata
     CHARACTER(LEN=15) :: ORIGIN
     CHARACTER(LEN=15) :: DATE
     CHARACTER(LEN=50) :: LABEL

     ! key parameters

     REAL     :: TEFF
     REAL     :: GRAV
     REAL     :: FRACHE      !  He / ( H + He ) 
     REAL     :: FEBYH       ! [ Fe / H ]
     REAL     :: ALBYFE      ! [ alpha / Fe ]

     REAL     :: WAV_REF = 0.0
     INTEGER  :: NDEPTH  = 0
     INTEGER  :: NWAVE   = 0
     INTEGER  :: NELEM   = 0
     INTEGER  :: NELEM_STRAT = 0


     ! opacity parameters
     CHARACTER(LEN=5)  :: OP_BF 
     CHARACTER(LEN=5)  :: OP_BB
     REAL              :: OP_ODFINT
     CHARACTER(LEN=20) :: OP_ODFNAME
     CHARACTER(LEN=20) :: OP_LINELIST
     REAL              :: OP_BBWIN      ! Interval dnu/nu to search for lines 
     REAL              :: OP_VTURB        

     ! global abundances
     REAL, ALLOCATABLE, DIMENSION(:) :: ABUNDS

     ! physical structure
     REAL, ALLOCATABLE, DIMENSION(:) :: TAU
     REAL, ALLOCATABLE, DIMENSION(:) :: TEMP
     REAL, ALLOCATABLE, DIMENSION(:) :: OPAC
     REAL, ALLOCATABLE, DIMENSION(:) :: PGAS
     REAL, ALLOCATABLE, DIMENSION(:) :: PEL
     REAL, ALLOCATABLE, DIMENSION(:) :: GRAD
     REAL, ALLOCATABLE, DIMENSION(:) :: MASS
     REAL, ALLOCATABLE, DIMENSION(:) :: Z

     ! stratified abundance structure
     INTEGER, ALLOCATABLE, DIMENSION(:) :: ZELEM_STRAT
     LOGICAL, ALLOCATABLE, DIMENSION(:) :: QELEM_STRAT
     REAL, ALLOCATABLE, DIMENSION(:,:) :: STRAT

     ! emergent spectrum
     REAL, ALLOCATABLE, DIMENSION(:) :: WAVE
     REAL, ALLOCATABLE, DIMENSION(:) :: FLUX
     REAL, ALLOCATABLE, DIMENSION(:) :: FCONT

     ! convergence
     REAL    :: CONV_EPS
     REAL    :: CONV_SIG
     LOGICAL :: CONV_OK   ! .true. if SIG < EPS

     ! units
     CHARACTER(LEN=10) :: UNITS_TEFF
     CHARACTER(LEN=10) :: UNITS_GRAV
     CHARACTER(LEN=10) :: UNITS_VTURB
     CHARACTER(LEN=10) :: UNITS_WAV_REF

     CHARACTER(LEN=10) :: UNITS_TAU
     CHARACTER(LEN=10) :: UNITS_TEMP
     CHARACTER(LEN=10) :: UNITS_PGAS
     CHARACTER(LEN=10) :: UNITS_PEL
     CHARACTER(LEN=10) :: UNITS_GRAD
     CHARACTER(LEN=10) :: UNITS_OPAC
     CHARACTER(LEN=10) :: UNITS_MASS
     CHARACTER(LEN=10) :: UNITS_Z

     CHARACTER(LEN=10) :: UNITS_WAVE
     CHARACTER(LEN=10) :: UNITS_FLUX

     CHARACTER(LEN=10) :: UNITS_ABUND


  END TYPE MODEL_ATMOS


  TYPE PULS_STATIC

     ! Represents structure of a static model created by PULS,
     ! the Bridger / Pilar pulsation code. PEL, KAP and TAU must 
     ! computed separately, since these are not directly available
     ! in the outputs from PULS. P, T, and R are given as functions of M.
     !
     ! Variables of this TYPE must be declared to be of dimension PULS__ND_STATIC, 
     ! the number of zones in the static models. 
     !
     ! This static structure would typically be read in at the same time
     ! as the dynamic (or hydro) model, and would be used to generate the
     ! static model as function of TAU.  
     ! The structure could be used to store static models of types
     ! 1) lagrangian        r_0   (m)
     ! [ 3) scaled eulerian   r_0   (tau) ] [ may not be required ]
     ! Ths subscript 0 is "required" in order to reduce coding errors.

     DOUBLE PRECISION :: M0  !  Mr
     DOUBLE PRECISION :: P0 
     DOUBLE PRECISION :: T0  
     DOUBLE PRECISION :: R0 
     DOUBLE PRECISION :: K0 
     DOUBLE PRECISION :: PEL0 
     DOUBLE PRECISION :: TAU0
     DOUBLE PRECISION :: DM0 !  dM
     DOUBLE PRECISION :: MM  !  M* - Mr

  END TYPE PULS_STATIC


  TYPE PULS_HYDRO

     ! Represents structure of a hydrodyanamic model created by PULS,
     ! the Bridger / Pilar pulsation code. PEL, KAP and TAU must 
     ! computed separately, since these are not directly available
     ! in the outputs from PULS. 
     ! P, T, R and V are given as functions of M and PHI, where
     ! PHI is some time- or phase-related variable. 
     ! In either case, the pulsation period must also be given,
     !
     ! Variables of this TYPE must be declared to be of dimension 
     ! (PULS__ND,PULS__NPHI) 
     ! where PULS__ND = the number of zones in the pulsations model,
     ! and PULS__NPHI = the number of phases to be treated. 
     !
     ! This dynamic structure would typically be read in at the same time
     ! as the static model, and would be used to generate the hydro
     ! model as function of TAU.  
     ! The structure could be used to store hydro models of types
     ! 1) lagrangian        r     (m,phi)
     ! 2) scaled lagrangian r/r_0 (m,phi)
     ! 3) scaled eulerian   r/r_0 (tau,phi)
     ! 4) renormalized      r_A * r/r_0 (tau,phi)   
     ! ** r_A is the value from an LTE model atmosphere, this (4) represents the
     !    model that will actually be used in the formal solution to compute a 
     !    spectrum. 

     DOUBLE PRECISION :: P   ! pressure
     DOUBLE PRECISION :: T   ! temperature
     DOUBLE PRECISION :: R   ! radius
     DOUBLE PRECISION :: L   ! luminosity
     DOUBLE PRECISION :: V   ! velocity
     DOUBLE PRECISION :: D   ! density
     DOUBLE PRECISION :: K   ! opacity (rosseland mean)

  END TYPE PULS_HYDRO


END MODULE LTE_STRUCTURES
