      SUBROUTINE CBLSQF (X,Y,NPTS, NORD, CBCOFF,CBXMIN,CBXMAX)
!+
!     PROGRAM:   CBLSQF
!     PURPOSE:      
!=                FIND THE CHEBYSHEV-SERIES APPROXIMATION FOR A SERIES OF DATA 
!                 BY THE METHOD OF LEAST SQUARES
!     REFERENCES: NAG LIBRARY MARK 12
!
!     HISTORY:
!     VERSION     MACHINE       DATE      AUTHOR/INSTITUTE
!     1.0         microVAX      22.02.88  C.S.JEFFERY / ST.ANDREWS
!     >>> WRAP UP FOR NAG LIBRARY CALL
!
!
!  SUBROUTINE ARGUMENTS
!  --------------------
!  X      - Array of X values 
!  Y      - Array of Y values 
! [W      - Array of weights - not used here]
!  NPTS   - Number of points
!  NORD   - Order of fit (0 < NORD+1 <= NPTS)
!
!  CBCOFF - Array of Chebyshev polynomial coefficients (dimension > NORD+1)
!  CBXMIN - Minimum of X-range for X-x normalisation
!  CBXMAX - Maximum of X-range for X-x normalisation
!
!  NAG LIBRARY CALL ARGUMENTS
!  --------------------------
!  XNAG   - Array of X-values
!  YNAG   - Array of Y-values
!  WNAG   - Array of weights (set = 1)
!  CBCOFS - Array of Chebyshev polynomial coefficients for NAG call
!  CBRMSE - Array of root mean square residuals for NAG call
!  NR     - Dimension of coefficient matrices (NR > NORD)
!  NC     - Dimension of coefficient matrices (NC > NORD)
!  IFAIL  - Error flag
!  WK1    - Work array (dimension > 3*NPTS)
!  WK2    - Work array (dimension > 2*NORD+2)
!-
      IMPLICIT REAL    (A-H,O-Z)
      IMPLICIT INTEGER (I-N)

!.....INPUT ARGUMENTS
      INTEGER NPTS, NORD
      REAL    X(*), Y(*), CBCOFF(*), CBXMIN, CBXMAX
!
!.....VARIABLES FOR NAG CALL
      PARAMETER (KMAX=9, NPMAX=1000)
      DOUBLE PRECISION   XNAG(NPMAX), YNAG(NPMAX), WNAG(NPMAX)
      DOUBLE PRECISION   CBCOFS(KMAX+1,KMAX+1), CBRMSE(KMAX+1)
      DOUBLE PRECISION   WK1(3*NPMAX), WK2(2*KMAX+2)
      INTEGER IFAIL, NO1, NR
!.

!.....DO SOME ERROR TRAPPING
      IF (NORD.GT.KMAX) THEN
         WRITE (*,*) 'CBLSQF: Order too large for current version'
         RETURN
      ELSEIF (NPTS.GT.NPMAX) THEN
         WRITE (*,*) 'CBLSQF: Too many data for current version'
         RETURN
      ENDIF

!.....IDENTIFY MINIMUM AND MAXIMUM OF X-RANGE TO RECOVER NORMALISATION
      CBXMIN = X(1)
      CBXMAX = X(NPTS)

!.....COPY X/Y ARRAYS INTO LOCAL ARRAYS
      DO 10 N = 1,NPTS
         XNAG(N) = X(N)
         YNAG(N) = Y(N)
         WNAG(N) = 1.0
10       CONTINUE

!     write (6,'(8f10.2)') (x(i),i=1,npts)


!.....MAKE CALL TO NAG LIBRARY
      NO1 = NORD+1
      NR  = KMAX+1
!          //////
      CALL E02ADF (NPTS, NO1, NR, XNAG,YNAG,WNAG, 
     1             WK1,WK2, CBCOFS, CBRMSE, IFAIL)
!          //////

!.....TRAP NAG FAILURE
      IF (IFAIL.NE.0) THEN
         WRITE (*,'(A,I4)') 'CBLSQF: Error in NAG/E02ADF: IFAIL = ',IFAIL
         RETURN
      ENDIF

!.....IDENTIFY BEST FIT
      RESMIN = 1.E38
      DO 20 I = 0,NORD
         IF (CBRMSE(I+1).LT.RESMIN) THEN
            RESMIN = CBRMSE(I+1)
            IORD   = I
         ENDIF
20    CONTINUE

!.....COPY REQUIRED CHEBYSHEV POLYNOMIAL COEFFICIENTS
      DO 30 J = 0,IORD
30       CBCOFF(J+1) = CBCOFS(IORD+1,J+1)
      DO 31 J = IORD+1,NORD
31       CBCOFF(J+1) = 0.0

!.....QUIT
      END
