      SUBROUTINE LSQPOL (X,Y,NPTS,NORDER,COEF,ERR,RMS,CH2,RML,FRT,R2B)
!+
!     PROGRAM:      LSQPOL
!     PURPOSE:      
!=    Performs a least squares fit of order up to ten.
!     REFERENCES:   BEVINGTON, P.140.
!
!     HISTORY:
!     VERSION       MACHINE       DATE      AUTHOR/INSTITUTE
!     0.0                           .  .69  P.R.BEVINGTON
!     >>> POLFIT
!     1.0           VAX 11/780      .  .87  S.A.BELL / ST.ANDREWS
!     1.1           microVAX           .87  C.S.JEFFERY / ST.ANDREWS
!     >>> STOP GAP MEASURE UNTIL I SORT OUT OWN VERSION
!
!
!   THIS ROUTINE PERFORMS A LEAST SQUARES FIT OF ORDER UP TO TEN.
!   NORMAL EQUATIONS ARE FORMED AND SOLUTION FOR COEFFICIENTS AND
!   ERRORS ARE MADE USING ROUTINE MATINVERR. VARIABLES ARE AS FOLLOWS.
!
!  X,Y    - X and Y data arrays forming pairs
!  NPTS   - Number of points
!  NORDER - Order of fit e.g. for 2, fit is of form y = a0 + a1.x + a2.x**2
!  NTERMS - Number of coefficients
!  COEF   - Array containing coefficients
!  ERR    - Array containing errors on the above
!  R      - Linear correlation coefficients for each term (except a0)
!  RMS    - Root mean square error of fit
!  CH2    - Chi**2 for fit
!  FRT    - F test statistic on fit
!  RML    - Multiple correlation coefficient
!-

      IMPLICIT REAL    (A-H,O-Z)
      IMPLICIT INTEGER (I-N)

      INTEGER NPTS,NORDER
      REAL    X(*),Y(*), COEF(11),ERR(11), RMS,CH2,RML,FRT,R2B
!             ......... CSJ MODIFIED 03.11.87
      REAL    XP(11),A(11,11),C(11),SUMXP(11),XMEAN(11),SIGMAX(11),R(11)
!.
      XP(1)  = 1.0
      SUMY   = 0.0
      SIGMA  = 0.0
      NTERMS = NORDER + 1

!  Zero summing arrays

      DO 2 I = 1,11
        SIGMAX(I) = 0.0
        XMEAN(I) = 0.0
        SUMXP(I) = 0.0
        C(I) = 0.0
        R(I) = 0.0
        COEF(I) = 0.0
        ERR(I) = 0.0
        DO 1 J = 1,11
          A(I,J) = 0.0
1       CONTINUE
2     CONTINUE

!  Accumulate sums for the following (XP contains X**M where M=0 to M=10)
!  SUMXP contains sum of X**(2*NORDER), SUMXPY contains sum of (X**NORDER)*Y,
!  SUMY contains sum of Y and SUMYY contains sum of Y**2.

      DO 7 N = 1,NPTS
        DO 3 M = 2,NTERMS
3          XP(M) = XP(M-1) * X(N)
        DO 4 I = 2,NTERMS
4         SUMXP(I) = SUMXP(I) + XP(I)
        SUMY = SUMY + Y(N)

!  Normal equations e.g.  S(x.y) = S(a0.x) + S(a1.x.x) + S(a2.x.x.x) + ...
!  are of the matrix form C = A.B where array C is LHS and array A is RHS.
!  B is the solution.

        DO 6 J = 1,NTERMS
          C(J) = C(J) + XP(J) * Y(N)
          DO 5 K = 1,NTERMS
5           A(J,K) = A(J,K) + XP(J) * XP(K)
6       CONTINUE
7     CONTINUE
!
      YMEAN = SUMY / NPTS
      DO 8 I = 2,NTERMS
8       XMEAN(I) = SUMXP(I) / NPTS
      DO 11 N = 1,NPTS
        SIGMA = SIGMA + (Y(N) - YMEAN) * (Y(N) - YMEAN)
        DO 9 M = 2,NTERMS
9         XP(M) = XP(M-1) * X(N)
        DO 10 J = 2,NTERMS
          SIGMAX(J) = SIGMAX(J) + (XP(J) - XMEAN(J)) 
     >                * (XP(J) - XMEAN(J))
          R(J) = R(J) + (XP(J) - XMEAN(J)) * (Y(N) - YMEAN)
10      CONTINUE
11    CONTINUE
      SIGMA = SQRT (SIGMA / (NPTS - 1))
      DO 12 J = 2,NTERMS
        SIGMAX(J) = SQRT (SIGMAX(J) / (NPTS - 1))
        R(J) = R(J) / ((NPTS - 1) * SIGMAX(J) * SIGMA)
12    CONTINUE

!  Call inversion routine to get matrix B i.e the array COEF and errors ERR.

      CALL MATINVERR (A,C,NTERMS,COEF,ERR)

!  Calculate the r.m.s error of the fit RMS and chi**2 of the fit CH2.

      RMS = 0.0
      DO 15 I = 1,NPTS
        DO 13 M = 2,NTERMS
13        XP(M) = XP(M-1) * X(I)
        YC = COEF(1)
        DO 14 J = 2,NTERMS
14         YC = YC + COEF(J) * XP(J)
        RMS = RMS + (YC - Y(I)) * (YC - Y(I))
15    CONTINUE
      CH2 = RMS / FLOAT (NPTS - NORDER - 1)
      RMS = SQRT (RMS / FLOAT (NPTS - NORDER - 1))

!  Calculate the multiple correlation coefficient RML and the F test statistic
!  FRT. Trap perfect straight lines with RML=1.0 for obvious reasons.

      RML = 0.0
      DO 16 J = 2,NTERMS
16      RML = RML +COEF(J) * R(J) * SIGMAX(J) / SIGMA
      IF(RML.EQ.1.0) THEN
        FRT = 9.99999E36
      ELSE
        FRT = (RML / NORDER) / ((1.0 - RML) / (NPTS - NORDER - 1))
      ENDIF
      R2B = 1.0 - (1.0 - RML) * (NPTS - 1) / (NPTS - NORDER - 1)
      RML = SQRT (RML)

!  Calculate the errors taking into account the r.m.s error of the fit.

      DO 17 I = 1,NTERMS
17      ERR(I) = ERR(I) * RMS

!  Finished

      END
