SUBROUTINE NR_AMBCHSQ (P,Y,MP,NP,NDIM,FTOL,ITER,FUNK, XX,YX,SX,NX)

  !+
  !
  !  NR_AMBCHSQ
  !
  !  PURPOSE:
  != Multidimensional minimization of a function of a n dimensional vector.
  !  
  !  DESCRIPTION:
  !
  !  AMOEBA performs a multidimensional minimization of the function 
  !  FUNK(X) where X is an NDIM-dimensional vector, by the downhill simplex 
  !  method of Nelder and Mead. 

  !  Input is a matrix P whose NDIM+1 rows are NDIM-dimensional
  !  vectors which are the vertices of the starting simplex. [Logical
  !  dimension of P are P(NDIM+1,NDIM); physical dimsnesions are input
  !  as P(MP,NP)]. Also input is the vector Y of length NDIM+1 whose 
  !  components must be pre-initialized to the values of FUNK evaluated 
  !  at the NDIM+1 vertices (rows) of P; and FTOL the fractional convergence 
  !  tolerance to be acheived in the function value (n.b.!).  
  !
  !  On output, P and Y will have been reset to NDIM+1 new points all within 
  !  FTOL of a minimum function value, and ITER gives the number of iterations 
  !  taken.
  !
  !  Press et al. 1989. Numerical Recipes
  !
  !  NR_AMBCHSQ is an extension to minimize the chi-squared residual between
  !  the function FUNK(P) where P are the function parameters to be found, 
  !  and the "observed" function YX(XX), where SX are the errors on the NX data 
  !  values.
  !
  !  This subroutine calls NR_CHISQ to evaluate chi^2, but does not require 
  !  the covariance matrix to be computed correctly. This is flagged by
  !  setting COVAR(1,1) = -99. 
  !
  !  Vincent Woolf / Simon Jeffery 
  !  2000 January 
  !
  !  2000 November 21:
  !       adapted to avoid problems when parameters may take zero values
  !  2000 December 22:
  !       avoid computing COVAR in NR_CHISQ - which can be costly.
  !
  !-

  !  Imported arrays (AMOEBA data)
  DIMENSION P(MP,NP),Y(MP)

  !  Imported arrays (FUNK data)
  DIMENSION XX(*), YX(*), SX(*)
  INTEGER   NX 

  !  Local constants.
  PARAMETER (NMAX=20,ALPHA=1.0,BETA=0.5,GAMMA=2.0,ITMAX=500)

  !  Local arrays.
  REAL, ALLOCATABLE :: PR(:), PRR(:), PBAR(:)

  !  Local variables.
  REAL, ALLOCATABLE :: COVAR(:,:) ! Array of covariances
  REAL      CHISQ             ! Chi-squared residual


  !.

  MPTS=NDIM+1
  ITER=0

  ALLOCATE (PR(MP))
  ALLOCATE (PRR(MP))
  ALLOCATE (PBAR(MP))
  ALLOCATE (COVAR(MP,MP))

 ITERATION: DO WHILE (ITER < ITMAX)
  !  Determine which point is the highest (worst), next-highest and lowest (best)
  ILO=1
  IF(Y(1).GT.Y(2))THEN
     IHI=1
     INHI=2
  ELSE
     IHI=2
     INHI=1
  ENDIF

  !  ... by looping over the points in the simplex
  DO I=1,MPTS
     IF(Y(I).LT.Y(ILO)) ILO=I
     IF(Y(I).GT.Y(IHI))THEN
        INHI=IHI
        IHI=I
     ELSE IF(Y(I).GT.Y(INHI))THEN
        IF(I.NE.IHI) INHI=I
     ENDIF
  ENDDO


  !  Compute the absolute/fractional range from highest to lowest 
  !  and return if satisfactory
  ABAR=0.5*(ABS(Y(IHI))+ABS(Y(ILO)))
  RTOL = ABS(Y(IHI)-Y(ILO))
  IF (ABAR.GT.FTOL) RTOL = RTOL / ABAR
  IF (RTOL.LT.FTOL) EXIT

  !  Compute the fractional change in parameters 
  !  and return if they have converged
  PTOL=0.
  DO J = 1,NDIM
     RMQ = 99999
     DO I = 1,NDIM+1 
        RMQ = MIN( RMQ, ABS( P(I,J) ) )
     ENDDO
     RMP = 0.
     DO I = 1,NDIM+1
        RMP = MAX( RMP, ABS( P(I,J) ) - RMQ  ) 
        IF (RMQ.GT.FTOL) RMP = RMP / RMQ
     ENDDO
     PTOL = MAX( PTOL, RMP ) 
  ENDDO
  IF(PTOL.LT.FTOL) EXIT

  ITER=ITER+1

  !  Begin a new iteration. Compute the vector average of all points except
  !  the highest, i.e. the center of the "face" of the simplex across from
  !  the high point. We will subsequently explore along the ray from the high
  !  point through that center.
  DO J=1,NDIM
     PBAR(J)=0.
  ENDDO

  DO I=1,MPTS
     IF(I.NE.IHI)THEN
        DO J=1,NDIM
           PBAR(J)=PBAR(J)+P(I,J)
        ENDDO
     ENDIF
  ENDDO

  !  Extrapolate by a factor ALPHA through the face, 
  !  i.e. reflect the simplex from the high point.
  DO J=1,NDIM
     PBAR(J)=PBAR(J)/NDIM
     PR(J)=(1.+ALPHA)*PBAR(J)-ALPHA*P(IHI,J)
  ENDDO

  !  Evaluate the function at the reflected point.
  COVAR(1,1) = -99.
  CALL NR_CHISQ (XX,YX,SX,NX, PR,NP, CHISQ,COVAR,NMAX, FUNK)

  YPR=CHISQ

  !  If the result is better than the best point, 
  !  try an additional extrapolation by a factor GAMMA,
  IF(YPR.LE.Y(ILO))THEN

     DO J=1,NDIM
        PRR(J)=GAMMA*PR(J)+(1.-GAMMA)*PBAR(J)
     ENDDO

     !     and check out the function there.
     COVAR(1,1) = -99.
     CALL NR_CHISQ (XX,YX,SX,NX, PRR,NP, CHISQ,COVAR,NMAX, FUNK)
     YPRR=CHISQ

     !     The additional extrapolation succeeded, and replaces the high point.
     IF(YPRR.LT.Y(ILO))THEN
        DO J=1,NDIM
           P(IHI,J)=PRR(J)
        ENDDO
        Y(IHI)=YPRR

        !     The additional extrapolation failed, but we can still use the 
        !     reflected point.
     ELSE
        DO J=1,NDIM
           P(IHI,J)=PR(J)
        ENDDO
        Y(IHI)=YPR
     ENDIF

     !  The reflected point is worse than the second-highest. 
  ELSE IF(YPR.GE.Y(INHI))THEN

     !     If its better than the highest, then replace the highest,
     IF(YPR.LT.Y(IHI))THEN
        DO J=1,NDIM
           P(IHI,J)=PR(J)
        ENDDO
        Y(IHI)=YPR
     ENDIF

     !     but look for an immediate lower point. In other words, perform
     !     an immediate contraction of the simplex along one dimension.
     DO J=1,NDIM
        PRR(J)=BETA*P(IHI,J)+(1.-BETA)*PBAR(J)
     ENDDO

     !     Then evaluate the function.
     COVAR(1,1) = -99.
     CALL NR_CHISQ (XX,YX,SX,NX, PRR,NP, CHISQ,COVAR,NMAX, FUNK)
     YPRR=CHISQ

     !     Contraction gives an improvement, so accept it.
     IF(YPRR.LT.Y(IHI))THEN
        DO J=1,NDIM
           P(IHI,J)=PRR(J)
        ENDDO
        Y(IHI)=YPRR

        !     Can't seem to get rid of that high point. Better contract around
        !     the lowest (best) point.
     ELSE
        DO I=1,MPTS
           IF(I.NE.ILO)THEN
              DO J=1,NDIM
                 PR(J)=0.5*(P(I,J)+P(ILO,J))
                 P(I,J)=PR(J)
              ENDDO
              COVAR(1,1) = -99.
              CALL NR_CHISQ (XX,YX,SX,NX, PR,NP, CHISQ,COVAR,NMAX, FUNK)
              Y(I)=CHISQ
           ENDIF
        ENDDO
     ENDIF

     !  We arrive here if the original refelction gives a middling point.
     !  Replace the old high point and continue
  ELSE
     DO J=1,NDIM
        P(IHI,J)=PR(J)
     ENDDO
     Y(IHI)=YPR
  ENDIF

  !  for the test of doneness and the next iteration
  ENDDO ITERATION

  !  report what's happening
  IF (ITER4s == ITMAX) WRITE (*,*)  'AMOEBA exceeding maximum iterations.'

  DEALLOCATE (PR)
  DEALLOCATE (PRR)
  DEALLOCATE (PBAR)
  DEALLOCATE (COVAR)

END SUBROUTINE NR_AMBCHSQ
