SUBROUTINE NR_CHISQ (X,Y,S,N, P,NP,CHISQ,COVAR,NC, FUNC)

  !+
  !  Program:
  !     NR_CHISQ

  !  Purpose:
  !=    Calculate chi-squared on data y(x) wrt a fitting function f(x)

  !  Type:
  !     SUBROUINE subprogram

  !  Description:
  !     Calculate chi-squared CHISQ on data Y(X), each datum with
  !     error S(X), with respect to a fitting function FUNC(X,P), 
  !     where P are NP independent parameters. The covariance matrix
  !     COVAR(NC,NC) is also computed for all NP independent parameters
  !
  !     On input, COVAR(1,1) carries a value that is passed to FUNC
  !     as the first element in the derivative vectory DYDA. This allows
  !     the calling routine to determine, for example, whether derivative
  !     need to be calculated. Its a rather messy trick, but was introduced
  !     to avoid much more substantial software engineering.

  !  Arguments given:
  !     X	(real)           array of x-values
  !     Y (real)           array of y-values
  !     S (real)           array of standard deviations on y-values
  !     N (integer)        number of data in X,Y,S
  !     P (real)           values of parameters to pass to function FUNC
  !     NP (integer)       number of parameters P


  !  Reference:
  !     Press et al. 1989, Numerical Recipes, p. 526

  !  Authors:
  !     Press et al. 
  !     C.S.Jeffery (CSJ) (St Andrews)

  !  History:
  !     21-DEC-1994: CSJ
  !        original version, adapted from Numerical Recipes
  !     22-DEC-2000: CSJ
  !        introduction of switch to avoid use of covariances

  !-

  !  Global typing:
  IMPLICIT NONE

  !  Subroutine arguments (given):

  REAL X(*)
  REAL Y(*)
  REAL S(*)
  INTEGER N

  REAL P(*)
  INTEGER NP
  INTEGER NC

  !  Subroutine arguments (returned):
  REAL CHISQ
  REAL COVAR(NC,NC)


  !  Local constants:
  INTEGER MMAX
  PARAMETER (MMAX=20)        ! maximum number of coefficients MA

  !  Local arrays:
  REAL DYDA(MMAX)

  !  Local variables:
  INTEGER I, J, K
  REAL  XX
  REAL  YY
  REAL  DY
  REAL  S2I, WT

  EXTERNAL FUNC

  !.

  !   Pass flag that covariances are not required
  DYDA(1) = COVAR(1,1)


  !   Initialize chi-squared and covariance matrix
  DO J = 1,NP
     DO K = 1,J
        COVAR(J,K) = 0.
     ENDDO
  ENDDO
  CHISQ = 0.

  !  Sum chi-squared and covariance matrix over all data
  DO I = 1,N
     XX = X(I)
     CALL FUNC (XX,P,YY,DYDA,NP)
     S2I = 1./(S(I)*S(I))
     DY = Y(I) - YY
     DO J = 1,NP
        WT = DYDA(J) * S2I
        DO K = 1,J
           COVAR(J,K) = COVAR(J,K) + WT*DYDA(K)
        ENDDO
     ENDDO

     !     and find chi-squared.
     CHISQ = CHISQ + DY*DY*S2I
  ENDDO


  !  Fill in the symmetric side.
  DO J = 2,NP
     DO K = 1,J-1
        COVAR(K,J) = COVAR(J,K)
     ENDDO
  ENDDO


END SUBROUTINE NR_CHISQ
