SUBROUTINE NR_LFIT (X,Y,SIG,NDATA, A,MA,LISTA,MFIT, &
     COVAR,NCVM,CHISQ, FUNCS)

  !+
  !  Program:
  !     NR_LFIT

  !  Purpose:
  !=    Carry out a general least squares regression.

  !  Type:
  !     SUBROUINE subprogram

  !  Description:
  !     Given a set of NDATA points X(I),Y(I) with individual standard deviations
  !     SIG(I), use chi-squared minimisation to determine MFIT of MA coefficient 
  !     A of a function that depends linearly on A, y = SUMi[ Ai*AFUNCi(x) ]. 
  !     The array LISTA renumbers the parameters so that the first MFIT elements 
  !     correspond to the parameters actually being determined; 
  !     the remaining MA-MFIT elements are held fixed at their input values. The 
  !     program returns values for the MA fit parameters A, chisquared=CHISQ, and
  !     the covariance matrix COVAR(I,J). NCVM is the physical dimension of
  !     COVAR(NCVM,NCVM) in the calling routine. The user supplies a subroutine
  !     FUNCS(X,AFUNC,MA) that returns the MA basis functions evaluated at
  !     x=X in the array AFUNC.

  !  Arguments given:
  !     X	(real)           array of x-values
  !     Y (real)           array of y-values
  !     SIG (real)         array of standard deviations on y-values
  !     NDATA (integer)    number of data in X,Y,SIG
  !     

  !  Reference:
  !     Press et al. 1989, Numerical Recipes, p. 513

  !  Authors:
  !     Press et al. 
  !     C.S.Jeffery (CSJ) (St Andrews)

  !  History:
  !     1-DEC-1994: CSJ
  !        original version, adapted from Numerical Recipes

  !-

  !  Constant and variable declarations:

  !  Subroutine arguments (given):
  INTEGER NDATA
  REAL X(*)
  REAL Y(*)
  REAL SIG (*)
  INTEGER MA
  INTEGER LISTA(*)
  INTEGER MFIT
  INTEGER NCVM

  !  Subroutine arguments (returned):
  REAL A(*)
  REAL COVAR(NCVM,NCVM)
  REAL CHISQ

  !  Local constants:
  PARAMETER (NMAX=50)        ! maximum number of coefficients MA

  !  Local arrays:
  REAL BETA(NMAX,1)
  REAL AFUNC(NMAX)

  !.

  !  Check to see that LISTA contains a proper permutation of the
  !  coefficients and fill in any missing members.
  KK = MFIT + 1
  DO J = 1,MA
     IHIT=0
     DO K=1,MFIT
        IF (LISTA(K).EQ.J) IHIT = IHIT + 1
     ENDDO
     IF (IHIT.EQ.0) THEN
        LISTA(KK) = J
        KK = KK + 1
     ELSEIF (IHIT.GT.1) THEN
        WRITE(*,*) 'nr_lfit: improper set in LISTA'
     ENDIF
  ENDDO
  IF (KK.NE.(MA+1)) WRITE(*,*) 'nr_lfit: improper set in LISTA'


  !  Initialize the (symmetric) matrix
  DO J = 1,NCVM
     DO K = 1,NCVM
        COVAR(J,K) = 0.
     ENDDO
     BETA(J,1) = 0.
  ENDDO


  !  Loop over data to accumulate coefficients of the normal equations.
  DO I = 1,NDATA
     CALL FUNCS (X(I),AFUNC,MA)
     YM = Y(I)

     !     Subtract off dependencies on known pieces of the fitting function
     IF (MFIT.LT.MA) THEN
        DO J = MFIT+1,MA
           YM = YM - A(LISTA(J))*AFUNC(LISTA(J))
        ENDDO
     ENDIF

     SIG2I = 1./SIG(I)**2
     DO J = 1,MFIT
        WT = AFUNC(LISTA(J)) * SIG2I
        DO K = 1,J
           COVAR(J,K) = COVAR(J,K) + WT * AFUNC(LISTA(K))
        ENDDO
        BETA(J,1) = BETA(J,1) + YM * WT
     ENDDO
  ENDDO


  !  Fill in above the diagonal from symmetry.
  IF (MFIT.GT.1) THEN
     DO J = 2,MFIT
        DO K = 1,J-1
           COVAR(K,J) = COVAR(J,K)
        ENDDO
     ENDDO
  ENDIF


  !  Matrix solution
  CALL NR_GAUSSJ (COVAR,MFIT,NCVM,BETA,1,1)


  !  Partition solution to appropriate ceofficients A
  DO J = 1,MFIT
     A(LISTA(J)) = BETA(J,1)
  ENDDO


  !  Evaluate chi-squared of the fit
  CHISQ = 0.
  DO I = 1,NDATA
     CALL FUNCS (X(I),AFUNC,MA)
     SUM = 0.
     DO J = 1,MA
        SUM = SUM + A(J) * AFUNC(J)
     ENDDO
     CHISQ = CHISQ + ((Y(I)-SUM)/SIG(I))**2
  ENDDO


  !  Sort covariance matrix to true order of fitting coefficients
  IF (MFIT.GT.1) CALL NR_COVSRT (COVAR,NCVM,MA,LISTA,MFIT)


END SUBROUTINE NR_LFIT
