SUBROUTINE NR_MRQMIN (X,Y,SIG,NDATA, A,MA,LISTA,MFIT, &
     COVAR,ALPHA,NCA,CHISQ,FUNCS,ALAMDA)

  !+
  !  Program:
  !     NR_MRQMIN

  !  Purpose:
  !=    Carry out a non-linear regression using the Levenberg-Marquardt method

  !  Type:
  !     SUBROUINE subprogram

  !  Description:
  !     Levenberg-Marquardt method, attempting to reduce the value of chi-squared
  !     of a fit between a set of NDATA points X(I), Y(I) with individual standard
  !     deviations SIG(I), and a nonlinear function dependent on the MA 
  !     coefficients A. The array LISTA numbers the parameters A such that the
  !     first MFIT elements correspond to values actually being adjusted; the
  !     remaining MA-MFIT parameters are held fixed at their input values. The
  !     program returns current best-fit values for the MA fit parameters A, and
  !     chi^2=CHISQ. The arrays COVAR(NCA,NCA), ALPHA(NCA,NCA) with physical
  !     dimensions NCA (>=MFIT) are used as working space during most iterations.
  !     Supply a subroutine FUNCS(X,A,YFIT,DYDA,MA) that evaluates parameters
  !     A at X. On the first call provide an initial guess for the parameters A,
  !     and set ALAMDA<0 for initializaion (which then sets ALAMDA=-ALAMDA). If a
  !     step succeeds CHISQ becomes smaller and ALAMDA decreases by a factor of
  !     10. If a step fails ALAMDA grows by a factor of 10. You must call this
  !     routine repeatedly until convergence is achieved. Then, make one final
  !     call with ALAMDA=0, so that COVAR(I,J) returns the covariance matrix,
  !     and ALPHA(I,J) the curvature matrix.

  !  Arguments given:
  !     X	(real)           array of x-values
  !     Y (real)           array of y-values
  !     SIG (real)         array of standard deviations on y-values
  !     NDATA (integer)    number of data in X,Y,SIG


  !  Reference:
  !     Press et al. 1989, Numerical Recipes, p. 526

  !  Authors:
  !     Press et al. 
  !     C.S.Jeffery (CSJ) (St Andrews)

  !  History:
  !     21-DEC-1994: CSJ
  !        original version, adapted from Numerical Recipes

  !-

  !  Constant and variable declarations:
  IMPLICIT NONE

  !  Subroutine arguments:
  REAL,DIMENSION(*),INTENT(IN)    :: X
  REAL,DIMENSION(*),INTENT(IN)    :: Y
  REAL,DIMENSION(*),INTENT(IN)    :: SIG
  INTEGER,INTENT(IN)              :: NDATA

  REAL,DIMENSION(*),INTENT(INOUT) :: A       !(MA)
  INTEGER,INTENT(IN)              :: MA
  INTEGER,DIMENSION(*),INTENT(INOUT) :: LISTA   !(MFIT)
  INTEGER,INTENT(IN)              :: MFIT

  REAL,DIMENSION(NCA,NCA),INTENT(OUT) :: COVAR  !(NCA,NCA)
  REAL,DIMENSION(NCA,NCA),INTENT(OUT) :: ALPHA   !(NCA,NCA)
  INTEGER,INTENT(IN)              :: NCA
  REAL,INTENT(OUT)                :: CHISQ

  EXTERNAL :: FUNCS
  REAL,INTENT(INOUT)              :: ALAMDA

  !! in the above, * and NCA could be replaced with ':' but this would mean
  !! creating a subroutine interface which woudl have to be included in the
  !! calling routine. 

  !  Local constants:
  INTEGER, PARAMETER :: MMAX=50  ! maximum number of coefficients MA

  !  Local arrays:
  REAL, SAVE, DIMENSION(MMAX) :: BETA   !! 
  REAL ATRY(MMAX)
  REAL DA(MMAX,1)


  !  Local variables:
  REAL, SAVE :: OCHISQ                ! previous value of chi squared, saved to next use

  !  Lop counters and temporary array pointers 
  INTEGER  :: KK             ! MFIT+1  
  INTEGER  :: J, K           ! loop counters
  INTEGER  :: IHIT

  !.
  
  !  Initialization  
  IF (ALAMDA < 0.) THEN
     KK = MFIT + 1
     DO J = 1,MA
        IHIT = 0
        DO K = 1,MFIT
           IF (LISTA(K) == J) IHIT = IHIT + 1
        ENDDO
        IF (IHIT == 0) THEN
           LISTA(KK) = J
           KK = KK + 1
        ELSEIF (IHIT > 1) THEN
           WRITE(*,*) 'nr_mrqmin: Improper set in LISTA'
        ENDIF
     ENDDO
     IF (KK /= (MA+1)) WRITE(*,*) 'nr_mrqmin: Improper set in LISTA'
     ALAMDA=MIN(-ALAMDA,0.001)
     CALL NR_MRQCOF (X,Y,SIG,NDATA,A,MA,LISTA,MFIT, &
          ALPHA,BETA,NCA,CHISQ,FUNCS)
     OCHISQ = CHISQ
     ATRY(1:MA) = A(1:MA)
  ENDIF

  !  Alter linearized fitting matrix, by augmenting the diagonal elements
  DO J = 1,MFIT
     COVAR(J,1:MFIT) = ALPHA(J,1:MFIT)
     COVAR(J,J) = ALPHA(J,J)*(1.0+ALAMDA)
     DA(J,1) = BETA(J)
  ENDDO
  
  !  Matrix solution
  CALL NR_GAUSSJ (COVAR,MFIT,NCA,DA,1,1)
 
  !  Once converged evaluate covariance matrix with ALAMDA=0.
  IF (ALAMDA == 0.0) THEN
     CALL NR_COVSRT (COVAR,NCA,MA,LISTA,MFIT)
     RETURN
  ENDIF

  !  Did the trial succeed?
  ATRY(LISTA(1:MFIT)) = A(LISTA(1:MFIT))+DA(1:MFIT,1)
  IF (MFIT < MA) THEN
     ATRY(LISTA(MFIT+1:MA)) = A(LISTA(MFIT+1:MA))
  ENDIF
  CALL NR_MRQCOF (X,Y,SIG,NDATA,ATRY,MA,LISTA,MFIT, &
       COVAR,DA,NCA,CHISQ,FUNCS)

  !  Success, accept the new solution
  IF (CHISQ < 1.1*OCHISQ) THEN
     !!                 ****   test
     ALAMDA = 0.1*ALAMDA
     OCHISQ = CHISQ
     DO J = 1,MFIT
        ALPHA(J,1:MFIT) = COVAR(J,1:MFIT)
        BETA(J) = DA(J,1)
        A(LISTA(J)) = ATRY(LISTA(J))
     ENDDO

  !  Failure, increase ALAMDA and return
  ELSE
     ALAMDA = 10.*ALAMDA
     CHISQ = OCHISQ
  ENDIF

 ! write(*,*) 'nr_mrqmin: ------- '  !! inserted to avoid an untraced memory leak 
                                    !! see notes for 3.v.2019

END SUBROUTINE NR_MRQMIN
