REAL FUNCTION PHOT_FILTER (WAVE,FILTER)

  !+
  !
  !  PHOT_FILTER
  !
  != Return transmission coefficient for a specified standard filter 
  !  at a specified wavlength (nm)
  !  If a filter is not defined, the transmission coefficient is 1.0. 
  !
  !  based on data taken from Kurucz programs ...
  !
  !  Filters currently available are: 
  !    Johnson:    U B V R I J K L 
  !    Stromgren:  u v b y
  !
  !   Dummy values (square) available for:
  !    Broad Band  H M N T = [12] 
  !
  !
  !-

  IMPLICIT NONE

  ! Function arguments
  CHARACTER(LEN=1),INTENT(IN) :: FILTER     ! Filter designation
  REAL            ,INTENT(IN) :: WAVE       ! wavelength (nm) 


  ! Johnson filter holders
  INTEGER, PARAMETER :: NU=24
  REAL, SAVE, DIMENSION(NU) :: UWAVE, UFILT
  INTEGER, PARAMETER :: NB=40
  REAL, SAVE, DIMENSION(NB) :: BWAVE, BFILT
  INTEGER, PARAMETER :: NV=54
  REAL, SAVE, DIMENSION(NV) :: VWAVE, VFILT
  INTEGER, PARAMETER :: NR=23
  REAL, SAVE, DIMENSION(NR) :: RWAVE, RFILT
  INTEGER, PARAMETER :: NI=27
  REAL, SAVE, DIMENSION(NI) :: IWAVE, IFILT
  INTEGER, PARAMETER :: NJ=32
  REAL, SAVE, DIMENSION(NJ) :: JWAVE, JFILT
  INTEGER, PARAMETER :: NK=18
  REAL, SAVE, DIMENSION(NK) :: KWAVE, KFILT
  INTEGER, PARAMETER :: NL=15
  REAL, SAVE, DIMENSION(NL) :: LWAVE, LFILT


  ! Stromgren filter holders
  INTEGER, PARAMETER :: NSU=29
  REAL, SAVE, DIMENSION(NSU) :: SUWAVE, SUFILT
  INTEGER, PARAMETER :: NSV=29
  REAL, SAVE, DIMENSION(NSV) :: SVWAVE, SVFILT
  INTEGER, PARAMETER :: NSB=29
  REAL, SAVE, DIMENSION(NSB) :: SBWAVE, SBFILT
  INTEGER, PARAMETER :: NSY=29
  REAL, SAVE, DIMENSION(NSY) :: SYWAVE, SYFILT

  ! Dummy values - to avoid failures
  INTEGER, PARAMETER :: NH=5
  REAL, SAVE, DIMENSION(NH) :: HWAVE, HFILT
  INTEGER, PARAMETER :: NM=5
  REAL, SAVE, DIMENSION(NM) :: MWAVE, MFILT
  INTEGER, PARAMETER :: NN=5
  REAL, SAVE, DIMENSION(NN) :: NWAVE, NFILT
  INTEGER, PARAMETER :: NT=5
  REAL, SAVE, DIMENSION(NT) :: TWAVE, TFILT


  ! Data initialisation flag
  INTEGER :: I
  LOGICAL, SAVE :: INIT_FLAG = .FALSE.


  ! Filter response curves

  ! Johnson filters [ U B V R I J K L ]

  DATA UWAVE / &
       305.,310.,315.,320.,325.,330.,335.,340.,345.,350.,355., &
       360.,365.,370.,375.,380.,385.,390.,395.,400.,405.,410.,415.,420./ 
  DATA UFILT / &
       0.,.009,.028,.071,.127,.208,.307,.415,.524,.632,.740,  &  
       .840,.929,.995,.990,.830,.613,.406,.212,.090,.033,.014,0.,0./     

  DATA BWAVE / &
       360.,365,370.,375.,380.,385.,390.,395.,400.,405.,410., &
       415.,420.,425.,430.,435.,440.,445.,450.,455.,460.,465.,470.,475., &
       480.,485.,490.,495.,500.,505.,510.,515.,520.,525.,530.,535.,540., &
       545.,550.,555./ 
  DATA BFILT / &
       0.,.006,.023,.045,.106,.254,.492,.752,.881,.923,.955,  &
       .977,.990,1.,1.,.997,.984,.958,.916,.871,.820,.775,.723,.672,.617, & 
       .569,.511,.457,.402,.347,.299,.244,.199,.154,.113,.084,.051,.029, & 
       .010,0./  

  DATA VWAVE / &
       475.,480.,485.,490.,495.,500.,505.,510.,515.,520.,525.,  &
       530.,535.,540.,545.,550.,555.,560.,565.,570.,575.,580.,585.,590., & 
       595.,600.,605.,610.,615.,620.,625.,630.,635.,640.,645.,650.,655., &
       660.,665.,670.,675.,680.,685.,690.,695.,700.,705.,710.,715.,720., &
       725.,730.,735.,740./    
  DATA VFILT / &
       0.,.030,.084,.163,.301,.458,.630,.780,.895,.967,.997,1.,  &
       .988,.958,.919,.877,.819,.765,.711,.657,.602,.545,.488,.434,.386, & 
       .331,.289,.250,.214,.181,.151,.120,.093,.069,.051,.036,.027,.021, &
       .018,.016,.014,.012,.011,.010,.009,.008,.007,.006,.005,.004,.003, &
       .002,.001,.000/  

  DATA RWAVE / &
       520.,540.,560.,580.,600.,620.,640.,660.,680.,700., &
       720.,740.,760.,780.,800.,820.,840.,860.,880.,900.,920.,940.,960./
  DATA RFILT / &
       .00,.06,.28,.50,.69,.79,.88,.94,.98,1.,.94,.85,.73, &
       .57,.42,.31,.17,.11,.06,.04,.02,.01,.00/

  DATA IWAVE / &
       680.,700.,720.,740.,760.,780.,800.,820.,840.,860., &
       880.,900.,920.,940.,960.,980.,1000.,1020.,1040.,1060.,1080., &
       1100.,1120.,1140.,1160.,1180.,1200./
  DATA IFILT / &
       .00,.01,.17,.36,.56,.76,.96,.98,.99,1.,.98,.93,.84, &
       .71,.58,.47,.36,.28,.20,.15,.10,.08,.05,.03,.02,.01,.00/

  DATA JWAVE / &
       960.,980.,1000.,1020.,1040.,1060.,1080.,1100.,1120., &
       1140.,1160.,1180.,1200.,1220.,1240.,1260.,1280.,1300.,1320., &
       1340.,1360.,1380.,1400.,1420.,1440.,1460.,1480.,1500.,1520., &
       1540.,1550.,1560./
  DATA JFILT / &
       .00,.02,.03,.06,.16,.35,.62,.93,.85,.78,.78,.80,.85, &
       .93,.75,.64,.63,.63,.66,.68,.70,.70,.66,.60,.46,.27,.14,.09,.06, &
       .02,.01,.00/

  DATA KWAVE/ &
       1800.,1850.,1900.,1950.,2000.,2050.,2100.,2150.,2200., &
       2250.,2300.,2350.,2400.,2450.,2500.,2550.,2575.,2600./
  DATA KFILT / &
       .00,.10,.48,.95,1.,.98,.96,.95,.97,.96,.94,.95,.95, &
       .84,.46,.08,.04,.00/

  DATA LWAVE / &
       2900.,3000.,3100.,3200.,3300.,3400.,3500.,3600.,3700., &
       3800.,3900.,4000.,4100.,4150.,4200./
  DATA LFILT / &
       .0,.14,.68,.95,1.,1.,.98,.85,.69,.39,.21,.1,.02,.01,.0/


  ! Stromgren filters [ u v b y ]
  DATA SUFILT/  & 
       .0000,.0060,.0592,.1180,.1780,.2380,.2912,.3332,.3608,.3812, &  
       .3928,.3944,.3908,.3852,.3708,.3532,.3308,.3008,.2616,.2168, &  
       .1692,.1176,.0768,.0468,.0216,.0088,.0040,.0000,.0000/         

  DATA SVFILT/  & 
       .0000,.0016,.0036,.0084,.0148,.0228,.0304,.0480,.0784,.1300, &  
       .2000,.2988,.4000,.4728,.4932,.4800,.4352,.3720,.2820,.1812, &  
       .1112,.0672,.0400,.0268,.0200,.0140,.0072,.0036,.0000/         

  DATA SBFILT/  & 
       .0000,.0040,.0096,.0160,.0232,.0360,.0500,.0800,.1240,.2000, &  
       .3020,.4040,.4588,.4688,.4504,.3820,.2784,.1740,.1100,.0688, &  
       .0392,.0252,.0152,.0124,.0092,.0072,.0040,.0020,.0000/         

  DATA SYFILT/  &  
       .0000,.0068,.0168,.0272,.0400,.0696,.1020,.1520,.2340,.3300, &  
       .4100,.4540,.4792,.5012,.5220,.5132,.4544,.3480,.2480,.1548, &  
       .1000,.0660,.0416,.0252,.0136,.0096,.0080,.0040,.0000/         



  ! Dummy Filters [ H M N [12] ]

  DATA HWAVE/  & 
       1170., 1171., 1650., 2129., 2130. /
  DATA HFILT / &
       0.0,   1.0,   1.0,   1.0,   0.0   / 

  DATA MWAVE/  & 
       3600., 3601., 4800., 5999., 6000. /
  DATA MFILT / &
       0.0,   1.0,   1.0,   1.0,   0.0   / 

  DATA NWAVE/  & 
       4500., 4501., 10200., 15899., 15900. /
  DATA NFILT / &
       0.0,   1.0,   1.0,   1.0,   0.0   / 

  DATA TWAVE/  & 
       6000., 6001., 12000., 17999., 18000. /
  DATA TFILT / &
       0.0,   1.0,   1.0,   1.0,   0.0   / 

  !.


  ! Initialize wavelength arrays for Stromgren filters
  IF ( .NOT. INIT_FLAG) THEN
    DO I=1,29  
      SUWAVE(I)=312.5+FLOAT(I)*2.5       
      SVWAVE(I)=372.5+FLOAT(I)*2.5       
      SBWAVE(I)=432.5+FLOAT(I)*2.5       
      SYWAVE(I)=512.5+FLOAT(I)*2.5       
    ENDDO
    INIT_FLAG = .TRUE.
  ENDIF


  


  ! Standard value outside filter windows

  PHOT_FILTER = 0.0


  ! Johnson filters

  IF (FILTER=='U') THEN; IF ( UWAVE(1)<WAVE .AND. WAVE<UWAVE(NU) ) &
       CALL PHOT_LINTER(UWAVE,UFILT,NU,WAVE,PHOT_FILTER,1)

  ELSEIF (FILTER=='B') THEN; IF ( BWAVE(1)<WAVE .AND. WAVE<BWAVE(NB) ) &
       CALL PHOT_LINTER(BWAVE,BFILT,NB,WAVE,PHOT_FILTER,1)

  ELSEIF (FILTER=='V') THEN; IF ( VWAVE(1)<WAVE .AND. WAVE<VWAVE(NV) ) &
       CALL PHOT_LINTER(VWAVE,VFILT,NV,WAVE,PHOT_FILTER,1)

  ELSEIF (FILTER=='R') THEN; IF ( RWAVE(1)<WAVE .AND. WAVE<RWAVE(NR) ) &
       CALL PHOT_LINTER(RWAVE,RFILT,NR,WAVE,PHOT_FILTER,1)

  ELSEIF (FILTER=='I') THEN; IF ( IWAVE(1)<WAVE .AND. WAVE<IWAVE(NI) ) &
       CALL PHOT_LINTER(IWAVE,IFILT,NI,WAVE,PHOT_FILTER,1)

  ELSEIF (FILTER=='J') THEN; IF ( JWAVE(1)<WAVE .AND. WAVE<JWAVE(NJ) ) &
       CALL PHOT_LINTER(JWAVE,JFILT,NJ,WAVE,PHOT_FILTER,1)

  ELSEIF (FILTER=='K') THEN; IF ( KWAVE(1)<WAVE .AND. WAVE<KWAVE(NK) ) &
       CALL PHOT_LINTER(KWAVE,KFILT,NK,WAVE,PHOT_FILTER,1)

  ELSEIF (FILTER=='L') THEN; IF ( LWAVE(1)<WAVE .AND. WAVE<LWAVE(NL) ) &
       CALL PHOT_LINTER(LWAVE,LFILT,NL,WAVE,PHOT_FILTER,1)


    ! Stromgren filters

  ELSEIF (FILTER=='u') THEN; IF ( SUWAVE(1)<WAVE .AND. WAVE<SUWAVE(NSU) ) &
       CALL PHOT_LINTER(SUWAVE,SUFILT,NSU,WAVE,PHOT_FILTER,1)

  ELSEIF (FILTER=='v') THEN; IF ( SVWAVE(1)<WAVE .AND. WAVE<SVWAVE(NSV) ) &
       CALL PHOT_LINTER(SVWAVE,SVFILT,NSV,WAVE,PHOT_FILTER,1)

  ELSEIF (FILTER=='b') THEN; IF ( SBWAVE(1)<WAVE .AND. WAVE<SBWAVE(NSB) ) &
       CALL PHOT_LINTER(SBWAVE,SBFILT,NSB,WAVE,PHOT_FILTER,1)

  ELSEIF (FILTER=='y') THEN; IF ( SYWAVE(1)<WAVE .AND. WAVE<SYWAVE(NSY) ) &
       CALL PHOT_LINTER(SYWAVE,SYFILT,NSY,WAVE,PHOT_FILTER,1)


    ! Dummy filters

  ELSEIF (FILTER=='H') THEN; IF ( HWAVE(1)<WAVE .AND. WAVE<HWAVE(NH) ) &
       CALL PHOT_LINTER(HWAVE,HFILT,NH,WAVE,PHOT_FILTER,1)

  ELSEIF (FILTER=='M') THEN; IF ( MWAVE(1)<WAVE .AND. WAVE<MWAVE(NM) ) &
       CALL PHOT_LINTER(MWAVE,MFILT,NM,WAVE,PHOT_FILTER,1)

  ELSEIF (FILTER=='N') THEN; IF ( NWAVE(1)<WAVE .AND. WAVE<NWAVE(NN) ) &
       CALL PHOT_LINTER(NWAVE,NFILT,NN,WAVE,PHOT_FILTER,1)

  ELSEIF (FILTER=='T') THEN; IF ( TWAVE(1)<WAVE .AND. WAVE<TWAVE(NT) ) &
       CALL PHOT_LINTER(TWAVE,TFILT,NT,WAVE,PHOT_FILTER,1)


    ! Filter not available: set transmission to unity
  ELSE
    PHOT_FILTER = 1.0

  ENDIF

END FUNCTION PHOT_FILTER

