      SUBROUTINE DP_VSHIFT (MU)

!+
!
!     Subroutine:
!     DP_VSHIFT
!
!     Package:
!     DP   subroutine library for SPECTRUM
!
!     Purpose:
!=    Apply depth dependent velocity shifts to line opacities
!
!     Method:
!     Apply differential velocity shift to line opacity, 
!     retaining original wavelength grid. Shifted opacity is
!     obtaining by cubic interpolation on the old opacity. 
!     
!     Radial velocity is assumed to be positive in the
!     outward direction (i.e. towards lower optical depth).and
!     hence results in a BLUE shift (assumes stellar CoM frame). 
!
!     This device allows for the calculation of line profiles in 
!     expanding or contracting atmospheres. It approximates the source
!     function by assuming the atmosperhe is plane parallele and
!     approximated by a single emergent ray at cosine angle MU.
!
!     We use the following arrays from SPC_GLOBAL
!     W(1:NW)              : the wavelength array
!     RLMU (1:NW,1:NDEPTH) : the opacity artray    (changed on exit). 
!
!     Use:
!     CALL DP_VSHIFT (MU)
!
!     Normally, thsi suubroutine would appear between the calculation of 
!     the line opacities, e.g. LIN_OP, and the calculation of the source 
!     function (INTEG). 
!
!     Authors:
!     C.S.Jeffery (CSJ) Armagh Observatory
!
!     History:
!     15-MAY-03: (CSJ) Original version
!
!-

!  Global constants
      USE TAPP_PAR

!  Global variables
      USE SPC_GLOBAL
      
!  Double precision version
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)

!  Subroutine arguments (gigen).
      DOUBLE PRECISION, INTENT(IN) :: MU          ! cosine theta

!  Temporary value holders
      DOUBLE PRECISION, ALLOCATABLE :: WOLD (:)   ! wavelengths
      DOUBLE PRECISION, ALLOCATABLE :: ROLD (:)   ! original opacity
      DOUBLE PRECISION, ALLOCATABLE :: RNEW (:)   ! shifted opacity
      DOUBLE PRECISION              :: WNEW

!.


!     check shifts are implemented
      IF ( QUB__VRAD(1) /= 0. .OR. QUB__VRAD(NDEPTH) /= 0.  ) THEN  

        ALLOCATE ( WOLD (NW) )  ! Allocate temporary array
        ALLOCATE ( ROLD (NW) )  ! Allocate temporary array
        ALLOCATE ( RNEW (NW) )  ! Allocate temporary array

        IF ( NMODE == 0 ) W = W + RLAMB
        WOLD = W(1:NW)

        DO J = 1,NDEPTH
          V = QUB__VRAD(J) * MU
          DW = (1. + V * 1.D3 / TAPP__C) ! v in km/s  +ve = blueshift
          ROLD = RLMU(1:NW,J)
          DO I = 1,NW
            WNEW = W(I)*DW
            IF ( WNEW < W(1) ) THEN
              RNEW(I) = ROLD(1)
            ELSEIF ( WNEW > W(NW) ) THEN
              RNEW(I) = ROLD(NW)
            ELSE 
!             use linear interpolation to avoid -ve opacities
              CALL MTH_D_TABINT ( NW, WOLD, ROLD, 1, WNEW, RNEW(I) )
!             RNEW(I) = MAX ( RNEW(I), 0. )    
            ENDIF
          ENDDO
          RLMU(1:NW,J) = RNEW(1:NW)
        ENDDO

        IF ( NMODE == 0 ) W = W - RLAMB

      ENDIF
      

      END
      
