      SUBROUTINE EWABUND (EWIDTH)
!+
!  Name:
!      EWABUND

!  Purpose:
!=     Derive the chemical abundance from an absorption line equivalent width

!  Module:
!      subroutine

!  Language:
!      FORTRAN 77

!  Arguments:
!      EWIDTH      Equivalent width of absorption line (mA)    (Given)

!  Description:
!      The program assumes a starting abundance (ABUND) for the chemical 
!      species in question (normally solar). For a given absorption line, the 
!      LTE equivalent width (EW) is calculated from the given model atmosphere,
!      abundance and microturbulent velocity (subroutine WHOLE).
!      The abundance is then modified, using a Newton Raphson iteration,
!      until EW is sufficiently close to EWIDTH. The derived abundance is then
!      returned in common (ABUND).
!
!      A second linear iteration is provided as a safety net in case of
!      slow convergence. This is the original algorithm used by PLD.
!
!      On repeat solutions for the same atomic species, the starting 
!      abundance is chosen to be the value found from the previous
!      line and equivalent width. This improves convergence where lines
!      give consistent values.
!
!      On a test using 60 oxygen lines, this routine has not achieved 
!      better efficiency than using  the simple linear iteration with
!      cosmic abundances as starting values. However it can be used in 
!      modes 0, 20 and 21, rather than simply in 0.      

!  Authors:
!      C.S.Jeffery (CSJ)

!  History:
!      8-FEB-1993  (CSJ)  Original version

!-

!  Global variables
      USE SPC_GLOBAL

!  Implicit typing
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)

!  Previous species identifier.and abundance
      DATA NSPL / 0 /
      SAVE NSPL, ABUNDL

!.


!  Remember mode before switching off output.
      NMODX=NMODE
      NMODE=1


!  Initial estimate for abundance.

!     Use previous abundance if same species
      IF (NSP.EQ.NSPL) THEN
         ABUND = ABUNDL

!     or solar abundance by default
      ELSE
         ABUND=10.**(COSMIC(NSP)-ANORM)
      ENDIF

!  Initialize derivative
      DABDEW = 0.

!  Iterate for abundance.
      DO I = 1,10

!     Get equivalent width for these estimates.
         CALL WHOLE
!         write(*,*) 'EW:',nsp,ewidth,rlamb,abund,ew
         ABN = ABUND
         EWN = EW

         IF ( RLMU(1,1) < 0. ) THEN
            WRITE (*,'(A,I2,F9.2,A)') 
     :            'EWABUND: ',NSP,RLAMB,' line not found'
            RETURN
         ENDIF 

!        For debugging.
!        WRITE (*,*) I, EWIDTH, ABN, EW

!     If EW less than 1.0E-6 try the hard way.
         IF (EW.LT.1.0E-6) GOTO 10

!     Optimise increment for calculating derivative.
         IF (EW.GT.EWIDTH) THEN
            ABUND = ABUND * 0.99
         ELSE 
            ABUND = ABUND * 1.01
         ENDIF

!      Calculate derivative
         CALL WHOLE
!         write(*,*) 'EW:',nsp,ewidth,rlamb,abund,ew
         DEWDAB = (EW - EWN) / (ABUND - ABN)

!     Derive new abundance.
!        Restrict change in log abundance
         DAB = (EWIDTH - EWN) / DEWDAB
         DAB = MIN (DAB, ABN*2.0)
         DAB = MAX (DAB, -ABN*0.3)
         ABUND = ABN + DAB       
         
!     Check for convergence.
         IF (ABS(DAB/ABN).LE.0.01) GOTO 100
         EWDRAT = ABS((EWN/EWIDTH)-1)
         IF (EWDRAT.LE.0.001) GOTO 100
         
      ENDDO


!  Not converged - do it the hard way.
      WRITE (NWRITE,*) 'EWABUND: - Newton-Raphson not converging'
      I = 0

10    ABUND = 0.1 * 10.**(COSMIC(NSP)-ANORM)
20    I = I + 1
      CALL WHOLE
!         write(*,*) 'EW:',nsp,ewidth,rlamb,abund,ew

!     If EW less than 1.0E-6 assume failed calculation.
      IF (EW.LT.1.0E-6) GOTO 200
      IF (I > 20) GOTO 200

!     Check for convergence
      EWDRAT = ABS((EW/EWIDTH)-1)
      IF(EWDRAT.GE.0.001) THEN
         ABUND = ABUND*EWIDTH/EW
         GO TO 20
      ENDIF


!  Converged - repeat calculation with output switched on.
100   NMODE = NMODX
      IF (DEWDAB > 1.E-10) DABDEW = 1. / DEWDAB
      CALL WHOLE
!         write(*,*) 'EW:',nsp,ewidth,rlamb,abund,ew

!  Store abundance for next try
      NSPL = NSP
      ABUNDL = ABUND
      RETURN


!  Error - calculation assumed to have failed
200   NMODE = NMODX
      WRITE(NWRITE,214) NSP, RLAMB, EWIDTH, EW, ABUND
      RETURN

  214 FORMAT('EWABUND: ',I3,F8.2,1P,3E10.2,
     *       ' - calculation abandoned')




      END

