      SUBROUTINE HEIDIF(TEMPHE,PEHE,VTRB)

!+
!
!  HEIDIF
!
!=      Calculates normalised line opacity for helium lines
!       both diffuse and non-diffuse.
!
!       Opacity stored in 
!          COMMON/WORKS/WORK
!
!       There are several modes of operation:
!       I. For lines with tabulated broadening profiles:
!        A. Low electron density use Voigt profile with 
!           electron broadening and triplet splitting data from 
!           Stark tables
!        B. Use quadratic interpolation in temperature, density and
!           wavelength in one of the following tables:
!           NHEDAT = 1:  BCS: 4026, 4388, 4471, 4921
!           NHEDAT = 2:  Beachamp et al., 21 lines
!        C. In case A, and in case B where the tabulated data do
!           not reach far enough, the contribution of forbidden 
!           components is introduced following Dufton 1972, A&A 18, 335
!
!       II. For lines without tabulated profiles but _with_ Stark 
!           broadening coefficients:
!        A.  Use Voigt profile with electron broadening and triplet 
!           splitting data from Stark tables, and add corrections for
!           ion broadening
!        C. In case A, and in case B where the tabulated data do
!           not reach far enough, the contribution of the forbidden 
!           compenents is introduced following Dufton 1972, A&A 18, 335
!
!       III. For lines with no broadening data: 
!           These should not get this far, but should be treated in
!           the same way as normal metal lines.
!
!       Impact approximation electron widths and line shifts are 
!       available from 
!           Dimitrijevic & Sahal-Brechot 1990, A&AS 82, 519 
!       and related publication. Software exists to retrieve these data
!       interpolated for T and Ne. At present ONLY the electron widths
!       are used at fixed values (T=20000, Ne=10**15) and then ONLY 
!       if no other value for EWIDHE has been supplied through the main 
!       data tables.
!
!-

!    Public constants
      USE SPC_CONSTANTS

!    Global variables
      USE SPC_GLOBAL

!  Double precision version
      IMPLICIT REAL*8 (A-H,O-Z)

!  Arguments
      DOUBLE PRECISION :: TEMPHE         ! temperature
      DOUBLE PRECISION :: PEHE           ! electron pressure
      DOUBLE PRECISION :: VTRB           ! microturbulence

!  Arrays for use during interpolations
      DIMENSION XVAR(3),YVAR(3),STE(3,KW),SWA(3,KW)

      DOUBLE PRECISION, DIMENSION(KW) :: FWORK ! forbidden line opacity 

!  Local variables
      DOUBLE PRECISION  EWIDS  ! Electron width from Impact approximation
      DOUBLE PRECISION  EDELS  ! Line shift from impact approximation
      DOUBLE PRECISION  EWID   ! Electron width to use
      DOUBLE PRECISION  EDEL   ! Line shift to use

!.


!  Calculate electron density and doppler width
      ELNUM=CELNUM*PEHE/TEMPHE
      DOPP=RLAMB*SQRT(CDOPPT*TEMPHE/ATW(IHE)+CDOPPV*VTRB*VTRB)

      ELLOG=15.0+LOG10(ELNUM)
      TELOG=LOG10(TEMPHE)


!  Set pointer abbreviations
      IJ = IDHE
      IL = IDHE1      ! tabulated profiles



!  0. Get electron widths from Dimitrijevic & Sahal-Brechot
!     but only use these if a value is not provided in the tables
      EWID = EWIDHE(IJ)
      IF ( IDHES.NE.0 .AND. (EWID.LT.0.0001 .OR. NHESTARK.EQ.2) ) 
     :    CALL HEI_EWID (IDHES, TELOG, ELLOG, EWID, EDEL, NHESTARK)

!%!      write(*,*) idhe,ewidhe(ij),idhes,ewid,idhe1

!  I. ** For lines with broadening data
      IF (  (NHEDAT.EQ.1 .AND. IDHE1.GT.0 .AND. IDHE1.LE.4)        
     :   .OR. (NHEDAT.EQ.2 .AND. IDHE1.GT.0 .AND. IDHE1.LE.21)  )    
     :     THEN



!     IA. ** Low density :
        IF(ELLOG.LT.(ELHE1(1,IL)-0.3)) THEN

!       Voigt profile including triplet splitting
          CALL HEIVOIGT (NW,W,WORK, 
     :                   ELNUM,EWID,DBLE(CORRHE(IJ)),
     :                   DBLE(DELHE(1,IJ)),DBLE(DELHE(2,IJ)),DOPP)

!       Forbidden component ....
          IF (ABS(DEL_HE1F(IJ)).GT.0.01) THEN
            CALL HEIVOIGT (NW,W-DEL_HE1F(IJ),FWORK,
     :                   ELNUM,EWID,0.0D0,0.0D0,0.0D0,DOPP)
            WORK = WORK + ALF_HE1F(IJ) * ELNUM * FWORK
          ENDIF


!     IB. ** High density : tabulated profiles
        ELSE


!       Position in temperature grid
          IT = 1
          IF (NHEDAT.EQ.1 .AND. TELOG.GT.0.5*(TEHE1(2,IL)+TEHE1(3,IL))) 
     :      IT=2

!       Position in electron density grid
          DO IN=1,NDHE1(IL)-2
            NEL = IN
            IF(ELLOG.LT.0.5*(ELHE1(NEL+1,IL)+ELHE1(NEL+2,IL))) EXIT 
          ENDDO

!       Quadratic interpolation/extrapolation against temperature
          XVAR(1:3)=DBLE(TEHE1(IT:IT+2,IL))
          DO IN=1,3
            NEJ=NEL+IN-1
            DO IW=1,NWHE1(NEJ,IL)
              YVAR(1:3)=SHE1(IW,IT:IT+2,NEJ,IL)
              CALL POLY(XVAR,YVAR,TELOG,STE(IN,IW))
            ENDDO
          ENDDO


!       Wavelength interpolation
!       or approximation assuming a lambda**-2.5 dependence 
!
!          The approximation was checked OK 10/04/01, however it won't cope
!          if there is tabulated data missing before the approximation is
!          valid, or if a satellite forbidden component is missing - CSJ
! 
          DO I=1,NW
            DO IN=1,3
              NEJ=NEL+IN-1
              NSE=NWHE1(NEJ,IL)

!           Position in the wavelength grid
              IF    (W(I).LE.WHE1(1,NEJ,IL)) THEN
                SWA(IN,I)=STE(IN,1)+2.5*LOG10(WHE1(1,NEJ,IL)/W(I))
              ELSEIF(W(I).GE.WHE1(NSE,NEJ,IL)) THEN
                SWA(IN,I)=STE(IN,NSE)+2.5*LOG10(WHE1(NSE,NEJ,IL)/W(I))
              ELSE
                DO K=1,NSE-3
                  WTEST=0.5*(WHE1(K+1,NEJ,IL)+WHE1(K+2,NEJ,IL))
                  IF(W(I).LT.WTEST) GOTO 13
                ENDDO
                K=NSE-2


!             Quadratic interpolation against wavelength
   13           CONTINUE
                XVAR(1:3)=WHE1(K:K+2,NEJ,IL)
                YVAR(1:3)=STE(IN,K:K+2)
                CALL POLY(XVAR,YVAR,W(I),SWA(IN,I))

              ENDIF
            ENDDO
          ENDDO


!       Introduce approximate forbidden component if necessary
          DO IN=1,3
            NEJ=NEL+IN-1
            NSE=NWHE1(NEJ,IL)
            IF ( (DEL_HE1F(IJ).LT.WHE1(1,NEJ,IL)) .OR.
     :           (DEL_HE1F(IJ).GT.WHE1(NSE,NEJ,IL)) ) THEN
              CALL HEIVOIGT (NW,W-DEL_HE1F(IJ),FWORK,
     :                       DBLE(ELHE1(NEJ,IL)),EWID,
     :                       0.0D0,0.0D0,0.0D0,DOPP)
              SWA(IN,1:NW) = SWA(IN,1:NW) 
     :                + ALF_HE1F(IJ) * ELHE1(NEJ,IL) * FWORK(I)
            ENDIF 
          ENDDO

!       Electron density interpolation 
          IF (ELLOG.LE.ELHE1(NEL+2,IL)) THEN
            XVAR(1:3)=DBLE(ELHE1(NEL:NEL+2,IL))
            DO I=1,NW
              YVAR(1:3)=SWA(1:3,I)
              CALL POLY(XVAR,YVAR,ELLOG,WORK(I))
            ENDDO
            WORK(1:NW)=10.0**WORK(1:NW)

!       Do not extrapolate above limit of numerical profiles
!       because of break down in the line core due to wavelength shifts
          ELSE
            WORK(1:NW)=10.**SWA(3,1:NW)
          ENDIF

        ENDIF




! II. ** Lines without tabulated broadening data (esp. higher series 
!        members) but with old style Stark coefficients

      ELSE


!     Voigt profile including triplet splitting
        CALL HEIVOIGT (NW,W,WORK, 
     :                 ELNUM,EWID,DBLE(CORRHE(IJ)),
     :                 DBLE(DELHE(1,IJ)),DBLE(DELHE(2,IJ)),DOPP)

!       Forbidden component ....
        IF (ABS(DEL_HE1F(IJ)).GT.0.01) THEN
          CALL HEIVOIGT (NW,W-DEL_HE1F(IJ),FWORK,
     :                   ELNUM,EWID,0.0D0,0.0D0,0.0D0,DOPP)
          WORK = WORK + ALF_HE1F(IJ) * ELNUM * FWORK
        ENDIF

 
!     Ion broadening
        IF (RATHE(IJ).GT.0.001) THEN

!       Find start and end wavelength for phasing in the ion broadening
          AVO = ELNUM*EWID*(1.0+CORRHE(IJ))/DOPP
          IF(AVO.LT.0.5) THEN
            START=2.0*DOPP
          ELSE
            START=4.0*AVO
          ENDIF

          AVION=SNHE(IJ)*(ELNUM**0.6666666)
          IF(AVION.GT.(2.0*START)) THEN
            FINISH=AVION
          ELSE
            FINISH=2.0*START
          ENDIF

          DO I=1,NW

!         -- Full contribution
            IF    (ABS(W(I)).GT.FINISH) THEN
              WORK(I)=WORK(I)+RATHE(IJ)*ELNUM/(ABS(W(I))**2.5)

!         -- Partial contribution
            ELSEIF(ABS(W(I)).GT.START) THEN 
              RATION=(ABS(W(I))-START)/(FINISH-START)
              WORK(I)=WORK(I)+
     :                RATION*RATHE(IJ)*ELNUM/(ABS(W(I))**2.5)

            ENDIF
          ENDDO
        ENDIF
      ENDIF



!  Debug output
!      IF(BUGLN) THEN
!       WRITE(NDEBUG,901)TELOG,ELLOG,(WORK(I),I=1,NW)
!      ENDIF


!  Format statements
  201 FORMAT(F10.2)
  202 FORMAT(7F10.2)
  203 FORMAT(I5)
  900 FORMAT(' ***** HEIDIF ***** LOW DENSITY REGIME',
     *       5X,'TELOG=',F10.2,5X,'ELLOG=',F10.2/
     *       (1P,8E10.2))      
  901 FORMAT(' ***** HEIDIF ***** HIGH DENSITY REGIME',
     *       5X,'TELOG=',F10.2,5X,'ELLOG=',F10.2/
     *       (1P,8E10.2))
  902 FORMAT(' ***** HEIDIF ***** ELECTRON BROADENING',
     *       5X,'TELOG=',F10.2,5X,'ELLOG=',F10.2/
     *       (1P,8E10.2))      
  903 FORMAT(' ***** HEIDIF ***** ELECTRON PLUS ION BROADENING',
     *       5X,'START=',F10.2,5X,'FINISH=',F10.2/
     *       (1P,8E10.2))      

      END
