SUBROUTINE SPC_COPYMODEL ( MODEL )

  !+
  !
  !  Purpose:     
  !=    Copy model atmosphere from model "structure" to SPECTRUM arrays
  !
  !  Description:
  !
  !
  !  Language:
  !     Fortran 95
  !
  !  Authors:
  !     C.S.Jeffery (CSJ) (Armagh Observatory)
  !
  !  History:
  !     15-SEP-2006: CSJ
  !         Original version
  !
  !  Use:
  !     CALL SPC_COPYMODEL ( MODEL )
  !     
  !  Subroutine Arguments (Given):
  !     MODEL  : TYPE(MODEL_ATMOS)  :  Data structure containing model atmosphere
  !
  !     
  !-

  !  LTE Structures
  USE LTE_STRUCTURES

  !   use LTE globals for passing to OP
  USE LTE_MODEL, ONLY : LTE_TEMP => TEMP
  USE LTE_MODEL, ONLY : LTE_PG => P_TOTAL
  USE LTE_MODEL, ONLY : LTE_PE => P_E

  !  SPC Globals
  USE SPC_GLOBAL

  !  Safe definitions
  IMPLICIT NONE

  !  Subroutine Arguments (Returned):
  TYPE (MODEL_ATMOS) MODEL ! Data structure containing model atmosphere

  !     Other local variables
  INTEGER :: I,J  ! loop index
  INTEGER :: ND   ! number of depth points
  INTEGER :: JD   ! depth point index
  INTEGER :: IZ   ! atomic number identifier

  CHARACTER(LEN=20) :: ZSYMB      ! chemical symbol
  INTEGER          :: TAP__STATUS = 0

  double precision :: xalpha(ki)

  !     External functions
  DOUBLE PRECISION :: TAP_AELEM  ! mean atomic weight of an element

  !     Local variables for composition normalisation
  DOUBLE PRECISION :: C__SOL, C__STAR, AH_SOL, AMU_SOL, AMU_STAR


  !.



  !-------------------------------------------------------------------------

  ! copy basic model parameters


  TEFF = MODEL%TEFF
  GRAV = MODEL%GRAV

  ALPHA(1:MODEL%NELEM) = MODEL%ABUNDS(1:MODEL%NELEM)

  NDEPTH = MODEL%NDEPTH
  ND     =       NDEPTH

  NOPTYP = 0
  IF ( MODEL%OP_BF(1:2) == 'KI') NOPTYP = 1
  IF ( MODEL%OP_BF(1:2) == 'OP') NOPTYP = 2

  TAU_WL = MODEL%WAV_REF

  TAU(1:ND)  = MODEL%TAU(1:ND)
  TEMP(1:ND) = MODEL%TEMP(1:ND)
  PG(1:ND)   = MODEL%PGAS(1:ND)
  PE(1:ND)   = MODEL%PEL(1:ND)
  OP(1:ND)   = MODEL%OPAC(1:ND)

  ! and also copy to OP 

  !     Make space for model in OP arrays
  IF ( .NOT. ALLOCATED(LTE_TEMP) ) ALLOCATE (LTE_TEMP(NDEPTH)) 
  IF ( .NOT. ALLOCATED(LTE_PG) )   ALLOCATE (LTE_PG(NDEPTH))   
  IF ( .NOT. ALLOCATED(LTE_PE) )   ALLOCATE (LTE_PE(NDEPTH))  

  !     Copy to LTE_MODEL for opacity calculations
  LTE_TEMP = TEMP(1:NDEPTH)
  LTE_PG   = PG(1:NDEPTH)
  LTE_PE   = PE(1:NDEPTH)

  !-----------------------------------------------------------------------

  !  get some composition dependent variables

  AMU =  AH * 1.00  +  AHE* 4.00  +  AC *12.01  +  AN *14.01 &
       +  AO *16.00  +  ANE*20.18  +  AMG*24.31  +  AAL*26.98 &
       +  ASI*28.09  +  AS *32.06  +  ACA*40.08  +  AFE*55.85

  !     find abundance normalisation constant such that
  !     : log (sum(mu(i).n(i))) = solar
  !     

  AH_SOL   = 1. - SUM( 10.**(COSMIC(2:26)-12.) )
  C__SOL   = 12. - LOG10(AH_SOL)
  ! write (*,*) 'ah, c',ah_sol, c__sol
  AMU_SOL  = LOG10( SUM( ATW(1:26)*10.**(COSMIC(1:26)-12.) ) )
  AMU_STAR = LOG10( SUM( ATW(1:26)*ALPHA(1:26) ) )
  C__STAR  = C__SOL + AMU_SOL - AMU_STAR
  ! write (*,*) 'c*, mu*, c, mu', c__star, amu_star, c__sol, amu_sol 
  ANORM = C__STAR


  !-----------------------------------------------------------------------

  !  distribute uniform composition through structure
  DO IZ = 1,MODEL%NELEM
     QUB__ABSTRAT(IZ,:) = ALPHA(IZ)
  ENDDO

  ! deal with stratified composition
  IF ( MODEL%NELEM_STRAT > 0 ) THEN

     ! copy in the stratified elements
     QUB__LSTRAT = .TRUE.
     DO IZ = 1,MODEL%NELEM_STRAT
        IF ( MODEL%QELEM_STRAT(IZ) ) THEN
           QUB__QSTRAT(IZ) = .TRUE.
           QUB__ABSTRAT(IZ,1:ND) = MODEL%STRAT(IZ,1:ND)
        ENDIF
     ENDDO

     !  compute the depthe-dependent mean atomic weight
     DO JD = 1,ND
        QUB__AMU(JD) = 0.0
        DO IZ=1,MODEL%NELEM_STRAT
           IF ( MODEL%QELEM_STRAT(IZ) ) THEN
              QUB__AMU(JD) = QUB__AMU(JD) + QUB__ABSTRAT(IZ,JD)*ATW(IZ)
           ELSE
              QUB__AMU(JD) = QUB__AMU(JD) + ALPHA(IZ)*ATW(IZ)
           ENDIF
        ENDDO
        DO IZ=MODEL%NELEM_STRAT+1,MODEL%NELEM
              QUB__AMU(JD) = QUB__AMU(JD) + ALPHA(IZ)*ATW(IZ)
        ENDDO
     ENDDO

  ENDIF

  !-----------------------------------------------------------------------

  !     Compute depth-dependent part of van der Waals' broadening coefficient.
  CALL SPC_VDWC (AH, AHE)

  !-------------------------------------------------------------------------

  !     ..Get more data required by OP opacity routines

  !     Load up common blocks for OP: homogeneous layers
  !     note: numbering in OP does not correspond to atomic number
  CALL OP_ALPSET (NDEPTH, ALPHA) 

  !     Load up common blocks for OP: stratified layers
  IF ( MODEL%NELEM_STRAT > 0 ) THEN

     DO IZ = 1,MODEL%NELEM_STRAT
        IF ( MODEL%QELEM_STRAT(IZ) ) THEN
           CALL OP_ALPSET_Z ( ND, IZ, QUB__ABSTRAT(IZ,1:ND))  
        ENDIF
     ENDDO

  ENDIF

  CALL OP_ALPMU (NDEPTH)

  !     Calculate ionisation fractions and atomic data
  CALL OP_IONFRN (NDEPTH)
  CALL OP_ATFILL (NDEPTH)

  !-------------------------------------------------------------------------

  !     Report model input

  WRITE (NWRITE,*)
  WRITE (NWRITE,'(2X,A)') 'SPC_COPYMODEL: '
  WRITE (NWRITE,201)ND,TEFF,LOG10(GRAV),AHE/MAX(AH,1.D-10), &
       AH,AHE,AC,AN,AO,ANE,AMG,ASI,AS,ACA,AFE,AMU, ANORM
  IF ( QUB__LSTRAT ) THEN 
     WRITE (NWRITE,'(A,L)') 'Model is stratified ',QUB__LSTRAT
     write (NWRITE,'(A,I2)') 'N elements ',MODEL%NELEM_STRAT 
     WRITE (NWRITE,*) QUB__QSTRAT(1:MODEL%NELEM)
     WRITE (NWRITE,*) 'amu_j: '
     WRITE (NWRITE,'(16F6.3)') QUB__AMU(1:ND)
  ENDIF


201 FORMAT(/' ND=',I5,3X,'TEFF=',0PF10.1, &
       3X,'LOG G=',0PF8.3,3X,'He/H=',0PF8.3//1P, &
       5X,' H=',E12.5,5X,'HE=',E12.5,5X,' C=',E12.5/ &
       5X,' N=',E12.5,5X,' O=',E12.5,5X,'NE=',E12.5/ &
       5X,'MG=',E12.5,5X,'SI=',E12.5,5X,' S=',E12.5/ &
       5X,'CA=',E12.5,5X,'FE=',E12.5,5X,/ &
       4X,'AMU=',0P,F9.4,8X,     15X,     2X,'ANORM=',F9.4/)


END SUBROUTINE SPC_COPYMODEL


