MODULE SPC_GLOBAL

  !+
  !  SPC_GLOBAL
  !
  !  Global declarations for SPeCtrum subroutines
  !-


  !  SPC_CONTROL
  !
  !     Global constants for input/output and debugging
  USE SPC_CONTROL


  !  SPC_DIMS
  !
  !  Global constants fpr defining program dimensions  !
  INTEGER, PARAMETER  :: KD=201    ! Maximum no of depth points in model
  INTEGER, PARAMETER  :: KI=92     ! Maximum no of atoms in LINES
  INTEGER, PARAMETER  :: KZ=6      ! Maximum no of ions for each atom
  INTEGER, PARAMETER  :: KL=150000 ! Maximum no of lines in LINES or SYNT
  INTEGER, PARAMETER  :: KW=500000 ! Maximum number of wavelength points
  INTEGER, PARAMETER  :: KA=1      ! Maximum number of abundances in SYNT
  INTEGER, PARAMETER  :: KV=2      ! Maximum number of vt in SYNT
  INTEGER, PARAMETER  :: KTH = 12  ! Maximum number of angles for intensities


  !     Atomic data and ionisation fractions
  !         Atomic Data for elements
  !         ------------------------
  !
  !           ATW(KI)              Atomic weight
  !           ELEM(KI)             Element identifier
  !           COSMIC(KI)           Cosmic abundnce with [H]=12.0
  !           RION(6,KI)           Ionization potentials in eV
  !           PARFUN(5,KI)         Partition functions
  !           IION(KI)             Pointers to atomic data for given species
  !           KI                   Maximum no. of species allowed

  CHARACTER*10  ELEM(KI)
  DOUBLE PRECISION, DIMENSION(KI) :: COSMIC 
  DOUBLE PRECISION, DIMENSION(KI) :: ATW
  DOUBLE PRECISION, DIMENSION(6,KI) :: RION
  INTEGER, DIMENSION(KI) :: IION

  INTEGER :: IH, IHE, IC, IN, IO, INE, IMG, ISI, IS, ICA, IFE 
  INTEGER :: INA, IAL, IP, ICL, IAR

  EQUIVALENCE   (IH ,IION( 1)), (IHE,IION( 2)), (IC ,IION( 6))
  EQUIVALENCE   (IN ,IION( 7)), (IO, IION( 8)), (INE,IION(10))
  EQUIVALENCE   (IMG,IION(12)), (ISI,IION(14)), (IS ,IION(16))
  EQUIVALENCE   (ICA,IION(20)), (IFE,IION(26)), (INA,IION(11))
  EQUIVALENCE   (IAL,IION(13)), (IP, IION(15)), (ICL,IION(17))
  EQUIVALENCE   (IAR,IION(18))

  !        Ionisation fractions and partition functions at each depth point
  !           U_ION(KEL, KION, KD)   Partition function
  !           F_ION(KEL, KION, KD)   Ion fraction
  !  DOUBLE PRECISION    U_ION(KI, KZ, KD), F_ION(KI, KZ, KD)


  !  SPC_HELIUM
  !
  !  Common blocks for providing neutral and ionized helium line data

  !  Maximum dimensions for tabulated line broadening data
  INTEGER, PARAMETER :: KHE  = 100 ! Max number of helium diffuse lines      
  INTEGER, PARAMETER :: KHES =  80 ! Max number of He lines with Stark data
  INTEGER, PARAMETER :: KHE1 =  21 ! Max number of tabulated helium I lines 
  INTEGER, PARAMETER :: KHE2 =  21 ! Max number of tabulated helium II lines 
  INTEGER, PARAMETER :: KTE  =   6 ! Max number of temperatures in helium data
  INTEGER, PARAMETER :: KEL  =  11 ! Max number of densities in helium data
  INTEGER, PARAMETER :: KLW  = 100 ! Max number of wavelengths in helium data 

  !  Neutral Helium Data - including fundamental data
  INTEGER NHE                    ! Number of tabulated neutral helium lines
  INTEGER IDHE                   ! Identified tabulated helium line
  INTEGER ITABHE(3,KHE)          ! Pointers for broadening tables 
  !  1: Stark data
  !  2: BCS tables
  !  3: BWB tables
  DOUBLE PRECISION  WAVHE(KHE)             ! Wavelengths (A)
  REAL    GFHE(KHE)              ! gf-values
  DOUBLE PRECISION  EXCHE(KHE)             ! Excitation potentials (eV)
  REAL    DELHE(2,KHE)           ! Triplet line splitting 
  REAL    EWIDHE(KHE)            ! Electron damping width for NE=10**15 cm-3
  REAL    CORRHE(KHE)            ! Correction for ion broadening
  REAL    RATHE(KHE)             ! Parameter for ion broadening fit
  REAL    SNHE(KHE)              !  Parameter for ion broadening fit

  !  Neutral Helium Forbidden Lines (low density approxn.)
  REAL   DEL_HE1F(KHE)          ! Wavelength shifts
  REAL   ALF_HE1F(KHE)          ! Oscillator strength ratio / Ne

  !  Neutral Helium Stark Broadening Data
  INTEGER IDHES                  ! Identifies Stark data
  INTEGER NDHES(KHES)            ! Number of densities for each line
  INTEGER NTHES(KHES)            ! Number of temperatures
  REAL    TEHES(KTE,KHES)        ! Temperatures for each line
  REAL    ELHES(KEL,KHES)        ! Electron densities for each line
  REAL    EWIDHES(KTE,KEL,KHES)  ! Electron damping width
  !     REAL    ECORHES(KTE,KEL,KHES)  ! Correction for ion broadening
  REAL    EDELHES(KTE,KEL,KHES)  ! Triplet line splitting

  !  Neutral Helium Line Profile Data
  INTEGER IDHE1                  ! Identifies tabulated data
  INTEGER NWHE1(KEL,KHE1)        ! Number of points in profiles for different Ne
  INTEGER NTHE1(KHE1)            ! Number of temperatures
  INTEGER NDHE1(KHE1)            ! Number of densities for each line

  REAL    TEHE1(KTE,KHE1)        ! Temperatures for profiles
  REAL    ELHE1(KEL,KHE1)        ! Electron densities for profiles
  REAL    WHE1(KLW,KEL,KHE1)     ! Wavelengths for profiles
  REAL    SHE1(KLW,KTE,KEL,KHE1) ! Profiles for transitions


  !  Ionized Helium Data
  INTEGER NHE2                   ! Number of tabulated ionized helium lines
  INTEGER IDHE2                  ! Identifies tabulated He2 data
  INTEGER NWHE2(KHE2)            ! Number of wavelength points
  INTEGER MHE2(KHE2)             ! Symmetry parameter
  !  0 = symmetric
  !  1 = antisymmetric
  INTEGER NTHE2(KHE2)            ! Number of temperatures
  INTEGER NDHE2(KHE2)            ! Number of electron densities
  DOUBLE PRECISION WAVHE2(KHE2)           ! Central wavelength
  REAL    GFHE2(KHE2)            ! gf-values
  DOUBLE PRECISION EXCHE2(KHE2)           ! Excitation potential
  REAL    SCHE2(KHE2)            ! Scaling factor (alphanumeric)
  REAL    THE2(KTE,KHE2)         ! Temperatures for profiles
  REAL    ELHE2(KEL,KHE2)        ! Electron densities for profiles
  REAL    WHE2(KLW,KHE2)         ! Wavelength grid
  REAL    SHE2(KLW,KEL,KTE,KHE2) ! Profiles


  !  SPC_ATOMIC
  !
  !         Atomic data for isolated lines
  !
  !           ISP(KL)              Species number
  !                                0 < ISP < 21
  !           NION(KL)             Ionization stage
  !                                1 - neutral
  !                                2 - once ionized etc.
  !           WAVE(KL)             Wavelength in Angstroms
  !           GF(KL)               gf-value
  !           EDAM(KL)             Electron damping
  !                                Half-width-half-maximum in Angstroms
  !                                for electron density 10**15 cm-3
  !           RDAM(KL)             Radiative damping
  !                                Half-width-half-maxima in Angstroms
  !           WDAM(KL)             van der Waal's damping
  !           EXCIT(KL)            Excitation potential of lower level in eV
  !           ZMULT(KL)            Multiplet number (from Moore)
  !                                UV multiplets are -ve, optical are +ve.
  !           KL                   Maximum number of lines allowed

  INTEGER, DIMENSION(KL) :: ISP
  INTEGER, DIMENSION(KL) :: NION
  DOUBLE PRECISION, DIMENSION(KL) :: WAVE
  DOUBLE PRECISION, DIMENSION(KL) :: GF
  DOUBLE PRECISION, DIMENSION(KL) :: EDAM
  DOUBLE PRECISION, DIMENSION(KL) :: RDAM
  DOUBLE PRECISION, DIMENSION(KL) :: WDAM
  DOUBLE PRECISION, DIMENSION(KL) :: EXCIT
  DOUBLE PRECISION, DIMENSION(KL) :: ZMULT

  !         Data to calculate isolated lines or blends
  !
  !           NSP                  Species number
  !           RLAMB                Wavelength in Angstroms
  !           ABUND                Element abundance
  !           VT                   Micrturbulent velocity in km/s
  !           NLINE                Position of first line identified in linelist
  !           RLAMBP               Wavelength in previous calculation
  !           WTOL                 Wavelength tolerance for line identification
  !           DABDEW               Derivative d_ab/d_ew in same units as ABUND

  INTEGER  :: NSP
  INTEGER  :: NLINE
  DOUBLE PRECISION :: RLAMB
  DOUBLE PRECISION :: ABUND
  DOUBLE PRECISION :: VT
  DOUBLE PRECISION :: RLAMBP
  DOUBLE PRECISION :: WTOL
  DOUBLE PRECISION :: DABDEW

  !         Data for spectral synthesis calculation
  !
  !           NWSY                 Number of wavelength points in spectrum
  !                                Maximum KW
  !           NWSYC                Number of continuum points in spectrum
  !                                Maximum KW
  !           WSY(KW)              Wavelength points in Angstroms
  !           WSYC(KW)             Continuum points in Angstroms
  !
  !           NELSY                Number of lines in spectrum
  !                                Maximum KL
  !           NABSY                Number of different sets of abundances
  !                                Obsolete: Maximum KA
  !           NVTSY                Number of different microturbulences
  !                                Obsolete: Maximum KV
  !
  !           NSPSY(KL)            Species number
  !           NIONSY(KL)           Ionization stage
  !           WAVSY(KL)            Wavelength in Anstroms
  !           EDAMSY(KL)           Electron damping as in COMMON /ATOMIC/
  !           RDAMSY(KL)           Radiative damping as in COMMON /ATOMIC/
  !           WDAMSY(KL)           van der Waal's damping as ...
  !           EXCSY(KL)            Excitation potential of lower level in eV
  !           ZMLTSY(KL)           Multiplet number as in COMMON /ATOMIC/
  !           ABUSY(KL)            Set of abundance values for each line
  !           VTSY(KV)             Set of microturbulences (km/s)
  !           IABSY                Counter for abundances
  !           IVTSY                Counter for microturbulences
  !
  !           GFN_THRESHOLD        Lines with n.gf < this value not included

  INTEGER :: NWSY,NWSYC
  DOUBLE PRECISION, DIMENSION(KW) :: WSY, WSYC

  INTEGER :: NELSY,NABSY,NVTSY,IABSY,IVTSY
  INTEGER, DIMENSION(KL) :: NSPSY, NIONSY
  DOUBLE PRECISION, DIMENSION(KL) :: WAVSY, GFSY
  DOUBLE PRECISION, DIMENSION(KL) :: EDAMSY, RDAMSY, WDAMSY
  DOUBLE PRECISION, DIMENSION(KL) :: EXCSY, ZMLTSY
  DOUBLE PRECISION, DIMENSION(KL) :: ABUSY
  DOUBLE PRECISION, DIMENSION(KV) :: VTSY
  DOUBLE PRECISION :: GFN_THRESHOLD


  !   SPC_MODEL
  !
  !         Model atmosphere data
  !           NDEPTH               Number of depth points
  !           TEFF                 Effective temperature
  !           GRAV                 Gravity (in cm/s**2)
  !           TAU_WL               Reference scale for optical depth
  !           TAU(KD)              Optical depth
  !           TEMP(KD)             Temperature (K)
  !           PG(KD)               Gas pressure (cgs units)
  !           PE(KD)               Electron pressure (cgs units)
  !           OP(KD)               Opacity (normally at 5000 Angstroms)
  !                                corresponding to optical depth scale
  !           KD                   Specicifies maximum depth of atmosphere
  !
  !           QUB__VRAD(KD)        Depth dependent radial velocity
  !           QUB__VTRB(KD)        Depth dependent microturbulent velocity


  DOUBLE PRECISION    TEFF, GRAV, TAU_WL
  INTEGER             NDEPTH

  DOUBLE PRECISION, DIMENSION(KD) :: TAU
  DOUBLE PRECISION, DIMENSION(KD) :: TEMP
  DOUBLE PRECISION, DIMENSION(KD) :: PG
  DOUBLE PRECISION, DIMENSION(KD) :: PE
  DOUBLE PRECISION, DIMENSION(KD) :: OP
  DOUBLE PRECISION, DIMENSION(KD) :: QUB__VRAD
  DOUBLE PRECISION, DIMENSION(KD) :: QUB__VTRB


  !         Fractional abundances
  !         ---------------------
  !           ALPHA(KI)            Fractional abundances of species in model
  !           AH                   Hydrogen
  !           AHE                  Helium
  !           AC                   Carbon...
  !                                etc.
  !           AMU                  Mean atomic weight
  !           ANORM                Abundance normalisation s.t. log(sum(mu.n)) = c
  !
  DOUBLE PRECISION :: ALPHA(KI)
  DOUBLE PRECISION :: AMU
  DOUBLE PRECISION :: ANORM
  DOUBLE PRECISION :: AH , AHE, AC , AN , AO , ANE, AMG
  DOUBLE PRECISION :: AAL, ASI, AP,  AS , ACA, AAR, AFE
  EQUIVALENCE   (AH ,ALPHA( 1)), (AHE,ALPHA( 2)), (AC ,ALPHA( 6))
  EQUIVALENCE   (AN ,ALPHA( 7)), (AO, ALPHA( 8)), (ANE,ALPHA(10))
  EQUIVALENCE   (AMG,ALPHA(12)), (AAL,ALPHA(13)), (ASI,ALPHA(14))
  EQUIVALENCE   (AP ,ALPHA(15)), (AS ,ALPHA(16)), (AAR,ALPHA(18))
  EQUIVALENCE   (ACA,ALPHA(20)), (AFE,ALPHA(26))

  !         Stratified abundance data

  LOGICAL                :: QUB__LSTRAT = .FALSE. 
  LOGICAL, DIMENSION(KI) :: QUB__QSTRAT = .FALSE.  ! is this element stratified 
  DOUBLE PRECISION, DIMENSION(KI,KD) :: QUB__ABSTRAT  ! stratified abundances
  DOUBLE PRECISION, DIMENSION(KD) :: QUB__AMU  ! stratified mean atomic weight


  !         Depth dependent velocity fields
  !
  !           LVF                  Depth-dependent velocity field flag (LOGICAL)
  !           LVT                  Depth-dep. microturbulent velocity (LOGICAL)
  !           VFIELD               Velocity shift in each zone
  !           VTURBD               microturbulent velocity in each zone
  !           VDWC                 van der Waal's constant

  LOGICAL :: LVF = .FALSE.
  LOGICAL :: LVT = .FALSE.
  DOUBLE PRECISION, DIMENSION(KD) :: VFIELD
  DOUBLE PRECISION, DIMENSION(KD) :: VTURBD
  DOUBLE PRECISION, DIMENSION(KD) :: VDWC


  !  SPC_SPECTRUM
  !
  !         Wavelength scale etc
  !
  !           NW                   Number of wavelength points
  !           SYMM                 Logical variable
  !                                   .true. for symmetric profile
  !           WAVMAX               Maximum wavelength from line centre
  !                                  0 - use default values
  !           W(KW)                Wavelength points
  !           FLUX(KW)             Profile information 
  !                                Varies depending on point in program
  !           TAUL(KW)             Continuum optical depth where
  !                                line optical depth is unity
  !           FLUXC(KW)            Continuum flux at each continuum wavelength
  !           FLUXN(KW)            Normalized flux in spectrum or profile
  !           CONFL                Ratio of continuum to line wing fluxes
  !           EW                   Equivalent width of entire profile

  INTEGER :: NW
  LOGICAL :: SYMM
  DOUBLE PRECISION :: WAVMAX
  DOUBLE PRECISION, DIMENSION(KW) :: W
  DOUBLE PRECISION, DIMENSION(KW) :: FLUX
  DOUBLE PRECISION, DIMENSION(KW) :: TAUL
  DOUBLE PRECISION, DIMENSION(KW) :: FLUXC
  DOUBLE PRECISION, DIMENSION(KW) :: FLUXN
  DOUBLE PRECISION :: CONFL
  DOUBLE PRECISION :: EW


  !         Specific intensities

  DOUBLE PRECISION  COSTHETA(KTH)    ! Cosine angles for intensities
  DOUBLE PRECISION  INTENS(KW,KTH)   ! Specific intensity in spectrum
  DOUBLE PRECISION  INTENC(KW,KTH)   ! Specific intensity in continuum
  INTEGER           NTHETA           ! Number of angles to compute


  !         Line opacity

  DOUBLE PRECISION, DIMENSION(KW,KD) :: RLMU   ! Line opacity


  !         Work space

  DOUBLE PRECISION, DIMENSION(KW) :: WORK
  INTEGER, DIMENSION(KW) :: IWORK


END MODULE SPC_GLOBAL
