SUBROUTINE SPC_LINES ( F__LINES , WMIN_IN, WMAX_IN )

  !+     
  !=    Input subroutine for atomic data for individual metal lines     
  !-
  !     Double precision version
  !     
  ! 
  !     12.iv.2018 - adjusted to restrict number of lines read from list.
  !     by addition of arguments WMIN and WMAX. 
  !

  !    Public constants
  USE SPC_CONSTANTS
  USE TAPM_PAR
  USE TAPP_PAR

  !    Global variables
  USE SPC_GLOBAL
  USE OP_GLOBAL, ONLY : OP_XION, OP_XUU
  USE OP_GLOBAL, ONLY : OP_MAXNZ, OP_MAXNA, OP_MAXNI

  !    Implicit typing (denegrated)
  IMPLICIT REAL*8 (A-H,O-Z)

  !    Filename
  CHARACTER (*), INTENT(IN) :: F__LINES

  !    Subprogram outputs
  REAL, INTENT(IN) ::   WMIN_IN, WMAX_IN   ! limits for wavelength grid (A)


  INTEGER        :: NUSED = 61
  CHARACTER (90) :: F__LINES_USED

  !    Local constants for line broadening

  !     Gamma to hwhm
  DOUBLE PRECISION GTOFW
  PARAMETER ( GTOFW = 1.0 / ( 4 * TAPM__PI * TAPP__C * 1.D10 ) )

  !     Classical value for Gamma_stark
  DOUBLE PRECISION XNF, XEH, XET, XEU
  PARAMETER ( XNF = 25.  )
  PARAMETER ( XEH = 13.595  )
  PARAMETER ( XET = 8067.6 )
  PARAMETER ( XEU = TAPP__H / TAPP__E * TAPP__C * 1.E10 )

  !     Classical value for Gamma_vdW
  DOUBLE PRECISION W02, W12, WX0
  PARAMETER ( W02 = 2.5  )
  PARAMETER ( W12 = 45.  )
  PARAMETER ( WX0 = 4.5D-9  )

  !     Cut off for n.gf to be used in line opacity calculation.
  !     the cutoff used is evaluated at tau=1/3. Need to test this value
  !      REAL GFNCUT
  !      PARAMETER (GFNCUT = 1.E-2)
  !     superceded by GFN_THRESHOLD in common block.
  DOUBLE PRECISION :: TAU_THRESHOLD = 0.1 ! Threshold at which lines should be checked
  INTEGER          :: JT1 = 0             ! Pointer to depth at which above holds

  !.

  !     If model exists, locate zone where cont optical depth approx 0.33
  IF ( NDEPTH > 0 ) THEN
     DO J = 1,NDEPTH
        IF ( TAU(J) < TAU_THRESHOLD ) JT1 = J
     ENDDO
  ELSE
     WRITE (*,*) 'SPC_LINES: ** Error reading linelist.'
     WRITE (*,*) '           ** Model atmosphere not available. Use "read_model"'
     RETURN
  ENDIF

  !     Verify wavelength limits 
  !  -- i.e. if both 0 set WMAX infinity
  WMIN = WMIN_IN
  WMAX = WMAX_IN
  IF ( WMIN_IN < 1.E-5 ) WMIN = 0.
  IF ( WMAX_IN < 1.E-5 ) WMAX = 999999.
  IF ( WMAX < WMIN ) THEN
     WW = WMIN
     WMIN = WMAX
     WMAX = WW
  ENDIF  

  !     Check GFN_THRESHOLD has been set (default in SPECTRUM is 0.01)
  GFN_THRESHOLD = MAX ( 0.00001, GFN_THRESHOLD )

  !     Open input stream
  WRITE (*,'(/,A)') ' SPC_LINES '
  IF ( NDEPTH > 0 ) WRITE (*,'(A,F5.2,A,1P,E7.1,A,0P,I3)')  &
       ' Selecting lines with gf.n(tau=',TAU_THRESHOLD, &
       ') > ',GFN_THRESHOLD,' jd =',JT1
  OPEN(UNIT=NATOM,FILE=F__LINES,STATUS='OLD')

  !     Get an output stream
  F__LINES_USED = F__LINES//'_USED'
  OPEN(UNIT=NUSED,FILE=F__LINES_USED,STATUS='UNKNOWN')


  !     Read atomic data for individual lines from stream NATOM
  !     Stored in COMMON/ATOMIC/


  LINE_STATUS = 0
  IL = 0
  I  = 1
  WRITE (*,'(A,2F8.0)') ' SPC_LINES: Reading lines in range ',WMIN,WMAX 
  DO WHILE ( LINE_STATUS >= 0 )
     READ(NATOM,*,IOSTAT=LINE_STATUS)ISP(I),NION(I), &
          WAVE(I),GF(I),EDAM(I),RDAM(I),WDAM(I),EXCIT(I),ZMULT(I)

     !     Detect last line in list (-1 0 0 0 0)
     IF (ISP(I).EQ.-1) EXIT

     !     If EOF or ISP = -1 stop reading
     IF (LINE_STATUS < 0) THEN 
        I = I - 1
        EXIT
     ENDIF

     !     Reject lines with -ve atomic numbers (these are "commented out")
     IF (ISP(I) < 0) CYCLE

     !     Reject lines with atomic numbers too large for opacity arrays
     IF (ISP(I) > OP_MAXNZ) CYCLE

     !     Reject lines with ionisation states too large opacity arrays
     IF (NION(I) > OP_MAXNI) CYCLE

     !     Reject lines outside specfied wavelength range
     IF (WMIN > WAVE(I) .OR. WAVE(I) > WMAX) CYCLE

     !     Track number of lines read
     IL = IL + 1

     !     Trap error reading line format
     IF (LINE_STATUS > 0) CYCLE

     !     Limit numner of lines, and allow for H, HeI and HeII.
     IF (I >= KL-200) THEN
        WRITE (*,'(A,2I3,F10.2)') 'Too many lines: last line:',ISP(I),NION(I),WAVE(I)
        EXIT
     ENDIF



     !     if model exists, test the line strength
     IF ( NDEPTH > 0 ) THEN
        FRAC = CLOPAC*WAVE(I)*WAVE(I)  &             ! constant bundle
             * 10.**GF(I)              &             ! gf - value
             * EXP(-CEVTOT*EXCIT(I)/TEMP(JT1)) &     ! excitation potential
             / OP_XUU(NION(I),ISP(I),JT1)       &     ! partition function
             * (COSMIC(ISP(I))/AMU)             &     ! reduced abundance
             * (1.0-EXP(-CSTIME/(TEMP(JT1)*WAVE(I)))) & ! stimulated emission
             * OP_XION(NION(I),ISP(I),JT1)            ! ion fraction

        !     only include this line if it is strong enough at tau=tau_threshold. 
        IF ( FRAC < GFN_THRESHOLD ) CYCLE
     ENDIF


     !     If NION < 5 OR ISP > 0: process the line
     IF (NION(I) <= 4 .AND. ISP(I) > 0) THEN

        !     Scale the data as follows

        !     log(gf) --> gf
        GF(I) = 10.0**GF(I)

        !     log(Gamma) --> Gamma 
        RDAM(I) = 10.0**RDAM(I)      
        EDAM(I) = 10.0**EDAM(I)
        WDAM(I) = 10.0**WDAM(I)

        !     Adopt classical damping constant where none given
        !     .formulae copied from SYNSPEC: subroutine inilin.f
        !     and need to be derived more rigorously

        IF (EDAM(I) < 1.D-90 .OR. WDAM(I) < 1.D-90) THEN
           EXCU = EXCIT(I) + XEU / WAVE(I)
           XNEFF2 = ISP(I)**2 * (XEH / RION(NION(I),ISP(I)) - EXCU/XET)
           IF (XNEFF2 <= 0. .OR. XNEFF2 > XNF) XNEFF2=XNF
        ENDIF

        IF(EDAM(I) < 1.D-90) THEN
           EDAM(I) = 1.D-8 * XNEFF2**2.5
        ENDIF

        IF(WDAM(I) < 1.D-90) THEN
           IF(ISP(I) < 21) THEN
              WX2 = W02 * (XNEFF2/FLOAT(NION(I)))**2
           ELSEIF(ISP(I) < 45) THEN
              WX2 = (W12-FLOAT(ISP(I)))/FLOAT(NION(I))
              !.Unsold approximation (about 2.E-15 too small!!)
           ELSE
              WX2 = 0.3D-30 *   &
                   ( 1./(RION(NION(I),ISP(I))-EXCIT(I)-1.24D4/WAVE(I))**2 - &
                   1./(RION(NION(I),ISP(I))-EXCIT(I))**2 )
           ENDIF
           WDAM(I) = WX0 * WX2**0.4
        ENDIF



        !     Gamma --> fwhm

        RDAM(I) = GTOFW*WAVE(I)*WAVE(I)*RDAM(I)
        !     According to Gray (p.207), in Delta lambda form, the
        !     half-half width is 0.59x1.e-4 Angstrom for all lines
        !     Previously we used a value four times this. 
        IF(RDAM(I).LT.0.59E-04) RDAM(I) = 0.59E-04

        EDAM(I) = GTOFW*WAVE(I)*WAVE(I)*EDAM(I)
        !     For electron damping scaled to Ne = 1E15
        EDAM(I) = 1.0E15*EDAM(I)

        WDAM(I) = GTOFW*WAVE(I)*WAVE(I)*WDAM(I)

        !      Write out the lines selected for the linelist
        WRITE(NUSED,'(2I3,F9.3,F7.3,4F8.3,F7.2,1P,4E10.2)',IOSTAT=LINE_STATUS) &
          ISP(I),NION(I),WAVE(I),GF(I),EDAM(I),RDAM(I),WDAM(I),EXCIT(I),ZMULT(I),FRAC, &
          OP_XUU(NION(I),ISP(I),JT1), OP_XION(NION(I),ISP(I),JT1), COSMIC(ISP(I))

        !     Increment line counter
        I = I + 1

     ENDIF

  ENDDO

  !     Close down linelist files
  CLOSE(UNIT=NATOM)
  CLOSE(UNIT=NUSED)

  !     Report lines selected
  WRITE (*,*) IL, ' lines read from linelist '
  WRITE (*,*) I, ' lines selected for current model'
  WRITE (*,'(A,2F7.1)') '    in range ', MINVAL(WAVE(:I)),MAXVAL(WAVE(:I))


  !     Debug output
  IF(BUGAT) THEN
     WRITE(NDEBUG,901)
     DO I=1,200
        WRITE(NDEBUG,902)ISP(I),NION(I),WAVE(I),GF(I),EDAM(I),RDAM(I),EXCIT(I)
     ENDDO
  ENDIF

  !     Debug statements
901 FORMAT(//7X,'ISP',6X,'NION',6X,'WAVE',8X,'GF',6X,'EDAM',6X,'RDAM',5X,'EXCIT')
902 FORMAT(2I10,0PF10.2,1P,5E10.3)
END SUBROUTINE SPC_LINES
