SUBROUTINE SPC_READPULS ( F__STATIC, F__MASSZ, F__HYDRO )

  !+
  !
  !  Purpose:     
  !=    Read in a PULS pulsation model
  !
  !  Description:
  !     This subroutine will read in a) a static model envelope and b)
  !     a sequence of equivalent models from a pulsating model envelope
  !     and convert them to a sequence of scaled models on a standard 
  !     optical depth scale. The result will be used to compute emergent 
  !     spectra consistne with the local temperature, pressure and velocity
  !     shifts in the model structure, primarily for investigating the 
  !     shock phase of the pulsation cycle.  
  !
  !  Language:
  !     Fortran 95
  !
  !  Authors:
  !     C.S.Jeffery (CSJ) (Armagh Observatory)
  !
  !  History:
  !     30-OCT-2012: CSJ
  !         Original version
  !
  !  Use:
  !     CALL SPC_READPULS ( F__STATIC, F__HYDRO, PULS__MODEL, PULS__ND, PULS__NPHI, IECHO, IOSTAT )
  !     
  !  Subroutine Arguments (Given):
  !     F__STATIC  : CHARACTER(*)  :  Name of file containing static model envelope
  !     F__MASSZ   : CHARACTER(*)  :  Name of file containing dynamic mass zones
  !     F__HYDRO   : CHARACTER(*)  :  Name of file containing dynamic model envelope
  !
  !-

  !   public constants
  USE TAPM_PAR, ONLY : TAPM__PI

  !   public control variables (not required here)
  ! USE SPC_CONTROL

  !   collect the pulsation data structures
  USE LTE_STRUCTURES, ONLY : PULS_HYDRO, PULS_STATIC

  !   public pulsation model in scaled lagrangian units
  USE SPC_PULSE_PUB


  !  Safe definitions
  IMPLICIT NONE

  !  Subroutine Arguments (Given):
  CHARACTER(*) :: F__STATIC  !  Name of file containing static model envelope
  CHARACTER(*) :: F__MASSZ   !  Name of file containing dynamic mass zoning
  CHARACTER(*) :: F__HYDRO   !   Name of file containing dynamic model envelope

  !  Function interfaces
  INTERFACE
     FUNCTION NR_LOCATE (XX,X)
       DOUBLE PRECISION, DIMENSION(:), INTENT(IN) :: XX
       DOUBLE PRECISION, INTENT(IN) :: X
       INTEGER :: NR_LOCATE
     END FUNCTION NR_LOCATE
     SUBROUTINE NR_POLINT (XA,YA,X,Y,DY)
!       USE NRUTIL, ONLY : NRERROR, IMINLOC
       DOUBLE PRECISION, DIMENSION(:), INTENT(IN) :: XA
       DOUBLE PRECISION, DIMENSION(:), INTENT(IN) :: YA
       DOUBLE PRECISION, INTENT(IN)               :: X
       DOUBLE PRECISION, INTENT(OUT) :: Y
       DOUBLE PRECISION, INTENT(OUT) :: DY
     END SUBROUTINE NR_POLINT
  END INTERFACE


  !  Static model envelope
  TYPE(PULS_STATIC), DIMENSION(:), ALLOCATABLE :: STAT__LG  ! Static lagrangian model
  TYPE(PULS_STATIC), DIMENSION(:), ALLOCATABLE :: STAT__LG0 ! Static lagrangian model at hydro masses
  INTEGER                           :: STAT__ND  ! Dimension for static model

  DOUBLE PRECISION, DIMENSION(:), ALLOCATABLE :: TMP__MM

  ! Dynamic model envelope
  ! DOUBLE PRECISION, DIMENSION(:),   ALLOCATABLE :: PULS__M   !
  ! DOUBLE PRECISION, DIMENSION(:),   ALLOCATABLE :: PULS__DM  !
  ! TYPE(PULS_HYDRO), DIMENSION(:,:), ALLOCATABLE :: PULS__LG  ! Dynamic lagrangian model


  !     Other local variables
  INTEGER :: I,J,K ! loop index
  INTEGER :: N,M   ! array pointers
  INTEGER :: ND   ! number of depth points
  INTEGER :: NT   ! number of times
  INTEGER :: NSTATIC = 20 ! unit number for input files
  INTEGER :: NMASSZ  = 20 ! unit number for input files
  INTEGER :: NHYDRO  = 20 ! unit number for input files
  DOUBLE PRECISION :: ERR ! error returned by interpolation routine


  !.

  ! I. Read in the static model, the mass zones for the hydro model, 
  !    and the hydro model itself

  CALL PULS_READ_STATIC
  CALL PULS_READ_MASSZ
  CALL PULS_READ_HYDRO
  WRITE (*,*)
  WRITE (*,*) 'SPC_READPULS: pulsation model read'  
  WRITE (*,*) 'SPC_READPULS: Model 1'  
  WRITE (*,'(A5,5(/1P,20E9.2))') 'M', PULS__ZONEM(1:PULS__ND)
  WRITE (*,'(A5,5(/1P,20E9.2))') 'R', PULS__MODEL(1:PULS__ND,1)%R
  WRITE (*,'(A5,5(/1P,20E9.2))') 'P', PULS__MODEL(1:PULS__ND,1)%P
  WRITE (*,'(A5,5(/1P,20E9.2))') 'T', PULS__MODEL(1:PULS__ND,1)%T
  WRITE (*,'(A5,5(/1P,20E9.2))') 'V', PULS__MODEL(1:PULS__ND,1)%V


  ! II. Rezone the static model onto the mass zones of the hydro model.
  ! --> this will probably not work because the outer mass zones are so close together.
  !     I think it will be necessary to rework this in terms of integrated mass
  !     from the surface (PULS__ZONEM for the puls model)

  WRITE (*,*) 'SPC_READPULS: mass rezoning on M* - Mr'
  !WRITE(*,*) 'STAT__LG%MM'  
  !WRITE(*,*) STAT__LG%MM  
  !WRITE(*,*) 'PULS__ZONEM'  
  !WRITE(*,*) PULS__ZONEM  
  ALLOCATE ( STAT__LG0(PULS__ND) )
  WRITE(*,*) 'STAT_LG0 - allocated'
  DO J = 1,PULS__ND
     !! I don't think we should be using nr_polint; better to use mth_d_tabint ***
     ! locate mass variable in array
     N =  NR_LOCATE (STAT__LG%MM,PULS__ZONEM(J))
     N =  MAX(1,N-1)
     N =  MIN(N,STAT__ND-3)
     CALL NR_POLINT (STAT__LG(N:N+3)%MM,STAT__LG(N:N+3)%R0,PULS__ZONEM(J),STAT__LG0(J)%R0,ERR)
     CALL NR_POLINT (STAT__LG(N:N+3)%MM,STAT__LG(N:N+3)%P0,PULS__ZONEM(J),STAT__LG0(J)%P0,ERR)
     CALL NR_POLINT (STAT__LG(N:N+3)%MM,STAT__LG(N:N+3)%T0,PULS__ZONEM(J),STAT__LG0(J)%T0,ERR)
  ENDDO
  STAT__LG0%MM = PULS__ZONEM
  WRITE(*,*) 'SPC_READPULS: mass rezoning done'

  ! III. Scale the pulsation model to the static model  

  DO K = 1,PULS__NT
     PULS__MODEL(1:PULS__ND,K)%R = PULS__MODEL(1:PULS__ND,K)%R / STAT__LG0(1:PULS__ND)%R0
     PULS__MODEL(1:PULS__ND,K)%P = PULS__MODEL(1:PULS__ND,K)%P / STAT__LG0(1:PULS__ND)%P0
     PULS__MODEL(1:PULS__ND,K)%T = PULS__MODEL(1:PULS__ND,K)%T / STAT__LG0(1:PULS__ND)%T0
  ENDDO
  WRITE (*,*)
  WRITE (*,*) 'SPC_READPULS: pulsation scaled to static model'
  WRITE (*,*) 'SPC_READPULS: Model 1'  
  WRITE (*,'(A5,5(/1P,20E9.2))') 'M   ', PULS__ZONEM(1:PULS__ND)
  WRITE (*,'(A5,5(/20F9.2))') 'R/Ro', PULS__MODEL(1:PULS__ND,1)%R
  WRITE (*,'(A5,5(/20F9.2))') 'P/Po', PULS__MODEL(1:PULS__ND,1)%P
  WRITE (*,'(A5,5(/20F9.2))') 'T/To', PULS__MODEL(1:PULS__ND,1)%T
  WRITE (*,'(A5,5(/20F9.2))') 'V   ', PULS__MODEL(1:PULS__ND,1)%V

  ! IV. Convert the mass zoning from spherical to plane parallel
  ! --> this means dividing the mass of each zone by 4.pi.r^2 in order to 
  !     bring the zone mass from the total shell to a unit area

  PULS__ZONES = PULS__ZONES / 4 / TAPM__PI / STAT__LG0%R0**2
  PULS__ZONEM = PULS__ZONEM / 4 / TAPM__PI / STAT__LG0%R0**2
  WRITE (*,*)
  WRITE (*,*) 'SPC_READPULS: mass units converted to plane parallel'
  WRITE (*,'(A5,5(/1P,20E9.2))') 'M   ', PULS__ZONEM(1:PULS__ND)


  ! V. Compute the optical depth scaling for the envelope from the static model.
  ! --> Unfortunately, the static model does not retain density or opacity information,
  !      although the hydro model does, so this requires us to compute the equation of
  !      state and local opacity. The other way around it is to use the adopted
  !      model atmosphere to generate a mass scale, and then work back from that. 
  !      However, that can't be done here, becasue we have not made any assumptions
  !      about the model atmosphere at this point. 


  !  Check temporary arrays are deallocated
  DEALLOCATE (STAT__LG, STAT__LG0)

  ! set flag to indicate pulsation model has been read and is required
  PULS__OK = .TRUE.
  RETURN


CONTAINS



  SUBROUTINE PULS_READ_STATIC

    ! Read output from nlpulse static model calculations 

    !   Local holder for static model. 
    TYPE(PULS_STATIC), DIMENSION(10000) :: STAT__TMP
    INTEGER :: J 

    !.

    ! Static model is unformatted
    WRITE(*,*) 'PULS_READ_STATIC: ',F__STATIC
    OPEN(UNIT=NSTATIC,FILE=F__STATIC,STATUS='OLD',FORM='UNFORMATTED')

    ! Number of mass zones is not defined a priori
    DO J = 1,10000
       READ (NSTATIC,END=91) STAT__TMP(J)%R0,STAT__TMP(J)%P0,STAT__TMP(J)%M0,STAT__TMP(J)%T0,STAT__TMP(J)%DM0
    ENDDO

91  CLOSE (NSTATIC)
    STAT__ND = J - 1
    WRITE(*,*) 'PULS_READ_STATIC: model read with ',STAT__ND,' zones'

    !  integrate dm to generate mass from surface
    STAT__TMP(1)%MM = STAT__TMP(J)%DM0
    DO J = 2,STAT__ND
       STAT__TMP(J)%MM = STAT__TMP(J-1)%MM - STAT__TMP(J-1)%DM0
    ENDDO

    ! truncate arrays to valid and required data only
    ALLOCATE (STAT__LG(STAT__ND))
    STAT__LG = STAT__TMP(1:STAT__ND)

  END SUBROUTINE PULS_READ_STATIC




  SUBROUTINE PULS_READ_MASSZ

    ! Read mass zones for nlpulse calculations

    ! Input:
    ! file - complete name of .models file to read

    ! Output: 
    ! puls__nd   - number of depth zones in dynamic model
    ! m  - total mass interior to the mass zone
    ! dm  - mass within the mass zone (AB 3.13)
    ! dmi  - mass between mass zones (AB 3.14)
    !        NB - the difference is subtle!

    !   Local holder for mass zones. 
    DOUBLE PRECISION, DIMENSION(1000) :: TEMP__M, TEMP__DM, TEMP__DMI
    INTEGER :: J 

    ! Mass zone file is unformatted
    WRITE(*,*) 'PULS_READ_MASSZ: ',F__MASSZ
    OPEN(UNIT=NMASSZ,FILE=F__MASSZ,STATUS='OLD',FORM='UNFORMATTED')

    ! Number of mass zones is not defined a priori
    DO J = 1,1000
       READ (NMASSZ,END=92) TEMP__M(J),TEMP__DM(J),TEMP__DMI(J)
    ENDDO

92  CLOSE (NMASSZ)
    PULS__ND = J - 1
    WRITE(*,*) 'PULS_READ_MASSZ: masses for ',PULS__ND,' zones defined'

    ! truncate arrays to valid and required data only
    ALLOCATE (PULS__ZONES(PULS__ND))
    ALLOCATE (PULS__ZONED(PULS__ND))
    ALLOCATE (PULS__ZONEM(PULS__ND))
    ! reverse the mass zones so that surface is at start
    PULS__ZONES = TEMP__M(PULS__ND:1:-1)
    PULS__ZONED = TEMP__DM(PULS__ND:1:-1)

    !  integrate dm to generate mass from surface
    PULS__ZONEM(1) = PULS__ZONED(1)
    DO J = 2,PULS__ND
       PULS__ZONEM(J) = PULS__ZONEM(J-1) + PULS__ZONED(J)
    ENDDO

  END SUBROUTINE PULS_READ_MASSZ




  SUBROUTINE PULS_READ_HYDRO

    ! Read output from nlpulse hydro calculation 

    !   Local holders for hydro model. 
    REAL, DIMENSION(:),   ALLOCATABLE :: TMP__TIMES
    TYPE(PULS_HYDRO), DIMENSION(:,:), ALLOCATABLE :: PULS__TMP

    !   Local variables.
    INTEGER :: NT = 10000
    INTEGER :: J, JJ, K 
    REAL :: rr, pp, tt, uu, vv, ll, ee, kk, xx, qq, dt
    INTEGER :: it, NZ

    INTEGER :: F__IOSTAT = 0

    !.

    ALLOCATE (TMP__TIMES(NT))
    ALLOCATE (PULS__TMP(PULS__ND,NT))

    ! Static model is unformatted
    WRITE(*,*) 'PULS_READ_HYDRO: ',F__HYDRO
    OPEN(UNIT=NHYDRO,FILE=F__HYDRO,STATUS='OLD',FORM='UNFORMATTED')


    DO K = 1,NT
       PULS__NT = K
       READ (NHYDRO,END=93,IOSTAT=F__IOSTAT) TMP__TIMES(K), DT, IT, NZ
       ! debug ! WRITE (*,*) TEMP__TIMES(K), DT, IT, NZ
       IF ( NZ /= PULS__ND ) WRITE (*,*) & 
            'PULS_READ_HYDRO: *warning* zones inconsistent: k,puls__nd,nz=',K,PULS__ND,NZ

       DO JJ = 1,NZ
          J = NZ+1-JJ
          READ (NHYDRO,END=94) rr, pp, tt, uu, vv, ll, ee, kk, xx, qq

          PULS__TMP(J,K)%R = rr    ! radius
          PULS__TMP(J,K)%P = pp    ! pressure
          PULS__TMP(J,K)%T = tt    ! temperature
          PULS__TMP(J,K)%V = uu/1.E5  ! velocity : km/s
          PULS__TMP(J,K)%D = 1./vv ! density
          PULS__TMP(J,K)%L = ll    ! luminosity
          PULS__TMP(J,K)%K = kk    ! opacity (rosseland mean)

       ENDDO
    ENDDO

94  WRITE (*,*) 'PULS_READ_HYDRO: *warning* incomplete model: jj,nz=',JJ,NZ
93  CLOSE (NHYDRO)
    PULS__NT = PULS__NT - 1
    WRITE(*,*) 'PULS_READ_HYDRO: model read with ',PULS__NT,' timesteps'

    ! truncate arrays to valid and required data only
    ALLOCATE ( PULS__TIMES(PULS__NT) )
    PULS__TIMES = TMP__TIMES(1:PULS__NT)
    DEALLOCATE (TMP__TIMES)

    ALLOCATE ( PULS__MODEL(PULS__ND,PULS__NT) )
    PULS__MODEL = PULS__TMP(1:PULS__ND,1:PULS__NT)
    DEALLOCATE (PULS__TMP)

  END SUBROUTINE PULS_READ_HYDRO



END SUBROUTINE SPC_READPULS


