SUBROUTINE SPC_READQUB ( F__MODEL )

  !+
  !     
  !=    Reads in model atmosphere in "qub" format
  !     
  !     F__MODEL = 'X' on input avoids opening or closing the model file.
  !     
  !-

  !  Global variables.
  USE SPC_GLOBAL
  USE OP_GLOBAL, ONLY : OP_ALPNN
  USE LTE_MODEL, ONLY : LTE_TEMP => TEMP
  USE LTE_MODEL, ONLY : LTE_PG => P_TOTAL
  USE LTE_MODEL, ONLY : LTE_PE => P_E


  !  Double precision version
  IMPLICIT REAL*8 (A-H,O-Z)

  !  Filename
  CHARACTER (*), INTENT(IN) :: F__MODEL

  !.


  !     Define number of depth points
  NDEPTH=25


  !     Re-initialize abundances


  !     Open input stream
  IF ( LEN_TRIM(F__MODEL) /= 0 ) THEN 
     OPEN(UNIT=NMODEL,FILE=F__MODEL,STATUS='OLD')
  ENDIF

  !     Read the model atmosphere parameters

  READ (NMODEL,100,END=10) TEFF,GRAV,AHE,AH,ACN


  !     Check type of model, ie whether it conatins detailed abundances

  IF (AH == 0.0.AND.ACN == 0.0) THEN



     !     Model assumed to originate from ATLAS6
     !     set number of depth points and opacity switch
     !     NOPTYP = 0: QUB opacity

     NDEPTH = 25
     NOPTYP =  0
     TAU_WL = 4000.

     !     Keep helium to hydrogen ratio
     !     Note AHE is changed to fractional helium abundance in COMPOS

     HETOH=AHE

     !     Hydrogen rich atmosphere - calculate fractional abundances,
     !     abundance normalisation constant (ANORM) and mean atomic 
     !     weight (AMU), assuming solar metallicity

     CALL COMPOS

  ELSE



     !     Helium-rich / metal-rich atmosphere - read fractional abundances
     !     for metals from second and third lines of model, and calculate
     !     the mean atomic weight

     READ(NMODEL,100)  AC, AN,AO ,ANE,AMG

     !     Model assumed to originate from D.Schoenberner's program STERNE
     !     with KIEL opacities: set number of depth points and opacity switch
     !     NOPTYP = 1: KIEL opacity
     IF (AC.LE.1.01) THEN
        NDEPTH = 50
        NOPTYP =  1
        TAU_WL = 4000.

        !     Model assumed to originate from D.Schoenberner's program STERNE
        !     use data line to define opacity switch and number of depth points
        !     
        !     NOPTYP = 1: KIEL opacity
        !     NOPTYP = 2: OP   opacity

     ELSEIF (ABS(AC-2.0) <= 0.01) THEN
        NOPTYP = INT(AC)
        NDEPTH = INT(AN)
        READ(NMODEL,100)  AC, AN,AO ,ANE,AMG

     ENDIF

     READ(NMODEL,100) AAL,ASI,AS ,ACA,AFE

     AMU =  AH * 1.00  +  AHE* 4.00  +  AC *12.01  +  AN *14.01 &
          +  AO *16.00  +  ANE*20.18  +  AMG*24.31  +  AAL*26.98 &
          +  ASI*28.09  +  AS *32.06  +  ACA*40.08  +  AFE*55.85

  ENDIF

  !     Remove previously allocated arrays
  IF ( ALLOCATED(LTE_TEMP) ) DEALLOCATE (LTE_TEMP) 
  IF ( ALLOCATED(LTE_PG)   ) DEALLOCATE (LTE_PG) 
  IF ( ALLOCATED(LTE_PE)   ) DEALLOCATE (LTE_PE)   

  !     Make space for model in LTE arrays
  ALLOCATE (LTE_TEMP(NDEPTH))  
  ALLOCATE (LTE_PG(NDEPTH))   
  ALLOCATE (LTE_PE(NDEPTH))  


  !     Read the model structure

  READ (NMODEL,100) (TAU(J),TEMP(J),PG(J),PE(J),OP(J),J=1,NDEPTH)

  write(*,*) ' model read '

  !     Copy to LTE_MODEL for opacity calculations
  LTE_TEMP(1:NDEPTH) = TEMP(1:NDEPTH)
  LTE_PG(1:NDEPTH)   = PG(1:NDEPTH)
  LTE_PE(1:NDEPTH)   = PE(1:NDEPTH)


  !     Compute ionization fractions for all ionic species.
  !     This is idependent of model abundances and is determined
  !     only by the electron pressure and temperature structure.
  !     It is important for doing spectral syntheses.

  CALL OP_IONFRN (NDEPTH)
 
  write(*,*) '  ionfrn done '

  !     Read ionization fractions and overwrite
  !     CALL DP_RD_IONFRN

  !     Compute depth-dependent part of van der Waals' broadening coefficient.
  CALL SPC_VDWC (AH, AHE)


  !     Composition normalization

  !     Find normalisation constant for abundance scales
  !     (ANORM for QUB models already calculated in COMPOS)

  IF (NOPTYP == 1.OR.NOPTYP == 2) THEN

     AH_SOL   = 1. - SUM( 10.**(COSMIC(2:26)-12.) )
     C__SOL   = 12. - LOG10(AH_SOL)
     ! write (*,*) 'ah, c',ah_sol, c__sol
     AMU_SOL  = LOG10( SUM( ATW(1:26)*10.**(COSMIC(1:26)-12.) ) )
     AMU_STAR = LOG10( SUM( ATW(1:26)*ALPHA(1:26) ) )
     C__STAR  = C__SOL + AMU_SOL - AMU_STAR
     ! write (*,*) 'c*, mu*, c, mu', c__star, amu_star, c__sol, amu_sol 
     ANORM = C__STAR

  ENDIF


  !     Load composition data into OP global
  !     (currently assumes homogeneous composition)
  CALL OP_ALPSET (NDEPTH, ALPHA)
  CALL OP_ALPMU (NDEPTH)

  !     Calculate ionisation fractions and atomic data
  CALL OP_IONFRN (NDEPTH)
  CALL OP_ATFILL (NDEPTH)


  !     Report model input
  WRITE (*,'(/ ''MODEL ATMOSPHERE''//''F__MODEL: '',A)') F__MODEL 
  IF (NOPTYP == 0) WRITE (*,'( ''OPACITY: '',I1,A)') NOPTYP,' ATLAS/QUB'
  IF (NOPTYP == 1) WRITE (*,'( ''OPACITY: '',I1,A)') NOPTYP,' ATLAS/KIEL'
  IF (NOPTYP == 2) WRITE (*,'( ''OPACITY: '',I1,A)') NOPTYP,' KIEL.FF+OP.BF'
  WRITE (NWRITE,201) NDEPTH,TEFF,LOG10(GRAV),AHE/MAX(AH,1.D-10), &
       AH,AHE,AC,AN,AO,ANE,AMG,ASI,AS,ACA,AFE,AMU, &
       ANORM

  !     Open input stream
  IF ( LEN_TRIM(F__MODEL) /= 0 ) THEN 
     CLOSE (UNIT=NMODEL)
  ENDIF

  !     
  !     Debug output
  !     
  IF(BUGMD) THEN
     WRITE(NDEBUG,900)NDEPTH,TEFF,GRAV, &
          AH,AHE,AC,AN,AO,ANE,AMG,ASI,AS,ACA,AFE,AMU, &
          (TAU(J),TEMP(J),PG(J),PE(J),OP(J),J=1,NDEPTH)
  ENDIF

  RETURN

  !     
  !     No more models
  !     
10 WRITE(NWRITE,200)
  STOP


  !     Format statements     
100 FORMAT(5E12.5)
200 FORMAT(' ********** NO MORE MODELS *********')
201 FORMAT(' NDEPTH=',I5,3X,'TEFF=',0PF10.1, &
       &  3X,'LOG G=',0PF8.3,3X,'He/H=',0PF8.3//1P, &
       &  5X,' H=',E12.5,5X,'HE=',E12.5,5X,' C=',E12.5/ &
       &  5X,' N=',E12.5,5X,' O=',E12.5,5X,'NE=',E12.5/ &
       &  5X,'MG=',E12.5,5X,'SI=',E12.5,5X,' S=',E12.5/ &
       &  5X,'CA=',E12.5,5X,'FE=',E12.5,5X,/ &
       &  4X,'AMU=',0P,F9.4,8X,     15X,     2X,'ANORM=',F9.4/)

  !     Output NDEBUG
900 FORMAT(//' MODEL ATMOSPHERE'//' NDEPTH=',I5,5X,'TEFF=',0PF10.1, &
       &  5X,'GRAV=',1P,E12.5// &
       &  5X,'H=',E12.5,5X,'HE=',E12.5,5X,'C=',E12.5, &
       &  5X,'N=',E12.5,5X,'O=',E12.5,5X,'NE=',E12.5/ &
       &  5X,'MG=',E12.5,5X,'SI=',E12.5,5X,'S=',E12.5, &
       &  5X,'CA=',E12.5,5X,'FE=',E12.5,5X,'AMU=',E12.5// &
       &  9X,'TAU',8X,'TEMP',10X,'PG',10X,'PE',10X,'OP'// &
       &  30(5E12.5/))


END SUBROUTINE SPC_READQUB
