SUBROUTINE SPC_TAUSCALE ( NTAU, TAUN, TAUW )

  !+        
  !     Subroutine:
  !     SPC_TAUSCALE
  !     
  !     Package:
  !     SPC   subroutine library for SPECTRUM
  !     
  !     Purpose:
  !=    Map exsisting model atmopshere onto new optical depth scale
  !     
  !     Method:
  !     Optical depths are provided as subroutine arguments. 
  !     The reference wavelength is also provided. This is to future proof
  !     in case it is requested to change this value in the optical depth scale.  
  !     
  !     The existing model atmosphser sturucture is then mapped onto this scale
  !     by spline interpolation in log(tau). 
  !     
  !     Use: 
  !     CALL SPC_TAUSCALE ( NTAU, TAUN TAUW  )
  !     
  !     Authors:
  !     C.S.Jeffery (CSJ) Armagh Observatory
  !     
  !     History:
  !     21-MAY-03: (CSJ) Original version
  !     
  !-   

  !  Global variables
  USE SPC_GLOBAL
  USE OP_GLOBAL, ONLY : OP_ALPNN
  USE LTE_MODEL, ONLY : LTE__TEMP => TEMP
  USE LTE_MODEL, ONLY : LTE__PG => P_TOTAL
  USE LTE_MODEL, ONLY : LTE__PE => P_E

  !  No global typing
  IMPLICIT NONE


  !  Function interfaces
  INTERFACE
     FUNCTION NR_LOCATE (XX,X)
       DOUBLE PRECISION, DIMENSION(:), INTENT(IN) :: XX
       DOUBLE PRECISION, INTENT(IN) :: X
       INTEGER :: NR_LOCATE
     END FUNCTION NR_LOCATE
     SUBROUTINE NR_POLINT (XA,YA,X,Y,DY)
       !       USE NRUTIL, ONLY : NRERROR, IMINLOC
       DOUBLE PRECISION, DIMENSION(:), INTENT(IN) :: XA
       DOUBLE PRECISION, DIMENSION(:), INTENT(IN) :: YA
       DOUBLE PRECISION, INTENT(IN)               :: X
       DOUBLE PRECISION, INTENT(OUT) :: Y
       DOUBLE PRECISION, INTENT(OUT) :: DY
     END SUBROUTINE NR_POLINT
  END INTERFACE


  !  Subroutiine arguments (input):
  INTEGER, INTENT(IN)                          :: NTAU ! number of depths
  DOUBLE PRECISION, DIMENSION (*), INTENT(IN)  :: TAUN ! new optical depths
  DOUBLE PRECISION, INTENT(IN)                 :: TAUW ! reference wavelength

  !  Temporary store for interpolating coefficient s and new values
  DOUBLE PRECISION, DIMENSION (KD) :: TAX ! optical depth
  DOUBLE PRECISION, DIMENSION (KD) :: TEN ! temperature
  DOUBLE PRECISION, DIMENSION (KD) :: PGN ! gas pressure
  DOUBLE PRECISION, DIMENSION (KD) :: PEN ! electron pressure
  DOUBLE PRECISION, DIMENSION (KD) :: OPN ! opacity
  DOUBLE PRECISION                 :: ERR

  INTEGER J, N ! interpolation pointer

  !  Sterne default abundance vector      
  INTEGER, DIMENSION(11) :: STN__K = (/ 1,2,6,7,8,12,13,14,16,20,26 /)
  DOUBLE PRECISION, DIMENSION(93) :: ALPHA__ND     ! Composition vector


  !. 

 
  IF ( NTAU > KD ) WRITE (*,'( ''SPC_TAUSCALE: NTAU > KD'',2I3)') NTAU,KD 

  !     Convert to logarithm
  TAX(1:NTAU)    = DLOG10(TAUN(1:NTAU))
  TAU(1:NDEPTH)  = DLOG10(TAU(1:NDEPTH))
  TEMP(1:NDEPTH) = DLOG10(TEMP(1:NDEPTH))
  PG(1:NDEPTH)   = DLOG10(PG(1:NDEPTH))
  PE(1:NDEPTH)   = DLOG10(PE(1:NDEPTH))
  OP(1:NDEPTH)   = DLOG10(OP(1:NDEPTH))

  ! Read current composition (assume homogeneous) ***
  ALPHA__ND(STN__K) = OP_ALPNN(STN__K,NDEPTH)

  !write (*,*) 'SPC_TAUSCALE'
  !      write (*,'(1p,20e9.2)') tau(1:ndepth)
  !      write (*,'(1p,20e9.2)') temp(1:ndepth)
  !      write (*,'(1p,20e9.2)') pg(1:ndepth)
  !      write (*,'(1p,20e9.2)') pe(1:ndepth)
  !      write (*,'(1p,20e9.2)') op(1:ndepth)

  !     Evaluate splines at new optical depths
  DO J = 1,NTAU
     N = MIN(MAX(1,NR_LOCATE(TAU(1:NDEPTH),TAX(J))-1),NDEPTH-3)
     CALL NR_POLINT (TAU(N:N+3),TEMP(N:N+3),TAX(J),TEN(J),ERR) 
     CALL NR_POLINT (TAU(N:N+3),PG(N:N+3),  TAX(J),PGN(J),ERR) 
     CALL NR_POLINT (TAU(N:N+3),PE(N:N+3),  TAX(J),PEN(J),ERR) 
     CALL NR_POLINT (TAU(N:N+3),OP(N:N+3),  TAX(J),OPN(J),ERR)
     !   write (*,'(2i5, 1p,20e9.2)') j,n,tax(j),ten(j),pgn(j),pen(j),opn(j)
  ENDDO

  !      write (*,'(1p,20e9.2)') tax(1:ntau)
  !      write (*,'(1p,20e9.2)') ten(1:ntau)
  !      write (*,'(1p,20e9.2)') pgn(1:ntau)
  !      write (*,'(1p,20e9.2)') pen(1:ntau)
  !      write (*,'(1p,20e9.2)') opn(1:ntau)

  !     Copy new scale back into model arrays
  TAU(1:NTAU)  = 10.**(TAX(1:NTAU))
  TEMP(1:NTAU) = 10.**(TEN(1:NTAU))
  PG(1:NTAU)   = 10.**(PGN(1:NTAU))
  PE(1:NTAU)   = 10.**(PEN(1:NTAU))
  OP(1:NTAU)   = 10.**(OPN(1:NTAU))
  NDEPTH = NTAU

  !     Remove previously allocated arrays
  IF ( ALLOCATED(LTE__TEMP) ) DEALLOCATE (LTE__TEMP) 
  IF ( ALLOCATED(LTE__PG)   ) DEALLOCATE (LTE__PG) 
  IF ( ALLOCATED(LTE__PE)   ) DEALLOCATE (LTE__PE)   
  !     Make space for model in LTE arrays
  ALLOCATE (LTE__TEMP(NDEPTH))  
  ALLOCATE (LTE__PG(NDEPTH))   
  ALLOCATE (LTE__PE(NDEPTH))  
  !     Copy to LTE_MODEL for opacity calculations
  LTE__TEMP(1:NDEPTH) = TEMP(1:NDEPTH)
  LTE__PG(1:NDEPTH)   = PG(1:NDEPTH)
  LTE__PE(1:NDEPTH)   = PE(1:NDEPTH)
  !     Reset composition
  CALL OP_ALPSET (NDEPTH, ALPHA__ND)
  ! write (*,*) 'SPC_TAUSC -- ALPHA__ND :', alpha__nd
  CALL OP_ALPMU (NDEPTH)
  !     Store ionisation fractions and level populations in OP data arrays
  CALL OP_IONFRN (NDEPTH)
  CALL OP_ATFILL (NDEPTH)
  CALL SPC_VDWC (AH, AHE)

END SUBROUTINE SPC_TAUSCALE
