      SUBROUTINE SPC_VDATA (VTYPE, NV, V_TAU, V_VEL)

!+     
!     Subroutine:
!     SPC_VDATA
!     
!     Package:
!     SPC   subroutine library for SPECTRUM
!     
!     Purpose:
!=    Propagate depth dependent velocities into model
!     or depth-dependent temperature perturbation
!     
!     Method:
!     Data are input in from subroutine arguments as
!     functions of tau. 
!     
!     Radial velocity is assumed to be positive in the
!     outward direction (i.e. towards lower optical depth). 
!     
!     For all depthe less than V_TAU(1), velocities are set to V_VEL(1)
!     For all depthe greater than V_TAU(NV), velocities are set to V_VEL(NV)
!     Between these values, V_VEL(TAU) is obtained by quadratic interpolatiion in V_TAU. 
!     
!     Use: 
!     CALL SPC_VDATA ( VTYPE,  NV, V_TAU, V_VEL  )
!     
!     Example:.
!     CALL SPC_VDATA ('VTRB', NV, SP_V_TAU, SP_V_VTRB )
!     CALL SPC_VDATA ('VRAD', NV, SP_V_TAU, SP_V_VRAD  )
!     
!     Authors:
!     C.S.Jeffery (CSJ) Armagh Observatory
!     
!     History:
!     15-MAY-03: (CSJ) Original version
!     04-SEP-12: (CSJ) Minor updates
!     
!-

!  Global variables
      USE SPC_GLOBAL

!  Double precision version     
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)

!  Subroutiine arguments (input):
      CHARACTER (*), INTENT(IN)                    :: VTYPE ! type of velocities
      INTEGER, INTENT(IN)                          :: NV ! number of velocities
      DOUBLE PRECISION, DIMENSION (*), INTENT(IN)  :: V_TAU ! optical depths
      DOUBLE PRECISION, DIMENSION (*), INTENT(IN)  :: V_VEL ! velocities

!  Temporary store for velocities       
      DOUBLE PRECISION                 :: TJ
      DOUBLE PRECISION, DIMENSION (KD) :: V_TMP ! interpolated velocities

!.

!      IF ( VTYPE(1:4) == 'TTAU' ) THEN
!      write (*,*) 'temp'
!      write (*,*) temp(1:ndepth)
!      ENDIF


!     Only one velocity for all depths...
      IF ( NV == 1 ) THEN
        V_TMP(1:NDEPTH) = (/ (V_VEL(1),J=1,NDEPTH) /)


      ELSE
        NP   = MIN(NV-1,2)
        DO J = 1,NDEPTH
          TJ = TAU(J)
          IF ( TJ <= V_TAU(1) ) THEN
            V_TMP(J) = V_VEL(1)
          ELSEIF ( TJ >= V_TAU(NV) ) THEN
            V_TMP(J) = V_VEL(NV)
          ELSE
            CALL MTH_D_TABINT ( NV,V_TAU,V_VEL,NP,TJ,V_TMP(J) )
          ENDIF
        ENDDO
      ENDIF

!     .. set depth-dependent microturbulent velocities
      IF ( VTYPE(1:4) == 'VTRB' ) QUB__VTRB = V_TMP

!     .. set depth-dependent radial velocities
      IF ( VTYPE(1:4) == 'VRAD' ) QUB__VRAD = V_TMP

!     .. use this subroutine for temperature and velocity poerturbations
      IF ( VTYPE(1:4) == 'TTAU' ) TEMP(1:NDEPTH) = TEMP(1:NDEPTH) * V_TMP(1:NDEPTH)

!     .. use this subroutine for temperature and velocity poerturbations
      IF ( VTYPE(1:4) == 'TINV' ) TEMP(1:NDEPTH) = TEMP(1:NDEPTH) / V_TMP(1:NDEPTH)


!      IF ( VTYPE(1:1) == 'T' ) THEN
!      write (*,*) vtype
!      write (*,*) v_tmp(1:ndepth)
!      write (*,*) temp(1:ndepth)
!      ENDIF

      END SUBROUTINE SPC_VDATA
