SUBROUTINE KEY_GET (KEYWRD, PARSTR, KPROMPT, KSEP, KUNIT, KECHO, KSTAT)

  !+
  !  Program Name:
  !     KEY_GET
  !
  !  Program Type:
  !     Fortran95 subroutine
  !
  !  Purpose:
  !=    Recover the next keyword from the command line
  !
  !  Method:
  !     The existing string PARSTR is inspected for unprocessed information.
  !     If it is blank, a new command string is requested (see Prompt) from unit 
  !     defined by KUNIT, otherwise from STDIN. This
  !     string is echoed to the terminal (see Echo). Any trailing comments are
  !     removed. Initial blanks are stripped and separators are isolated from 
  !     adjoining text. The first complete word, terminated either by a blank 
  !     or an "=" sign, is detached from PARSTR and placed in KEYWRD. The 
  !     remainder of PARSTR is retained.
  !
  !     Comments: exclamation mark strings following an exclamation mark
  !               will be ignored
  !     Prompt: New commands will be prompted with KPROMPT as required, 
  !               if undefined, the prompt is silent
  !     Echo: Commands are echoed to the terminal if KECHO > 0
  !     Separators: characters defined in KSEP will be treated as separators.
  !               to avoid parsing errors, additional blank characters are
  !               inserted either side of a separator
  !
  !  Use:
  !     CALL KEY_GET  (KEYWRD, PARSTR, KPROMPT, KSEP, KUNIT, KECHO, KSTAT) 
  !
  !  Arguments:
  !     KEYWRD (Character, Returned, Required) 
  !           : the command extracted as first word of the current command string
  !     PARSTR (Character, Given/Returned, Required) 
  !           : the current command string
  !     KPROMPT (Character, Given, Optional)
  !           : String echoed to terminal as prompt (Default is "")
  !     KSEP (Character, Given, Optional)
  !           : String used to separate commands in PARSTR (Default is "")
  !     KUNIT  (Integer, Given, Optional) 
  !           : Unit on which PARSTR is to be read (Default is "stdin")
  !     KECHO (Integer, Given, Optional)
  !           : Switch governing whteher input string is echoed to "stdout": 
  !             0 = echo off, 1 = echo on (Default = 0).
  !     KSTAT (Integer, Returned, Optional)     
  !           : Status flag indicating read error state: 0 = OK, 1 = EOF, 2 = ERR
  !
  !  Examples:
  !     CALL KEY_GET  (KEYWRD, PARSTR, 'enter>', ',', 5, 0, KSTAT) 
  !     CALL KEY_GET  (KEYWRD, PARSTR)
  !     CALL KEY_GET  (KEYWRD, PARSTR, ksep=';', kecho='1')
  !
  !  Note:
  !     It is necessary to use an INTERFACE to call this routine 
  !
  !INTERFACE
  !
  !  SUBROUTINE KEY_GET (KEYWRD, PARSTR, KPROMPT, KSEP, KUNIT, KECHO, KSTAT) 
  !
  !    CHARACTER, INTENT(OUT)          :: KEYWRD*(*)  ! (Returned).
  !    CHARACTER, INTENT(INOUT)        :: PARSTR*(*)  ! (Given and Returned).
  !    CHARACTER, INTENT(IN), OPTIONAL :: KPROMPT*(*) ! (Given).
  !    CHARACTER, INTENT(IN), OPTIONAL :: KSEP*(*)    ! (Given).
  !    INTEGER, INTENT(IN), OPTIONAL   :: KUNIT       ! (Given).
  !    INTEGER, INTENT(IN), OPTIONAL   :: KECHO       ! (Given).
  !    INTEGER, INTENT(OUT), OPTIONAL  :: KSTAT       ! (Returned).
  !
  !  END SUBROUTINE KEY_GET
  !
  !END INTERFACE
  !
  !
  !  History:
  !     16-OCT-2004: CSJ
  !        Added "optional" attributes, Separator handling
  !     13-OCT-2004: CSJ 
  !        Added "prompt" facility
  !     16-OCT-2004: CSJ
  !        Added minor upgrades  to maintain compatibility with STR library
  !     xx-xxx-1987: CSJ 
  !        Original and Principal component of "KEY" package
  !
  !  Author:
  !     C. S. Jeffery (CSJ) Kiel, St Andrews, Armagh. 
  !
  !-

  CHARACTER, INTENT(OUT)           :: KEYWRD*(*)
  CHARACTER, INTENT(INOUT)         :: PARSTR*(*)
  CHARACTER, INTENT(IN),  OPTIONAL :: KPROMPT*(*)
  CHARACTER, INTENT(IN),  OPTIONAL :: KSEP*(*)
  INTEGER,   INTENT(IN),  OPTIONAL :: KUNIT
  INTEGER,   INTENT(IN),  OPTIONAL :: KECHO
  INTEGER,   INTENT(OUT), OPTIONAL :: KSTAT


  !.

  IF ( PRESENT (KSTAT) ) KSTAT = 0

  !   ..Check status of command line
  CALL STR_LEN ( PARSTR, LP )

  IF (LP == 0) THEN

    !     ..Read input data, skipping blank lines
1   IF ( PRESENT (KPROMPT) ) WRITE (*,'(A,'' '',$)') KPROMPT
    IF ( PRESENT (KUNIT) ) THEN
      READ (KUNIT,'(A)',END=998,ERR=999) PARSTR
    ELSE
      READ (*,'(A)',END=998,ERR=999) PARSTR
    ENDIF
    CALL STR_LEN ( PARSTR, LP )
    IF ( PRESENT (KECHO) ) THEN 
      IF (KECHO /= 0) CALL STR_TEXT ( PARSTR(1:LP) )
    ENDIF

    !     ..Remove trailing comments
    LC = INDEX (PARSTR,'!')
    IF (LC == 1) GOTO 1
    IF (LC  > 1) CALL STR_TRIM ( PARSTR, LC-1 )

    !     ..Skip blank lines
    CALL STR_LEN ( PARSTR, LS )
    IF (LS.EQ.0) GOTO 1
  ENDIF

  !   ..Strip leading blanks
  LP=LEN(PARSTR)
  DO I = 1,LP
    IF (PARSTR(I:I) /= ' ') EXIT
  ENDDO
  PARSTR(1:1+LP-I) = PARSTR(I:LP)

  !   ..Isolate separators with blanks
  CALL STR_LEN(PARSTR, LP)
  LS = LP
  I  = 1
  IF ( PRESENT (KSEP) ) THEN
    DO I = LP,2,-1
      IF (PARSTR(I:I) == KSEP ) THEN
        ! check for blank space after separator
        IF (PARSTR(I+1:I+1) /= ' ') THEN
          PARSTR(I+2:LS) = PARSTR(I+1:LS-1)
          PARSTR(I+1:I+1)  = ' '
          LS = MIN(LP,LS + 1)
        ENDIF
        ! check for blank space before separator
        IF (PARSTR(I-1:I-1) /= ' ') THEN
          PARSTR(I+1:LS) = PARSTR(I:LS-1)
          PARSTR(I:I)  = ' '
          LS = MIN(LP,LS + 1)
        ENDIF
      ENDIF
    ENDDO
  ENDIF
  LP = LS

  !   ..Check tail is padded with blanks
  CALL STR_NULL ( PARSTR(2+LP-I:LEN(PARSTR)) )

  !.....Decode the first keyword (delimited by blank or equals)
  CALL STR_LEN ( PARSTR, ICLEN )
  ICEQN = INDEX (PARSTR, '=')
  ICBLN = INDEX (PARSTR, ' ')
  IF (ICEQN > 0.AND.ICEQN < ICBLN) ICBLN = ICEQN
  IF (ICBLN > 0.AND.ICBLN < ICLEN) THEN
    KEYWRD = PARSTR(1:ICBLN-1)
    CALL STR_TRIM ( KEYWRD, ICBLN-1 )
    PARSTR = PARSTR(ICBLN+1:ICLEN)
    CALL STR_TRIM ( PARSTR, ICLEN-ICBLN )
  ELSE
    KEYWRD = PARSTR
    CALL STR_NULL ( PARSTR ) 
  ENDIF

  RETURN


  !.....Error actions

  !   ..End of input
998 IF ( PRESENT (KSTAT) ) KSTAT = 1
  KEYWRD = 'END'
  CALL STR_NULL ( PARSTR )
  RETURN

  !   ..Error reading input
999 IF ( PRESENT (KSTAT) ) KSTAT = 2
  CALL STR_NULL ( KEYWRD )
  CALL STR_NULL ( PARSTR )
  RETURN

END SUBROUTINE KEY_GET
