DOUBLE PRECISION FUNCTION TAP1_POWERG ( Z, STATUS )

!+
!  Program:
!     TAP1_POWERG

!  Purpose:
!=    Evaluate power series expansion for ln Gamma (z)

!  Language:
!     Fortran 90

!  Invocation:
!     Should not be invoked except by TAP_GAMMLN
 
!  Package:
!     TAP: Theoretical Astrophysics Package

!  Module type:
!     double precision function
!     auxiliary subprogram

!  Description:
!     The Gamma function is evaluated by means of a power series expansion
!     valid for |z|<2
!
!        ln (Gamma(1+z)) = -ln(1+z) + z(1-gamma) + sum(n=2,...)(Cn.z^n)
!
!        Cn = (-1)**n * (Zeta(n)-1) / n
!
!        Zeta(n) is the Riemann Zeta function. 
!
!     The constants Cn have been evaluated to 34 d.p. using the symbolic 
!     mathematical computation package MAPLEV...
!
!     It is designed to converge to machine precision, but this depends on
!     the number of terms (Cn) available. In the current version, the
!     TAP parameter TAP__DSMAL is used as convergence criterion. In the
!     range 1 < z < 2, the function is accurate to at least 10 decimal 
!     places in a comparison with the tables in Abramowitz and Stegun (1971). 
!
!     In the range 0.5 < z < 1.5, the series converges to 1 part in 10^10
!     after 15 terms. For z = 2, approximately 30 terms are required.

!  Arguments:
!     Z = DOUBLE PRECISION (Given)
!        The real number Z for which ln(GAMMA(X)) is required.
!     STATUS = INTEGER (Given and returned)
!        The inheritied global status.

!  Returned Value:
!     TAP1_POWERG = DOUBLE PRECISION
!        LOG (GAMMA(Z)) as a real number

!  Authors:
!     CSJ: C.S.Jeffery (CCP7)
!     {enter_new_authors_here}

!  History:
!     5-AUG-1993 (CSJ):
!        Original version.
!     8-JAN-2002 (CSJ):
!        Convert to F90.
!     {enter_changes_here}

!  Bugs:
!     {note_any_bugs_here}

!  Notes:
!     This function subprogram must _only_ be used in association with
!     TAP_GAMMLN or an equivalent front end, in order to check for
!     argument validity.

!-

!  Global constants.
      USE TAP_PAR
      USE TAPM_PAR
      USE TAP_ERR

!  Type definitions.
      IMPLICIT NONE

!  Global status.
      INTEGER :: STATUS

!  Arguments given.
      DOUBLE PRECISION :: Z

!  Local constants.
      INTEGER, PARAMETER :: NTERMS = 40

!  Local variables.
      INTEGER  :: N
      DOUBLE PRECISION :: H,S,X,XN

      DOUBLE PRECISION :: C(NTERMS)
      DATA (C(N),N=1,10)                                   &
             / 0.0                                       , &
               0.3224670334241132182362075833230126      , &
              -0.0673523010531980951332460538371500      , &
               0.0205808084277845478790009241352920      , &
              -0.0073855510286739852662730972914068      , &
               0.0028905103307415232857529882984868      , &
              -0.0011927539117032609771139356928281      , &
               0.0005096695247430424223356548135816      , &
              -0.0002231547584535793797614188036013      , &
               0.0000994575127818085337145958900319      /

      DATA (C(N),N=11,20)                                  &
             /-0.00004492623673813314170020750240636     , &
               0.00002050721277567069155316650397836     , &
                   -0.943948827526839590398742510438D-05 , &
                    0.437486678990748780418179322400D-05 , &
                   -0.203921575380136623678190070967D-05 , &
                     0.95514121304074198328571797734D-06 , &
                    -0.44924691987645660432942903312D-06 , &
                     0.21207184805554665869231359015D-06 , &
                    -0.10043224823968099608720830500D-06 , &
                      0.4769810169363980565760193423D-07 /

      DATA (C(N),N=21,30)                                  &
             /       -0.2271109460894316491031998114D-07 , &
                      0.1083865921489695409107491763D-07 , &
                      -0.518347504197004665512124865D-08 , &
                       0.248367454380247831718500871D-08 , &
                      -0.119214014058609120744254820D-08 , &
                        0.57313672416788620133301954D-09 , &
                       -0.27595228851242331451781496D-09 , &
                        0.13304764374244489481497163D-09 , &
                        -0.6422964563838100022082448D-10 , &
                         0.3104424774732227276239223D-10 /

      DATA (C(N),N=31,40)                                  &
             /          -0.1502138408075414217093300D-10 , &
                          0.727597448023907966250459D-11 , &
                         -0.352774247657591508361506D-11 , &
                          0.171199179055961790860114D-11 , &
                          -0.83153858414202848197983D-12 , &
                           0.40422005252894400655366D-12 , &
                          -0.19664756310966164904111D-12 , &
                            0.9573630387838555763788D-13 , &
                           -0.4664076026428374224577D-13 , &
                            0.2273736960065972320639D-13 /

!.

!  initialize variables.
      H = DBLE(0)
      N = 1

!  actually evaluate Gamma(1+x) = Gamma(z).
      X = Z - 1.0D+00

!  first term.
      S = - LOG (1.0D+00+X) + X * (1.0D+00 - TAPM__GAMMA)
      XN = X

!  continue summation until series converges.
      DO N = 2,NTERMS
        H = S
        XN = XN * X
        S = S + C(N) * XN
        IF ( ABS(S-H).LE.TAP__DSMAL ) GOTO 20
      ENDDO

!  unconverged result
      STATUS = TAP__BADCONV

!  set result.
20    TAP1_POWERG = S
      END




