DOUBLE PRECISION FUNCTION TAP_ABCOS (ZELEM)  

!+
!  Name:
!     TAP_ABCOS

!  Purpose:
!=    Return the relative cosmi abundance for a given element.

!  Language:
!     Fortran 90

!  Invocation:
!     RESULT = TAP_ABCOS( ZELEM )

!  Description:
!     Given the atomic number of the element, the recommended value of
!     the cosmic element abundance relative to hydrogen is returned.
!     The abundances are based primarily on the compilation of solar
!     and meteorite data by Anders & Grevesse (1989), supplemented in a
!     few cases by more recent observations.
!
!     In general, the solar and meteorite data are in good agreement.
!     Some exceptions (e.g. Lithium) are obvious, whilst others (e.g.
!     Indium and Tungsten) are less so. The solar data are returned
!     where possible, meteoritic data are returned where there are no
!     solar data.
!
!     All solar and meteoritic abundances are given in logarithmic form
!     relative to the solar hydrogen abundance, which is defined to be
!     12.00.  Cases where no data are available for a given element
!     from either source are indicated by a returned value of
!     TAP__DNULL.

!  Arguments:
!     ZELEM = INTEGER (Given)
!        The atomic number (Z) for which the abundance is required.

!  Returned Value:
!     TAP_ABSOL = DOUBLE PRECISION
!        The cosmic abundance (logarithm with H = 12.00).

!  Accuracy:
!     See references for full details.
!
!     Solar  abundances: errors lie between +/- 0.03 and +/- 0.10 dex.
!            Exceptions:  F, Cl,  K, Ge, Rb, Rh, Ag, Cd, In, Sn, Sb, Ce,
!                        Tb, Dy, Ho, Tm, Yb, Lu,  W, Pt, Au, Tl.
!
!     Meteor abundances: errors lie between +/- 0.01 and +/- 0.05 dex.
!            Exceptions: F, Cl, Br, Kr, Sb, I, Xe, Au.

!  References:
!     Anders E. & Grevesse N. 1989. Geochim. Cosmochim. Acta 53, 197.
!     Holden N.E., Martin R.L., & Barnes I.L. 1984. Pure Appl. Chem. 56,
!        675.
!
!     Solar Fe:
!        Holweger H., Heise C. & Kock M. 1990. Astron.Astrophys. 232,
!           510.
!
!     Solar C:
!        Stuerenberg S. & Holweger H. 1990. Astron.Astrophys. 237, 125.

!  Authors:
!     CSJ: C.S.Jeffery (St Andrews)
!     PCTR: P.C.T. Rees (STARLINK)
!     {enter_new_authors_here}

!  History:
!     6-MAY-1992 (CSJ):
!        Original version.
!     8-JUL-1992 (PCTR):
!        Changed to a DOUBLE PRECISION function and removed the status
!        argument.
!     8-JAN-2002 (CSJ):
!        Conversion to Fortan 90
!     {enter_further_changes_here}

!  Bugs:
!     {note_any_bugs_here}

!-


!  Global Constants:
   USE TAP_PAR        ! TAP public constants

!  Type Definitions:
   IMPLICIT NONE      ! No implicit typing

!  Arguments Given:
   INTEGER :: ZELEM  

!  External References:
                                 
   DOUBLE PRECISION :: TAP_ABMET  ! Meteoritic abundances
   DOUBLE PRECISION :: TAP_ABSOL  ! Solar abundances

  
!  Local Constants:

   ! A small DOUBLE PRECISION constant
   DOUBLE PRECISION, PARAMETER :: SMALL = TAP__DNULL * TAP__DSMAL       


!.
!  Check for a bad atomic number and missing data.
   IF ( (ZELEM.LE.0) .OR. (ZELEM.GT.TAP__NELEM) ) THEN
  
   !  Bad atomic number.
      TAP_ABCOS = TAP__DNULL  

   ELSE  

   !  Get a solar abundance and check for a NULL value.
      TAP_ABCOS = TAP_ABSOL (ZELEM)  

   !  Missing solar abundance data, so get a meteoritic abundance.
      IF ( (TAP_ABCOS.LT.TAP__DNULL + ABS (SMALL) ) .AND. & 
           (TAP_ABCOS.GT.TAP__DNULL - ABS (SMALL) ) ) THEN
        TAP_ABCOS = TAP_ABMET (ZELEM)  
      ENDIF  

   ENDIF  

END FUNCTION TAP_ABCOS
