DOUBLE PRECISION FUNCTION TAP_ABMET( ZELEM )

!+
!  Name:
!     TAP_ABMET

!  Purpose:
!=    Return the relative meteoritic abundance for a given element.

!  Language:
!     Fortran 90

!  Invocation:
!     RESULT = TAP_ABMET( ZELEM )

!  Description:
!     Given the atomic number of the element, the recommended value for
!     the meteoritic element abundance relative to hydrogen is
!     returned.  The abundances are based primarily on the compilation
!     of solar and meteorite data by Anders & Grevesse (1989),
!     supplemented in a few cases by more recent observations.
!
!     In general, the solar and meteorite data are in good agreement.
!     Some exceptions (e.g. Lithium) are obvious, whilst others (e.g.
!     Indium and Tungsten) are less so. The solar data will find
!     applications in stellar theory, whilst it is expected that the
!     meteorite data should be more useful for ISM studies.
!     
!     All solar and meteoritic abundances are given in logarithmic form
!     relative to the solar hydrogen abundance, which is defined to be
!     12.00.  Cases where no data are available for a given element
!     are indicated by a returned value of TAP__DNULL.

!  Arguments:
!     ZELEM = INTEGER (Given)
!        The atomic number (Z) for which the abundance is required

!  Returned Value:
!     TAP_ABMET = DOUBLE PRECISION
!        The solar system meteorite abundance (logarithm with H =
!        12.00).

!  Accuracy:
!     See references for full details.
!
!     Solar  abundances: errors lie between +/- 0.03 and +/- 0.10 dex.
!            Exceptions:  F, Cl,  K, Ge, Rb, Rh, Ag, Cd, In, Sn, Sb, Ce,
!                        Tb, Dy, Ho, Tm, Yb, Lu,  W, Pt, Au, Tl.
!                       
!     Meteor abundances: errors lie between +/- 0.01 and +/- 0.05 dex.
!            Exceptions: F, Cl, Br, Kr, Sb, I, Xe, Au.

!  References:
!     Anders E. & Grevesse N. 1989. Geochim. Cosmochim. Acta 53, 197.
!     Holden N.E., Martin R.L., & Barnes I.L. 1984. Pure Appl. Chem. 56,
!        675.
!
!     Solar Fe:
!        Holweger H., Heise C. & Kock M. 1990. Astron.Astrophys. 232,
!           510.
!
!     Solar C:
!        Stuerenberg S. & Holweger H. 1990. Astron.Astrophys. 237, 125.

!  Authors:
!     CSJ: C.S.Jeffery (St Andrews)
!     PCTR: P.C.T. Rees (STARLINK)
!     {enter_new_authors_here}

!  History:
!     6-MAY-1992 (CSJ):
!        Original version.
!     8-JUL-1992 (PCTR):
!        Changed to a DOUBLE PRECISION function and removed the status
!        argument.
!     8-JAN-2002 (CSJ):
!        Convert to Fortran 90.
!     {enter_further_changes_here}

!  Bugs:
!     {note_any_bugs_here}

!-

!  Global Constants:
      USE TAP_PAR          ! TAP public constants
      
!  Type Definitions:
      IMPLICIT NONE              ! No implicit typing

!  Arguments Given:
      INTEGER :: ZELEM

!  Local Constants:
! A small REAL constant
      REAL, PARAMETER :: SMALL = TAP__RNULL * TAP__RSMAL 
      
!  Local Variables:
      REAL :: ABUNDM( TAP__NELEM )  ! Meteoritic element abundance ratios

!  Local Data:
!     Meteoritic element abundances:

!  H
      DATA ABUNDM( 1 ) / TAP__RNULL /

!  He
      DATA ABUNDM( 2 ) / TAP__RNULL /

!  Li
      DATA ABUNDM( 3 ) / 3.31 /

!  Be
      DATA ABUNDM( 4 ) / 1.42 /

!  B
      DATA ABUNDM( 5 ) / 2.88 /

!  C
      DATA ABUNDM( 6 ) / TAP__RNULL /

!  N
      DATA ABUNDM( 7 ) / TAP__RNULL /

!  O
      DATA ABUNDM( 8 ) / TAP__RNULL /

!  F
      DATA ABUNDM( 9 ) / 4.48 /

!  Ne
      DATA ABUNDM( 10 ) / TAP__RNULL /

!  Na
      DATA ABUNDM( 11 ) / 6.31 /

!  Mg
      DATA ABUNDM( 12 ) / 7.58 /

!  Al
      DATA ABUNDM( 13 ) / 6.48 /

!  Si
      DATA ABUNDM( 14 ) / 7.55 /

!  P
      DATA ABUNDM( 15 ) / 5.57 /

!  S
      DATA ABUNDM( 16 ) / 7.27 /

!  Cl
      DATA ABUNDM( 17 ) / 5.27 /

!  Ar
      DATA ABUNDM( 18 ) / TAP__RNULL /

!  K
      DATA ABUNDM( 19 ) / 5.13 /

!  Ca
      DATA ABUNDM( 20 ) / 6.34 /

!  Sc
      DATA ABUNDM( 21 ) / 3.09 /

!  Ti
      DATA ABUNDM( 22 ) / 4.93 /

!  V
      DATA ABUNDM( 23 ) / 4.02 /

!  Cr
      DATA ABUNDM( 24 ) / 5.68 /

!  Mn
      DATA ABUNDM( 25 ) / 5.53 /

!  Fe
      DATA ABUNDM( 26 ) / 7.51 /

!  Co
      DATA ABUNDM( 27 ) / 4.91 /

!  Ni
      DATA ABUNDM( 28 ) / 6.25 /

!  Cu
      DATA ABUNDM( 29 ) / 4.27 /

!  Zn
      DATA ABUNDM( 30 ) / 4.65 /

!  Ga
      DATA ABUNDM( 31 ) / 3.13 /

!  Ge
      DATA ABUNDM( 32 ) / 3.63 /

!  As
      DATA ABUNDM( 33 ) / 2.37 /

!  Se
      DATA ABUNDM( 34 ) / 3.35 /

!  Br
      DATA ABUNDM( 35 ) / 2.63 /

!  Kr
      DATA ABUNDM( 36 ) / 3.23 /

!  Rb
      DATA ABUNDM( 37 ) / 2.40 /

!  Sr
      DATA ABUNDM( 38 ) / 2.93 /

!  Y
      DATA ABUNDM( 39 ) / 2.22 /

!  Zr
      DATA ABUNDM( 40 ) / 2.61 /

!  Nb
      DATA ABUNDM( 41 ) / 1.40 /

!  Mo
      DATA ABUNDM( 42 ) / 1.96 /

!  Tc
      DATA ABUNDM( 43 ) / TAP__RNULL /

!  Ru
      DATA ABUNDM( 44 ) / 1.82 /

!  Rh
      DATA ABUNDM( 45 ) / 1.09 /

!  Pd
      DATA ABUNDM( 46 ) / 1.70 /

!  Ag
      DATA ABUNDM( 47 ) / 1.24 /

!  Cd
      DATA ABUNDM( 48 ) / 1.76 /

!  In
      DATA ABUNDM( 49 ) / 0.82 /

!  Sn
      DATA ABUNDM( 50 ) / 2.14 /

!  Sb
      DATA ABUNDM( 51 ) / 1.04 /

!  Te
      DATA ABUNDM( 52 ) / 2.24 /

!  I
      DATA ABUNDM( 53 ) / 1.51 /

!  Xe
      DATA ABUNDM( 54 ) / 2.23 /

!  Cs
      DATA ABUNDM( 55 ) / 1.12 /

!  Ba
      DATA ABUNDM( 56 ) / 2.21 /

!  La
      DATA ABUNDM( 57 ) / 1.20 /

!  Ce
      DATA ABUNDM( 58 ) / 1.61 /

!  Pr
      DATA ABUNDM( 59 ) / 0.78 /

!  Nd
      DATA ABUNDM( 60 ) / 1.47 /

!  Pm
      DATA ABUNDM( 61 ) / TAP__RNULL /

!  Sm
      DATA ABUNDM( 62 ) / 0.97 /

!  Eu
      DATA ABUNDM( 63 ) / 0.54 /

!  Gd
      DATA ABUNDM( 64 ) / 1.07 /

!  Tb
      DATA ABUNDM( 65 ) / 0.33 /

!  Dy
      DATA ABUNDM( 66 ) / 1.15 /

!  Ho
      DATA ABUNDM( 67 ) / 0.50 /

!  Er
      DATA ABUNDM( 68 ) / 0.95 /

!  Tm
      DATA ABUNDM( 69 ) / 0.13 /

!  Yb
      DATA ABUNDM( 70 ) / 0.95 /

!  Lu
      DATA ABUNDM( 71 ) / 0.12 /

!  Hf
      DATA ABUNDM( 72 ) / 0.73 /

!  Ta
      DATA ABUNDM( 73 ) / 0.13 /

!  W
      DATA ABUNDM( 74 ) / 0.68 /

!  Re
      DATA ABUNDM( 75 ) / 0.27 /

!  Os
      DATA ABUNDM( 76 ) / 1.38 /

!  Ir
      DATA ABUNDM( 77 ) / 1.37 /

!  Pt
      DATA ABUNDM( 78 ) / 1.68 /

!  Au
      DATA ABUNDM( 79 ) / 0.83 /

!  Hg
      DATA ABUNDM( 80 ) / 1.09 /

!  Tl
      DATA ABUNDM( 81 ) / 0.82 /

!  Pb
      DATA ABUNDM( 82 ) / 2.05 /

!  Bi
      DATA ABUNDM( 83 ) / 0.71 /

!  Po
      DATA ABUNDM( 84 ) / TAP__RNULL /

!  At
      DATA ABUNDM( 85 ) / TAP__RNULL /

!  Rn
      DATA ABUNDM( 86 ) / TAP__RNULL /

!  Fr
      DATA ABUNDM( 87 ) / TAP__RNULL /

!  Ra
      DATA ABUNDM( 88 ) / TAP__RNULL /

!  Ac
      DATA ABUNDM( 89 ) / TAP__RNULL /

!  Th
      DATA ABUNDM( 90 ) / 0.08 /

!  Pa
      DATA ABUNDM( 91 ) / TAP__RNULL /

!  U
      DATA ABUNDM( 92 ) / -0.49 /

!.

!  Check for a bad atomic number and missing data.
      IF ( ( ZELEM .LE. 0 ) .OR. ( ZELEM .GT. TAP__NELEM ) ) THEN

!     Bad atomic number.
         TAP_ABMET = TAP__DNULL
      ELSE IF ( ( ABUNDM( ZELEM ) .LT. TAP__RNULL+ABS( SMALL ) ) &
                .AND. ( ABUNDM( ZELEM ) &
                        .GT. TAP__RNULL-ABS( SMALL ) ) ) THEN

!     Meteorite data are missing, return TAP__DNULL.
         TAP_ABMET = TAP__DNULL
      ELSE

!     Return the meteorite abundance.
         TAP_ABMET = DBLE( ABUNDM( ZELEM ) )
      END IF
      
    END FUNCTION TAP_ABMET
