DOUBLE PRECISION FUNCTION TAP_ABSOL ( ZELEM )

!+
!  Name:
!     TAP_ABSOL

!  Purpose:
!=    Return relative solar photospheric abundance for a given element.

!  Language:
!     Starlink Fortran 77

!  Invocation:
!     RESULT = TAP_ABSOL( ZELEM )

!  Description:
!     Given the atomic number of the element, the recommended value
!     of the solar photospheric element abundance relative to hydrogen
!     is returned. The abundances are based primarily on the
!     compilation of solar and meteorite data by Anders & Grevesse
!     (1989), supplemented in a few cases by more recent observations.
!
!     In general, the solar and meteorite data are in good agreement.
!     Some exceptions (e.g. Lithium) are obvious, whilst others (e.g.
!     Indium and Tungsten) are less so. The solar data will find
!     applications in stellar theory, whilst it is expected that the
!     meteorite data should be more useful for ISM studies.
!     
!     All solar and meteoritic abundances are given in logarithmic form
!     relative to the solar hydrogen abundance, which is defined to be
!     12.00.  Cases where no data are available for a given element
!     are indicated by a returned value of TAP__DNULL.

!  Arguments:
!     ZELEM = INTEGER (Given)
!        The atomic number (Z) for which the abundance is required.

!  Returned Value:
!     TAP_ABSOL = DOUBLE PRECISION
!        The solar photospheric abundance (logarithm with H = 12.00).

!  Accuracy:
!     See references for full details.
!
!     Solar  abundances: errors lie between +/- 0.03 and +/- 0.10 dex.
!            Exceptions:  F, Cl,  K, Ge, Rb, Rh, Ag, Cd, In, Sn, Sb, Ce,
!                        Tb, Dy, Ho, Tm, Yb, Lu,  W, Pt, Au, Tl.
!                       
!     Meteor abundances: errors lie between +/- 0.01 and +/- 0.05 dex.
!            Exceptions: F, Cl, Br, Kr, Sb, I, Xe, Au.

!  References:
!     Anders E. & Grevesse N. 1989. Geochim. Cosmochim. Acta 53, 197.
!     Holden N.E., Martin R.L., & Barnes I.L. 1984. Pure Appl. Chem. 56,
!        675.
!
!     Solar Fe:
!        Holweger H., Heise C. & Kock M. 1990. Astron.Astrophys. 232,
!           510.
!
!     Solar C:
!        Stuerenberg S. & Holweger H. 1990. Astron.Astrophys. 237, 125.

!  Authors:
!     CSJ: C.S.Jeffery (St Andrews)
!     PCTR: P.C.T. Rees (STARLINK)
!     {enter_new_authors_here}

!  History:
!     6-MAY-1992 (CSJ):
!        Original version.
!     8-JUL-1992 (PCTR):
!        Changed to a DOUBLE PRECISION function and removed the status 
!        argument.
!     {enter_further_changes_here}

!  Bugs:
!     {note_any_bugs_here}

!-

!  Global Constants:
      USE TAP_PAR            ! TAP public constants
      
!  Type Definitions:
      IMPLICIT NONE          ! No implicit typing

!  Arguments Given:
      INTEGER, INTENT(IN) :: ZELEM

!  Local Constants:
      ! A small REAL constant 
      REAL, PARAMETER :: SMALL = TAP__RNULL * TAP__RSMAL 
      
!  Local Variables:
      DOUBLE PRECISION :: ABUNDS( TAP__NELEM ) ! Solar element abundance ratios

!  Local Data:
!     Solar photospheric element abundances:

!  H
      DATA ABUNDS( 1 ) / 12.00 /

!  He
      DATA ABUNDS( 2 ) / 10.99D00 /

!  Li
      DATA ABUNDS( 3 ) / 1.16D00 /

!  Be
      DATA ABUNDS( 4 ) / 1.15D00 /

!  B
      DATA ABUNDS( 5 ) / 2.60D00 /

!  C
      DATA ABUNDS( 6 ) / 8.58D00 /

!  N
      DATA ABUNDS( 7 ) / 8.05D00 /

!  O
      DATA ABUNDS( 8 ) / 8.93D00 /

!  F
      DATA ABUNDS( 9 ) / 4.56D00 /

!  Ne
      DATA ABUNDS( 10 ) / 8.09D00 /

!  Na
      DATA ABUNDS( 11 ) / 6.33D00 /

!  Mg
      DATA ABUNDS( 12 ) / 7.58D00 /

!  Al
      DATA ABUNDS( 13 ) / 6.47D00 /

!  Si
      DATA ABUNDS( 14 ) / 7.55D00 /

!  P
      DATA ABUNDS( 15 ) / 5.45D00 /

!  S
      DATA ABUNDS( 16 ) / 7.21D00 /

!  Cl
      DATA ABUNDS( 17 ) / 5.50D00 /

!  Ar
      DATA ABUNDS( 18 ) / 6.56D00 /

!  K
      DATA ABUNDS( 19 ) / 5.12D00 /

!  Ca
      DATA ABUNDS( 20 ) / 6.36D00 /

!  Sc
      DATA ABUNDS( 21 ) / 3.10D00 /

!  Ti
      DATA ABUNDS( 22 ) / 4.99D00 /

!  V
      DATA ABUNDS( 23 ) / 4.00D00 /

!  Cr
      DATA ABUNDS( 24 ) / 5.67D00 /

!  Mn
      DATA ABUNDS( 25 ) / 5.39D00 /

!  Fe
      DATA ABUNDS( 26 ) / 7.48D00 /

!  Co
      DATA ABUNDS( 27 ) / 4.92D00 /

!  Ni
      DATA ABUNDS( 28 ) / 6.25D00 /

!  Cu
      DATA ABUNDS( 29 ) / 4.21D00 /

!  Zn
      DATA ABUNDS( 30 ) / 4.60D00 /

!  Ga
      DATA ABUNDS( 31 ) / 2.88D00 /

!  Ge
      DATA ABUNDS( 32 ) / 3.41D00 /

!  As
      DATA ABUNDS( 33 ) / TAP__RNULL /

!  Se
      DATA ABUNDS( 34 ) / TAP__RNULL /

!  Br
      DATA ABUNDS( 35 ) / TAP__RNULL /

!  Kr
      DATA ABUNDS( 36 ) / TAP__RNULL /

!  Rb
      DATA ABUNDS( 37 ) / 2.60D00 /

!  Sr
      DATA ABUNDS( 38 ) / 2.90D00 /

!  Y
      DATA ABUNDS( 39 ) / 2.24D00 /

!  Zr
      DATA ABUNDS( 40 ) / 2.60D00 /

!  Nb
      DATA ABUNDS( 41 ) / 1.42D00 /

!  Mo
      DATA ABUNDS( 42 ) / 1.92D00 /

!  Tc
      DATA ABUNDS( 43 ) / TAP__RNULL /

!  Ru
      DATA ABUNDS( 44 ) / 1.84D00 /

!  Rh
      DATA ABUNDS( 45 ) / 1.12D00 /

!  Pd
      DATA ABUNDS( 46 ) / 1.69D00 /

!  Ag
      DATA ABUNDS( 47 ) / 0.94D00 /

!  Cd
      DATA ABUNDS( 48 ) / 1.86D00 /

!  In
      DATA ABUNDS( 49 ) / 1.66D00 /

!  Sn
      DATA ABUNDS( 50 ) / 2.00D00 /

!  Sb
      DATA ABUNDS( 51 ) / 1.00D00 /

!  Te
      DATA ABUNDS( 52 ) / TAP__RNULL /

!  I
      DATA ABUNDS( 53 ) / TAP__RNULL /

!  Xe
      DATA ABUNDS( 54 ) / TAP__RNULL /

!  Cs
      DATA ABUNDS( 55 ) / TAP__RNULL /

!  Ba
      DATA ABUNDS( 56 ) / 2.13D00 /

!  La
      DATA ABUNDS( 57 ) / 1.22D00 /

!  Ce
      DATA ABUNDS( 58 ) / 1.55D00 /

!  Pr
      DATA ABUNDS( 59 ) / 0.71D00 /

!  Nd
      DATA ABUNDS( 60 ) / 1.50D00 /

!  Pm
      DATA ABUNDS( 61 ) / TAP__RNULL /

!  Sm
      DATA ABUNDS( 62 ) / 1.00D00 /

!  Eu
      DATA ABUNDS( 63 ) / 0.51D00 /

!  Gd
      DATA ABUNDS( 64 ) / 1.12D00 /

!  Tb
      DATA ABUNDS( 65 ) / -0.10D00 /

!  Dy
      DATA ABUNDS( 66 ) / 1.10D00 /

!  Ho
      DATA ABUNDS( 67 ) / 0.26D00 /

!  Er
      DATA ABUNDS( 68 ) / 0.93D00 /

!  Tm
      DATA ABUNDS( 69 ) / 0.00D00 /

!  Yb
      DATA ABUNDS( 70 ) / 1.08D00 /

!  Lu
      DATA ABUNDS( 71 ) / 0.76D00 /

!  Hf
      DATA ABUNDS( 72 ) / 0.88D00 /

!  Ta
      DATA ABUNDS( 73 ) / TAP__RNULL /

!  W
      DATA ABUNDS( 74 ) / 1.03D00 /

!  Re
      DATA ABUNDS( 75 ) / TAP__RNULL /

!  Os
      DATA ABUNDS( 76 ) / 1.45D00 /

!  Ir
      DATA ABUNDS( 77 ) / 1.35D00 /

!  Pt
      DATA ABUNDS( 78 ) / 1.80D00 /

!  Au
      DATA ABUNDS( 79 ) / 1.01D00 /

!  Hg
      DATA ABUNDS( 80 ) / TAP__RNULL /

!  Tl
      DATA ABUNDS( 81 ) / 0.90D00 /

!  Pb
      DATA ABUNDS( 82 ) / 1.85D00 /

!  Bi
      DATA ABUNDS( 83 ) / TAP__RNULL /

!  Po
      DATA ABUNDS( 84 ) / TAP__RNULL /

!  At
      DATA ABUNDS( 85 ) / TAP__RNULL /

!  Rn
      DATA ABUNDS( 86 ) / TAP__RNULL /

!  Fr
      DATA ABUNDS( 87 ) / TAP__RNULL /

!  Ra
      DATA ABUNDS( 88 ) / TAP__RNULL /

!  Ac
      DATA ABUNDS( 89 ) / TAP__RNULL /

!  Th
      DATA ABUNDS( 90 ) / 0.12D00 /

!  Pa
      DATA ABUNDS( 91 ) / TAP__RNULL /

!  U
      DATA ABUNDS( 92 ) / -0.47D00 /

!.

!  Check for a bad atomic number and missing data.

!     Bad atomic number.
      IF ( ( ZELEM .LE. 0 ) .OR. ( ZELEM .GT. TAP__NELEM ) ) THEN
         TAP_ABSOL = TAP__DNULL

!     Missing solar abundance data.
      ELSE IF( ( ABUNDS( ZELEM ) .LT. TAP__RNULL+ABS( SMALL ) ) &
               .AND. ( ABUNDS( ZELEM )                          &
                       .GT. TAP__RNULL-ABS( SMALL ) ) ) THEN
         TAP_ABSOL = TAP__DNULL

!     Return the solar photospheric abundance.
      ELSE
         TAP_ABSOL = ABUNDS( ZELEM )

      END IF

      
    END FUNCTION TAP_ABSOL
