DOUBLE PRECISION FUNCTION TAP_GAMMA ( X, STATUS )

!+
!  Name:
!     TAP_GAMMA

!  Purpose:
!=    Evaluate the gamma function Gamma (x) for a real argument.

!  Language:
!     Fortran 90

!  Invocation:
!     DOUBLE PRECISION TAP_GAMMA
!     RESULT = TAP_GAMMA( X, STATUS )

!  Package:
!     TAP: Theoretical Astrophysics Package

!  Module type:
!     double precision function

!  Description:
!     The gamma function (Gamma (x)) is evaluated for a single real
!     argument.
!
!     The gamma function for positive x using the TAP function
!     TAP_GAMMLN, which uses a power series expansion. If x < 0
!     the reflection formula
!
!         Gamma(x) * Gamma(1-x) = pi.cosec(pi.x)
!
!     is applied. If x = -n, n = 0,1,2,..., Gamma(x) has a pole.
!     An error status is returned, and TAP_GAMMA (x) is not defined.

!  Arguments:
!     X = DOUBLE PRECISION (Given)
!        The real number X for which GAMMA(X) is required.
!     STATUS = INTEGER (Given and Returned)
!        The global status.

!  Returned Value:
!     TAP_GAMMA = DOUBLE PRECISION
!        GAMMA(X) as a real number

!  Subprograms used:
!     DOUBLE PRECISION FUNCTION TAP_GAMMLN ( Z, STATUS )
!     DOUBLE PRECISION FUNCTION TAP1_POWERG ( Z, STATUS )

!  Authors:
!     CSJ: C.S.Jeffery (CCP7)
!     {enter_new_authors_here}

!  History:
!     10-AUG-1993 (CSJ):
!        Original version.
!     {enter_changes_here}

!  Bugs:
!     Will overflow close to poles of Gamma function.
!     {note_any_bugs_here}

!-

!  Global constants:
      USE SAE_PAR
      USE MCH_PAR
      USE TAP_ERR
      USE TAP_PAR
      USE TAPM_PAR 

!  Type Definitions
      IMPLICIT NONE

!  Arguments given:
      DOUBLE PRECISION :: X    ! the function argument x

!  Status:
      INTEGER :: STATUS

!  Local constants:
   ! Maximum value allowed for X determined from machine model on first call
      DOUBLE PRECISION :: XMAX = -1.0D+00 
      SAVE :: XMAX

!  Local variables:
      DOUBLE PRECISION :: Y    ! local variable holders
      DOUBLE PRECISION :: Z    !  
      DOUBLE PRECISION :: G    ! 
      INTEGER :: N             ! Index in recursion formula
      INTEGER :: ND            ! number of terms in recurrence formula

!  External functions:
      DOUBLE PRECISION :: TAP_GAMMLN    ! ln(Gamma)
      EXTERNAL TAP_GAMMLN
      DOUBLE PRECISION :: TAP1_POWERG   ! power series expansion for ln(Gamma)
      EXTERNAL TAP1_POWERG


!.

!  Check inherited global status.
      IF ( STATUS .NE. SAI__OK ) RETURN 

!  Calculate x_max on first call, using Newton's method.
      IF ( XMAX.LE.0.0D+00 ) THEN
         G = LOG (MCH__LARGE)
         Z = G
10       Y = Z * LOG(Z) - Z / 2.0D+00 - Z + 0.5D+00 * (TAPM__LNPI+TAPM__LN2)
         Z = Z + (G-Y) / (LOG(Z) - 0.5D+00/Z)
         IF ( ABS(G-Y).GT.TAP__DSMAL ) GOTO 10
         XMAX = Z
      ENDIF

!  Gamma(x) for 0 < x < x_max, where Gamma(x_max) <= MCH__LARGE.
      IF ( X.GT.0.0D+00 .AND. X.LT.XMAX ) THEN
         TAP_GAMMA = EXP ( TAP_GAMMLN (X,STATUS) )

!  Poles of Gamma function, x = -n :  n = 0,1,2,.. 
      ELSEIF ( ABS(MOD(X,1.0D+00 )).LE.MCH__EPS ) THEN
         STATUS = TAP__SINGULAR

!  Use reflection formula, -1 < x < 0.
      ELSEIF ( X.LT.0.0D+00 .AND. X.GT.-1.0D+00 ) THEN
         TAP_GAMMA = TAPM__PI / SIN (TAPM__PI*X) & 
                              / EXP (TAP_GAMMLN (1.0D+00-X,STATUS) )

!  Use reflection and recurrence formulae, 1-xmax < x < -1.
      ELSEIF ( X.LT.0.0D+00 .AND. X.GT.(1.0D+00-XMAX) ) THEN
         Z = MOD (X,1.0D+00) + 1.0D+00
         IF ( Z.LT.0.5D+00 ) Z = Z + 1.0D+00
         ND = NINT (X-Z)
         G = TAPM__PI / SIN (TAPM__PI*Z) &
                      / EXP (TAP1_POWERG (1.0D+00-Z, STATUS))
         DO N=ND,-1
            Z = Z - 1.0D+00
            G = G / Z
         ENDDO
         TAP_GAMMA = G

!  Otherwise x is too large for Gamma(x).
      ELSE
         STATUS = TAP__BADX
 

      ENDIF

      END


