DOUBLE PRECISION FUNCTION TAP_GAMMLN ( X, STATUS )

!+
!  Name:
!     TAP_GAMMLN

!  Purpose:
!=    Evaluate the gamma function ln Gamma (x) for a positive real argument.

!  Language:
!     Fortran 90

!  Invocation:
!     DOUBLE PRECISION TAP_GAMMLN
!     RESULT = TAP_GAMMLN( X, STATUS )

!  Package:
!     TAP: Theoretical Astrophysics Package

!  Module type:
!     double precision function

!  Description:
!     The gamma function (Gamma (x)) is evaluated for a single real
!     positive argument, the logarithm (ln (Gamma(x))) is returned.
!
!     The gamma function is evaluated using a power series expansion
!     valid over the interval 0 < x < 2. However, the expnasion is only
!     used in the interval 1/2 < x < 3/2, where it is most rapidly
!     convergent. For other values of the argument x, a combination of 
!     the reflection formula
!
!         Gamma(x) * Gamma(1-x) = pi.cosec(pi.x)
!
!     and the recurrence relation
!
!         Gamma(x+1) = x.Gamma(x)
!
!     are applied. The algorithm is not as fast as obtained with
!     a series approximation, (eg Stirling's formula or the Lanczos
!     approximation - see Press et al.), but is adopted here
!     in order to achieve maximum arithmetic precision.

!  References.
!     Abramowitz M., &  Stegun I.A., 1974, Handbook of Mathematical Functions,
!        Dover.
!     Press et al. 1989. Numerical Recipes  

!  Arguments:
!     X = DOUBLE PRECISION (Given)
!        The real number X for which GAMMA(X) is required.
!     STATUS = INTEGER (Given and Returned)
!        The global status.

!  Returned Value:
!     TAP_GAMMLN = DOUBLE PRECISION
!        LOG (GAMMA(X)) as a real number

!  Subprograms used:
!     DOUBLE PRECISION FUNCTION TAP1_POWERG ( Z, STATUS )

!  Authors:
!     CSJ: C.S.Jeffery (CCP7)
!     {enter_new_authors_here}

!  History:
!     5-AUG-1993 (CSJ):
!        Original version.
!     8-JAN-2002 (CSJ):
!        Converted to F90.
!     {enter_changes_here}

!  Bugs:
!     {note_any_bugs_here}

!-

!  Global constants:
      USE SAE_PAR
      USE MCH_PAR
      USE TAP_ERR
      USE TAPM_PAR

!  Type Definitions
      IMPLICIT NONE

!  Arguments given:
      DOUBLE PRECISION :: X    ! the function argument x

!  Status:
      INTEGER :: STATUS

!  Local variables:
      DOUBLE PRECISION :: Z    !  
      DOUBLE PRECISION :: G    ! local holder for ln(Gamma(x))
      INTEGER :: N             ! Index in recursion formula
      INTEGER :: ND            ! number of terms in recurrence formula

!  External functions:
      DOUBLE PRECISION :: TAP1_POWERG   ! power series expansion for ln(Gamma)
      EXTERNAL TAP1_POWERG


!.

!  Check inherited global status.
      IF ( STATUS .NE. SAI__OK ) RETURN 

!  Ensure x is positive for ln(Gamma). 
      IF ( X.LT.MCH__EPS ) THEN
         STATUS = TAP__BADX

!  simple power series expansion, 0.5 <= x <= 1.5.
      ELSEIF ( X.GE.0.5D+00 .AND. X.LE.1.5D+00 ) THEN
         TAP_GAMMLN = TAP1_POWERG (X, STATUS)

!  Recursion formula, x > 1.5.
      ELSEIF ( X.GT.1.5D+00 ) THEN
         Z = MOD (X,1.0D+00)
         IF ( Z.LT.0.5D+00 ) Z = Z + 1.0D+00
         ND = NINT (X-Z)
         G = TAP1_POWERG (Z, STATUS)
         DO N=1,ND
            G = G + LOG (Z)
            Z = Z + 1.0D+00
         ENDDO
         TAP_GAMMLN = G

!  Reflection formula, 0 < x < 0.5.
      ELSEIF ( X.LT.0.5D+00 ) THEN
         TAP_GAMMLN = TAPM__LNPI - LOG ( SIN (TAPM__PI*X)) &
                                 - TAP1_POWERG (1.0D+00-X, STATUS)

!  broken logic.
      ELSE
         STATUS = TAP__LOGIC
 

      ENDIF

      END


