DOUBLE PRECISION FUNCTION TAP_IONPT ( ZELEM, ZION )

!+
!  Name:
!     TAP_IONPT

!  Purpose:
!=    Provides ionisation potential for a given ion

!  Language:
!     Fortran 90

!  Invocation:
!     RESULT = TAP_IONPT( atom_z, ion_z, status )

!  Description:
!     This function provides ionisation potentials expressed in electron
!     volts. Recommended values are derived from analyses of optical 
!     spectra. The following sources have been used, in descending priority
!     1: Moore 1970, table 1
!     2: Allen 1973, table 17 (electron affinities)
!     3. Allen 1973, table 16
!
!     The ionisation potentials given by Moore were derived by multiplying 
!     the observed series limits by 0.000123981, i.e., by using the
!     conversion factor
!
!        1eV = 806573.    m**(-1)
!
!     The 1986 CODATA (Cohen & Taylor 1987) recommended value for this
!     quantity is
!
!        1eV = 806544.10  m**(-1)
!          +/-       .24
!
!     In order to preserve the form of the data tabulated by Moore,
!     and thus to to allow easy verification, but also to be achieve the
!     highest possible standard of accuracy, this version applies
!     the correction at execution time. With 1986 CODATA values, the
!     difference amounts to about 36 parts per million, as anticipated
!     by Moore. 
     
!  Arguments:
!     ZELEM = INTEGER (Given)
!        The atomic number of the ion  ( eg C: = 6)
!     ZION = INTEGER (Given)
!        The atomic charge on the ion  ( eg CI: = 0, OVI: = 5 )

!  Returned Value:
!     TAP_IONPT = DOUBLE PRECISION
!        Ionisation potential in electron volts (eV)

!  References:
!     -  Moore C.E., 1970. NSRDS - NBS 34 "Ionisation Potentials and
!     Ionization Limits Derived from the Analyses of Optical Spectra"
!     -  Cohen E.R. & Taylor B.N., 1987. Journal of Research of the
!     National Bureau of Standards, 92, 85.
!     -  Allen C.W.. 1973. Astrophysical Quantities, 3rd Edition
!     atlas9
!     -  Lester version of Kurucz code in CCP7 library

!  Accuracy:
!     The accuracy of the data stored reflects the number of
!     significant figures given by the sources, such that the final
!     significant figure is believed to be meaningful.

!  Implementation Deficiencies:
!     The uncertainty of 30 ppm in the conversion factor used by Moore
!     has been eliminated by applying a correction factor. This process
!     arbitrarily increases the number of "significant figures" in the
!     returned value cf. the stored value, and has the effect of
!     slightly increasing the execution time. These effects should be
!     removed in a future version.
!
!     In order to allow ionizations to be calculated for rare-earths,
!     a third and sometimes even a second ionization potential has been 
!     adopted as in atlas9. Cases are marked -  #1  -

!  Pitfalls:
!     If unusual results are obtained, check that the ion identifier has
!     been entered correctly: i.e., 0 for neutrals, 1 for singly
!     ionized, etc.. Do not use the frequently used "spectrum" identifier,
!     i.e. I for neutrals, II for singly ionized, etc.

!  Error conditions:
!     The status flag will be set if any of the following conditions are
!     encountered:
!        an illegal value for the atomic number ( zelem )
!        an illegal value for the ion charge    ( zion  )
!        a datum is unavailable
!
!     In all cases, the result of the function call will be 0 eV.

!  [optional_function_items]...

!  Authors:
!     CSJ: C.S.Jeffery (St. Andrews)
!     {enter_new_authors_here}

!  History:
!     8-MAY-1992 (CSJ):
!        Original version.
!     6-MAY-1994 (CSJ):
!        Added electron affinities
!     8-JAN-2002 (CSJ):
!        Converted to F90.
!     {enter_changes_here}

!  Bugs:
!     Data from Allen for higher ionization levels than given by Moore
!     has not yet been entered.
!
!     {note_any_bugs_here}

!-
      
!  Global Constants:
      USE SAE_PAR         ! Standard SAE constants
      USE TAP_PAR         ! TAP public constants
      USE TAPP_PAR        ! TAP physical constants

!  Type Definitions:
      IMPLICIT NONE            ! No omplicit typing

!  Arguments Given:
      INTEGER :: ZELEM
      INTEGER :: ZION


!  Local Constants:
!    Correction factor for revised value of electron volt
!    obtained from TAP physical constants.
      DOUBLE PRECISION, PARAMETER :: EV_COR = TAPP__E / TAPP__H / TAPP__C / 806573.D0

!  Local Variables:
!      INTEGER :: I                  ! Loop index
!      INTEGER :: ION_POINT( TAP__NELEM+1 )  ! Nuclide table pointer
!      INTEGER :: ION_OFFSET( TAP__NELEM+1 ) ! Nuclide mass offset
!      INTEGER :: ION                ! Array pointer

!    Number of ions for each element
      INTEGER :: ION_CNT( TAP__NELEM, 2 ) 
!    Ionisation potentials in electron volts
      REAL :: ION_POT( TAP__NELEM, -1:TAP__NELEM-1 ) 

!    Dummy array index 
      INTEGER I

!  Local Data:
!    Ionisation potentials in electron volts

!  H
      DATA ION_CNT ( 1,1:2 ) / -1, 0 /
      DATA (ION_POT ( 1,I), I=-1, 0) / 0.754, 13.598 /
!  He
      DATA ION_CNT ( 2,1:2 ) / -1, 1 /
      DATA (ION_POT ( 2,I), I=-1, 1) / -0.3, 24.587, 54.416 /
!  Li
      DATA ION_CNT ( 3,1:2 ) / -1, 2 /
      DATA (ION_POT ( 3,I), I=-1, 2) / 0.65, 5.392, 75.638, 122.451/
!  Be
      DATA ION_CNT ( 4,1:2 ) / -1, 3 /
      DATA (ION_POT ( 4,I), I=-1, 3) / -0.4, 9.322, 18.211, 158.893, &
                                      217.713/
!  B
      DATA ION_CNT ( 5,1:2 ) / -1, 4 /
      DATA (ION_POT ( 5,I), I=-1, 4) / 0.38, 8.298, 25.154, 37.930, &
                                      259.368, 340.217/
!  C
      DATA ION_CNT ( 6,1:2 ) / -1, 5 /
      DATA (ION_POT ( 6,I), I=-1, 5) / 1.24, 11.260, 24.383, 47.887, &
                                       64.492, 392.077, 489.981/
!  N
      DATA ION_CNT ( 7,1:2 ) / -1, 6 /
      DATA (ION_POT ( 7,I), I=-1, 6) / -0.2, 14.534, 29.601, 47.448, &
                                       77.472, 97.888, 552.057, 667.029/
!  O
      DATA ION_CNT ( 8,1:2 ) / -1, 7 /
      DATA (ION_POT ( 8,I), I=-1, 7) / 1.46, 13.618, 35.116, 54.934, &
                                       77.412,113.896, 138.116, 739.315, &
                                      871.387/
!  F
      DATA ION_CNT ( 9,1:2 ) / -1, 8 /
      DATA (ION_POT ( 9,I), I=-1, 8) / 3.47, 17.422, 34.970, 62.707, &
                                       87.138,114.240, 157.161, 185.182, &
                                      953.886,1103.089/
!  Ne
      DATA ION_CNT ( 10,1:2 ) / -1, 9 /
      DATA (ION_POT (10,I), I=-1, 9) / -0.7, 21.564, 40.962, 63.45, &
                                       97.11,126.21, 157.93, 207.27, &
                                      239.09,1195.797,1362.164/
!  Na
      DATA ION_CNT ( 11,1:2 ) / -1, 10 /
      DATA (ION_POT (11,I), I=-1,10) / 0.5, &
                                       5.139, 47.286, 71.64, 98.91, &
                                     138.39, 172.15, 208.47, 264.18, &
                                     299.87,1465.091,1648.659/
!  Mg
      DATA ION_CNT ( 12,1:2 ) / -1, 11 /
      DATA (ION_POT (12,I), I=-1,11) / -0.4, &
                                       7.646, 15.035, 80.143, 109.24, &
                                     141.26, 186.50, 224.94, 265.90, &
                                     327.95, 367.53,1761.802,1962.613/
!  Al
      DATA ION_CNT ( 13,1:2 ) / -1, 12 /
      DATA (ION_POT (13,I), I=-1,12) /-0.7, &
                                       5.986, 18.828, 28.447, 119.99, &
                                     153.71, 190.47, 241.43, 284.59, &
                                     330.21, 398.57, 442.07,2085.983, &
                                    2304.080/
!  Si
      DATA ION_CNT ( 14,1:2 ) / -1, 13 /
      DATA (ION_POT (14,I), I=-1,13) / 1.42, &
                                       8.151, 16.345, 33.492, 45.141, &
                                     166.77, 205.05, 246.52, 303.17, &
                                     351.10, 401.43, 476.06, 523.50, &
                                    2437.676,2673.108/
!  P
      DATA ION_CNT ( 15,1:2 ) / -1, 14 /
      DATA (ION_POT (15,I), I=-1,14) / 1.0, &
                                      10.486, 19.725, 30.18, 51.37, &
                                      65.023, 220.43, 263.22, 309.41, &
                                     371.73, 424.50, 479.57, 560.41, &
                                     611.85,2816.943,3069.762/
!  S
      DATA ION_CNT ( 16,1:2 ) / -1, 15 /
      DATA (ION_POT (16,I), I=-1,15) / 2.3, &
                                      10.360, 23.33, 34.83, 47.30, &
                                      72.68, 88.049, 280.93, 328.23, &
                                     379.10, 447.09, 504.78, 564.65, &
                                     651.63, 704.14,3223.836,3494.099/
!  Cl
      DATA ION_CNT ( 17,1:2 ) / -1, 16 /
      DATA (ION_POT (17,I), I=-1,16) / 3.62, &
                                      12.967, 23.81, 39.61, 53.46, &
                                      67.8, 97.03, 114.193, 348.28, &
                                     400.05, 455.62, 529.26, 591.97, &
                                     656.69, 749.74, 809.39,3658.425, &
                                    3946.193/
!  Ar
      DATA ION_CNT ( 18,1:2 ) / 0, 17 /
      DATA (ION_POT (18,I), I= 0,17) / 15.579, 27.629, 40.74, 59.81, &
                                      75.02, 91.007, 124.319, 143.456, &
                                     422.44, 478.68, 538.95, 618.24, &
                                     686.09, 755.73, 854.75, 918., &
                                    4120.778,4426.114/
!  K
      DATA ION_CNT ( 19,1:2 ) / 0, 18 /
      DATA (ION_POT (19,I), I= 0,18) / 4.341, 31.625, 45.72, 60.91, &
                                      82.66, 100.0, 117.56, 154.86, &
                                     175.814, 503.44, 564.13, 629.09, &
                                     714.02, 787.13, 861.77, 968., &
                                    1034.,4610.955,4933.931/
!  Ca
      DATA ION_CNT ( 20,1:2 ) / 0, 19 /
      DATA (ION_POT (20,I), I= 0,19) / 6.113, 11.871, 50.908, 67.10, &
                                      84.41, 108.78, 127.7, 147.24, &
                                     188.54, 211.270, 591.25, 656.39, &
                                     726.03, 816.61, 895.12, 974., &
                                    1087.,1157.,5129.045,5469.738/
!  Sc
      DATA ION_CNT ( 21,1:2 ) / 0, 14 /
      DATA (ION_POT (21,I), I= 0,14) / 6.54, 12.80, 24.76, 73.47, &
                                      91.66, 111.1, 138.0, 158.7, &
                                     180.02, 225.32, 249.832, 685.89, &
                                     755.47, 829.79, 926.00 /
!  Ti
      DATA ION_CNT ( 22,1:2 ) / 0, 14 /
      DATA (ION_POT (22,I), I= 0,14) / 6.82, 13.58, 27.491, 43.266, &
                                      99.22, 119.36, 140.8, 168.5, &
                                     193.2, 215.91, 265.23, 291.497, &
                                     787.33, 861.33, 940.36 /
!  V
      DATA ION_CNT ( 23,1:2 ) / 0, 14 /
      DATA (ION_POT (23,I), I= 0,14) / 6.74, 14.65, 29.310, 46.707, &
                                      65.23, 128.12, 150.17, 173.7, &
                                     205.8, 230.5, 255.04, 308.25, &
                                     336.267, 895.58, 974.02 /
!  Cr
      DATA ION_CNT ( 24,1:2 ) / 0, 14 /
      DATA (ION_POT (24,I), I= 0,14) / 6.766, 16.50, 30.96, 49.1, &
                                      69.3, 90.56, 161.1, 184.7, &
                                     209.3, 244.4, 270.8, 298.0, &
                                     355., 384.30,1010.64 /
!  Mn
      DATA ION_CNT ( 25,1:2 ) / 0, 15 /
      DATA (ION_POT (25,I), I= 0,15) / 7.435, 15.640, 33.667, 51.2, &
                                      72.4, 95., 119.27, 196.46, &
                                     221.8, 243.3, 286.0, 314.4, &
                                     343.6, 404., 435.3,1136.2 /
!  Fe
      DATA ION_CNT ( 26,1:2 ) / 0, 16 /
      DATA (ION_POT (26,I), I= 0,16) / 7.870, 16.18, 30.651, 54.8, &
                                      75.0, 99., 125., 151.06, &
                                     235.04, 262.1, 290.4, 330.8, &
                                     361.0, 392.2, 457., 489.5, &
                                    1266.1 /
!  Co
      DATA ION_CNT ( 27,1:2 ) / 0, 17 /
      DATA (ION_POT (27,I), I= 0,17) / 7.86, 17.06, 33.50, 51.3, &
                                      79.5, 102., 129., 157., &
                                     186.13, 276., 305., 336., &
                                     379., 411., 444., 512., &
                                     546.8,1403.0 /
!  Ni
      DATA ION_CNT ( 28,1:2 ) / 0, 18 /
      DATA (ION_POT (28,I), I= 0,18) / 7.635, 18.168, 35.17, 54.9, &
                                      75.5, 108., 133., 162., &
                                     193., 224.5, 321.2, 352., &
                                     384., 430., 464., 499., &
                                     571., 607.2,1547.  /
!  Cu
      DATA ION_CNT ( 29,1:2 ) / 0, 19 /
      DATA (ION_POT (29,I), I= 0,19) / 7.726, 20.292, 36.83, 55.2, &
                                      79.9, 103., 139., 166., &
                                     199., 232., 266., 368.8, &
                                     401., 435., 484., 520., &
                                     557., 633., 671.,1698.  /
!  Zn
      DATA ION_CNT ( 30,1:2 ) / 0, 20 /
      DATA (ION_POT (30,I), I= 0,20) / 9.394, 17.964, 39.722, 59.4, &
                                      82.6, 108., 134., 174., &
                                     203., 238., 274., 310.8, &
                                     419.7, 454., 490., 542., &
                                     579., 619., 698., 738., &
                                    1856.  /
!  Ga
      DATA ION_CNT ( 31,1:2 ) / 0, 3 /
      DATA (ION_POT (31,I), I= 0, 3) / 5.999, 20.51, 30.71, 64.  /

!  Ge
      DATA ION_CNT ( 32,1:2 ) / 0, 4 /
      DATA (ION_POT (32,I), I= 0, 4) / 7.899, 15.934, 34.22, 45.71, &
                                      93.5 /
!  As
      DATA ION_CNT ( 33,1:2 ) / 0, 5 /
      DATA (ION_POT (33,I), I= 0, 5) / 9.81, 18.633, 28.351, 50.13, &
                                      62.63, 127.6 /
!  Se
      DATA ION_CNT ( 34,1:2 ) / 0, 6 /
      DATA (ION_POT (34,I), I= 0, 6) / 9.752, 21.19, 30.820, 42.944, &
                                      68.3, 81.70, 155.4 /
!  Br
      DATA ION_CNT ( 35,1:2 ) / -1, 7 /
      DATA (ION_POT (35,I), I=-1, 7) / 3.48, &
                                      11.814, 21.8, 36., 47.3, &
                                      59.7, 88.6, 103.0, 192.8 /
!  Kr
      DATA ION_CNT ( 36,1:2 ) / 0, 8 /
      DATA (ION_POT (36,I), I= 0, 8) / 13.999, 24.359, 36.95, 52.5, &
                                      64.7, 78.5, 111.0, 126., &
                                     230.9 /
!  Rb
      DATA ION_CNT ( 37,1:2 ) / 0, 9 /
      DATA (ION_POT (37,I), I= 0, 9) / 4.177, 27.28, 40., 52.6, &
                                      71.0, 84.4, 99.2, 136., &
                                     150., 277.1 /
!  Sr
      DATA ION_CNT ( 38,1:2 ) / 0, 10 /
      DATA (ION_POT (38,I), I= 0,10) / 5.695, 11.030, 43.6, 57., &
                                      71.6, 90.8, 106., 122.3, &
                                     162., 177., 324.1 /
!  Y
      DATA ION_CNT ( 39,1:2 ) / 0, 11 /
      DATA (ION_POT (39,I), I= 0,11) / 6.38, 12.24, 20.52, 61.8, &
                                      77.0, 93.0, 116., 129., &
                                     146.2, 191., 206., 374.0 /
!  Zr
      DATA ION_CNT ( 40,1:2 ) / 0, 4 /
      DATA (ION_POT (40,I), I= 0, 4) / 6.84, 13.13, 22.99, 34.34, &
                                      81.5 /
!  Nb
      DATA ION_CNT ( 41,1:2 ) / 0, 6 /
      DATA (ION_POT (41,I), I= 0, 6) / 6.88, 14.32, 25.04, 38.3, &
                                      50.55, 102.6, 125.  /
!  Mo
      DATA ION_CNT ( 42,1:2 ) / 0, 7 /
      DATA (ION_POT (42,I), I= 0, 7) / 7.099, 16.15, 27.16, 46.4, &
                                      61.2, 68., 126.8, 153.  /
!  Tc
      DATA ION_CNT ( 43,1:2 ) / 0, 2 /
      DATA (ION_POT (43,I), I= 0, 2) / 7.28, 15.26, 29.54 /
!  Ru
      DATA ION_CNT ( 44,1:2 ) / 0, 2 /
      DATA (ION_POT (44,I), I= 0, 2) / 7.37, 16.76, 28.47 /
!  Rh
      DATA ION_CNT ( 45,1:2 ) / 0, 2 /
      DATA (ION_POT (45,I), I= 0, 2) / 7.46, 18.08, 31.06 /
!  Pd
      DATA ION_CNT ( 46,1:2 ) / 0, 2 /
      DATA (ION_POT (46,I), I= 0, 2) / 8.34, 19.43, 32.93 /
!  Ag
      DATA ION_CNT ( 47,1:2 ) / 0, 2 /
      DATA (ION_POT (47,I), I= 0, 2) / 7.576, 21.49, 34.83 /
!  Cd
      DATA ION_CNT ( 48,1:2 ) / 0, 2 /
      DATA (ION_POT (48,I), I= 0, 2) / 8.993, 16.908, 37.48 /
!  In
      DATA ION_CNT ( 49,1:2 ) / 0, 3 /
      DATA (ION_POT (49,I), I= 0, 3) / 5.786, 18.869, 28.03, 54.  /
!  Sn
      DATA ION_CNT ( 50,1:2 ) / 0, 4 /
      DATA (ION_POT (50,I), I= 0, 4) / 7.344, 14.632, 30.502, 40.734, &
                                      72.28 /
!  Sb
      DATA ION_CNT ( 51,1:2 ) / 0, 5 /
      DATA (ION_POT (51,I), I= 0, 5) / 8.641, 16.53, 25.3, 44.2, &
                                      56., 108.  /
!  Te
      DATA ION_CNT ( 52,1:2 ) / 0, 6 /
      DATA (ION_POT (52,I), I= 0, 6) / 9.009, 18.6, 27.96, 37.41, &
                                      58.75, 70.7, 137.  /
!  I
      DATA ION_CNT ( 53,1:2 ) / 0, 2 /
      DATA (ION_POT (53,I), I=-1, 2) / 3.17, &
                                       10.451, 19.131, 33.  /
!  Xe
      DATA ION_CNT ( 54,1:2 ) / 0, 2 /
      DATA (ION_POT (54,I), I= 0, 2) / 12.130, 21.21, 32.1 /
!  Cs
      DATA ION_CNT ( 55,1:2 ) / 0, 2 /
      DATA (ION_POT (55,I), I= 0, 2) / 3.894, 25.1, 35. /  ! #1
!  Ba
      DATA ION_CNT ( 56,1:2 ) / 0, 2 /
      DATA (ION_POT (56,I), I= 0, 2) / 5.212, 10.004, 37. /  ! #1
!  La
      DATA ION_CNT ( 57,1:2 ) / 0, 2 /
      DATA (ION_POT (57,I), I= 0, 2) / 5.577, 11.06, 19.175/
!  Ce
      DATA ION_CNT ( 58,1:2 ) / 0, 3 /
      DATA (ION_POT (58,I), I= 0, 3) / 5.47, 10.85, 20.20, 36.72 /
!  Pr
      DATA ION_CNT ( 59,1:2 ) / 0, 4 /
      DATA (ION_POT (59,I), I= 0, 4) / 5.42, 10.55, 21.62, 38.95, &
                                      57.45 /
!  Nd
      DATA ION_CNT ( 60,1:2 ) / 0, 2 /
      DATA (ION_POT (60,I), I= 0, 2) / 5.49, 10.72, 20. /  ! #1
!  Pm
      DATA ION_CNT ( 61,1:2 ) / 0, 2 /
      DATA (ION_POT (61,I), I= 0, 2) / 5.55, 10.90, 20. /  ! #1
!  Sm
      DATA ION_CNT ( 62,1:2 ) / 0, 2 /
      DATA (ION_POT (62,I), I= 0, 2) / 5.63, 11.07, 20. /  ! #1
!  Eu
      DATA ION_CNT ( 63,1:2 ) / 0, 2 /
      DATA (ION_POT (63,I), I= 0, 2) / 5.67, 11.25, 20. /  ! #1
!  Gd
      DATA ION_CNT ( 64,1:2 ) / 0, 2 /
      DATA (ION_POT (64,I), I= 0, 2) / 6.14, 12.1, 20. /  ! #1
!  Tb
      DATA ION_CNT ( 65,1:2 ) / 0, 2 /
      DATA (ION_POT (65,I), I= 0, 2) / 5.85, 11.52, 20. /  ! #1
!  Dy
      DATA ION_CNT ( 66,1:2 ) / 0, 2 /
      DATA (ION_POT (66,I), I= 0, 2) / 5.93, 11.67, 20. /  ! #1
!  Ho
      DATA ION_CNT ( 67,1:2 ) / 0, 2 /
      DATA (ION_POT (67,I), I= 0, 2) / 6.02, 11.80, 20. /  ! #1
!  Er
      DATA ION_CNT ( 68,1:2 ) / 0, 2 /
      DATA (ION_POT (68,I), I= 0, 2) / 6.10, 11.93, 20. /  ! #1
!  Tm
      DATA ION_CNT ( 69,1:2 ) / 0, 2 /
      DATA (ION_POT (69,I), I= 0, 2) / 6.18, 12.05, 23.71 /
!  Yb
      DATA ION_CNT ( 70,1:2 ) / 0, 2 /
      DATA (ION_POT (70,I), I= 0, 2) / 6.254, 12.17, 25.2 /
!  Lu
      DATA ION_CNT ( 71,1:2 ) / 0, 2 /
      DATA (ION_POT (71,I), I= 0, 2) / 5.426, 13.9, 19. /  ! #1
!  Hf
      DATA ION_CNT ( 72,1:2 ) / 0, 3 /
      DATA (ION_POT (72,I), I= 0, 3) / 7.0, 14.9, 23.3, 33.3 /
!  Ta
      DATA ION_CNT ( 73,1:2 ) / 0, 2 /
      DATA (ION_POT (73,I), I= 0, 2) / 7.89, 16.2, 24. /  ! #1
!  W
      DATA ION_CNT ( 74,1:2 ) / 0, 2 /
      DATA (ION_POT (74,I), I= 0, 2) / 7.98, 17.7, 25. / ! #1
!  Re
      DATA ION_CNT ( 75,1:2 ) / 0, 2 /
      DATA (ION_POT (75,I), I= 0, 2) / 7.88, 16.6, 26. / ! #1
!  Os
      DATA ION_CNT ( 76,1:2 ) / 0, 2 /
      DATA (ION_POT (76,I), I= 0, 2) / 8.7, 17., 27. / ! #1
!  Ir
      DATA ION_CNT ( 77,1:2 ) / 0, 2 /
      DATA (ION_POT (77,I), I= 0, 2) / 9.1, 20., 28. / ! #1
!  Pt
      DATA ION_CNT ( 78,1:2 ) / 0, 2 /
      DATA (ION_POT (78,I), I= 0, 2) / 9.0, 18.563, 29. / ! #1
!  Au
      DATA ION_CNT ( 79,1:2 ) / 0, 2 /
      DATA (ION_POT (79,I), I= 0, 2) / 9.225, 20.5, 30. / ! #1
!  Hg
      DATA ION_CNT ( 80,1:2 ) / 0, 2 /
      DATA (ION_POT (80,I), I= 0, 2) / 10.437, 18.756, 34.2 /
!  Tl
      DATA ION_CNT ( 81,1:2 ) / 0, 2 /
      DATA (ION_POT (81,I), I= 0, 2) / 6.108, 20.428, 29.83 /
!  Pb
      DATA ION_CNT ( 82,1:2 ) / 0, 4 /
      DATA (ION_POT (82,I), I= 0, 4) / 7.416, 15.032, 31.937, 42.32, &
                                      68.8 /
!  Bi
      DATA ION_CNT ( 83,1:2 ) / 0, 5 /
      DATA (ION_POT (83,I), I= 0, 5) / 7.289, 16.69, 25.56, 45.3, &
                                      56.0, 88.3 /
!  Po
      DATA ION_CNT ( 84,1:2 ) / 0, 0 /
      DATA (ION_POT (84,I), I= 0, 0) / 8.42 /
!  At
      DATA ION_CNT ( 85,1:2 ) / 0, 0 /
      DATA (ION_POT (85,I), I= 0, 0) / 9.3 /
!  Rn
      DATA ION_CNT ( 86,1:2 ) / 0, 0 /
      DATA (ION_POT (86,I), I= 0, 0) / 10.748/
!  Fr
      DATA ION_CNT ( 87,1:2 ) / 0, 0 /
      DATA (ION_POT (87,I), I= 0, 0) / 4.0727/
!  Ra
      DATA ION_CNT ( 88,1:2 ) / 0, 1 /
      DATA (ION_POT (88,I), I= 0, 1) / 5.279, 10.147/
!  Ac
      DATA ION_CNT ( 89,1:2 ) / 0, 1 /
      DATA (ION_POT (89,I), I= 0, 1) / 6.9, 12.1 /
!  Th
      DATA ION_CNT ( 90,1:2 ) / 0, 2 /
      DATA (ION_POT (90,I), I=0, 2) / 11.5, 20.0, 28.8 /

!  Pu
!     DATA ION_CNT ( 94,1:2 ) / 0, 2 /
!     DATA (ION_POT (94,I), I= 0, 0) / 5.8 /
!  Am
!     DATA ION_CNT ( 95,1:2 ) / 0, 2 /
!     DATA (ION_POT (95,I), I= 0, 0) / 6.0 /

!.


!  A default value
      TAP_IONPT = TAP__DNULL

!  Check for bad atomic number.
!     The element is not tabulated, return a NULL value.
      IF ( ZELEM.LE.0 .OR. ZELEM.GT.TAP__NELEM ) THEN
         TAP_IONPT = TAP__DNULL

!  Check for bad atomic charge.
!     The ion does not exist, return a NULL value.
      ELSEIF ( ZION.LT.-1 .OR. ZION.GT.ZELEM-1 ) THEN
         TAP_IONPT = TAP__DNULL

!     The ion is not tabulated, return a NULL value.
      ELSEIF ( ZION.LT.ION_CNT(ZELEM,1) .OR. ZION.GT.ION_CNT(ZELEM,2) ) THEN
         TAP_IONPT = TAP__DNULL

!  Pick up stored ionisation potential and apply correction factor.
      ELSE
         TAP_IONPT = ION_POT ( ZELEM, ZION ) * EV_COR

      ENDIF

    END FUNCTION TAP_IONPT
      
