DOUBLE PRECISION FUNCTION TAP_PARFN ( ZELEM, ZION, TEMP, PEL )

  !+
  !  Name:
  !     TAP_PARFN

  !  Purpose:
  !=    Calculate the partition function for a given ion
  !     at a given temperature and electron pressure

  !  Language:
  !     Fortran 90

  !  Invocation:
  !     U = TAP_PARFN ( zelem, zion, temp, pel )

  !  Description:
  !     Calculate partition functions using approximation formulae
  !     due to Traving et al. (1966a,b) and Unsoeld (1948).
  !     If these formulae have not been included (coded) then
  !     a ground-state partition function is returned. 
  !

  !  Arguments:
  !     ZELEM = INTEGER (Given)
  !        Atomic number (ie hydrogen = 1, helium = 2, etc )
  !     ZION = INTEGER (Given)
  !        Atomic charge (ie neutral = 0, singly ionised = 1, etc )
  !     TEMP = DOUBLE PRECISION (Given)
  !        Temperature (degrees K)
  !     PEL = DOUBLE PRECISION (Given)
  !        Electron pressure (N m**-2) 

  !  Returned Value:
  !     TAP_PARFN = DOUBLE PRECISION
  !        Partition function U(T,Pe)

  !  Algorithm:
  !     U = g0 + Sum(i=1,ns)[U'(i)]
  !
  !     U' = Sum(j=1,nk)[g(j).10^(-Chi(j).theta)]
  !          + 2.gpr.Qas(l,z).10^(-Chi(ion).theta)
  !
  !     theta = 5040 / T
  !
  !     Chebyshev approximation
  !
  !     f(theta) = Sum(j=1,nk)[g(j).10^(-Chi(j).theta)]
  !
  !     ~ psi(theta) = Sum(k=1,nm)[alpha(k).10^(-gamma(k).theta)]

  !  Accuracy:
  !     Chebyshev coefficients are valid for  0 <= Chi.theta <= 800
  !     In most cases, the maximum error at max(theta) is limited to
  !     |f(theta) - psi(theta)| < 0.01

  !  Limitations:
  !
  !     If Chi.theta > 800, the ground state statistical weight is returned.
  !        
  !     Where Chebyshev polynomials are not yet entered,
  !     the ground state statistical weight is returned. 
  !
  !     (07.12.2018) If the electron pressure is not available, 
  !     the ground state statistical weight is returned. 
  !
  !     If ground state statistical weights are unavailable
  !     a default value U = 1.0 is returned.

  !  Species currently included as a Chbyshev poynomial
  !  (indicated by ion identifier):
  !     ZELEM   ZION + I
  !     1   H   I
  !     2   He  I  II
  !     3   Li  I  II
  !     4   Be  I  II III
  !     5   B   I  II III  IV
  !     6   C   I  II III  IV   V
  !     7   N   I  II III  IV   V
  !     8   O   I  II III  IV   
  !     9   F   I  II III  IV   V
  !     10  Ne  I  II III  IV   
  !     11  Na  I  II III  IV   V
  !     12  Mg  I  II III  IV
  !     13  Al  I  II III  IV
  !     14  Si  I  II III  IV   V
  !     15  P   I  II III  IV
  !     16  S   I  II III  IV
  !     17  Cl  I  II III  IV   V
  !     18  A   I  II III  IV
  !     19  K   I  II III  IV   V
  !     20  Ca  I  II III  IV 
  !     21  Sc  I  II III  IV   V
  !     22  Ti  I  II III  IV   V
  !     23  V   I  II III  IV   V
  !     24  Cr  I  II III  IV   V  VI
  !     25  Mn  I  II III  IV   V  VI
  !     26  Fe  I  II III  IV
  !     27  Co  I  II III  IV
  !     28  Ni  I  II III  IV
  !     29  Cu  I  II III  IV
  !     30  Zn  I  II III  IV
  !     31  Ga  I  II III  IV
  !     32  Ge 
  !     33  As
  !     34  Se 
  !     35  Br
  !     36  Kr  I  II III
  !     37  Rb 
  !     38  Sr  I  II III  IV
  !     39  Y   I  II III  IV
  !     40  Zr  I  II III  IV
  !         Nb
  !         Ba

  !  Note:
  !     An accurate expression for THETA is given by
  !
  !         THETA = EV_K / TEMP
  !
  !     but in this subroutine it is more appropriate to use the 
  !     expression used by Traving et al. (1966a) 
  !
  !         THETA = 5040. / TEMP

  !  References:
  !       Traving et al. 1966a. Abh.der.Hamburg.Sternw. 8,3
  !       Traving et al. 1966b. Abh.der.Hamburg.Sternw. 8,26
  !       Unsold, 1948. Zeit.f.Astrophysik. 24,356.

  !  External Routines Used:
  !     TAP:
  !        TAP_PAR, [routine_used]...

  !  [optional_subroutine_items]...

  !  Authors:
  !     UH:    U. Heber (Kiel)
  !     CSJ:   C.S.Jeffery (St Andrews/CCP7)
  !     RED:   R.E.Dudley (St Andrews)       : Chebychev polynomial data
  !     RWJR:  R.W.J.Rolleston (QUB)         : Chebychev polynomial data
  !     CSJ:   C.S.Jeffery (St Andrews/CCP7) : Version for TAP
  !     AMIR:  A.Ahmad (Armagh Observatory)  : more polynomial data
  !     {enter_new_authors_here}

  !  History:
  !     12-MAR-1992 (CSJ):
  !        TAP version.
  !     29-FEB-1993 (CSJ):
  !        Sr, Y and Zr added
  !     9-MAY-1994 (CSJ):
  !        Converted to double precision function for 
  !        consistency with other TAP subprograms.
  !    29-SEP-2000 (CSJ)
  !        gsw's added for Z > 40.
  !           2002 (CSJ) 
  !        converted to F95
  !    12-MAR-2003 (CSJ)
  !        added more Bascek et al. PFs (data from TLUSTY)
  !    14-MAR-2003 (AMIR)
  !        Na, Cl, K, Ca, Sc, Co, Ni, Cu, Zn, Ga and Kr added
  !     {enter_changes_here}

  !  Bugs:
  !     Ground state statistical weights provided for first three ionisation
  !     states only, for Z >= Sc.
  !     No ground state statistical weights for As and Br, bad values returned.
  !     No ground state statistical weights for negative ions.
  !     For As, Br and Z>40, gsw's are taken from Kurucz code
  !     (PFSAHA).
  !     There are probably thousands of app. formulae for
  !     pf's in the literature, the OP are known to have done
  !     their own thing as well. This code really need cleaning 
  !     up and making completely general - for given conditions.
  !     Traving et al. pfs for Nb and Ba are still to be done
  !     {note_any_bugs_here}

  !-

  !  Global Constants:
  USE  SAE_PAR           ! Standard SAE constants
  USE  TAP_PAR           ! TAP public constants
  USE  TAPM_PAR          ! TAP Mathematical constants
  USE  TAPP_PAR          ! TAP Physical constants

  !  Type Definitions:
  IMPLICIT NONE              ! No implicit typing


  !  Arguments Given:
  INTEGER          ::  ZELEM
  INTEGER          ::  ZION 
  DOUBLE PRECISION ::  TEMP
  DOUBLE PRECISION ::  PEL

  !  Local Constants:
  ! EO / k * log e - Russell's constant
  DOUBLE PRECISION, PARAMETER :: EV_K = TAPP__EV * TAPM__LGE / TAPP__K 
  ! a0 - Radius for first Bohr orbit (cm)
  DOUBLE PRECISION, PARAMETER :: A_0 = TAPP__A0 * 100.

  !  Local Variables:
  INTEGER :: I                  ! array index / loop counter
  ! .. species or princ.quant.no.
  INTEGER :: J                  ! array index 
  ! .. ions
  INTEGER :: K                  ! array index / loop counter
  ! .. terms in Chebyshev expansions
  INTEGER :: M                  ! loop counter
  ! .. terms in quasi asymptotic formulae
  DOUBLE PRECISION :: PEX       ! electron pressure (cgs) 
  DOUBLE PRECISION :: U         ! internal holder for partition function U 
  DOUBLE PRECISION :: THETA     ! theta = 5040 / temp
  DOUBLE PRECISION :: QL        ! effective atomic charge
  DOUBLE PRECISION::  QJ        ! effective electron charge + 1
  DOUBLE PRECISION :: Q         ! temporary summation for U
  DOUBLE PRECISION :: QAS       ! quasi-asymptotic expansion
  DOUBLE PRECISION :: H         ! temporary summation for Qas
  DOUBLE PRECISION :: CHIT      ! Chi.theta
  DOUBLE PRECISION :: EXPON     ! exponent in Qas
  DOUBLE PRECISION :: MM        ! M*M
  INTEGER :: IZ                 ! local form for ZELEM
  INTEGER :: IJ                 ! local form for ZION
  INTEGER :: LAS                ! integer approximation for QL
  INTEGER :: NH                 ! integer approximation for H
  ! 

  !  Local Data:

  !  Declarations for polynomial coefficients:

  INTEGER, PARAMETER :: MZ=40, MJ=6, MS=5, MK=15

  INTEGER  :: ITBH, NS, NK
  REAL     :: G0, GPR, CHI, GL, ALP, GAM

  DIMENSION :: ITBH(MJ,MZ), NS(MJ,MZ), NK(MJ,MZ), G0(MJ,MZ)
  DIMENSION :: GPR(MS,MJ,MZ),CHI(MS,MJ,MZ),GL(MS,MJ,MZ)
  DIMENSION :: ALP(MK,MJ,MZ),GAM(MK,MJ,MZ)

  !  Declarations for ground state statistical weights:

  INTEGER, PARAMETER :: NZ=92, NJ=6
  INTEGER :: IG0(NJ,NZ)

  !-----------------------------------------------------------------------+-------

  !  Data for ground state statistical weights.

  DATA  ((IG0(J,I),J=1,NJ),I=1,10) & 
       /  2,  1,  1,  1,  1,  1,  & !  H 
       1,  2,  1,  1,  1,  1,  & !  He
       2,  1,  2,  1,  1,  1,  & !  Li
       1,  2,  1,  2,  1,  1,  & !  Be
       6,  1,  2,  1,  2,  1,  & !  B 
       9,  6,  1,  2,  1,  2,  & !  C
       4,  9,  6,  1,  2,  1,  & !  N
       9,  4,  9,  6,  1,  2,  & !  O
       6,  9,  4,  9,  6,  1,  & !  F
       1,  6,  9,  4,  9,  6  / !  Ne

  DATA  ((IG0(J,I),J=1,NJ),I=11,20) & 
       /  2,  1,  6,  9,  4,  9,  & !  Na
       1,  2,  1,  6,  9,  4,  & !  Mg
       6,  1,  2,  1,  6,  9,  & !  Al
       9,  6,  1,  2,  1,  6,  & !  Si
       4,  9,  6,  1,  2,  1,  & !  P
       9,  4,  9,  6,  1,  2,  & !  S
       6,  9,  4,  9,  6,  1,  & !  Cl
       1,  6,  9,  4,  9,  6,  & !  Ar
       2,  1,  6,  9,  4,  9,  & !  K
       1,  2,  1,  6,  9,  4  / !  Ca

  DATA  ((IG0(J,I),J=1,NJ),I=21,30) & 
       / 10, 15, 10,  1,  1,  1,  & !  Sc 
       21, 28, 21,  1,  1,  1,  & !  Ti
       28, 25, 28,  1,  1,  1,  & !  V
       7,  6, 25,  1,  1,  1,  & !  Cr
       6,  7,  6,  1,  1,  1,  & !  Mn
       25, 30, 25,  6, 25,  1,  & !  Fe
       28, 21, 28,  1,  1,  1,  & !  Co
       21, 10, 21,  1,  1,  1,  & !  Ni
       2,  1, 10,  1,  1,  1,  & !  Cu
       1,  2,  1,  1,  1,  1  / !  Zn

  DATA  ((IG0(J,I),J=1,NJ),I=31,40) & 
       /  6,  1,  2,  1,  1,  1,  & !  Ga 
       9,  6,  1,  1,  1,  1,  & !  Ge
       4,  8,  7,  1,  1,  1,  & !  As
       9,  4,  9,  1,  1,  1,  & !  Se  
       5,  8,  7,  1,  1,  1,  & !  Br 
       1,  6,  9,  1,  1,  1,  & !  Kr
       2,  1,  5,  1,  1,  1,  & !  Rb
       1,  2,  1,  1,  1,  1,  & !  Sr
       10, 15, 10,  1,  1,  1,  & !  Y
       21, 28, 21,  1,  1,  1  / !  Zr 

  DATA  ((IG0(J,I),J=1,NJ),I=41,50) & 
       / 36, 57, 25,  1,  1,  1,  & !  Nb
       7, 14, 45,  1,  1,  1,  & !  Mo
       14, 18,217,  1,  1,  1,  & !  Tc 
       24, 35, 23,  1,  1,  1,  & !  Ru  
       20, 21, 37,  1,  1,  1,  & !  Rh   
       2,  9, 23,  1,  1,  1,  & !  Pd
       2,  1,  9,  1,  1,  1,  & !  Ag
       1,  2,  1,  1,  1,  1,  & !  Cd
       3,  1,  2,  1,  1,  1,  & !  In  
       4,  4,  1,  1,  1,  1  / !  Sn 

  DATA  ((IG0(J,I),J=1,NJ),I=51,60) & 
       /  4,  5,  4,  1,  1,  1,  & !  Sb
       6,  6,  5,  1,  1,  1,  & !  Te
       4,  7,  7,  1,  1,  1,  & !  I 
       1,  4,  8,  1,  1,  1,  & !  Xe  
       2,  1,  5,  1,  1,  1,  & !  Cs   
       1,  4,  1,  1,  1,  1,  & !  Ba
       12, 30, 12,  1,  1,  1,  & !  La
       59,181, 40,  1,  1,  1,  & !  Ce
       94, 80,105,  1,  1,  1,  & !  Pr  
       23, 94, 80,  1,  1,  1  / !  Nd 

  DATA  ((IG0(J,I),J=1,NJ),I=61,70) & 
       / 20, 23, 94,  1,  1,  1,  & !  Pm
       16, 60, 38,  1,  1,  1,  & !  Sm
       8, 16,  8,  1,  1,  1,  & !  Eu 
       33, 94, 45,  1,  1,  1,  & !  Gd  
       67, 67, 67,  1,  1,  1,  & !  Tb   
       48, 48, 48,  1,  1,  1,  & !  Dy
       35, 35, 35,  1,  1,  1,  & !  Ho
       15, 35, 35,  1,  1,  1,  & !  Er
       8, 19, 17,  1,  1,  1,  & !  Tm  
       6,  2,  5,  1,  1,  1  / !  Yb 

  DATA  ((IG0(J,I),J=1,NJ),I=71,80) & 
       /  6,  2,  5,  1,  1,  1,  & !  Lu
       9, 18, 28,  1,  1,  1,  & !  Hf
       11, 38, 18,  1,  1,  1,  & !  Ta 
       8, 29, 38,  1,  1,  1,  & !  W  
       6, 11, 29,  1,  1,  1,  & !  Re   
       16, 26, 11,  1,  1,  1,  & !  Os
       18, 16, 26,  1,  1,  1,  & !  Ir
       18, 15, 15,  1,  1,  1,  & !  Pt
       2,  2,  8,  1,  1,  1,  & !  Au  
       1,  1,  1,  1,  1,  1  / !  Hg 

  DATA  ((IG0(J,I),J=1,NJ),I=81,90) & 
       /  2,  1,  2,  1,  1,  1,  & !  Tl
       1,  2,  1,  1,  1,  1,  & !  Pb
       4,  1,  1,  1,  1,  1,  & !  Bi 
       5,  5,  5,  1,  1,  1,  & !  Po 
       4,  7,  6,  1,  1,  1,  & !  At   
       1,  4,  7,  1,  1,  1,  & !  Rn
       2,  1,  4,  1,  1,  1,  & !  Fr
       1,  2,  1,  1,  1,  1,  & !  Ra
       6,  5,  9,  1,  1,  1,  & !  Ac  
       10, 10, 10,  1,  1,  1  / !  Th

  DATA  ((IG0(J,I),J=1,NJ),I=91,92) & 
       / 78, 78, 78,  1,  1,  1,  & !  Pa
       30, 30, 30,  1,  1,  1  / !  U 

  !-----------------------------------------------------------------------+-------

  !  Data for Chebyshev approximations.


  !  Hydrogen.

  DATA   (ITBH(J,1),J=1,6)  / 1, 5*0 /

  DATA   NS(1,1)    / 1 /
  DATA   G0(1,1)    / 2. /
  DATA   GPR(1,1,1) / 2. /
  DATA   GL(1,1,1)  / 11. /
  DATA   CHI(1,1,1) / 13.595 /

  DATA   NK(1,1)    / 2 /
  DATA  (GAM(K,1,1),K=1,2) / 10.853,13.342 /
  DATA  (ALP(K,1,1),K=1,2) / 20.4976,747.5023 /


  !  Helium.

  DATA  (ITBH(J,2),J=1,6)  / 1, 1, 4*0 /

  DATA  (NS(J,2),J=1,2)    / 1, 1 /
  DATA  (G0(J,2),J=1,2)    / 1, 2/ 
  DATA  (GPR(1,J,2),J=1,2) / 4, 2 /
  DATA  (GL(1,J,2),J=1,2)  / 8, 12 /
  DATA  (CHI(1,J,2),J=1,2) / 24.580, 54.403 /

  DATA  (NK(J,2),J=1,2)   / 2, 2 /
  DATA ((GAM(K,J,2),K=1,2),J=1,2) & 
       / 21.170,24.125,43.708,53.542 /
  DATA ((ALP(K,J,2),K=1,2),J=1,2) & 
       / 28.1703,527.8296,22.2809,987.7189 /


  !  Lithium.

  DATA  (ITBH(J,3),J=1,6)  / 1, 1, 1, 3*0 /

  DATA  (NS(J,3),J=1,3)    / 1, 1, 1 /
  DATA  (G0(J,3),J=1,3)    / 2, 1, 2 / 
  DATA  (GPR(1,J,3),J=1,3) / 2, 4, 2 /
  DATA  (GL(1,J,3),J=1,3)  / 6, 6, 8 /
  DATA  (CHI(1,J,3),J=1,3) / 5.390, 75.619, 122.420 /

  DATA  (NK(J,3),J=1,3)    / 2, 2, 2/
  DATA ((GAM(K,J,3),K=1,2),J=1,3) & 
       / 2.022,   4.604,  62.032,  72.624, 96.149, 118.333 /
  DATA ((ALP(K,J,3),K=1,2),J=1,3) & 
       /  8.4915,  97.5015,  23.3299, 192.6701, 16.0047, 247.9952 /


  !  Beryllium.

  DATA  (ITBH(J,4),J=1,6)  / 1, 1, 1, 1, 2*0 /

  DATA  (NS(J,4),J=1,4)    / 2, 1, 1, 1 /
  DATA  (G0(J,4),J=1,4)    / 1, 2, 1, 2 / 
  DATA  (GPR(1,J,4),J=1,4) / 4, 2, 4, 2 /
  DATA  (GL(1,J,4),J=1,4)  / 6, 8, 6, 8 /
  DATA  (CHI(1,J,4),J=1,4) /  9.320, 18.206, 153.850, 217.657 /
  DATA  (GPR(2,J,4),J=1,4) / 12, 3*0 /
  DATA  (GL(2,J,4),J=1,4)  /   4, 3*0  /
  DATA  (CHI(2,J,4),J=1,4) / 13.278, 3*0 /

  DATA  (NK(J,4),J=1,4)   / 5, 3, 2, 2 /
  DATA ((GAM(K,J,4),K=1,5),J=1,4) & 
       /  2.735,   6.774,   8.569,  10.750,  11.672,  &
       3.967,  12.758,  16.692, 2*0.,  &
       123.719, 147.167, 3*0., &
       171.066, 210.558, 3*0.  /
  DATA ((ALP(K,J,4),K=1,5),J=1,4) & 
       /   9.1849,  32.9263, 183.8887,  19.9563,  88.0437,  &
       6.0478,  35.9723, 233.9798, 2*0., &
       23.1331, 192.8669, 3*0., &
       16.1551, 261.8348, 3*0. /


  !  Boron.

  DATA  (ITBH(J,5),J=1,6)  / 1, 1,1, 1, 1, 0 /

  DATA  (NS(J,5),J=1,5)    / 1, 2, 1, 1, 1 /
  DATA  (G0(J,5),J=1,5)    / 2, 1, 2, 1, 2 / 
  DATA  (GPR(1,J,5),J=1,5) / 2, 4, 2, 4, 2 /
  DATA  (GL(1,J,5),J=1,5)  / 9, 6, 6, 5, 6 /
  DATA  (CHI(1,J,5),J=1,5) / 8.296, 25.149, 37.920, 259.298, 340.127 /
  DATA  (GPR(2,J,5),J=1,5) / 0,12, 0, 0, 0 /
  DATA  (GL(2,J,5),J=1,5)  / 0, 4, 0, 0, 0 /
  DATA  (CHI(2,J,5),J=1,5) / 0., 31.146, 3*0. /

  DATA  (NK(J,5),J=1,5)   / 3, 5, 3, 2, 2 /
  DATA ((GAM(K,J,5),K=1,5),J=1,5) & 
       / 0.002,   3.971,   7.882, 2*0.,  &
       4.720,  13.477,  22.103,  23.056,  24.734,  & 
       6.000,  24.540,  32.300, 2*0.,  &
       204.371, 243.028, 3*0., &
       255.201, 319.715, 3*0. /
  DATA ((ALP(K,J,5),K=1,5),J=1,5) & 
       / 4.0086,  19.6741, 402.3110, 2*0., &
       9.7257,  30.9262, 186.3466,  44.1629,  60.8371, &
       6.0084,  23.5767,  76.4149, 2*0., &
       19.5530,  96.4470, 3*0., &
       8.0000, 100.0000, 3*0.  /  


  !  Carbon.

  DATA  (ITBH(J,6),J=1,6) / 1, 1, 1, 1, 1, 0 /

  DATA  (NS(J,6),J=1,5)   / 1, 2, 2, 1, 1 /
  DATA  (G0(J,6),J=1,5)   / 1., 2., 1., 2., 1. /
  DATA ((GPR(I,J,6),I=1,2),J=1,5) & 
       / 12., 0., 2., 18., 4., 12., 2., 0., 4., 0. /
  DATA ((GL(I,J,6),I=1,2),J=1,5) & 
       /  6., 0., 6., 5., 6.1, 5., 6., 0., 4., 0. /
  DATA ((CHI(I,J,6),I=1,2),J=1,5) & 
       /11.256, 0.0  ,24.376,30.868,47.871,55.873, & 
       64.476, 0.0, 391.986, 0.   /

  DATA  (NK(J,6),J=1,5)   / 4, 6, 6, 3, 2 /
  DATA ((GAM(K,J,6),K=1,6),J=1,5) & 
       / 0.004, 1.359, 6.454,10.376,  0.  ,  0.  , & 
       0.008,16.546,21.614, 5.688, 15.801, 26.269, & 
       6.691,25.034,40.975,17.604, 36.180, 47.133, & 
       8.005,40.804,54.492, 0.  ,  0.  ,  0.  , & 
       303.772, 354.208, 4*0. /
  DATA ((ALP(K,J,6),K=1,6),J=1,5) & 
       / 8.0158, 5.8833, 33.7521,595.3432,  2*0., & 
       4.0003,17.0841, 82.9154, 15.9808, 48.2044,435.8093, & 
       10.0281,15.7574,186.2109, 15.4127, 55.9559,243.6311, & 
       6.0057,23.5757, 76.4185, 3*0., & 
       15.7995,36.2005, 4*0. /


  !  Nitrogen.

  DATA  (ITBH(J,7),J=1,6) / 1, 1, 1, 1, 1, 0 /

  DATA  (NS(J,7),J=1,5)   / 2, 2, 3, 2, 1 /
  DATA  (G0(J,7),J=1,5)   / 4., 1., 2., 1., 2. /
  DATA ((GPR(I,J,7),I=1,3),J=1,5) & 
       / 18.,10.,0., 12.,24.,0., 2.,18.,6., 4.,12.,0., & 
       2.0, 0., 0. /
  DATA ((GL(I,J,7),I=1,3),J=1,5) & 
       /  6.1,4.,0., 5.,3.9,0., 6.,5.,4., 6.,6.3,0., & 
       6., 0., 0. /
  DATA ((CHI(I,J,7),I=1,3),J=1,5) & 
       /14.529,16.428, 0.  ,29.593,36.693, 0.  , & 
       47.426,55.765,63.626,77.450,87.445, 0.  , & 
       97.863, 0., 0. /

  DATA  (NK(J,7),J=1,5)   / 6, 7, 9, 5, 3 /
  DATA ((GAM(K,J,7),K=1,9),J=1,5) & 
       / 2.554,  9.169, 13.651, 12.353, 13.784, 14.874,  3*0., & 
       0.014,  2.131, 15.745, 24.949,  6.376, 14.246, & 
       29.465,  2*0., & 
       0.022, 31.259, 41.428,  7.212, 15.228, 34.387, & 
       46.708, 46.475, 49.468, & 
       8.693, 37.650, 65.479, 61.155, 79.196,  4*0., & 
       9.999, 60.991, 82.262, 6*0. /
  DATA ((ALP(K,J,7),K=1,9),J=1,5) & 
       /14.0499, 30.8008,883.1443, 10.   , 16.   , 64.   , & 
       0.   ,  0.   ,  0.   , &
       8.0462,  6.2669, 17.8696,282.8084,  7.3751, 33.1390, & 
       215.4829,  0.   ,  0.   , & 
       4.0003, 19.3533, 80.6462, 13.0998, 19.6425, 94.3035, & 
       370.9539, 16.00 , 38.00 , & 
       10.3289, 14.5021,187.1624,108.1615,191.8383,  0.   , & 
       0.   ,  0.   ,  0.   , & 
       6.0044, 23.5612, 76.4344, 6*0. /


  !  Oxygen.

  DATA  (ITBH(J,8),J=1,6) / 1, 1, 1, 1, 0, 0  /

  DATA  (NS(J,8),J=1,4)   / 3, 4, 3, 5 /
  DATA  (G0(J,8),J=1,4)   / 5., 4., 1., 2. /
  DATA ((GPR(I,J,8),I=1,5),J=1,4) & 
       /  8.,20.,12., 0., 0.,   18.,10., 2.,10., 0.,  & 
       12.,24.,20., 0., 0.,    2.,18., 6.,18.,10. /
  DATA ((GL(I,J,8),I=1,5),J=1,4) & 
       / 8.,6.,3.4,0.,0.,   6.,5.,3.9,3.9,0.,  & 
       6.,4.9,4.,0.,0.,   5.9,5.,4.9,4.,4. /
  DATA ((CHI(I,J,8),I=1,5),J=1,4) & 
       / 13.614,16.938,18.630, 0.  , 0.  ,  & 
       35.108,37.621,40.461,42.584, 0.  , & 
       54.886,63.733,70.556, 0.  , 0.  , & 
       77.394,87.609,97.077,103.911,106.116 /

  DATA  (NK(J,8),J=1,4)   / 8, 12, 10, 13 /
  DATA ((GAM(K,J,8),K=1,13),J=1,4) & 
       /  0.022, 2.109, 9.812,13.087,13.804,16.061,14.293, & 
       16.114, 0.  , 0.  , 0.  , 0.  , 0.  , & 
       3.472, 7.437,22.579,32.035,27.774,33.678,28.118, & 
       31.019,34.204,30.892,33.189,36.181, 0.  , & 
       0.032, 2.760,35.328,48.277, 7.662,16.786,42.657, & 
       54.522,50.204,56.044, 0.  , 0.  , 0.  , & 
       0.048,50.089,66.604, 8.954,18.031,57.775,72.594, & 
       68.388,82.397,31.960,76.876,75.686,80.388           /
  DATA ((ALP(K,J,8),K=1,13),J=1,4) & 
       / 4.0029,5.3656,36.2853,1044.3447,131.0217,868.9779, & 
       14.8533,93.1466, 5*0., & 
       12.7843,5.6828,98.0919,829.4396,50.9878,199.0120, & 
       2.,6.,10.,10.,30.,50.,                     0., & 
       8.0703,5.7144,84.1156,529.0927,5.6609,28.9355, & 
       111.3620,494.0413,45.5249,134.4751,      3*0., & 
       4.0003,21.2937,78.7058,12.8293,16.2730,123.6578, & 
       327.2396,48.7883,102.2117,20.0060,161.9903,28.4184, & 
       61.5816                                             /


  !  Fluorine.

  DATA  (ITBH(J,9),J=1,6) / 1, 1, 1, 1, 1, 0 /

  DATA  (NS(J,9),J=1,5)   / 2, 3, 4, 3, 5 /
  DATA  (G0(J,9),J=1,5)   / 4, 5, 4, 1, 2 /
  DATA ((GPR(I,J,9),I=1,5),J=1,5) & 
       / 18,10,0,0,0,   8,20,12,0,0, 18,10,2,10,0, 12,24,20,0,0, 2,18,6,18,10 /
  DATA ((GL(I,J,9),I=1,5),J=1,5) & 
       / 4,4,0,0,0, 5,4,4,0,0, 5,4,3.9,4,0, 5,5,4,0,0, 5,5,4,4,4  /
  DATA ((CHI(I,J,9),I=1,5),J=1,5) & 
       /     17.418,  20.009,  3*0., &
       34.977,  39.204,  41.368, 2*0., &
       62.646,  65.774,  69.282,  71.882,   0., &
       87.139,  97.852, 106.089, 2*0., &
       114.214, 126.256, 137.373, 145.435, 148.250  /

  DATA  (NK(J,9),J=1,5)   / 6, 8, 11, 9, 12 /
  DATA ((GAM(K,J,9),K=1,12),J=1,5) & 
       / 0.050, 13.317, 15.692, 15.361, 17.128, 18.498,6*0.,                    &
       0.048,  2.735, 20.079, 30.277, 27.548, 32.532, 30.391, 34.707,4*0.,    &
       4.479, 12.072, 31.662, 51.432, 44.283, 50.964, 46.193, 50.436, 54.880, &
       50.816, 57.479,  0.  , &
       0.058,  3.434, 14.892, 37.472, 69.883, 67.810, 83.105, 72.435, 79.747, &
       3*0.,                    &
       0.098, 11.913, 31.240, 86.166, 83.035,103.846, 90.117, 97.030, 99.602, &
       106.700,101.990,108.179  /
  DATA ((ALP(K,J,9),K=1,12),J=1,5) & 
       /  2.0001, 39.9012,122.0986, 10.0000, 30.0000, 50.0000,6*0.,&
       4.0199,  5.5741, 22.1839,190.2179, 53.0383,126.9616,     &
       31.6894, 75.3105,4*0.,                                    &
       13.5014,  7.9936, 55.7981,298.7039, 26.2496, 63.7503,     &
       2.0000,  6.0000, 10.0000, 28.7150, 71.2850,  0.,         &
       8.0153,  6.1931, 21.7287, 48.7780,278.2782, 178.5560,    &
       421.4435, 51.7632, 95.2368, 3*0.,                          &
       4.0266, 17.5284, 27.4353, 49.0023,143.1411,306.8586,     &
       14.9231, 39.0769, 56.1826, 75.8174, 22.4437, 49.5563      /


  !  Neon.

  DATA  (ITBH(J,10),J=1,6) / 1, 1, 1, 1, 0, 0 /

  DATA  (NS(J,10),J=1,4)   / 2, 2, 3, 2 /
  DATA  (G0(J,10),J=1,4)   / 1, 4, 5, 4 /
  DATA ((GPR(I,J,10),I=1,3),J=1,4) & 
       / 8.,4.,0., 18.,10.,0.,  8.,20.,12., 18.,10.,0. /
  DATA ((GL(I,J,10),I=1,3),J=1,4) & 
       / 6.,6.,0., 5.,4.,0., 3.9,4.,4., 5.,5.,0.  /
  DATA ((CHI(I,J,10),I=1,3),J=1,4) & 
       / 21.559,21.656,0., 41.071,44.274,0., & 
       63.729,68.806,71.434, 97.162,100.917,0.  /

  DATA  (NK(J,10),J=1,4)   / 4, 5, 8, 6 /
  DATA ((GAM(K,J,10),K=1,8),J=1,4) & 
       / 17.796, 20.730, 17.879, 20.855, 4*0., & 
       0.097, 29.878, 37.221, 31.913, 37.551, 3*0., & 
       0.092, 3.424, 24.806, 46.616, 45.643, 54.147,  & 
       48.359, 57.420, & 
       5.453, 18.560, 46.583, 80.101, 70.337, 85.789, 2*0. /
  DATA ((ALP(K,J,10),K=1,8),J=1,4) & 
       / 34.5080, 365.4919, 16.5768, 183.4231, 4*0., & 
       2.0007, 89.5607, 380.4381, 26.4473, 63.5527, 3*0., & 
       4.0342, 5.6162, 11.5176, 72.8273, 48.5684, 131.4315, & 
       31.1710, 76.8290, & 
       14.0482, 13.3077, 52.7897, 467.8487, 54.2196,  & 
       195.7800, 2*0.  /


  !  Sodium.

  DATA  (ITBH(J,11),J=1,6) / 1, 1, 1, 1, 1, 0 /

  DATA  (NS(J,11),J=1,5)   / 1, 2, 2, 3, 5 /
  DATA  (G0(J,11),J=1,5)   / 2, 1, 4, 5, 4 /
  DATA ((GPR(I,J,11),I=1,5),J=1,5) & 
       / 2., 4*0., 8., 4., 3*0., 18., 10., 3*0., 8., 20., &
       12., 2*0., 18., 10., 2., 10., 30. /
  DATA ((GL(I,J,11),I=1,5),J=1,5) & 
       / 7., 4*0., 4., 4., 3*0., 4., 4., 3*0., 5., 5., 5., &
       2*0., 5., 5., 4., 4., 4. /
  DATA ((CHI(I,J,11),I=1,5),J=1,5) & 
       /  5.138, 4*0., 47.290, 47.459, 3*0., 71.647, 75.504, &
       3*0., 98.880, 104.778, 107.864, 2*0., 138.597, 142.980, &
       147.803, 151.427, 163.906 /

  DATA  (NK(J,11),J=1,5)   / 2, 4, 5, 8, 13 /
  DATA ((GAM(K,J,11),K=1,13),J=1,5) & 
       / 2.400, 4.552, 11*0., &
       34.367, 40.566, 34.676, 40.764, 9*0., &
       0.170,  44.554, 57.142, 51.689, 60.576, 8*0., &
       0.152,   4.260, 36.635, 83.254, 72.561, 89.475, &
       75.839, 92.582, 5*0., &
       5.930, 9.127, 94.219, 115.690, 97.783, 12.110, 92.769, &
       108.400, 28.372, 49.002, 111.473, 110.226, 125.690  /
  DATA ((ALP(K,J,11),K=1,13),J=1,5) & 
       /  11.6348, 158.3593,  11*0., &
       21.0453,  50.9546,  10.1389,  25.8611,  9*0., &
       2.0019,  38.0569, 137.9398,  28.3106,  61.6893, 8*0., &
       4.0334,   5.8560,  18.1786, 208.9142,  &
       93.6895, 406.3095,  60.4276, 239.5719,  5*0., &
       10.3174, 5.6967, 121.0726, 340.9131, 63.7918, 186.2078, 2.0000, &
       16.0000, 17.9266, 18.7036, 91.3687, 36.8524, 233.1475  /


  !  Magnesium.

  DATA  (ITBH(J,12),J=1,6) / 1, 1, 1, 1, 0, 0 /

  DATA  (NS(J,12),J=1,4)   / 1, 1, 2, 2 /
  DATA  (G0(J,12),J=1,4)   / 1, 2, 1, 4 /
  DATA ((GPR(I,J,12),I=1,2),J=1,4) & 
       / 4., 0., 2., 0., 8., 4., 18.,10. /
  DATA ((GL(I,J,12),I=1,2),J=1,4) & 
       / 7., 0., 7., 0., 5., 5., 5., 5. /
  DATA ((CHI(I,J,12),I=1,2),J=1,4) & 
       / 7.644, 0.0, 15.031, 0.0, 80.117, 80.393,  & 
       109.294, 113.799  /

  DATA  (NK(J,12),J=1,4)   / 3, 3, 4, 5 /
  DATA ((GAM(K,J,12),K=1,5),J=1,4) & 
       / 2.805, 6.777, 9.254,     0.,    0.,  & 
       4.459, 9.789, 13.137,    0.,    0.,  & 
       57.413, 71.252, 58.010, 71.660, 0., & 
       0.276, 74.440, 94.447, 54.472, 95.858  /
  DATA ((ALP(K,J,12),K=1,5),J=1,4) & 
       / 10.7445, 291.5057, 53.7488,  0.,       0., & 
       6.2270, 31.1291, 132.6438,   0.,       0., & 
       40.4379, 159.5618, 20.3845, 79.6154,   0., & 
       2.0007, 106.8977, 343.1010, 10.1326, 237.8581  /


  !  Aluminium.

  DATA  (ITBH(J,13),J=1,6) / 1, 1, 1, 1, 0, 0 /

  DATA  (NS(J,13),J=1,4)   / 2, 2, 1, 2 /
  DATA  (G0(J,13),J=1,4)   / 2, 1, 2, 1 /
  DATA ((GPR(I,J,13),I=1,2),J=1,4) & 
       / 2., 18., 4., 12., 2., 0., 8., 4.  /
  DATA ((GL(I,J,13),I=1,2),J=1,4) & 
       / 7., 4., 7., 4., 7., 0., 5., 5.  /
  DATA ((CHI(I,J,13),I=1,2),J=1,4) & 
       / 5.984, 10.634, 18.823, 25.496, 28.441, 0., & 
       119.957, 120.383  /

  DATA  (NK(J,13),J=1,4)   / 5, 5, 3, 4 /
  DATA ((GAM(K,J,13),K=1,5),J=1,4) & 
       / 0.014, 3.841, 5.420, 3.727, 8.833, & 
       4.749, 11.902, 16.719, 11.310, 18.268, & 
       6.751, 16.681, 24.151, 2*0., & 
       83.551, 104.787, 84.293, 105.171, 0.  /
  DATA ((ALP(K,J,13),K=1,5),J=1,4) & 
       / 4.0009, 11.7804, 142.2179, 13.6585, 96.3371, & 
       10.0807, 49.5843, 285.3343, 14.6872, 59.3122, & 
       6.3277, 29.5086, 134.1634, 2*0., & 
       46.3164, 153.6833, 22.9896, 77.0103, 0.  /


  !  Silicon.

  DATA  (ITBH(J,14),J=1,6) / 1, 1, 1, 1, 1, 0 /

  DATA  (NS(J,14),J=1,5)   / 1, 2, 2, 1, 2 /
  DATA  (G0(J,14),J=1,5)   / 1, 2, 1, 2, 1 /
  DATA ((GPR(I,J,14),I=1,2),J=1,5) & 
       / 12., 0., 2., 18., 4., 12., 2., 0., 8., 4. /
  DATA ((GL(I,J,14),I=1,2),J=1,5) & 
       / 6.1, 0., 5.9, 5.0, 5.0, 5.0, 7.0, 0.,  & 
       4.9, 4.9  /
  DATA ((CHI(I,J,14),I=1,2),J=1,5) & 
       / 8.149, 0., 16.339, 22.894, 33.459, 42.333, & 
       45.130, 0., 166.725, 167.357  /

  DATA  (NK(J,14),J=1,5)   / 5, 8, 7, 3, 4 /
  DATA ((GAM(K,J,14),K=1,8),J=1,5) & 
       / 0.020, 0.752, 1.614, 5.831, 7.431, 3*0., & 
       0.036, 8.795, 11.208, 13.835, 5.418, 7.825, & 
       14.440, 19.412, & 
       6.572, 11.449, 18.424, 25.457, 15.682, 27.010, & 
       34.599, 0., & 
       9.042, 24.101, 37.445, 5*0., & 
       115.608, 142.257, 118.377, 143.084, 4*0. /
  DATA ((ALP(K,J,14),K=1,8),J=1,5) & 
       / 7.9658, 4.6762, 1.3512, 123.2267, 443.7797, 3*0., & 
       4.00, 7.4186, 24.1754, 60.4060, 14.4695, 11.9721, & 
       26.5062, 269.0521, & 
       9.1793, 4.8766, 29.1442, 52.7998, 13.2674, 36.0417, & 
       180.6910, 0., & 
       6.4839, 27.6851, 135.8301, 5*0., & 
       36.9821, 163.0176, 19.1161, 80.8837, 4*0.  /


  !  Phosphorus.

  DATA  (ITBH(J,15),J=1,6) / 1, 1, 1, 1, 0, 0 /

  DATA  (NS(J,15),J=1,4)   / 2, 1, 1, 1 /
  DATA  (G0(J,15),J=1,4)   / 4, 1, 2, 1 /
  DATA ((GPR(I,J,15),I=1,2),J=1,4) & 
       / 18., 10., 12., 0., 2., 0., 4., 0.  /
  DATA ((GL(I,J,15),I=1,2),J=1,4) & 
       / 5., 5., 5., 0., 7., 0., 8.6, 0.  /
  DATA ((CHI(I,J,15),I=1,2),J=1,4) & 
       / 10.474, 11.585, 19.720, 0., 30.156, 0., 51.354, 0.  /

  DATA  (NK(J,15),J=1,4)   / 5, 4, 4, 3 /
  DATA ((GAM(K,J,15),K=1,5),J=1,4) & 
       / 1.514, 5.575, 9.247, 8.076, 10.735, & 
       0.043, 1.212, 8.545, 15.525, 0., & 
       0.074, 7.674, 16.639, 25.118, 0., & 
       8.992, 24.473, 40.704, 2*0.  /
  DATA ((ALP(K,J,15),K=1,5),J=1,4) & 
       / 13.5211, 22.2130, 353.2583, 10.00, 150.00, & 
       8.0241, 5.8085, 51.7542, 252.4002, 0., & 
       4.0021, 20.7985, 62.4194, 200.7786, 0., & 
       11.7414, 63.5124, 179.7420, 2*0.  /


  !  Sulphur.

  DATA  (ITBH(J,16),J=1,6) / 1, 1, 1, 1, 0, 0 /

  DATA  (NS(J,16),J=1,4)   / 3, 2, 1, 2 /
  DATA  (G0(J,16),J=1,4)   / 5, 4, 1, 2 /
  DATA ((GPR(I,J,16),I=1,3),J=1,4) & 
       / 8.,20.,12., 18.,10.,0., 12.,0.,0., 2.,18.,0.  /
  DATA ((GL(I,J,16),I=1,3),J=1,4) & 
       / 6., 5., 5., 5., 5., 0., 3.5, 0., 0., 5.0, 14.4, 0.  /
  DATA ((CHI(I,J,16),I=1,3),J=1,4) & 
       / 10.357, 12.200, 13.401, 23.405, 24.807, 0., & 
       35.047, 0., 0., 47.292, 57.681, 0.  /

  DATA  (NK(J,16),J=1,4)   / 8, 6, 5, 6 /
  DATA ((GAM(K,J,16),K=1,8),J=1,4) & 
       / 0.053, 1.121, 5.812, 9.425, 8.936, 11.277, & 
       9.600, 12.551, & 
       1.892, 3.646, 13.550, 19.376, 16.253, 21.062, 2*0., & 
       0.043, 0.123, 1.590, 13.712, 22.050, 3*0., & 
       0.118, 9.545, 18.179, 31.441, 30.664, 56.150, 2*0.  /
  DATA ((ALP(K,J,16),K=1,8),J=1,4) & 
       / 3.9615, 5.0780, 15.0944, 362.8588, 51.5995, 268.4002, & 
       12.00, 276.00, &
       11.4377, 5.5126, 141.0009, 254.0478, 33.0518, & 
       126.9479, 2*0., & 
       4.0707, 4.0637, 5.7245, 144.6376, 106.4909, 3*0., & 
       4.0011, 19.2813, 27.5990, 35.1179, 94.7454,  & 
       283.2486, 2*0.  / 


  !  Chlorine.

  DATA  (ITBH(J,17),J=1,6) / 1, 1, 1, 1, 1, 0 /

  DATA  (NS(J,17),J=1,5)   / 2, 3, 2, 1, 1 /
  DATA  (G0(J,17),J=1,5)   / 4, 5, 4, 1, 2 /
  DATA ((GPR(I,J,17),I=1,3),J=1,5) & 
       / 18., 10., 0., 8., 20., 12., 18., 10., 0., 12., 2*0., 2., 2*0. /
  DATA ((GL(I,J,17),I=1,3),J=1,5) & 
       / 6., 5., 0., 5., 5., 5., 5., 5., 0., 5., 2*0., 5.2, 2*0. /
  DATA ((CHI(I,J,17),I=1,3),J=1,5) & 
       / 13.014, 14.458, 0., 23.798, 26.041, 27.501, 39.904, 41.610, 0., &
       53.450, 2*0., 67.801, 2*0. /

  DATA  (NK(J,17),J=1,5)   / 5, 9, 6, 4, 4 /
  DATA ((GAM(K,J,17),K=1,9),J=1,5) & 
       /   0.110,   9.919,  12.280,  11.017,  13.532, 4*0., &
       0.092,  0.581,   1.620,  13.121,  19.787, &
       16.365,21.988,  18.065,  23.594, &
       2.358,  5.708,  19.084,  30.683, &
       24.880, 33.229, 3*0., &
       0.102,   1.391,  14.709,  36.968, 5*0., &
       0.185,  11.783,  25.653,  44.698, 5*0.  /
  DATA ((ALP(K,J,17),K=1,9),J=1,5) & 
       /   2.0007,  62.5048, 669.4942,  29.0259, 130.9740, 4*0., &
       3.9064,  0.3993,   5.3570,  60.3424, 119.9913, &
       138.1567, 278.8418, 102.3681, 158.6314, &
       12.6089,   5.9527, 110.5635, 262.8715, &
       69.2035, 100.7960, 3*0., &
       7.3458,   5.6638,  44.1256, 202.7846, 5*0., &
       4.0037,  21.8663,  40.5363,  57.5919, 5*0.  /


  !  Argon.

  DATA  (ITBH(J,18),J=1,6) / 1, 1, 1, 1, 0, 0 /

  DATA  (NS(J,18),J=1,4)   / 2, 2, 3, 1 /
  DATA  (G0(J,18),J=1,4)   / 1, 4, 5, 4 /
  DATA ((GPR(I,J,18),I=1,3),J=1,4) & 
       / 8.,4.,0., 18.,10.,0., 8.,20.,12., 18.,0.,0.  /
  DATA ((GL(I,J,18),I=1,3),J=1,4) & 
       / 6.,6.,0., 5.1,5.,0., 5.,5.,5., 5.,0.,0.  /
  DATA ((CHI(I,J,18),I=1,3),J=1,4) & 
       / 15.755, 15.933,  0.0 , 27.619, 29.355, 0., & 
       40.899, 42.407, 45.234, 59.793,  0., 0.  /

  DATA  (NK(J,18),J=1,4)   / 4, 5, 8, 4 /
  DATA ((GAM(K,J,18),K=1,8),J=1,4) & 
       / 12.638, 14.958, 12.833, 15.139, 4*0., & 
       0.178, 17.522, 23.584, 20.464, 25.150, 3*0.,  & 
       0.151,  1.561, 17.399, 30.871, 24.684, 33.978, & 
       27.091, 36.481,  & 
       2.810,  8.877, 24.351, 44.489, 4*0. /
  DATA ((ALP(K,J,18),K=1,8),J=1,4) & 
       / 43.6623, 324.3375, 20.8298, 163.1701, 4*0., & 
       2.0026, 137.4515, 258.5445, 62.8129, 149.1867, 3*0., & 
       4.0495, 14.4466, 46.8234, 124.6651,  & 
       151.9828, 268.0157, 101.1302, 150.8691, & 
       13.3718, 8.6528, 60.4614, 285.5072, 4*0. /


  !  Potassium.

  DATA  (ITBH(J,19),J=1,6) / 1, 1, 1, 1, 1, 0 /

  DATA  (NS(J,19),J=1,5)   / 1, 2, 3, 3, 2 /
  DATA  (G0(J,19),J=1,5)   / 2, 1, 4, 5, 4 /
  DATA ((GPR(I,J,19),I=1,3),J=1,5) & 
       / 2., 2*0., 8., 4., 0., 18., 10., 2., 8., 20., 12., 18., 10., 0. /
  DATA ((GL(I,J,19),I=1,3),J=1,5) & 
       / 7., 2*0., 5., 5., 0., 6., 6., 5., 6., 5., 5., 3.6, 4., 0. /
  DATA ((CHI(I,J,19),I=1,3),J=1,5) & 
       / 4.339, 2*0., 31.810, 32.079, 0., 45.738, 47.768, 50.515, 60.897, &
       63.890, 65.849, 82.799, 85.150, 0. /

  DATA  (NK(J,19),J=1,5)   / 3, 5, 8, 8, 6 /
  DATA ((GAM(K,J,19),K=1,8),J=1,5) & 
       /1.871,  3.713,  18.172,  5*0., &
       21.185, 27.705,   2.059,  23.709,  28.542, 3*0., &
       0.273,  26.709,  39.640,  31.220,  41.865, 29.955,  37.557,  42.862, &
       0.228,   2.274,  21.703,  50.191,  32.145, 49.262,  34.155,  51.718, &
       3.043,   5.479,  20.547,  30.680,  36.275, 47.345,  2*0. /
  DATA ((ALP(K,J,19),K=1,8),J=1,5) & 
       / 12.9782, 148.6673, 6.3493,  5*0., &
       66.3444, 101.6553,   4.0001,  13.4465,  46.5534, 3*0., &
        2.0171, 116.4767, 713.4965,  63.5907, 396.4079,   2.0000, 10.0000,  30.0000, &
        4.0702,   5.7791,  52.6795, 327.4539,  62.8604, 357.1331, 55.9337, 196.0646, &
       10.9275,   5.5398,  43.2761,  76.2560,  42.0000,  18.0000, 2*0. /


  !  Calcium.

  DATA  (ITBH(J,20),J=1,6) / 1, 1, 1, 1, 0, 0 /

  DATA  (NS(J,20),J=1,4)   / 2, 1, 2, 2 /
  DATA  (G0(J,20),J=1,4)   / 1, 2, 1, 4 /
  DATA ((GPR(I,J,20),I=1,2),J=1,4) & 
       /  4., 20., 2., 0., 8., 4., 18., 10. /
  DATA ((GL(I,J,20),I=1,2),J=1,4) & 
       /  5.9, 6., 7., 0., 5., 4.9, 5., 4.3/
  DATA ((CHI(I,J,20),I=1,2),J=1,4) & 
       /  6.111, 7.808, 11.868, 0., 51.207, 51.596, 67.181, 69.536 /

  DATA  (NK(J,20),J=1,4)   / 5, 3, 4, 5 /
  DATA ((GAM(K,J,20),K=1,5),J=1,4) & 
       /2.050,   3.349,   5.321,   4.873,   7.017, &
        1.769,   5.109,   9.524,   2*0., &
       27.271,  41.561,  29.172,  42.140, 0., &
        0.394,  28.930,  52.618,  38.593,  49.646 /
  DATA ((ALP(K,J,20),K=1,5),J=1,4) & 
       /18.2366,  27.5012, 149.2617,  94.5242, 705.4711, &
       11.8706,  14.0710, 106.0547,  2*0., &
       57.2414, 110.7567,  29.8121,  54.1874, 0., &
        2.0184,  97.5784, 282.3939, 209.1871, 252.8129 /


  !  Scandium.

  DATA  (ITBH(J,21),J=1,6) / 1, 1, 1, 1, 1, 0 /

  DATA  (NS(J,21),J=1,5)   / 3, 1, 1, 1, 1 /
  DATA  (G0(J,21),J=1,5)   / 4, 3, 4, 1, 4 /
  DATA ((GPR(I,J,21),I=1,3),J=1,5) & 
       /  30., 42., 18., 20., 2*0., 2., 2*0., 12., 2*0., 18., 2*0. /
  DATA ((GL(I,J,21),I=1,3),J=1,5) & 
       /  4.9, 4.9, 5., 5., 2*0., 6., 2*0., 4.6, 2*0., 3.8, 2*0.  /
  DATA ((CHI(I,J,21),I=1,3),J=1,5) & 
       /  6.538, 7.147, 8.042, 12.891, 2*0., 24.752, 2*0., 74.090, 2*0., 91.847, 2*0. /

  DATA  (NK(J,21),J=1,5)   / 10, 5, 4, 2, 3 /
  DATA ((GAM(K,J,21),K=1,10),J=1,5) & 
       / 0.021, 2.056,  3.551, 5.465, 1.535, 3.797, 6.203, 2.389, 4.858,  7.141, &
         0.011, 0.430,  1.156,   3.711, 8.863, 5*0., &
         0.025, 3.499, 10.463,  18.606, 6*0., &
        41.779,  57.217, 8*0., &
         0.539,  24.442, 51.079, 7*0. /
  DATA ((ALP(K,J,21),K=1,10),J=1,5) &   
       / 6.0014,  83.1958,  67.3666, 329.4354, &
         44.0793, 169.9969, 533.9195, &
         34.1642, 124.8475, 228.9879, &
         11.9979,  16.9280,  28.4778,  82.0418, 234.5360, 5*0., &
         6.0042,   2.7101,  13.9801,  65.3039, 6*0., &
         12.0000,  12.0000, 8*2., &
         2.0051,   2.9621,  29.0306, 7*0. /


  !  Titanium.

  DATA  (ITBH(J,22),J=1,6) / 1, 1, 1, 1, 1, 0 /

  DATA  (NS(J,22),J=1,5)   / 3, 2, 1, 1, 1 /
  DATA  (G0(J,22),J=1,5)   / 5, 4, 5, 4, 1 /
  DATA ((GPR(I,J,22),I=1,3),J=1,5) & 
       / 56,56,28, 42,10, 0, 20,0,0, 2,0,0, 12,0,0  /
  DATA ((GL(I,J,22),I=1,3),J=1,5) & 
       / 5.0,4.7,5.0, 5.0,5.0,0., 5.0,0.,0., 6.0,0.,0., 4.8,0.,0.  /
  DATA ((CHI(I,J,22),I=1,3),J=1,5) & 
       / 6.818,6.953,7.411, 13.635,14.685,0., &
       28.137,0.,0., 43.236,0.,0., 100.083,0.,0. /

  DATA  (NK(J,22),J=1,5)   / 13, 9, 5, 4, 2 /
  DATA ((GAM(K,J,22),K=1,13),J=1,5) & 
       /  0.021,  0.048,  1.029,  2.183,  4.109,  5.785,  0.846,  &
       1.792,  3.836,  5.787,  2.561,  4.869,  6.340,          &
       0.023,  0.124,  0.774,  1.810,  4.980,  9.585,  1.082,  &
       4.928, 11.279,  4*0.,                                   &
       0.041,  1.375,  4.768, 10.985, 19.769,  8*0.,           &
       0.048, 11.577, 24.531, 36.489, 9*0.,                    &
       54.436, 75.373, 11*0.                                    /
  DATA ((ALP(K,J,22),K=1,13),J=1,5) & 
       /   7.0887,  8.9186,  17.5633, 206.6832, 438.5735, 654.1721, &
       38.0462,  69.6271, 364.2845, 832.0408,           &
       98.8562,  57.9934, 442.1498,                     &
       19.7843,  32.0637,  37.0895, 110.6682, 288.4946, &
       521.8837,  10.0000,  34.0000, 120.0000, 4*0.,     &
       16.1691,  22.3550,  24.1646,  83.5128, 222.7963, 8*0., &
       6.0020,   4.6177,  25.2636,  52.1162, 9*0.,     &
       12.0000,   8.0000, 11*0.                           /


  !  Vanadium.

  DATA  (ITBH(J,23),J=1,6) / 1, 1, 1, 1, 1, 0 /

  DATA  (NS(J,23),J=1,5)   / 2, 3, 1, 1, 1 /
  DATA  (G0(J,23),J=1,5)   / 4, 1, 4, 5, 4 /
  DATA ((GPR(I,J,23),I=1,3),J=1,5) & 
       / 50,70,0, 56,72,64, 42,0,0, 20,0,0, 2,0,0  /
  DATA ((GL(I,J,23),I=1,3),J=1,5) & 
       / 5.0,5.0,0.0, 5.0,5.0,5.0, 5.0,0.,0., 5.0,0.,0., 11.0,0.,0.  /
  DATA ((CHI(I,J,23),I=1,3),J=1,5) & 
       / 6.738,7.101,0., 14.205,15.670,16.277, &
       29.748,0.,0., 48.464,0.,0., 65.198,0.,0. /

  DATA  (NK(J,23),J=1,5)   / 8, 13, 4, 4, 4 /
  DATA ((GAM(K,J,23),K=1,13),J=1,5) & 
       /   0.026,   0.145,   0.718,   2.586,   5.458, &
       2.171,   4.153,   6.097,   5* 0.,           &
       0.009,   0.366,   1.504,   5.294,  10.126, &
       1.796,   2.353,   6.068,  12.269, &
       2.560,   3.674,   6.593,  12.880, &
       0.045,   1.684,   8.162,  21.262, 9*0., &
       0.065,   1.746,  15.158,  33.141, 9*0., &
       0.077,  21.229,  44.134,  60.203, 9*0.  /
  DATA ((ALP(K,J,23),K=1,13),J=1,5) &    
       /  15.2627,  23.9869,  51.3053, 570.3384,1650.9417, &
       162.2829, 298.8303, 908.8852, 5* 0., &
       23.6736,  37.1624,  86.8011, 300.7440, 864.5880, &
       57.8961,  79.4605, 214.9007, 864.7425,  &
       61.8508,  64.0845, 192.8298, 718.2349,  &
       23.8116,  68.2495, 135.0613, 536.7632, 9*0.,  &
       15.9543,  22.5542,  71.4921, 248.9544, 9*0.,  &
       6.0006,   5.8785,  50.5077,  97.6129, 9*0.   /


  !  Chromium.

  DATA  (ITBH(J,24),J=1,6) / 1, 1, 1, 1, 1, 1 /

  DATA  (NS(J,24),J=1,6)   / 3, 2, 1, 1, 1, 1 /
  DATA  (G0(J,24),J=1,6)   / 7, 6, 1, 4, 5, 4 /
  DATA ((GPR(I,J,24),I=1,3),J=1,6) & 
       / 12,60,40, 50,18,0, 56,0,0, 42,0,0, 20,0,0, 2,0,0 /
  DATA ((GL(I,J,24),I=1,3),J=1,6) & 
       / 5.0,5.0,5.0, 5.0,5.0,0.0, 5.0,0.,0., 5.0,0.,0., 5.2,0.,0.,  4.9,0.,0. /
  DATA ((CHI(I,J,24),I=1,3),J=1,6) & 
       / 6.763,8.285,9.221, 16.493,18.662,0., 30.950,0.,0., &
       49.580,0.,0., 73.093,0.,0., 90.595,0.,0. / 

  DATA  (NK(J,24),J=1,6)   / 9, 8, 4, 4, 4, 3 /
  DATA ((GAM(K,J,24),K=1,9),J=1,6) & 
       /   0.993,   3.070,   5.673, &
       3.339,   4.801,   7.198, &
       2.829,   4.990,   7.643, &
       1.645,   3.727,   7.181,  12.299,       &
       2.902,   4.273,   8.569,  14.912,   0., &
       0.047,   2.566,   9.441,  21.198, 5*0., &
       0.078,   2.242,  15.638,  32.725, 5*0., &
       0.103,   2.146,  26.153,  49.381, 5*0., &
       0.119,  32.711,  58.117, 6*0.  /
  DATA ((ALP(K,J,24),K=1,9),J=1,6) &    
       /  30.1842,  79.2847, 149.5293, &
       215.3696, 119.1974, 741.4321, &
       184.9946,1352.5038, 784.4937, &
       46.6191, 160.1361, 488.0449, 657.1928,      &
       47.1742, 267.0275, 441.1324, 150.6650,   0., &
       24.3768, 122.8359, 285.5092, 794.1654, 5*0., &
       24.2296,  75.0258, 172.9452, 543.6511, 5*0., &
       15.9819,  17.6800,  95.2003, 225.0947, 5*0., &
       6.0008,   6.8734,  15.1253, 6* 0.  /


  !  Manganese

  DATA  (ITBH(J,25),J=1,6) / 1, 1, 1, 1, 1, 1 /

  DATA  (NS(J,25),J=1,6)   / 3, 2, 1, 1, 1, 1 /
  DATA  (G0(J,25),J=1,6)   / 6, 7, 6, 1, 4, 5 /
  DATA ((GPR(I,J,25),I=1,3),J=1,6) & 
       / 14,10,50, 12,72,0, 50,0,0, 56,0,0, 42,0,0, 20,0,0  /
  DATA ((GL(I,J,25),I=1,3),J=1,6) & 
       / 6.0,5.0,6.0, 7.0,5.0,0.0, 5.0,0.,0., 5.0,0.,0., 5.0,0.,0., 3.7,0.,0.  /
  DATA ((CHI(I,J,25),I=1,3),J=1,6) & 
       / 7.432,8.606,9.240, 15.636,18.963,0., 33.690,0.,0., &
       53.001,0.,0., 76.006,0.,0., 109.002,0.,0. /

  DATA  (NK(J,25),J=1,6)   / 8, 8, 4, 4, 4, 4 /
  DATA ((GAM(K,J,25),K=1,8),J=1,6) & 
       /   2.527,   4.204,   6.602, &
       4.155,   7.321,   2.285,   5.631,   8.448, &
       1.496,   3.839,   7.751,  13.484, &
       3.681,   6.054,   9.934,  14.936, &
       3.531,   6.967,  15.222,  25.069, 4* 0., &
       0.071,   2.896,  20.725,  37.383, 4* 0., &
       0.126,   2.660,  28.528,  53.413, 4* 0., &
       0.117,   0.350,   2.539,  40.301, 4* 0.   /
  DATA ((ALP(K,J,25),K=1,8),J=1,6) &    
       /  53.9107,  81.3931, 546.6945, &
       144.1893, 407.8029,  45.6177, 298.4423,2410.9335, &
       22.6382,  93.8419, 183.9367, 907.5765, &
       137.0409, 168.6783, 329.0287, 773.2513, &
       70.1925,  72.3372, 213.9512, 539.5165, 4* 0., &
       24.2373,  93.5415, 456.6167, 506.5484, 4* 0., &
       24.7687,  66.9896, 264.1853, 484.0161, 4* 0., &
       12.3142,   4.3050,  22.2424,  60.1198, 4* 0.   /


  !  Iron.

  DATA  (ITBH(J,26),J=1,6) / 1, 1, 1, 1, 0, 0 /

  DATA  (NS(J,26),J=1,4)   / 3, 2, 2, 1 /
  DATA  (G0(J,26),J=1,4)   / 9, 10, 9, 6 /
  DATA ((GPR(I,J,26),I=1,3),J=1,4) & 
       / 60.,56.,40., 50.,18.,0., 12.,72.,0., 50.,2*0.  /
  DATA ((GL(I,J,26),I=1,3),J=1,4) & 
       / 5., 5., 5., 5., 5., 0., 6., 5., 0., 3.6, 2*0.  /
  DATA ((CHI(I,J,26),I=1,3),J=1,4) & 
       / 7.896, 8.195, 8.927, 16.178, 18.662, 0., & 
       30.640, 34.607, 0., 56.001, 2*0.  /

  DATA  (NK(J,26),J=1,4)   / 11, 8, 8, 4 /
  DATA ((GAM(K,J,26),K=1,11),J=1,4) & 
       / 0.066, 0.339, 2.897, 6.585, 0.923, 1.679, & 
       4.620, 7.053, 4.249, 5.875, 7.781, & 
       0.062, 0.283, 1.504, 5.430, 11.210, 2.792, & 
       7.627, 13.623, 3*0., & 
       0.077, 3.723, 12.137, 23.700, 2.688, 7.595, & 
       15.444, 25.587, 3*0., & 
       3.982, 4.677, 6.453, 23.561, 7*0.  /
  DATA ((ALP(K,J,26),K=1,11),J=1,4) & 
       / 14.4102,2.7050,421.6612,940.1484,36.2187,22.8883, & 
       239.5997, 825.2919, 110.0242, 992.3040, 640.6715, & 
       17.0494,32.3783,34.3184,420.9626,1067.2064,154.0059, & 
       462.1117, 329.8618, 3*0., & 
       15.7906,47.1186,279.9292,692.1005,91.0206,206.3082, & 
       706.9927, 836.6689, 3*0., & 
       40.0790, 27.6965, 28.2243, 18.0001, 7*0.  /


  !  Cobalt.

  DATA  (ITBH(J,27),J=1,6) / 1, 1, 1, 1, 0, 0 /

  DATA  (NS(J,27),J=1,4)   /  4, 2,  1,  1 /
  DATA  (G0(J,27),J=1,4)   / 10, 9, 10, 20 /
  DATA ((GPR(I,J,27),I=1,4),J=1,4) & 
       /  42., 70., 42., 18., 56., 24., 2*0., 50., 3*0., 12., 3*0. /
  DATA ((GL(I,J,27),I=1,4),J=1,4) & 
       /  5., 5., 5., 5., 5., 5., 2*0., 5., 3*0., 3., 3*0. /
  DATA ((CHI(I,J,27),I=1,4),J=1,4) & 
       /  7.863, 8.378, 9.160, 9.519, 17.052, 18.958, 2*0., 33.491, 3*0., &
       53.001, 3*0. /

  DATA  (NK(J,27),J=1,4)   / 14, 9, 5, 0 /
  DATA ((GAM(K,J,27),K=1,14),J=1,4) & 
       / 0.112, 0.341, 0.809,3.808,   6.723, &
       2.057, 3.484, 7.210,  2.405,   5.133, 8.097, &
       2.084, 5.291, 8.426, &
       0.135, 0.517, 1.606,  6.772,  12.622, &
       2.512, 4.348, 8.253, 15.377,    5*0., &
       0.132, 0.863, 3.086, 11.789,  23.263, 9*0., &
       0., 13*0. /
  DATA ((ALP(K,J,27),K=1,14),J=1,4) & 
       / 11.9120,  20.4424,  28.3863, 132.5038, 600.7461, &
       33.3092, 237.4331, 977.2502, &
       55.5396, 318.8169, 619.6366, &
       32.6900,  83.8694, 107.4378, &
       11.2593,  38.2239,  22.9964, 261.3486, 637.1485, &
       23.0233,  41.6599, 264.6460, 181.6699,     5*0., &
       16.0356,   7.8633,  70.3158, 423.3512, 742.3553, 9*0., &
        0., 13*0. /


  !  Nickel.

  DATA  (ITBH(J,28),J=1,6) / 1, 1, 1, 1, 0, 0 /

  DATA  (NS(J,28),J=1,4)   /  2, 2,  1,  1 /
  DATA  (G0(J,28),J=1,4)   /  9, 6,  9, 28 /
  DATA ((GPR(I,J,28),I=1,2),J=1,4) & 
       /  20., 56., 42., 18., 56., 0., 50., 0. /
  DATA ((GL(I,J,28),I=1,2),J=1,4) & 
       /  5.4, 5., 9., 5., 5., 0., 3., 0./
  DATA ((CHI(I,J,28),I=1,2),J=1,4) & 
       /  7.633, 8.793, 18.147, 20.233, 35.161, 0., 56.025, 0. /

  DATA  (NK(J,28),J=1,4)   /  9, 8, 5, 0 /
  DATA ((GAM(K,J,28),K=1,9),J=1,4) & 
       / 0.026, 0.137,  0.315,  1.778,  4.029, 6.621, &
         2.249, 4.042,  7.621,  &
         0.191, 1.235,  3.358,  8.429, &
        17.096, 3.472,  9.065, 16.556, 0., &
         0.194, 1.305,  5.813, 14.172, 26.169, 4*0., &
         0., 8*0. /
  DATA ((ALP(K,J,28),K=1,9),J=1,4) &
       / 7.1268,  12.4486,  11.9953,  10.0546, 114.1658, &
       391.2064, 26.3908, 213.8081, 938.7927, &
         4.1421,  37.3781,  25.9712, 333.3397, & 
       311.1633,  33.1031, 184.1854, 136.7072, 0., &
        11.1915,   5.4174,  53.6793, 460.6781, 380.0056, 4*0., &
         0., 8*0. /


  !  Copper.

  DATA  (ITBH(J,29),J=1,6) / 1, 1, 1, 1, 0, 0 /

  DATA  (NS(J,29),J=1,4)   /  3, 2, 1,  1 /
  DATA  (G0(J,29),J=1,4)   /  2, 1, 6, 21 /
  DATA ((GPR(I,J,29),I=1,3),J=1,4) & 
       /  2., 30., 10., 20., 56., 0., 42., 2*0., 56., 2*0. /
  DATA ((GL(I,J,29),I=1,3),J=1,4) & 
       /  8., 6., 5., 7., 5., 0., 5., 2*0., 2.9, 2*0. /
  DATA ((CHI(I,J,29),I=1,3),J=1,4) & 
       /  7.724, 10.532, 10.980, 20.286, 27.985, 0., 36.826, 2*0., &
       61.975, 2*0. /

  DATA  (NK(J,29),J=1,4)   /  8, 7, 4, 0 /
  DATA ((GAM(K,J,29),K=1,8),J=1,4) & 
       /4.212,  7.227,  1.493,  5.859, 9.709,  7.081,  9.362, 10.130, &
        2.865,  8.260, 14.431, 18.292, 9.650, 14.640, 24.320, 0., &
        0.337,  8.520, 16.925, 28.342, 4*0., &
        0., 7*0. /
  DATA ((ALP(K,J,29),K=1,8),J=1,4) & 
       /11.0549, 238.9423,  10.3077, 126.2990, 1073.3876, &
        30.0000,  50.0000,  60.0000, &
        19.2984,  50.5974, 240.2021, 1216.9016, &
        48.3048, 583.2011, 320.4931, 0., &
         4.0155,  70.3264, 313.1213, 536.5331, 4*0., &
        0., 7*0. /


  !  Zinc.

  DATA  (ITBH(J,30),J=1,6) / 1, 1, 1, 1, 0, 0 /

  DATA  (NS(J,30),J=1,4)   /  3, 3, 1,  1 /
  DATA  (G0(J,30),J=1,4)   /  1, 2, 1, 10 /
  DATA ((GPR(I,J,30),I=1,3),J=1,4) & 
       /  4., 8., 12., 2., 30., 10., 20., 2*0., 42., 2*0. /
  DATA ((GL(I,J,30),I=1,3),J=1,4) & 
       /  8., 5., 5., 8., 5., 5., 5., 2*0., 2.8, 2*0. /
  DATA ((CHI(I,J,30),I=1,3),J=1,4) & 
       /  9.391, 17.503, 17.166, 17.959, 27.757, 28.310, 39.701, 2*0., &
       65.074, 2*0. /

  DATA  (NK(J,30),J=1,4)   / 6, 8, 3, 0 /
  DATA ((GAM(K,J,30),K=1,8),J=1,4) & 
       / 4.546,  8.840, 10.247, 16.620, 11.175, 16.321, 2*0., &
         6.113, 12.964, 16.444,  7.926, 13.633, 24.353, 16.286, 24.910, &
        10.291, 20.689, 32.077,  5*0., &
         0., 7*0. /
  DATA ((ALP(K,J,30),K=1,8),J=1,4) &
       /15.9880, 484.0042,  18.5863, 123.4134, 3.0000, 189.0000, 2*0., &
         6.1902,  38.9317, 204.8780,  10.2588, &
        89.3771, 370.3640,  30.0000, 128.0000, &
        24.6904, 106.7491, 439.5586, 5*0., &
         0., 7*0. /


  !  Gallium.

  DATA  (ITBH(J,31),J=1,6) / 1, 1, 1, 1, 0, 0 /

  DATA  (NS(J,31),J=1,4)   / 2, 1, 1, 1 /
  DATA  (G0(J,31),J=1,4)   / 2, 1, 2, 1 /
  DATA ((GPR(I,J,31),I=1,2),J=1,4) & 
       /  2., 18., 4., 0., 2., 0., 20., 0. /
  DATA ((GL(I,J,31),I=1,2),J=1,4) & 
       /  10., 5., 7., 0., 6., 0., 5., 0. /
  DATA ((CHI(I,J,31),I=1,2),J=1,4) & 
       /  5.997, 11.982, 20.509, 0., 30.702, 0., 64.157, 0. /

  DATA  (NK(J,31),J=1,4)   / 6, 3, 3, 3 /
  DATA ((GAM(K,J,31),K=1,6),J=1,4) & 
       / 0.102, 1.854, 4.531, 5.793, 5.126, 11.014, &
       6.102, 13.989, 18.438, 3*0., &
       8.239, 18.899, 25.142, 3*0., & 
       18.801, 28.656, 50.560, 3*0. /
  DATA ((ALP(K,J,31),K=1,6),J=1,4) &
       / 4.0002, 2.0966, 34.9646, 496.9385, 16.4913, 229.5019, &
       10.1998, 58.6050, 224.1949, 3*0., &
       6.0724, 23.0304, 50.8971, 3*0., &
       19.9887, 59.9442, 240.0671, 3*0. /

 
  !  Krypton.

  DATA  (ITBH(J,36),J=1,6) / 1, 1, 1, 0, 0, 0 /

  DATA  (NS(J,36),J=1,3)   / 2, 3, 1 /
  DATA  (G0(J,36),J=1,3)   / 1, 4, 5 /
  DATA ((GPR(I,J,36),I=1,3),J=1,3) & 
       /  8., 4., 0., 18., 10., 2., 8., 2*0. /
  DATA ((GL(I,J,36),I=1,3),J=1,3) & 
       /  10., 10., 0., 8., 7., 7., 5.5, 2*0. /
  DATA ((CHI(I,J,36),I=1,3),J=1,3) & 
       /  13.996, 14.661, 0., 24.565, 26.380, 28.665, 36.940, 2*0. /

  DATA  (NK(J,36),J=1,3)   / 4, 7, 4 /
  DATA ((GAM(K,J,36),K=1,7),J=1,3) & 
       / 11.409, 13.690, 12.064, 14.369, 3*0., &
       0.675, 16.518, 22.940, 18.636, 24.293, 20.502, 26.753, &
       0.617, 2.202, 19.856, 29.183, 3*0. /
  DATA ((ALP(K,J,36),K=1,7),J=1,3) &
       / 81.1676, 2054.8318, 39.4009, 968.5989, 3*0., &
       2.0215, 243.4886, 2038.4851, 139.2137, &
       590.7840, 19.7365, 126.2633, & 
       4.5228, 5.4786, 296.9358, 569.0524, 3*0. /


  !  Strontium.

  DATA  (ITBH(J,38),J=1,6)  / 1, 1, 1, 1, 0, 0 /

  DATA  (NS(J,38),J=1,4)    / 2, 1, 1, 1 /
  DATA  (G0(J,38),J=1,4)    / 1, 2, 1, 4 /
  DATA ((GPR(I,J,38),I=1,2),J=1,4) & 
       / 4.,20., 2.,0., 12.,0., 18.,0. /
  DATA ((GL(I,J,38),I=1,2),J=1,4) & 
       / 9.,6., 9.,0., 2.7,0., 6.,0. /
  DATA ((CHI(I,J,38),I=1,2),J=1,4) & 
       / 5.693,7.518, 11.027,0., 43.001,0., 57.017,0. /

  DATA  (NK(J,38),J=1,4)  / 5, 4, 0, 3 /
  DATA ((GAM(K,J,38),K=1,5),J=1,4) & 
       / 2.015,3.777,5.348,4.429,6.964, & 
       0.675,2.044,6.735,9.792,0.  , & 
       5*0., & 
       1.232,26.142,47.993,0.,0.    /
  DATA ((ALP(K,J,38),K=1,5),J=1,4) & 
       / 24.3150,47.4338,668.2505,89.4004,425.5987,  & 
       6.2782,13.7844,36.0151,313.9210,0., & 
       5*0., & 
       2.0349,129.4944,408.4637,2*0. /


  !  Yttrium.

  DATA  (ITBH(J,39),J=1,6)  / 1, 1, 1, 1, 0, 0 / 

  DATA  (NS(J,39),J=1,4)    / 1, 2, 1, 1 /
  DATA  (G0(J,39),J=1,4)    / 4, 1, 4, 1 /
  DATA ((GPR(I,J,39),I=1,2),J=1,4) & 
       / 6.,0., 20.,4., 2.,0., 12.,0. /
  DATA ((GL(I,J,39),I=1,2),J=1,4) & 
       / 6.,0., 6.,6., 4.9,0., 2.7,0. /
  DATA ((CHI(I,J,39),I=1,2),J=1,4) & 
       / 6.377,0., 12.233,13.158, 20.154,0., 76.105,0. /

  DATA  (NK(J,39),J=1,4)  / 4, 8, 4, 0 /
  DATA ((GAM(K,J,39),K=1,8),J=1,4) & 
       / 0.066,1.620,3.139,5.015, 4*0., & 
       0.133,0.447,1.509,4.379,9.715,2.999,7.394,10.873, & 
       0.090,0.936,5.454,11.911, 4*0., & 
       8*0. /
  DATA ((ALP(K,J,39),K=1,8),J=1,4) & 
       / 6.0310,102.0874,230.6864,349.1883, 4*0., & 
       13.3733,10.2739,43.588,88.0471,397.7009,9.2791, & 
       25.5740,70.1467, & 
       6.0039,2.0213,6.6309,25.3433, 4*0., & 
       8*0. /


  !  Zirconium.

  DATA  (ITBH(J,40),J=1,6)  / 1, 1, 1, 1, 0, 0 /

  DATA  (NS(J,40),J=1,4)    / 1, 2, 1, 1 /
  DATA  (G0(J,40),J=1,4)    / 5, 4, 5, 4 /
  DATA ((GPR(I,J,40),I=1,2),J=1,4) & 
       / 56.,0., 42.,18., 20.,0., 2.,0. /
  DATA ((GL(I,J,40),I=1,2),J=1,4) & 
       / 6.,0., 6.,6., 6.,0., 5.7,0. /
  DATA ((CHI(I,J,40),I=1,2),J=1,4) & 
       / 6.835,0., 14.028,15.398, 22.980,0., 34.330,0. /

  DATA  (NK(J,40),J=1,4)  / 5, 8, 5, 4 /
  DATA ((GAM(K,J,40),K=1,8),J=1,4) & 
       / 0.084,0.401,1.170,2.864,5.852,3*0., & 
       0.068,0.448,3.200,11.169,0.687,1.728,5.735,11.943, & 
       0.102,0.401,1.655,7.737,17.096,3*0., & 
       0.155,4.962,11.822,20.015,4*0. /
  DATA ((ALP(K,J,40),K=1,8),J=1,4) & 
       /  11.6004,24.5594,111.5330,570.8658,1185.4027,3*0., & 
       17.786,53.6729,148.5349,875.6878, & 
       26.8482,101.3551,155.2734,316.4934, & 
       11.7757,9.5110,34.7245,100.9139,477.0311,3*0., & 
       6.0011,2.3714,8.3077,25.3194,4*0. /

  !.


  !  Declare local variables
  IZ    = ZELEM
  IJ    = ZION + 1
  THETA = 5040. / TEMP

  !  Check attempt to treat atom too heavy or highly ionized
  IF ( IZ.GT.NZ .OR. IJ.GT.NJ ) THEN
     !     set partition function to unity
     U      = TAP__DNULL

     !  Check Chebyshev coefficients exist
  ELSEIF ( IZ.GT.MZ .OR. IJ.GT.MJ ) THEN
     !     set partition function to ground-state statistical weight
     U      = IG0(IJ,IZ)

  ELSEIF ( ITBH(IJ,IZ).NE.1 ) THEN
     !     set partition function to ground-state statistical weight
     U      = IG0(IJ,IZ)

     !  Check Chebyshev expansion is valid
  ELSEIF ( CHI(NS(IJ,IZ),IJ,IZ)*THETA.GT.800 ) THEN
     !     set partition function to ground-state statistical weight
     U      = IG0(IJ,IZ)

     !  Check whether electron pressue is valid
  ELSEIF ( PEL .LE. 0. ) THEN
     !     set partition function to ground-state statistical weight
     U      = IG0(IJ,IZ)

     !  Use Traving et al. approximation formulae
  ELSE
     PEX   = PEL * 10.
     U     = G0(IJ,IZ)
     DO I = 1,NS(IJ,IZ)
        QAS  = 0.0
        QJ   = IJ
        !!        ..h    = sqrt ( (4/3.pi.Pe/kT)^(-1/3) * ez^(4/3) / 6a_0 )
        H    = (TAPM__4PI/ (TAPP__K*1.E7) / 3.) * (PEX / TEMP)
        H    = H**(-1./3.) * QJ**(4./3.) / (6. * TAPP__A0*100. )
        H    = SQRT ( H )
        NH   = INT(H) + 1
        QL   = GL(I,IJ,IZ)
        LAS  = NINT(QL)
        IF (NH.GT.LAS) THEN
           EXPON = QJ*QJ * 31.326 * THETA
           DO M = LAS,NH
              MM  = M * M
              Q   = MM * EXP ( EXPON/MM )
              QAS = QAS + Q
           ENDDO
        ENDIF
        Q    = GPR(I,IJ,IZ) * QAS * 10.**(-CHI(I,IJ,IZ)*THETA)
        U    = U + Q
     ENDDO
     Q   = 0.
     DO K = 1,NK(IJ,IZ)
        Q   = Q + ALP(K,IJ,IZ) * 10.**(-GAM(K,IJ,IZ)*THETA)
     ENDDO
     U   = U + Q

     !  That's all the cases:
  ENDIF

  TAP_PARFN = U

END FUNCTION TAP_PARFN
