MODULE TAPP_PAR

!+
!  Name:
!     TAPP_PAR

!  Purpose:
!=    Theoretical Astrophysics Library public physical constants.

!  Language:
!     Starlink Fortran 77

!  Type of Module:
!     Global contants module.

!  Description:
!     This file defines the physical global constants used by the 
!     Theoretical Astrophysics Library. Most values were taken from the 
!     1986 CODATA Recommended Values of the Fundamental Physical 
!     Constants (Cohen, E.R., and Taylor, B.N., 1987, Journal of 
!     Research of the National Bureau of Standards, Vol. 92, 85).
!     Other sources were used where necessary and given in the comments
!     for the constants concerned.  All values are declared as DOUBLE
!     PRECISION constants.  Experimentally determined values are are
!     accurate to their specified standard deviations given in the
!     comment script.  Exact values are given to 34 decimal digits and
!     were evaluated using the MAPLE symbolic computation package
!     (Char, B.W., Geddes, K.O., Gonnet, G.H., Monagan, M.B., and Watt,
!     S.M., 1988, Watcom Publications, Waterloo, Ontario, Canada).

!  Authors:
!     PCTR: P.C.T. Rees (STARLINK)
!     CSJ : C.S. Jeffery (Armagh Observatory)
!     {enter_new_authors_here}

!  History:
!     24-JAN-1992 (PCTR):
!        Original version.
!     08-JAN-2001 (CSJ):
!        Conversion to f95 MODULE
!     27-FEB-2003 (CSJ):
!        Introduction of 1998 CODATA values
!     {enter_changes_here}

!  Bugs:
!     {note_any_bugs_here}

!-

!  Speed of light in a vacuum: c = 2.99792458E+08 m/s (exact).
      DOUBLE PRECISION, PARAMETER :: TAPP__C = 2.99792458D+08 

!  Permiability of a vacuum: mu(0) = 4.0E-07Pi N/A**2 (exact).
      DOUBLE PRECISION, PARAMETER :: TAPP__M0 = 1.256637061435917295385057353311801D-06 

!  Permittivity of a vacuum: epsilon(0) = 1/mu(0)c**2 F/m (exact).
      DOUBLE PRECISION, PARAMETER :: TAPP__E0 = 8.854187817620389850536563031710750D-12 

!  Newtonian constant of gravitation: G = 6.67259E-11 m**3/kg s**2,
!                                 error = 0.00085E-11.
      DOUBLE PRECISION, PARAMETER :: TAPP__G = 6.67259D-11 

!  Planck constant: h = 6.6260755E-34 J s,
!               error = 0.0000040E-34.
      DOUBLE PRECISION, PARAMETER :: TAPP__H = 6.6260755D-34 

!  Planck constant: h(bar) = h/2Pi = 1.05457266E-34 J s,
!                            error = 0.00000063E-34.
      DOUBLE PRECISION, PARAMETER :: TAPP__HB2PI = 1.05457266D-34 

!  Elementary charge: e = 1.60217733E-19 C,
!                 error = 0.00000049E-19.
      DOUBLE PRECISION, PARAMETER :: TAPP__E = 1.60217733D-19 

!  Magnetic flux quantum: Phi(0) = h/2e = 2.06783461E-15 Wb,
!                                 error = 0.00000061E-15.
      DOUBLE PRECISION, PARAMETER :: TAPP__P0 = 2.06783461D-15 

!  Electron mass: m(e) = 9.1093897E-31 kg,
!                error = 0.0000054E-31.
      DOUBLE PRECISION, PARAMETER :: TAPP__ME = 9.1093897D-31 

!  Proton mass: m(p) = 1.6726231E-27 kg,
!              error = 0.0000010E-27.
      DOUBLE PRECISION, PARAMETER :: TAPP__MP = 1.6726231D-27 

!  Proton-electron mass ratio: m(p)/m(e) = 1.836152701E+03 
!                                  error = 0.000000037E+03.
      DOUBLE PRECISION, PARAMETER :: TAPP__MPBME = 1.836152701D+03 

!  Neutron mass: m(n) = 1.6749286E-27 kg,
!               error = 0.0000010E-27.
      DOUBLE PRECISION, PARAMETER :: TAPP__MN = 1.6749286D-27 

!  Fine-structure constant: alpha = mu(0)ce**2/2h = 7.29735308E-03,
!                                           error = 0.00000033E-03.
      DOUBLE PRECISION, PARAMETER :: TAPP__A = 7.29735308D-03 

!  Rydberg constant: R(infty) = m(e)c alpha**2/2h = 1.0973731568549E+07 m**(-1),
!                                           error = 0.0000000000083E+07.
      DOUBLE PRECISION, PARAMETER :: TAPP__RYD = 1.0973731568549D+07 

!  Bohr radius: a(0) = alpha/((4pi R(infty)) = 0.529177249E-10 m
!                                      error = 0.000000024E-10.
      DOUBLE PRECISION, PARAMETER :: TAPP__A0 = 5.29177249D-11 

!  Avogadro constant: N(A) = L = 6.0221367E+23 mol**(-1),
!                        error = 0.0000036E+23.
      DOUBLE PRECISION, PARAMETER :: TAPP__NA = 6.0221367D+23 

!  Faraday constant: F = N(A)e = 9.6485309E+04 C/mol,
!                        error = 0.0000029E+04.
      DOUBLE PRECISION, PARAMETER :: TAPP__F = 9.6485309D+04 

!  Molar gas constant: R = 8.314510 J/mol K,
!                  error = 0.000070.
      DOUBLE PRECISION, PARAMETER :: TAPP__R = 8.314510D+00 

!  Boltzmann constant: k = R/N(A) = 1.380658E-23 J/K,
!                           error = 0.000012E-23.
      DOUBLE PRECISION, PARAMETER :: TAPP__K = 1.380658D-23 

!  Stefan-Boltzmann constant: sigma = (Pi**2/60) k**4/h**3 c**2
!                                   = 5.67051E-08 W/m**2 K**4,
!                             error = 0.00019E-08.
      DOUBLE PRECISION, PARAMETER :: TAPP__S = 5.67051D-08 

!  Electron volt: eV = e/C = 1.60217733E-19 J,
!                    error = 0.00000049E-19.
      DOUBLE PRECISION, PARAMETER :: TAPP__EV = 1.60217733D-19 

!  (Unified) atomic mass unit: u = m(u) = m(12C)/12 = 1.6605402E-27 kg,
!                                             error = 0.0000010E-27.
      DOUBLE PRECISION, PARAMETER :: TAPP__U = 1.6605402D-27 

!.

END MODULE
