      SUBROUTINE FLUX_PRED (W,PARS,F,DERIVS,NP)

!+
!
!  Programme:
!     flux_pred
!
!  Purpose:
!=    Returns the predicted flux at a given wavelength
!     and for a given combination of Teff, theta, Eb-v.
! 
!     This subroutine works on the basis that the wavelength
!     grid for observations and models is the same 
!  
!  Language:
!     Fortran 90
! 
!  Authors:
!     C.S.Jeffery (CSJ) (Armagh Observatory)
!
!  History:
!     27-May-1999: CSJ
!        original version
!
!-


!  Global variables:

      USE SFIT_MOD_TGH


!  Subroutine arguments:
      REAL               ::  W       ! wavelength (given)
      REAL, DIMENSION(*) ::  PARS    ! parameters (given)
      REAL               ::  F       ! flux       (returned)
      REAL, DIMENSION(*) ::  DERIVS  ! derivatives wrt PARS (returned)
      INTEGER            ::  NP      ! number of parameters in PARS (given)


!  Local variables:
      REAL     ::  TEFF         ! Effective temperature (K)
      REAL     ::  THETA        ! Angular diameter (radians)
      REAL     ::  EBV          ! E(B-V) extinction coefficient



      REAL     ::  FTTAB(NTMAX) ! Single wavelength flux holder
      REAL     ::  F0           ! Flux from atmosphere
      REAL     ::  DF           ! Flux for calculating derivatives
      REAL     ::  DFDT         ! Temperature derivative of flux
      REAL     ::  DRDE         ! Reddening derivative of extinction

      INTEGER  ::  NTP          ! degree of interpolating polynomial


!  Local constants:
      DOUBLE PRECISION, PARAMETER :: TAPM__PI = 3.141592653589793238462643383279503D+00

!.


!   Currently assume NG = NH = 1
      NG = 1
      NH = 1

!   Map local values of parameters
      TEFF = PARS(1); EBV = PARS(2); THETA=PARS(3) 

!   Identify NW from wavelength
      DO NW = 1,NWMOD
         NWX = NW
         IF (ABS((W-WMOD(NW))/W)<0.000001) EXIT
      END DO

!   Interpolate model fluxes at Teff
      DO NT = 1,NTMOD 
         FTTAB(NT) = LOG ( FMOD(NWX,NT,NG,NH) )
      ENDDO
      NTP = MIN ( 4, NTMOD-1 )  !! ensures polynomial ok
      CALL TABINT (NTMOD,TMOD,FTTAB,NTP,TEFF,     F0)
      CALL TABINT (NTMOD,TMOD,FTTAB,NTP,TEFF*1.01,DF)
      F0 = EXP(F0); DF = EXP(DF)
      DFDT  = (DF-F0)/(0.01*TEFF)

!   Calculate insterstellar reddening
      CALL ISRED (W,EBV,     RED, A, 3.1,0)
      CALL ISRED (W,EBV+0.01,REDX,A, 3.1,0)
      DRDE      = (REDX-RED)/0.01

         
!   Redden fluxes and multiply by angular radius squared
      F         = F0   * RED  * THETA * THETA
      DERIVS(1) = DFDT * RED  * THETA * THETA 
      DERIVS(2) = F0   * DRDE * THETA * THETA 
      DERIVS(3) = F0   * RED  * THETA * 2     


      END SUBROUTINE FLUX_PRED


