!!!! Name    : mod_model_list.f90
!!!! Author  : Chris Winter
!!!!
!!!! Description: 
!!!!    Array-based vector implementation for storing a list of
!!!!    models. The array is dynamically extended when its storage
!!!!    limit has been exceeded. The array-based vector allows
!!!!    much faster access than a linked list.
!!!!

module mod_model_list

  implicit none

  private
  public :: model, modlist, modslist, modlist_new, modlist_destroy, modlist_length, &
       modlist_nth, modlist_add, modlist_reverse, modlist_flatten, &
       modlist_load, modlist_save, modlist_print, modlist_rescale_hmod, &
       modlist_rescale_list, modlist_is_dupe, modlist_chisq, modlist_open_grid, &
       modlist_close_grid, modlist_read_model, modlist_scale_point, &
       modlist_unscale_point, test_gen_list

  !!! Type Definitions
  type model ! A model
     double precision, dimension(:), pointer :: data
     integer, dimension(:), pointer :: recnums
  end type model

  type modlist ! A vector of models
     type(model), dimension(:), pointer :: models, models_tmp
     integer :: length = 0 ! Number of used elements in models
     integer :: size   = 0 ! Total elements in models, used+unused
     double precision, dimension(6) :: scale_facs ! Parameter scaling factors
  end type modlist


  !!! SFIT2 Global variable modslist
  type(modlist), pointer :: modslist


  !!! Local Parameters
  integer :: new_list_size = 5000
  integer :: resize_chunk  = 5000

  character(len=128) :: grdfile  ! filename for model grid


  !!! Procedure Definitions
contains

  !!! Initialise a list
  subroutine modlist_new(l)
    type(modlist), pointer :: l

    allocate( l )
    allocate( l%models( new_list_size ) )
    l%size = new_list_size
    l%scale_facs = 0.0
  end subroutine modlist_new


  !!! Completely destroy a list
  subroutine modlist_destroy(l)
    type(modlist), pointer :: l

    if ( associated( l ) ) then
       if ( associated( l%models ) ) then
          !!! Do I need to traverse the array and deallocate
          !!! all the model array pointers too?
          deallocate( l%models )
       end if

       deallocate( l )
    end if

    call modlist_close_grid()

  end subroutine modlist_destroy


  !!! Open the specified grid file for reading
  subroutine modlist_open_grid(f)

    use sfit_mod_tgh

    character :: f*(*)

    grdfile = f

    open(20, file=grdfile, access="direct", recl=4*nwmod, &
         status="unknown", form="unformatted")

  end subroutine modlist_open_grid


  !!! Close the grid file after operations are complete
  subroutine modlist_close_grid()

    close(20)

  end subroutine modlist_close_grid


  integer function modlist_length(l)
    type(modlist), pointer :: l

    modlist_length = l%length
  end function modlist_length


  !!! Adds a new element to the end of the list
  subroutine modlist_add(l, vector, recs)
    type(modlist), pointer :: l
    double precision, dimension(:), pointer :: vector
    integer, dimension(:), pointer :: recs

    if ( .not. associated(l) ) then
       call modlist_new(l)
    end if

    l%length = l%length + 1

    if( l%length .gt. l%size ) then
       call modlist_extend(l)
    end if

    !!! Allocate space for the data and recnums arrays
    !!! Then copy across the values.
    allocate( l%models( l%length)%data(size(vector)) )
    allocate( l%models( l%length)%recnums(size(recs)) )

    l%models( l%length)%data    = vector
    l%models( l%length)%recnums = recs
  end subroutine modlist_add


  logical function modlist_is_dupe(l, vector)
   !!! Perform a linear search to test if a model is
   !!! already in the list. This algorithm is, at worst,
   !!! O(n), where n is the number of elements in the list.
   !!! In other words, in the worst case scenario that no
   !!! duplicate exists, or the duplicate resides at the last
   !!! list element, then the entire list must be traversed
   !!! before the function can return.
    type(modlist), pointer :: l
    double precision, dimension(:), pointer :: vector
    integer :: i

    modlist_is_dupe = .false.

    i = 1
    do while ( (modlist_is_dupe .eqv. .false.) .and. (i .le. l%length) )
       if ( ( l%models(i)%data(1) .eq. vector(1) ) .and. &
            ( l%models(i)%data(2) .eq. vector(2) ) .and. &
            ( l%models(i)%data(3) .eq. vector(3) ) ) then
          modlist_is_dupe = .true.
       end if

       i = i + 1
    end do

  end function modlist_is_dupe


  !!! Extend the vector when it's full
  subroutine modlist_extend(l)
    type(modlist), pointer :: l

    !!! Setup the temp array
    allocate( l%models_tmp( l%size + resize_chunk ) )
    l%size = l%size + resize_chunk

    !!! Copy the contents of models.
    !!! Length will always include the new element to
    !!! be added, so this is why we subtract 1
    l%models_tmp(1:(l%length - 1)) = l%models

    !!! Resize the primary array, then switch pointers
    deallocate(l%models)

    l%models => l%models_tmp

    nullify(l%models_tmp)
  end subroutine modlist_extend


  !!! Returns element, the nth model in list l
  !!! nth = 0 refers to the first element in the list.
  subroutine modlist_nth(l, nth, element)
    type(modlist), pointer :: l
    integer :: nth
    type(model) :: element

    if( associated(l) ) then
       element = l%models(nth + 1)
    end if
  end subroutine modlist_nth


  !!! Reverse the elements in the vector
  subroutine modlist_reverse(l)
    type(modlist), pointer :: l
    integer :: i, j

    if( associated(l) ) then
       !!! Set up the temp list, then copy backwards
       allocate( l%models_tmp( l%size ) )

       i = l%length
       j = 1
       do while (i .gt. 0)
          l%models_tmp(j) = l%models(i)

          i = i - 1
          j = j + 1
       end do

       deallocate(l%models)

       l%models => l%models_tmp

       nullify(l%models_tmp)
    end if
  end subroutine modlist_reverse


  !!! Fast nearest neighbour searching
  subroutine modlist_chisq(l, npars, pars, chisq, results)

    use sfit_bindata
    use sfit_obsdata
    use sfit_mod_tgh
    use xfit_dims

    type(modlist), pointer :: l
    integer :: npars
    real, dimension(npars) :: pars
    real :: chisq
    double precision, dimension (3) :: results

    real, dimension(l%length) :: chisq_surface
    integer :: i, minimum

    real, dimension(nwmod) :: modl_tmp, modl
    real :: beta = 1.5    ! limb darkening - assumed

    real :: redshift
    real :: wred(nwmax)

    double precision, parameter :: TAPP__C = 2.99792458D+08

    real :: teff1, logg1, anhe1, vsin1, q1, deltv1


    teff1  = pars(1) 
    logg1  = pars(2)
    anhe1  = pars(3) 
    vsin1  = pars(4)
    q1     = pars(5)
    deltv1 = pars(6)

    
    chisq_surface = 0.0
    

    i = 0
    minimum = i + 1
    do while (i .lt. l%length)
       ! 1. Read Model
       ! 2. Process Model
       ! 3. Calculate chisq
       ! 4. Update minimum info

       call modlist_read_model(l, i, 1, modl_tmp(1:nwmod))

       ! 1  Resample model onto wavelength grid of observations
       call resample(wmod, modl_tmp, nwmod, wobs, modl(1:nwobs), nwobs, 1.0)
       modl_tmp = 0.0

       ! 2  Convolution of mode with VSINI
       call vsini_conv (wobs, modl, nwobs, vsin1, beta)

       ! 3  Apply redshift 
       redshift =  deltv1 * 1.0E+03 /  TAPP__C 
       wred(1:nwobs) = wobs(1:nwobs) * ( 1. + redshift )
       if( abs( wobs(nwobs) * redshift ) > SINST/20.) then 
          call resample( wred(1:nwobs), modl, nwobs, wobs(1:nwobs), modl_tmp(1:nwobs), nwobs, 1.0)
          modl = modl_tmp
          modl_tmp = 0.0
       end if

       ! 4  Convolution of each spectrum with systemic change in velocity shift
       if( DVSYS .GT. SINST ) then
          call boxcar (wobs, modl, modl_tmp, nwobs, DVSYS)
          modl = modl_tmp
          modl_tmp = 0.0
       end if

       ! 5  Convolution of each spectrum with instrumental profile.
       call qsmooth (wobs, modl, modl_tmp, nwobs, SINST)

       ! 6  Calculate chisq
       chisq_surface(i + 1) = ml_chisq(modl_tmp, FOBS, SOBS, nwobs)

       call modlist_unscale_point(l, l%models(i+1)%data, results)

       write (*, '(2X, 3F7.3, 1X, F12.3)') results, chisq_surface(i + 1)

       ! Update minimum
       if( chisq_surface(i + 1) .lt. chisq_surface(minimum) ) then
          minimum = i + 1
       end if

       i = i + 1
    end do

    call modlist_unscale_point(l, l%models(minimum)%data, results)

    print *, "Best Fit"
    print *, results
    print *, "CHISQ: ", chisq_surface(minimum), chisq_surface(minimum)/(nwobs-npars)

  end subroutine modlist_chisq


  real function ml_chisq (X,Y,S, N)
    !     Calculate chi-squared CHISQ on data Y(X), each datum with
    !     error S(X), with respect to a fitting function FUNC(X,P), 
    !     where P are NP independent parameters. The covariance matrix
    !     COVAR(NC,NC) is also computed for all NP independent parameters
    !
    !     On input, COVAR(1,1) carries a value that is passed to FUNC
    !     as the first element in the derivative vectory DYDA. This allows
    !     the calling routine to determine, for example, whether derivative
    !     need to be calculated. Its a rather messy trick, but was introduced
    !     to avoid much more substantial software engineering.

    !  Arguments given:
    !     X	(real)           array of x-values
    !     Y (real)           array of y-values
    !     S (real)           array of standard deviations on y-values
    !     N (integer)        number of data in X,Y,S
    !     P (real)           values of parameters to pass to function FUNC
    !     NP (integer)       number of parameters P

    !  Subroutine arguments (given):

    integer :: N
    real :: X(N)
    real :: Y(N)
    real :: S(N)

    !   Initialize chi-squared and covariance matrix
    ml_chisq = 0.0

    !  Sum chi-squared and covariance matrix over all data

    ml_chisq = sum( ( (X - Y) / S ) ** 2 )

  end function ml_chisq


  !!! Read in a model from the binary file
  subroutine modlist_read_model(l, nth, ftype, model_arr)

    use sfit_mod_tgh

    type(modlist), pointer :: l
    integer :: nth
    integer :: ftype
    real, dimension(nwmod) :: model_arr

    !integer :: stat
    type(model) :: mod_nth

    !!! Read in the model
    call modlist_nth(l, nth, mod_nth)
    read(20, rec=mod_nth%recnums(ftype)) model_arr(1:nwmod)

  end subroutine modlist_read_model



  !!! Make a copy of list L1, returning the copy in L2.
  subroutine modlist_copy(l1, l2)
    type(modlist), pointer :: l1, l2
    integer :: i

    do i = 0,  (modlist_length(l1) - 1)
       call modlist_add(l2, l1%models(i)%data, l1%models(i)%recnums)
    end do

    l2%scale_facs = l1%scale_facs

  end subroutine modlist_copy


  !!! Convert the vector for passing to tetgen
  subroutine modlist_flatten(l1, vec)
    use iso_c_binding
    type(modlist), pointer :: l1
    real(kind=c_double), allocatable, target :: vec(:)
    ! double precision :: vec(*)
    integer :: i, j, k

    !pointer(ptr, vec)
    !ptr = malloc(modlist_length(l1) * 3 * 8) !!! 3 reals per list element
                                             !!! 8 bytes per double precision

    print *, "modlist_flatten"

    allocate(vec(modlist_length(l1) * 3))

    k = 1
    do i = 1, modlist_length(l1)
       do j = 1, 3
          vec(k) = l1%models(i)%data(j)
          k = k + 1
       end do
    end do

  end subroutine modlist_flatten


  !!! Loads a list of models from a file already opened for reading
  subroutine modlist_load(unit, l)
    type(modlist), pointer :: l
    integer :: unit, length, i
    double precision, dimension(:), pointer :: data
    integer, dimension (:), pointer :: recs

    allocate(data(3), recs(3))

    call modlist_new(l)

    read (unit) length
    read (unit) l%scale_facs

    do i = 1, length
       read (unit) data
       read (unit) recs

       call modlist_add(l, data, recs)
    end do

    if ( modlist_length(l) .ne. length ) then
       print *, "MODLIST_LOAD: Reconstructed list has erroneous length!"
       stop
    end if
    print *,"MODLIST_LOAD: successful - length=",length
  end subroutine modlist_load

  !!! Saves a list of models to a file already opened for writing
  subroutine modlist_save(l, unit)
    type(modlist), pointer :: l
    integer :: unit, i

    write (unit) modlist_length(l)
    write (unit) l%scale_facs

    do i = 1, modlist_length(l)
       write (unit) l%models(i)%data
       write (unit) l%models(i)%recnums
    end do
  end subroutine modlist_save


  !!! Rescale the model parameters using the supplied
  !!! scaling factors, sf
  subroutine modlist_rescale_list(l, sf)
    type(modlist), pointer :: l
    double precision, dimension(6) :: sf
    !real :: t0, dt, g0, dg, h0, dh
    integer :: i

    do i = 1, modlist_length(l)
       l%models(i)%data(1) = (( l%models(i)%data(1) - sf(1) ) / (sf(2) - sf(1)) )
       l%models(i)%data(2) = (( l%models(i)%data(2) - sf(3) ) / (sf(4) - sf(3)) )
       l%models(i)%data(3) = (( l%models(i)%data(3) - sf(5) ) / (sf(6) - sf(5)) )
    end do

    l%scale_facs = sf
  end subroutine modlist_rescale_list

  subroutine modlist_scale_point(l, p, sp)
    type(modlist), pointer :: l
    double precision, dimension(3) :: p, sp

    sp(1) = ( p(1) - l%scale_facs(1) ) / ( l%scale_facs(2) - l%scale_facs(1) )
    sp(2) = ( p(2) - l%scale_facs(3) ) / ( l%scale_facs(4) - l%scale_facs(3) )
    sp(3) = ( p(3) - l%scale_facs(5) ) / ( l%scale_facs(6) - l%scale_facs(5) )
  end subroutine modlist_scale_point

  subroutine modlist_unscale_point(l, sp, p)
    type(modlist), pointer :: l
    double precision, dimension(3) :: sp, p

    p(1) = sp(1) * ( l%scale_facs(2) - l%scale_facs(1) ) + l%scale_facs(1)
    p(2) = sp(2) * ( l%scale_facs(4) - l%scale_facs(3) ) + l%scale_facs(3)
    p(3) = sp(3) * ( l%scale_facs(6) - l%scale_facs(5) ) + l%scale_facs(5)
  end subroutine modlist_unscale_point
    

  subroutine modlist_rescale_hmod(l, v0, dv)
    type(modlist), pointer :: l
    real :: v0, dv
    integer :: i

    do i = 1, modlist_length(l)
       l%models(i)%data(3) = (( l%models(i)%data(3) - v0 ) / dv)
    end do
  end subroutine modlist_rescale_hmod



  !!! *************************************** !!!
  !!! ********** Utility Routines *********** !!!
  !!! *************************************** !!!


  !!! Traverse a list and print out each data slot.
  subroutine modlist_print(l)
    type(modlist), pointer :: l
    integer :: i

    if ( .not. associated(l) ) then
       print *, "*NULL*"
    else
       do i = 1, modlist_length(l)
          print *, "Data: ", l%models(i)%data, " Recnums: ", l%models(i)%recnums
       end do
    endif
  end subroutine modlist_print


  subroutine test_gen_list(nsamples, ndims, l)

    integer :: nsamples, ndims, i
    double precision, dimension(:), pointer :: sample, tmp
    integer, dimension (:), pointer :: recs
    type(modlist), pointer :: l

    allocate(sample(ndims))
    allocate(tmp(ndims))
    allocate(recs(3))

    call random_seed()

    do i = 1, nsamples
       call random_number(sample)
       call random_number(tmp)
       recs = (/99, 99, 99/)

       sample = (100 * sample) / tmp
       recs = recs * 10

       call modlist_add(l, sample, recs)
    enddo

  end subroutine test_gen_list

end module mod_model_list
