      SUBROUTINE resample (x, y, nx, xx, yy, nxx, mode)
     
!+
!
!  Program:
!     RESAMPLE
!
!  Purpose:
!=    Resample a function y(x) onto  a new set of x-values xx.
!     The new function is returned as yy. nx and nxx are the
!     number of values in arrays x and xx respectively.
!
!  Method: 
!     If the resampling interval is smaller than the data interval, 
!     the result is linearly interpolated.
!     If the resampling interval is larger than the data interval,
!     the result is the average flux in corresponding data bins,
!     with the boundary data bins appropriately weighted
!     If the resampled function specifies points outside the given data
!     the missing points are set = "mode" EXCEPT if "mode == 0" in 
!     which case the missing data is set = to the given data at the
!     appropriate upper or lower boundary
!    
!  Accuracy:
!     The program was tested by rebinning a theroretical
!     spectrum sampled very unevenly (dx = 0.001A in line cores,
!     increasing to ~1 A in continuum regions) onto a
!     slightly uneven observed grid with <dx>~0.225 A.
!     Integrating the equivalent width over several lines
!     provided accuracy of 0.01-0.1 per cent, with the error
!     increasing with the number of lines.  (CSJ: 2000-FEB-09)
!  
!  Language:
!     Fortran 90
!
!  Author:
!     CSJ: Simon Jeffery, Armagh Observatory
!     (adapted from a similar routine written for IDL)
!
!  History
!     20-02-2000: CSJ 
!        Original IDL code
!     06-01-2005: CSJ 
!        Fixed bug in end-value corrections
!     17-04-2106: CSJ
!        Added extra comments in the preamble
!
!-
      

!  Subroutine arguments (given).
      REAL, DIMENSION(*) :: x     ! input x values 
      REAL, DIMENSION(*) :: y     ! input y values
      INTEGER            :: nx    ! number of x values

      REAL, DIMENSION(*) :: xx    ! output xx values
      INTEGER            :: nxx   ! number of xx values

      REAL               :: mode  ! default value outside valid range

!  Subroutine arguments (returned).
      REAL, DIMENSION(*) :: yy    ! output yy values

!  Local variables.
      REAL               :: b1,b2
      INTEGER            :: c1,c2

!.
      
        c1 = 1
        DO ix = 1,nxx  ! loop over new grid

          IF (xx(ix) >= x(1) .AND. xx(ix) <= x(nx)) THEN 
      
            IF (ix.EQ.1) THEN
              dx = (xx(2)-xx(1))/2.  
              b1 = xx(1)-dx    ;  b2 = b1+2*dx
            ELSEIF (ix.EQ.nxx) THEN
              dx = (xx(nxx)-xx(nxx-1))/2.
              b1 = xx(nxx)-dx  ;  b2 = b1+2*dx
            ELSE
              b1 = (xx(ix-1)+xx(ix))/2.  ;  b2 = (xx(ix)+xx(ix+1))/2.
            ENDIF
            c2 = 1

            DO jx = max(2,c1),nx-1        ! locate data
              xg = x(jx) 
              dg = (xg-x(jx-1))/2.
              IF (xg < b1-dg) c1 = jx+1
              dg = (x(jx+1)-xg)/2.
              IF (xg < b2+dg) c2 = jx
              IF (xg > b2+dg) EXIT
            END DO

      
            IF (c2 > c1) THEN                      ! bin up 
        
              s = y(c1)*((x(c1)+x(c1+1))/2. - b1)
              IF (c2 > c1+1) THEN 
                DO jx = c1+1,c2-1
                  s = s + y(jx)*(x(jx+1)-x(jx-1))/2.
                END DO
              ENDIF
              s = s + y(c2)*(b2-(x(c2-1)+x(c2))/2.)
              s = s / (b2-b1)    
       
            ELSEIF  ( x(c1+1) /= x(c1) ) THEN         ! interpolate

              c1 = min( c1, nx-1 )
              s = y(c1) + (y(c1+1)-y(c1))*(xx(ix)-x(c1))/(x(c1+1)-x(c1))

            ELSE                                   ! average

              s = (y(c1)+y(c1+1)) / 2.

            ENDIF

      
          ELSE 

            s = mode
            IF ( abs(mode)<1.e-6 ) THEN
              IF (xx(ix) < x(1) ) s = y(1) 
              IF (xx(ix) > x(nx)) s = y(nx) 
            ENDIF

          ENDIF

          yy(ix) = s

        END DO
      
      END SUBROUTINE
