SUBROUTINE SFIT_RDGRID  ( MODE, &
     NWMAX,NTMAX,NGMAX,NHMAX, &
     NWMOD,NTMOD,NGMOD,NHMOD, &
     WMOD, TMOD, GMOD, HMOD, &
     FMOD, CMOD )

  !+
  !  Program:
  !     sfit_rdgrid
  !
  !  Purpose:
  !=    Given a list of files on stdin (unit *), read a 3 dimensional grid of model spectra.
  !
  !     Different types of model grid may be specified:
  !     MODE = 0:   Spectrum.000  (including fluxes, continuum and residual)
  !     MODE = 1:   Spectrum.001  (residual or normalized flux only)
  !     MODE = 2:   Sterne.s      (total flux only)
  !     MODE = 3:   Spectrum.sp2  (total flux, parameters not in header)
  !     MODE = 4:   Synthe        (including fluxes, continuum and residual)
  !     MODE = 5:   Synthe        (residual or normalized flux only)
  !
  !     Format of data on stdin must be as follows
  !     NTMOD NGMOD NHMOD
  !     WMIN  WMAX  DW      [ default 0 0 0 ]
  !     filename[1]
  !     filename[2]
  !     ...
  !     filename[NT*NG*NH]
  !
  !  Language:
  !     Fortran 90
  ! 
  !  Reference:
  !
  !  Authors:
  !     C.S.Jeffery (CSJ) (Armagh Observatory)
  !
  !  History:
  !     05-Sep-2000: CSJ:        adapted from SFIT_TGV
  !                  CSJ:        various extensions to read different formats
  !     06-Mar-2003: CSJ: V2.0:  changed to allocatable internal array dimensions
  !     22-Feb-2005: CSJ: V3.0:  changed to eliminate "AMOD" arrays
  !     07-Dec-2005: CSJ: V3.0a: fixed some typos 
  !
  !-

  !  Implicit declaration
  IMPLICIT NONE

  !  Pi.
  !  TAPM__PI = PI
  DOUBLE PRECISION :: TAPM__PI
  PARAMETER ( TAPM__PI = 3.141592653589793238462643383279503D+00 )


  !  Global Variables.
  !     This subroutine should contain no global variables. 


  ! Subroutine Arguments.

  INTEGER,INTENT(IN)   :: MODE              ! The operation of this subroutine
  ! depends on the type of grid to be read

  INTEGER,INTENT(IN)   :: NWMAX             ! maximum number of wavelengths
  INTEGER,INTENT(IN)   :: NTMAX             ! maximum number of temperatures
  INTEGER,INTENT(IN)   :: NGMAX             ! maximum number of gravities
  INTEGER,INTENT(IN)   :: NHMAX             ! maximum number of helium abundances 

  INTEGER,INTENT(OUT)  :: NWMOD             ! number of wavelengths
  INTEGER,INTENT(OUT)  :: NTMOD             ! number of temperatures 
  INTEGER,INTENT(OUT)  :: NGMOD             ! number of gravities
  INTEGER,INTENT(OUT)  :: NHMOD             ! number of helium abundances

  REAL,INTENT(OUT)     :: TMOD(NTMAX)       ! temperatures in grid
  REAL,INTENT(OUT)     :: GMOD(NGMAX)       ! gravities in grid
  REAL,INTENT(OUT)     :: HMOD(NHMAX)       ! helium abundances in grid
  REAL,INTENT(OUT)     :: WMOD(NWMAX)       ! wavelengths in model

  REAL,INTENT(OUT)     :: FMOD(NWMAX,NTMAX,NGMAX,NHMAX)  ! emergent fluxes
  REAL,INTENT(OUT)     :: CMOD(NWMAX,NTMAX,NGMAX,NHMAX)  ! continuum fluxes
!  REAL,INTENT(OUT)     :: AMOD(NWMAX,NTMAX,NGMAX,NHMAX)  ! normalized fluxes

  ! Local variables
  CHARACTER(LEN=128) :: MODFILE   ! filename for model spectra
  INTEGER           :: LS, IB    ! length of string, pos of blank
  CHARACTER(LEN=72) :: MODHEAD1  ! header from sp2 file
  CHARACTER(LEN=72) :: MODHEAD2  ! header from sp2 file

  INTEGER           :: BADVAL    ! Status flag for i/o
  INTEGER           :: GRID_ORDER ! Identify grid - read order

  REAL      :: TAU               ! optical depth, not saved

  ! Temporary spectrum arrays
  INTEGER                       :: NWLOC  ! size of data arrays
  REAL,ALLOCATABLE,DIMENSION(:) ::  WLOC  ! wavelengths
  REAL,ALLOCATABLE,DIMENSION(:) ::  FLOC  ! emergent fluxes
  REAL,ALLOCATABLE,DIMENSION(:) ::  CLOC  ! continuum fluxes
  REAL,ALLOCATABLE,DIMENSION(:) ::  ALOC  ! normalized fluxes

  !  Model grid wavelength definition
  REAL                  :: WSTT  ! Start wavelength of stored model grid
  REAL                  :: WEND  ! End   wavelength of stored model grid
  REAL                  :: WDEL  ! Wavelength increment of stored model grid

  !  Loop counters
  INTEGER   :: NW                ! loop counter for wavelength
  INTEGER   :: NT                ! loop counter for temperature
  INTEGER   :: NG                ! loop counter for gravity
  INTEGER   :: NH                ! loop counter for helium abundance

  INTEGER   :: LF                ! length of filename

  ! Local constants
  CHARACTER :: MODEL_FMT(0:5)*80
  DATA MODEL_FMT(0) /'Spectrum.000  (including fluxes, continuum and residual) '/
  DATA MODEL_FMT(1) /'Spectrum.001  (residual or normalized flux only) ' /
  DATA MODEL_FMT(2) /'Sterne.s      (total flux, output from Sterne) ' /
  DATA MODEL_FMT(3) /'sp2           (spectrum format 2: header useless) ' /
  DATA MODEL_FMT(4) /'Synthe        (including fluxes, continuum and residual) '/
  DATA MODEL_FMT(5) /'Synthe        (residual or normalized flux only) '/


  !.

    WRITE (*,*) 'SFIT_RDGRID: Version 3.0a: 07-Dec-2005'

  ! Initialize the model grid
  TMOD = 0.
  GMOD = 0.
  HMOD = 0.
  WMOD = 0.
  CMOD = 0.
  FMOD = 0.
  NWMOD = 0


  ! Define the model grid

  WRITE (*,'(a,i2,2x,a)') ' sfit_rdgrid: mode = ',MODE,MODEL_FMT(MODE)
  WRITE (*,*) 'Number of temperatures, gravities, helium abundances'
  READ  (*,*) NTMOD, NGMOD, NHMOD
  WRITE (*,*) NTMOD, NGMOD, NHMOD

  WRITE (*,*) 'Enter grid order : tgh = 123, htg = 312 : last varies fastest'
  WRITE (*,*) ' ** only implemented for mode 0 ** '
  READ  (*,*) GRID_ORDER
  IF ( GRID_ORDER == 123 ) WRITE (*,*) 'GRID_ORDER - TtttGgggHehhh', GRID_ORDER
  IF ( GRID_ORDER == 312 ) WRITE (*,*) 'GRID_ORDER - HehhhTtttGggg', GRID_ORDER
  IF ( GRID_ORDER /= 123 .AND. GRID_ORDER /= 312) THEN
     WRITE (*,*) 'GRID_ORDER - ** Illegal ** - ',GRID_ORDER
     STOP
  ENDIF 

  WRITE (*,*) 'Start and end wavelengths and wavelength increment'
  WRITE (*,*) '0 0 0 to define wavelength grid from first model'
  READ  (*,*) WSTT, WEND, WDEL
  WRITE (*,*) WSTT, WEND, WDEL
  WRITE (*,*)

  !  Setup model wavelength grid
  IF ( WSTT > 0 .AND. WEND > 0 .AND. WDEL > 0 ) THEN 
    NWMOD = 1 + (WEND - WSTT) / WDEL
    DO NW = 1,MIN(NWMOD,NWMAX)
      WMOD(NW) = WSTT + (NW-1) * WDEL
    ENDDO

    ! Otherwise use WMOD as provided by first model
  ELSE
    WRITE (*,*) 'SFIT_RDGID: *Warning* : using wavelength grid from first model'
  ENDIF

  ! Check grid within limits
  IF (NTMOD>NTMAX .OR. NGMOD>NGMAX .OR. NHMOD>NHMAX .OR. NWMOD>NWMAX) THEN
    WRITE (*,'(A,I6,3I3)') &
         ' model grid exceeded : maximum dimensions:',NWMAX,NTMAX,NGMAX,NHMAX
    WRITE (*,'(A,I6,3I3)') &
         '                     :   given dimensions:',NWMOD,NTMOD,NGMOD,NHMOD
    STOP
  ENDIF


  ! MODE = 0:   Spectrum.000  (including fluxes, continuum and residual)
  IF ( MODE.EQ.0 ) THEN

    IF ( GRID_ORDER == 123 ) THEN 
    MODE0_123_INPUT: DO NT = 1,NTMOD
      DO NG = 1,NGMOD
        DO NH = 1,NHMOD

          CALL STR_NULL ( MODFILE )
          !           WRITE (*,*) 'File containing model spectrum?'
          READ  (*,'(A)') MODFILE
          MODFILE = MODFILE(1:INDEX(MODFILE,' ')-1)
          !            MODFILE = MODFILE(1:INDEX(MODFILE,' ')-1)
          !            WRITE(*,*) MODFILE

          OPEN  (1,FILE=MODFILE,STATUS='OLD')

          ! Read the header lines
          READ  (1,'(7X,F6.0,10x,F5.3,13X,F5.3)') &
               TMOD(NT), GMOD(NG), HMOD(NH)

          ! Read the dimension of the spectrum
          READ  (1,*) NWLOC

          ! Allocate the array sizes
          ALLOCATE ( WLOC(NWLOC),ALOC(NWLOC),FLOC(NWLOC),CLOC(NWLOC) )

          ! Read the data
          READ  (1,*) 
          READ  (1,*) 
          READ  (1,*)
          READ  (1,*) ( WLOC(NW),ALOC(NW),FLOC(NW),CLOC(NW), TAU, NW = 1,NWLOC )
          CLOSE (1)

          ! Use initial model to define wavelength grid
          IF ( NWMOD == 0 ) THEN
             NWMOD = NWLOC
             WMOD(1:NWMOD) = WLOC(1:NWMOD)
          ENDIF

          ! Resample arrays onto prespecified grid
          CALL RESAMPLE (WLOC,FLOC(1:NWLOC),NWLOC, &
               WMOD,FMOD(1:NWMOD,NT,NG,NH),NWMOD, 0.)
          CALL RESAMPLE (WLOC,CLOC(1:NWLOC),NWLOC, &
               WMOD,CMOD(1:NWMOD,NT,NG,NH),NWMOD, 0.)

          ! Arrays are resampled, free up for next use
          DEALLOCATE ( WLOC, ALOC, FLOC, CLOC )

          ! Report the spectrum 
          WRITE (*,'(A,''  Teff='',F6.0,'' log g='',F5.3, '' nHe='',F6.3,'' nw='',I6)') &
               TRIM(MODFILE), TMOD(NT), GMOD(NG), HMOD(NH), NWLOC
          WRITE (11,'(A,''  Teff='',F7.0,'' log g='',F4.1,'' nHe='',F6.3,'' nw='',I6)') &
               TRIM(MODFILE), TMOD(NT), GMOD(NG), HMOD(NH), NWLOC

        END DO
      END DO
    END DO MODE0_123_INPUT

    ELSEIF ( GRID_ORDER == 312 ) THEN 
    MODE0_312_INPUT: DO NH = 1,NHMOD
      DO NT = 1,NTMOD
        DO NG = 1,NGMOD

          CALL STR_NULL ( MODFILE )
          !           WRITE (*,*) 'File containing model spectrum?'
          READ  (*,'(A)') MODFILE
          MODFILE = MODFILE(1:INDEX(MODFILE,' ')-1)
          !            MODFILE = MODFILE(1:INDEX(MODFILE,' ')-1)
          !            WRITE(*,*) MODFILE

          OPEN  (1,FILE=MODFILE,STATUS='OLD')

          ! Read the header lines
          READ  (1,'(7X,F6.0,10x,F5.3,13X,F5.3)') &
               TMOD(NT), GMOD(NG), HMOD(NH)

          ! Read the dimension of the spectrum
          READ  (1,*) NWLOC

          ! Allocate the array sizes
          ALLOCATE ( WLOC(NWLOC),ALOC(NWLOC),FLOC(NWLOC),CLOC(NWLOC) )

          ! Read the data
          READ  (1,*) 
          READ  (1,*) 
          READ  (1,*)
          READ  (1,*) ( WLOC(NW),ALOC(NW),FLOC(NW),CLOC(NW), TAU, NW = 1,NWLOC )
          CLOSE (1)

          ! Use initial model to define wavelength grid
          IF ( NWMOD == 0 ) THEN
             NWMOD = NWLOC
             WMOD(1:NWMOD) = WLOC(1:NWMOD)
          ENDIF

          ! Resample arrays onto prespecified grid
          CALL RESAMPLE (WLOC,FLOC(1:NWLOC),NWLOC, &
               WMOD,FMOD(1:NWMOD,NT,NG,NH),NWMOD, 0.)
          CALL RESAMPLE (WLOC,CLOC(1:NWLOC),NWLOC, &
               WMOD,CMOD(1:NWMOD,NT,NG,NH),NWMOD, 0.)

          ! Arrays are resampled, free up for next use
          DEALLOCATE ( WLOC, ALOC, FLOC, CLOC )

          ! Report the spectrum 
          WRITE (*,'(A,''  Teff='',F6.0,'' log g='',F5.3, '' nHe='',F6.3,'' nw='',I6)') &
               TRIM(MODFILE), TMOD(NT), GMOD(NG), HMOD(NH), NWLOC
          WRITE (11,'(A,''  Teff='',F7.0,'' log g='',F4.1,'' nHe='',F6.3,'' nw='',I6)') &
               TRIM(MODFILE), TMOD(NT), GMOD(NG), HMOD(NH), NWLOC

        END DO
      END DO
    END DO MODE0_312_INPUT
    ENDIF



    !  convert to astrophysical fluxes (F_nu, C_nu)
    FMOD = FMOD * TAPM__PI
    CMOD = CMOD * TAPM__PI

    ! MODE = 1:   Spectrum.001  (residual or normalized flux only)
  ELSEIF ( MODE.EQ.1 ) THEN

    MODE1_INPUT: DO NT = 1,NTMOD
      DO NG = 1,NGMOD
        DO NH = 1,NHMOD 

          READ  (*,'(A)') MODFILE
          MODFILE = MODFILE(1:INDEX(MODFILE,' ')-1)

          OPEN  (1,FILE=MODFILE,STATUS='OLD')
          READ  (1,'(2X,F5.0,2x,F5.0,3X,F6.0)') &
               TMOD(NT), GMOD(NG), HMOD(NH)
          READ  (1,*) 
          READ  (1,*) NWLOC
          ALLOCATE ( WLOC(NWLOC),ALOC(NWLOC) )
          READ  (1,*) (WLOC(NW),ALOC(NW),NW=1,NWLOC)
          CLOSE (1)

          ! Use initial model to define wavelength grid
          IF ( NWMOD == 0 ) THEN
             NWMOD = NWLOC
             WMOD(1:NWMOD) = WLOC(1:NWMOD)
          ENDIF

          ! Resample the data
          CALL RESAMPLE (WLOC,ALOC(1:NWLOC),NWLOC, &
               WMOD,FMOD(1:NWMOD,NT,NG,NH),NWMOD, 1.)
          CMOD(1:NWMOD,NT,NG,NH) = 1.0
          DEALLOCATE ( WLOC, ALOC )

          WRITE (*,'(A,''  Teff='',F5.1,'' log g='',F4.1, '' nHe='',F6.3)') &
               TRIM(MODFILE), TMOD(NT), GMOD(NG), HMOD(NH)
          WRITE (11,'(''Model '',A,''  Teff='',F7.0,'' log g='',F4.1,'' nHe='',F6.3)') &
               TRIM(MODFILE), TMOD(NT), GMOD(NG), HMOD(NH)

        END DO
      END DO
    END DO MODE1_INPUT

    ! mode 1 input reads T in kK.
    TMOD = TMOD * 1000.


    ! MODE = 2:  Sterne.s  (total flux)
  ELSEIF ( MODE.EQ.2 ) THEN

    MODE2_INPUT: DO NT = 1,NTMOD
      DO NG = 1,NGMOD
        DO NH = 1,NHMOD 

          READ  (*,'(A)') MODFILE
          MODFILE = MODFILE(1:INDEX(MODFILE,' ')-1)

          OPEN  (1,FILE=MODFILE,STATUS='OLD')
          READ  (1,'(13X,F6.0,7x,F3.0,3X,F4.0)') &
               TMOD(NT), GMOD(NG), HMOD(NH)
          READ  (1,*) 
          READ  (1,*) NWLOC
          ALLOCATE ( WLOC(NWLOC),FLOC(NWLOC) )
          DO NW = 1,NWLOC
            READ  (1,*,IOSTAT=BADVAL) WLOC(NW),FLOC(NW)
          ENDDO
          CLOSE (1)

          ! Use initial model to define wavelength grid
          IF ( NWMOD == 0 ) THEN
             NWMOD = NWLOC
             WMOD(1:NWMOD) = WLOC(1:NWMOD)
          ENDIF

          CALL RESAMPLE (WLOC,FLOC,NWLOC, &
               WMOD,FMOD(1:NWMOD,NT,NG,NH),NWMOD, 1.)
          DEALLOCATE ( WLOC, FLOC )

          WRITE (*,'(A,''  Teff='',F7.0,'' log g='',F5.2, '' nHe='',F6.3)') &
               TRIM(MODFILE), TMOD(NT), GMOD(NG), HMOD(NH)
          WRITE (11,'(''Model '',A,''  Teff='',F7.0,'' log g='',F5.2,'' nHe='',F6.3)') &
               TRIM(MODFILE), TMOD(NT), GMOD(NG), HMOD(NH)


        END DO
      END DO
    END DO MODE2_INPUT

    !  convert to astrophysical fluxes (F_lambda)
    FMOD = TAPM__PI * FMOD


    ! MODE = 3:  sp2  (total flux: spectrum format 2)
  ELSEIF ( MODE.EQ.3 ) THEN


    WRITE (*,*) 'Enter the grid parameters: T, G and H'
    READ (*,*) TMOD(1:NTMOD)
    READ (*,*) GMOD(1:NGMOD)
    READ (*,*) HMOD(1:NHMOD)

    MODE3_INPUT: DO NT = 1,NTMOD
      DO NG = 1,NGMOD
        DO NH = 1,NHMOD 

          READ  (*,'(A)') MODFILE
          MODFILE = MODFILE(1:INDEX(MODFILE,' ')-1)

          OPEN  (1,FILE=MODFILE,STATUS='OLD')
          READ  (1,'(A)') MODHEAD1
          READ  (1,*) 
          READ  (1,*) NWLOC
          ALLOCATE ( WLOC(NWLOC),FLOC(NWLOC) )
          READ  (1,*) (WLOC(NW),FLOC(NW),NW=1,NWLOC)
          CLOSE (1)

          WRITE (*,'(A)')   TRIM(MODFILE),MODHEAD1
          WRITE (11,'(A)')  TRIM(MODFILE),MODHEAD1

          ! Use initial model to define wavelength grid
          IF ( NWMOD == 0 ) THEN
             NWMOD = NWLOC
             WMOD(1:NWMOD) = WLOC(1:NWMOD)
          ENDIF

          CALL RESAMPLE (WLOC,FLOC,NWLOC, &
               WMOD,FMOD(1:NWMOD,NT,NG,NH),NWMOD, 1.)
          DEALLOCATE ( WLOC, FLOC )

        END DO
      END DO
    END DO MODE3_INPUT

    !  convert to astrophysical fluxes (F_lambda)
    FMOD = TAPM__PI * FMOD


    ! MODE = 4:   Synthe        (including fluxes, continuum and residual)
  ELSEIF (MODE.EQ.4) THEN

    ! convert required grid to nm
    WMOD = WMOD * 0.1

    MODE4_INPUT: DO NT = 1,NTMOD
      DO NG = 1,NGMOD
        DO NH = 1,NHMOD 

          READ  (*,'(A)') MODFILE
          MODFILE = MODFILE(1:INDEX(MODFILE,' ')-1)

          OPEN  (1,FILE=MODFILE,STATUS='OLD')
          READ  (1,'(15X,F4.1)') HMOD(NH)
          READ  (1,'(10X,F5.0)') TMOD(NT) 
          READ  (1,'(10X,F3.1)') GMOD(NG)
          READ  (1,*) 
          READ  (1,'(27X,I8)') NWLOC
          ALLOCATE ( WLOC(NWLOC),ALOC(NWLOC),FLOC(NWLOC),CLOC(NWLOC) )
          READ  (1,*) 
          READ  (1,*) 
          READ  (1,*)
          READ  (1,*) (WLOC(NW),FLOC(NW),CLOC(NW),ALOC(NW),NW=1,NWLOC)
          CLOSE (1)

          ! Use initial model to define wavelength grid
          IF ( NWMOD == 0 ) THEN
             NWMOD = NWLOC
             WMOD(1:NWMOD) = WLOC(1:NWMOD)
          ENDIF

          CALL RESAMPLE (WLOC,FLOC(1:NWLOC),NWLOC, &
               WMOD,FMOD(1:NWMOD,NT,NG,NH),NWMOD, 1.)
          CALL RESAMPLE (WLOC,CLOC(1:NWLOC),NWLOC, &
               WMOD,CMOD(1:NWMOD,NT,NG,NH),NWMOD, 1.)
          DEALLOCATE ( WLOC, ALOC, FLOC, CLOC )

          WRITE (*,'(A,''  Teff='',F5.0,'' log g='',F3.1, '' nFe='',F6.3)') &
               TRIM(MODFILE), TMOD(NT), GMOD(NG), HMOD(NH)
          WRITE (11,'(''Model '',A,''  Teff='',F7.0,'' log g='',F4.1,'' nHe='',F6.3)') &
               TRIM(MODFILE), TMOD(NT), GMOD(NG), HMOD(NH)

        END DO
      END DO
    END DO MODE4_INPUT

    ! convert wavelengths from nm to A 
    WMOD = WMOD * 10.    


    ! MODE = 5:   Synthe        (residual or normalized flux only)
    ! not implemented

  ENDIF


  !  Convert temperature grid to kK

  TMOD = TMOD / 1000.



  !  Report grid

  WRITE (*,'(/'' Model grid:'')')
  WRITE (*,'(4x,''NMOD '',I5)') NTMOD*NGMOD*NHMOD
  WRITE (*,'(4x,''WMOD '',I5,'' ('',I5,'')'',10F10.2)') NWMOD,NWMAX,WMOD(1),WMOD(NWMOD)
  WRITE (*,'(4x,''TMOD '',I5,'' ('',I5,'')'',10F10.2,3(/10F10.2))') NTMOD,NTMAX,TMOD(1:NTMOD)
  WRITE (*,'(4x,''GMOD '',I5,'' ('',I5,'')'',10F10.2)') NGMOD,NGMAX,GMOD(1:NGMOD)
  WRITE (*,'(4x,''AMOD '',I5,'' ('',I5,'')'',2X,10F10.4)') NHMOD,NHMAX,HMOD(1:NHMOD)
  WRITE (*,*)

  WRITE (11,'(/'' Model grid:'')')
  WRITE (11,'(4x,''NMOD '',I5)') NTMOD*NGMOD*NHMOD
  WRITE (11,'(4x,''WMOD '',I5,10F10.2)') NWMOD,WMOD(1),WMOD(NWMOD)
  WRITE (11,'(4x,''TMOD '',I5,10F10.2,3(/10F10.2))') NTMOD,TMOD(1:NTMOD)
  WRITE (11,'(4x,''GMOD '',I5,10F10.2)') NGMOD,GMOD(1:NGMOD)
  WRITE (11,'(4x,''AMOD '',I5,2X,10F10.4)') NHMOD,HMOD(1:NHMOD)
  WRITE (11,*)


  !  Check grid ordinates are monotonically increasing
  DO NH=2,NHMOD
     IF ( HMOD(NH) <= HMOD(NH-1) ) THEN
        WRITE(*,*) 'SFIT_RDGRID: *** Severe Warning *** '
        WRITE(*,*) '    AMOD - axis is not monotonically increasing '
        EXIT
     ENDIF
  ENDDO
  DO NT=2,NTMOD
     IF ( TMOD(NT) <= TMOD(NT-1) )  THEN
        WRITE(*,*) 'SFIT_RDGRID: *** Severe Warning *** '
        WRITE(*,*) '    TMOD - axis is not monotonically increasing '
        EXIT
     ENDIF
  ENDDO
  DO NG=2,NGMOD
     IF ( GMOD(NG) <= GMOD(NG-1) ) THEN
        WRITE(*,*) 'SFIT_RDGRID: *** Severe Warning *** '
        WRITE(*,*) '    GMOD - axis is not monotonically increasing '
        EXIT
     ENDIF
  ENDDO


END SUBROUTINE SFIT_RDGRID








