SUBROUTINE SPEC_PRED (W,PARS,F,DERIVS,NP)

!+
!
!  Programme:
!     spec_pred
!
!  Purpose:
!=    Returns the predicted normalized flux at a given wavelength
!     and for a given combination of Teff, log g, and vsini 
!
!  Method: 
!     This subroutine must operate in two modes; the mode is determined
!     by comparing PARS with the values of PARS last used.
!
!     1.A On first call with a new set of parameters PARS it must
!         create FOUR new synthetic spectra, as follows:
!
!     1.A.1 Interpolation in tables for {TEFF,LOGG}, {TEFF+DT,LOGG} 
!           and {TEFF,LOGG+DG}.
!  [        Initially this was doen wil a 2d polynomial interpolation, which  ]
!  [        is optimized for smoothness. We currently use a bicubic spline    ]
!  [        interpolation, which is better for smoothness; since the spline   ]
!  [        coefficient can be pretabulated, this should be more efficient    ]
!  [                                                      CSJ  10/02/00       ]
!
!     1.A.2 Resampling of these spectra onto WOBS 
!
!  [        This step was replaced by resampling BEFORE starting chi-squared  ]
!  [        minimization, but resulted in very little apparent saving in time ]
!  [                                                      CSJ  09/02/00       ]
!  [        The apparent lack of saving was due to problems in 1.A.3 (see     ]
!  [        below). resmapling is now done routinely before chi-squared....   ]
!  [                                                      CSJ  11/02/00       ]
!
!     1.A.3 Convolution of each spectrum with Instrumental profile 
!
!  [        Originally  assumed a 2 pixel FWHM, but the choice of which pixel ]
!  [        to use, and whether 2 pixel is appropriate meant that the user is ]
!  [        now asked to provide this parameter.                              ]
!  [                                                      CSJ  09/02/00       ]
!
!  [        The use of GAUSS_CONV consumed a huge amount of cpu time (64%)    ]
!  [        and was replaced by QSMOOTH (~1%), resolving the spped issue.     ]
!  [                                                      CSJ  11/02/00       ]
!
!     1.A.3.5 Convolution of each spectrum with systemic change in velocity 
!             shift
!
!     1.A.4 Convolution of each spectrum with VSINI
!
!     1.A.5 Convolution of {TEFF,LOGG} spectrum with VSINI+DV
!
!     1.A.6 Computation of derivatives wrt TEFF, LOGG and VSINI
!
!     1.B It must then return the predicted flux F at the given wavelength W,
!         and derivatives, DERIVS, wrt each parameter.
!
!     2.  On subsequent calls with the same parameters PARS it must
!         simply return the predicted flux F at the given wavelength W.
!         and derivatives, DERIVS, wrt each parameter.
!
!  Notes:
!
!     1) to be efficient the subroutine should be used when the
!     external loop is over wavelength... and not over parameters.
!
!     2) the convolution routines appear to be accurate ONLY
!     if the models are sampled evenly - thus the placing of step
!     1.A.2 IS important.
!
!     3) 07-FEB-2000. It may be sufficient to rebin the model grid onto
!     the observed wavelength grid before starting the chi-squared solution.
!     The istrumental convolution could be done here as well. Then only the
!     interpolation for T,G and convolution with VSINI is needed, and its
!     only done on NWOBS wavelengths instead of NWMOD. 
!
!     4) 07-FEB-2000. Some authors will renormalize the fit to the observations
!     (or vice versa). It would be necessary, in the first instance, to generate
!     the ratio FOBS/FINT, to smooth it using a large window (GAUSS_CONV), and
!     then multiply FINT by the result. This might work, unless a large residual
!     eg HeI line cores, messes things up really badly. In that case, large
!     residuals have to be cut.
!  
!  Language:
!     Fortran 90
! 
!  Authors:
!     C.S.Jeffery (CSJ) (Armagh Observatory)
!
!  History:
!     03-FEB-2000: CSJ
!        original version
!
!-


!  Global variables:

  USE SFIT_MODDATA
  USE SFIT_BINDATA
  USE SFIT_OBSDATA

  USE XFIT_DIMS

!  Subroutine arguments:
  REAL               ::  W       ! wavelength (given)
  REAL, DIMENSION(*) ::  PARS    ! parameters (given)
  REAL               ::  F       ! flux       (returned)
  REAL, DIMENSION(*) ::  DERIVS  ! derivatives wrt PARS (returned)
  INTEGER            ::  NP      ! number of parameters in PARS (given)


!  Local variables:
  REAL     ::  TEFF,  TEFF0=0    ! Effective temperature (K)
  REAL     ::  LOGG,  LOGG0=0    ! Surface gravity (cgs)
  REAL     ::  VSINI, VSIN0=0    ! Projected rotation velocity (km/s)
  SAVE  TEFF0, LOGG0, VSINO

  REAL     ::  FINT(NWMAX,4) ! Interpolated fluxes
      
  INTEGER  ::  NW                ! Wavelength counter
  INTEGER  ::  I                 ! Tabulated flux index
  SAVE  NW, I


!  Interpolation done in double precision
  DOUBLE PRECISION  TG__T(NTMAX)        ! T vals for interpolation
  DOUBLE PRECISION  TG__G(NGMAX)        ! g values for interpolation 
  DOUBLE PRECISION  TG__F(NTMAX,NGMAX)  ! Flux interpolation tabl
  DOUBLE PRECISION  T__D                ! temperature
  DOUBLE PRECISION  G__D                ! gravity
  DOUBLE PRECISION  F__D                ! flux
  DOUBLE PRECISION  S__D                ! flux error

!.

!   Map local values of parameters
  TEFF  = PARS(1) 
  LOGG  = PARS(2) 
  VSINI = PARS(3) 

  IF (NW.GE.NWOBS) NW = 0

!   Test whether parameters have changed and compute new spectra
  IF (ABS(TEFF0-TEFF)>1 .OR. &
       ABS(LOGG0-LOGG)>0.01 .OR. &
       ABS(VSIN0-VSINI)>0.1 ) THEN


!      Principal diagnostic when trouble occurs 
!         WRITE (*,*) TEFF,LOGG,VSINI

!1.A On first call with a new set of parameters PARS
!         interpolate FOUR new synthetic spectra, as follows:


!1.A.1 Interpolation in tables for {TEFF,LOGG}, {TEFF+DT,LOGG} 
!      and {TEFF,LOGG+DG}.

     TG__T(1:NTMOD) = TMOD(1:NTMOD,1); DT = 0.01 * TEFF
     TG__G(1:NGMOD) = GMOD(1,1:NGMOD); DG = 0.01

!      Loop over wavelength
     DO NW = 1,NWBIN

!        Copy fluxes to 2d interpolation array
        TG__F (1:NTMOD,1:NGMOD) = FBIN(NW,1:NTMOD,1:NGMOD)

!        Loop over TEFF, LOGG and increments usin 2d polynomial interpolation
        T__D = TEFF; G__D = LOGG
        CALL NR_POLIN2 (TG__T(1:NTMOD),TG__G(1:NGMOD), &
             TG__F(1:NTMOD,1:NGMOD),NTMOD,NGMOD,T__D,G__D,F__D,S__D)
        FINT(NW,1) = F__D

        T__D = TEFF+DT; G__D = LOGG
        CALL NR_POLIN2 (TG__T(1:NTMOD),TG__G(1:NGMOD), &
             TG__F(1:NTMOD,1:NGMOD),NTMOD,NGMOD,T__D,G__D,F__D,S__D)
        FINT(NW,2) = F__D

        T__D = TEFF; G__D = LOGG+DG
        CALL NR_POLIN2 (TG__T(1:NTMOD),TG__G(1:NGMOD), &
             TG__F(1:NTMOD,1:NGMOD),NTMOD,NGMOD,T__D,G__D,F__D,S__D)
        FINT(NW,3) = F__D


!        Loop over TEFF, LOGG and increments using bicubic spline interpolation
!
!            T__D = TEFF; G__D = LOGG
!            CALL NR_SPLIN2 (TMOD(1:NTMOD,1),GMOD(1,1:NGMOD),&
!                            FSPL(NW,1:NTMOD,1:NGMOD), &
!                            FSP2(NW,1:NTMOD,1:NGMOD), NTMOD, NGMOD, &
!                             TEFF, LOGG, FINT(NW,1) )
!            FINT(NW,1) = F__D

!            T__D = TEFF+DT; G__D = LOGG
!            CALL NR_SPLIN2 (TMOD(1:NTMOD,1),GMOD(1,1:NGMOD),&
!                            FSPL(NW,1:NTMOD,1:NGMOD), &
!                            FSP2(NW,1:NTMOD,1:NGMOD), NTMOD, NGMOD, &
!                            TEFF+DT, LOGG, FINT(NW,2) )
!            FINT(NW,2) = F__D

!            T__D = TEFF; G__D = LOGG+DG
!            CALL NR_SPLIN2 (TMOD(1:NTMOD,1),GMOD(1,1:NGMOD),&
!                            FSPL(NW,1:NTMOD,1:NGMOD), &
!                            FSP2(NW,1:NTMOD,1:NGMOD), NTMOD, NGMOD, &
!                            TEFF, LOGG+DG, FINT(NW,3) )
!            FINT(NW,3) = F__D


!           write (*,'(i6,f10.3,4f8.3)' ) nw,wbin(nw),fbin(nw,3,3),(fint(nw,i),i=1,3)

     ENDDO


     NO = NWOBS             ! abbreviation for NWOBS

!1.A.2 Resampling of above spectra onto WOBS
!         DO I = 1,3
!            CALL RESAMPLE (WMOD,FINT(1:NWMOD,I),NWMOD,WOBS,FINT(1:NO,4),NO)
!            FINT(1:NO,I) = FINT(1:NO,4)
!         ENDDO

!1.A.3 Convolution of each spectrum with Instrumental profile.
!!      ! default instrumental profile (FWHM = 2 pixels)
     DO I = 1,3
        CALL QSMOOTH (WOBS,FINT(1:NO,I),FINT(1:NO,4),NO,SINST)
        FINT(1:NO,I) = FINT(1:NO,4)   ! use spare slot as work area
     ENDDO
         
!1.A.3.5 Convolution of each spectrum with systemic change in velocity shift
     DO I = 1,3
        CALL BOXCAR (WOBS,FINT(1:NO,I),FINT(1:NO,4),NO,DVSYS)
        FINT(1:NO,I) = FINT(1:NO,4)   ! use spare slot as work area
     ENDDO

!1.A.4 Convolution of each spectrum with VSINI
     BETA = 1.5                       ! assumed value for limb darkening
     FINT(1:NO,4) = FINT(1:NO,1)      ! copy F{TEFF,LOGG} for VSINI+DV
     DO I = 1,3
        CALL VSINI_CONV (WOBS,FINT(1:NO,I),NO,VSINI,BETA)
     ENDDO

!1.A.5 Convolution of {TEFF,LOGG} spectrum with VSINI+DV
     DV   = 10                        ! increment for derivatives
     CALL VSINI_CONV (WOBS,FINT(1:NO,4),NO,VSINI+DV,BETA)

!1.A.6 Replace fluxes by derivatives in spectra 2 - 4
     FINT(1:NO,2) = (FINT(1:NO,2) - FINT(1:NO,1)) / DT
     FINT(1:NO,3) = (FINT(1:NO,3) - FINT(1:NO,1)) / DG
     FINT(1:NO,4) = (FINT(1:NO,4) - FINT(1:NO,1)) / DV

!  Assumed this is called first time with W = WOBS(1), and with
!  incrementing NW thereafter
     NW = 0

  ENDIF


!   Identify NW from wavelength
  NW = NW + 1

!   Look up flux and derivatives
  F         = FINT(NW,1)
  DERIVS(1) = FINT(NW,2) 
  DERIVS(2) = FINT(NW,3) 
  DERIVS(3) = FINT(NW,4)     

!   Save local values of parameters to check on next call
  TEFF0 = PARS(1) 
  LOGG0 = PARS(2) 
  VSIN0 = PARS(3) 

END SUBROUTINE SPEC_PRED



