SUBROUTINE SPEC_SYNTH (W,PARS,F,DERIVS,NPAR)

  !+
  !
  !  Programme:
  !     spec_synth
  !
  !  Purpose:
  !=    Returns the predicted normalized flux at a given wavelength
  !     and for a given model atmosphere and abundances. It also gives
  !     derivatives wrt to each abundance, if required.
  !
  !  Method: 
  !     This subroutine must operate in two modes; the mode is determined
  !     by comparing PARS with the values of PARS last used.
  !
  !     1.A On first call with a new set of parameters PARS it must
  !         create NPAR+1 new synthetic spectra, as follows:
  !
  !     1.A.1 Generation of spectrum for {TEFF,LOGG,ABUNDS,VTURB} and variants.
  !           This is done by a direct call to SPECTRUM routines
  !
  !           [ In the initial version, it is assumed that TEFF and LOGG are
  !           [ fixed, and that the name of a file containing a model atmosphere
  !           [ is given
  !
  !     1.A.1a Resampling of these spectra onto WOBS 
  !
  !     1.A.2 Convolution of each spectrum with SYN__VSINI, SYN__BETA
  !
  !     1.A.3 Computation of derivatives wrt abundances
  !
  !     1.A.4 Shift of each spectrum with SYN__VRAD
  !
  !     [  1.A.5 Convolution of each spectrum with systemic change in velocity]
  !     [          shift                                                        ]
  !
  !     1.A.6 Convolution of each spectrum with Instrumental profile 
  !
  !     1.B It must then return the predicted flux F at the given wavelength W,
  !         and derivatives, DERIVS, wrt each parameter.
  !
  !     2.  On subsequent calls with the same parameters PARS it must
  !         simply return the predicted flux F at the given wavelength W.
  !         and derivatives, DERIVS, wrt each parameter.
  !
  !  Notes:
  !  
  !  Language:
  !     Fortran 90
  ! 
  !  Authors:
  !     C.S.Jeffery (CSJ) (Armagh Observatory)
  !
  !  History:
  !     17-FEB-2000: CSJ
  !     1.0   original version: derived from SPEC_PRED
  !                          (see prologue for further notes)
  !     1.2   adapted to avoid returning derivatives, if not required.
  !
  !     20-OCT-2004: CSJ
  !     1.3   modifies SOBS using the abundance derivatives in an attempt 
  !           to make the chi^2 more efficient. Works, but needs further 
  !           investigation and mathematical description.
  !
  !-


  !  Global variables:

  USE XFIT_DIMS
!  USE SFIT_GLOBAL

  USE SFIT_OBSDATA
  USE SFIT_SYNDATA


  !  Switch off implicit typing
  IMPLICIT NONE


  !  Subroutine arguments:
  REAL               ::  W       ! wavelength (given)
  REAL, DIMENSION(*) ::  PARS    ! parameters (given)
  REAL               ::  F       ! flux       (returned)
  REAL, DIMENSION(*) ::  DERIVS  ! derivatives wrt PARS (returned)
  INTEGER            ::  NPAR    ! number of parameters in PARS (given)


  !  Local variables:

  !  Local values for parameters
  REAL, DIMENSION(NAMAX+2) :: PARS0 = 0 ! Saved parameters
  SAVE  PARS0
  
  INTEGER :: NELS                ! set to NPAR - 3
  REAL :: VTURB                  ! microtrubulent velcoity
  REAL :: VSINI                  ! rotational velocity
  REAL :: VRAD                   ! radial velocity

  REAL :: DPAR                   ! test changes in PARS
  REAL :: DVT                    ! microturbulent velocity increment
  REAL :: DAB                    ! increment for an abundance
  REAL :: DVS                    ! increment for v_sini_i
  REAL :: DDV                    ! increment for radial velcoity

  !  Flags for what needs doing
  LOGICAL  :: LDERIVS = .TRUE.   ! Flag whether derivatives are needed
  LOGICAL  :: NEWGRID = .TRUE.   ! Flag generation of a new wavelength grid
  SAVE  NEWGRID




  !  Temporary store for synthetic spectra
  INTEGER            :: NSMAX         ! Dimension for synthetic spectrum
  REAL               :: WSS, WSE      ! Wavelength limits for synthetic spectrum
  REAL, ALLOCATABLE, SAVE :: WS(:)    ! Synthetic spectrum wavelengths
  REAL, ALLOCATABLE  :: FL(:)         ! Synthetic spectrum fluxes
  REAL, ALLOCATABLE  :: FC(:)         ! Synthetic spectrum continuum
  REAL, ALLOCATABLE  :: FN(:)         ! Synthetic spectrum normalized
  INTEGER, SAVE      :: NWS           ! Number of wavelengths in synthetic spectrum 

  !  Temporary store for redshifts
  REAL               :: BETA = 1.5    ! limb darkening - assumed
  REAL               :: REDSHIFT      ! redshift
  REAL               :: DRS           ! delta redshift
  REAL, ALLOCATABLE  :: WR(:)         ! Redshift spectrum wavelengths
  REAL, ALLOCATABLE  :: FR(:)         ! Redshift spectrum 

  ! Long term store for synthetic spectrum and derivatives
  INTEGER, PARAMETER :: NFMAX = NAMAX + 4
  REAL               :: FSYN(NOMAX,NFMAX) ! Synthetic fluxes + derivatives
  REAL               :: DFMAX             ! largest absolute derivative  

  ! Various counters
  INTEGER  ::  NW = 0            ! Wavelength counter
  INTEGER  ::  NP                ! Parameter index
  INTEGER  ::  NO                ! proxy for NWOBS
  INTEGER  ::  NB                ! spare loop counter
  SAVE  NW, NP


  ! Average spectral resolution required for mode
  REAL, PARAMETER :: WRES = 1.E6   ! 1 million



  !  Local constants:
  DOUBLE PRECISION, PARAMETER :: TAPP__C = 2.99792458D+08 


  !.

  !   Reset PARS0
  IF (NPAR == 0) THEN
     PARS0 = 0
     NW = 0
     RETURN
  ENDIF

  !   Check if starting a new spectrum
  IF (NW >= NWOBS) NW = 0

  !   Check if derivatives are needed
  LDERIVS = .TRUE.
  IF (DERIVS(1) <= -98.) LDERIVS = .FALSE. 

  !   Check if parameters changed for this spectrum
  DPAR = 0.0
  IF (NW == 0) THEN
              write (*,*) 'spec_synth: parameters for this run'  
              write (*,'(0P,20f8.3)') pars(1:npar)
     DO NP = 1,NPAR
        DPAR = DPAR + ABS(PARS(NP)-PARS0(NP))
     ENDDO
     ! DPAR = DPAR + ABS(WOBS(1)-PARS0(NPAR+1))  ! checks for new spectrum
            write (*,'(0P,1X,a,e12.3))') 'dpar :: ',dpar
  ENDIF

  !   Test whether parameters have changed and compute new spectra
  IF (DPAR > 1.E-30) THEN

     NO = NWOBS             ! abbreviation for NWOBS
     WSS = WOBS(1) - 10.    ! wavelength range for synthetic spectra
     WSE = WOBS(NO) + 10. 
     NSMAX = ( WSE - WSS ) * WRES * 2 / ( WSS + WSE ) 

     IF ( .NOT. ALLOCATED (WS) ) ALLOCATE ( WS ( NSMAX ) )
     IF ( .NOT. ALLOCATED (FL) ) ALLOCATE ( FL ( NSMAX ) )
     IF ( .NOT. ALLOCATED (FC) ) ALLOCATE ( FC ( NSMAX ) )
     IF ( .NOT. ALLOCATED (FN) ) ALLOCATE ( FN ( NSMAX ) )

     !      Map local values of parameters
     PARS0(1:NPAR) = PARS(1:NPAR)
     VTURB = PARS(NPAR-2)
     VSINI = PARS(NPAR-1)
     VRAD  = PARS(NPAR)
     NELS  = NPAR - 3

     !      Define variational increments
     DAB = 0.1
     DVT = MAX(0.2*VTURB,1.0)
     DVS = 10
     !1.A On first call with a new set of parameters PARS
     !       compute NPAR+1 new synthetic spectra, as follows:

     ! 1  Generate spectra for {TEFF, LOGG, ABUNDS, VTURB} and variances
     ! 2  Resample immediately onto observed wavelengths to conserve memory 

     write (*,'(A/,A)') 'SPEC_SYNTH','abundances and vturb'
     write (*,'(9(i3,f6.2))') (syn__elid(np),pars(np),np=1,npar-1)
     write (*,'(f6.2)') vturb
   !  write (*,*)

     CALL SPC_SYNTH ( SPC__NMODE, SPC__NOPAC, SPC__NSOURCE, SPC__NTHETA, SPC__NHEDAT, &
          SPC__NHESTARK, NELS, SYN__ELID(1:NELS), PARS(1:NELS), VTURB, &
          WSS, WSE, WS, FL, FC, FN, NWS, NEWGRID, & 
          SYN__FILE )
     CALL RESAMPLE (WS,FN,NWS, WOBS,FSYN(:,1),NO, 1.0)
     NEWGRID       = .FALSE.

   !  write (*,*)'SPEC_SYNTH : spectrum with',NWS,'wavelengths'

     !      Compute variational spectra in order to find derivatives...
     IF (LDERIVS) THEN

        !        Vary abundances in turn
        DO NP = 1,NPAR-1
           IF ( SYN__DABU(NP) > 0.0 ) THEN 
              PARS(NP) = PARS(NP) + DAB
              CALL SPC_SYNTH ( SPC__NMODE, SPC__NOPAC, SPC__NSOURCE, SPC__NTHETA, SPC__NHEDAT, &
                   SPC__NHESTARK, NELS, SYN__ELID(1:NELS), PARS(1:NELS), VTURB, &
                   WSS, WSE, WS, FL, FC, FN, NWS, NEWGRID, & 
                   SYN__FILE )
        ! 2008/03/05
        ! OK if apectrum is normalized
        ! need to use FL and THETA and EBV if not 
              CALL RESAMPLE (WS,FN,NWS, WOBS,FSYN(:,NP+1),NO, 1.0)
              PARS(NP) = PARS0(NP) 
           ELSE
              FSYN(:,NP+1) = FSYN(:,1)
           ENDIF
        ENDDO

        !        Vary microturbulence
        IF ( SYN__DABU(NPAR-2) > 0.0 ) THEN 
          CALL SPC_SYNTH  ( SPC__NMODE, SPC__NOPAC, SPC__NSOURCE, &
               SPC__NTHETA, SPC__NHEDAT, &
               SPC__NHESTARK, NELS, SYN__ELID(1:NELS), PARS(1:NELS), VTURB + DVT, &
               WSS, WSE, WS, FL, FC, FN, NWS, NEWGRID, & 
               SYN__FILE )
        CALL RESAMPLE (WS,FN,NWS, WOBS,FSYN(:,NPAR+1),NO, 1.0)
        ENDIF

     ENDIF


     IF ( ALLOCATED (FL) ) DEALLOCATE (FL)
     IF ( ALLOCATED (FC) ) DEALLOCATE (FC) 
     IF ( ALLOCATED (FN) ) DEALLOCATE (FN) 

!     DEALLOCATE (WS)   ! do not deallocate WS; SP_SYNTH more efficient if it exsits
!     NEWGRID = .TRUE.


     ! 2 Convolution of each spectrum with VSINI
     ! ** 2008/03/05
     ! ** BETA could be replaced by true wavelength-dependent 
     !    limb-darkneing coefficient as computed by spectrum
     FSYN(1:NO,NPAR+1) = FSYN(1:NO,1)  
     DO NP = 1,NPAR
        CALL VSINI_CONV (WOBS,FSYN(1:NO,NP),NO,VSINI,SYN__BETA)
     ENDDO
 

     ! 3 Replace fluxes by derivatives in spectra 2 - NPAR+1
     IF (LDERIVS) THEN
       DO NP = 1,NELS   !! abundances only
         IF ( SYN__DABU(NP) > 0.0 ) THEN 
           FSYN(1:NO,NP+1) = (FSYN(1:NO,NP+1) - FSYN(1:NO,1)) / DAB
         ENDIF
       ENDDO
     ENDIF

     IF (LDERIVS .AND. SYN__DABU(NPAR-1) > 0.0 ) THEN
        FSYN(1:NO,NPAR-1) = (FSYN(1:NO,NPAR-1) - FSYN(1:NO,1)) / DVT
     ENDIF

     ! 2a  Convolution of {TEFF,LOGG} spectrum with VSINI+DV
     ! ** 2008/03/05
     ! ** BETA ** see above
     IF (LDERIVS .AND. SYN__DABU(NPAR-1) > 0.0 ) THEN
       CALL VSINI_CONV (WOBS,FSYN(1:NO,NPAR+1),NO,VSINI+DVS,SYN__BETA)
       FSYN(1:NO,NPAR) = (FSYN(1:NO,NPAR+1) -  FSYN(1:NO,1)) / DVS
     ENDIF

     ! 4  Apply redshift
     IF ( .NOT. ALLOCATED (WR) ) ALLOCATE ( WR ( NO ) )
     IF ( .NOT. ALLOCATED (FR) ) ALLOCATE ( FR ( NO ) )

     REDSHIFT = VRAD * 1.0E+03 /  TAPP__C
     WR(1:NO) = WOBS(1:NO) * ( 1. + REDSHIFT )
     FR(1:NO) = 1.0
     IF ( ABS(WR(NO)*REDSHIFT) > SINST/20. ) THEN
        DO NP = 1,NPAR
           CALL RESAMPLE (WR,FSYN(:,NP),NO, WOBS,FR,NO, 1.)
           FSYN(1:NO,NP) = FR(1:NO)
        ENDDO
     ENDIF

     ! 4a. Compute derivative of redshift
     IF (LDERIVS .AND.  SYN__DABU(NPAR) > 0.0 ) THEN
       DRS = SINST / WOBS(NO/2) 
       WR(1:NO) = WOBS(1:NO) *  (1. + DRS) 
       CALL RESAMPLE (WR,FSYN(1:NO,1),NO, WOBS(1:NO),FR,NO, 1.)
       FSYN(1:NO,NPAR+1) = FR(1:NO)
       DDV = DRS * TAPP__C / 1.0E+03
       FSYN(1:NO,NPAR+1) = (FSYN(1:NO,NPAR+1) - FSYN(1:NO,1)) / DDV
     END IF


     ! 6 Convolution of each spectrum with Instrumental profile.
     !     ! default instrumental profile (FWHM = 2 pixels)
     ! This is interesting ... convolving derivatives with rotation? 
     ! not sure if that makes sense.  
     CALL QSMOOTH (WOBS,FSYN(1:NO,1),FR(1:NO),NO,SINST)
     FSYN(1:NO,1) = FR(1:NO)   ! use spare slot as work area
     DO NP = 1,NPAR
       IF ( SYN__DABU(NP) > 0.0 ) THEN 
         CALL QSMOOTH (WOBS,FSYN(1:NO,NP+1),FR(1:NO),NO,SINST)
         FSYN(1:NO,NP+1) = FR(1:NO)   ! use spare slot as work area
       ENDIF
     ENDDO


     IF ( ALLOCATED (WR) ) DEALLOCATE (WR) 
     IF ( ALLOCATED (FR) ) DEALLOCATE (FR) 

     !  Assumed this is called first time with W = WOBS(1), and with
     !  incrementing NW thereafter
     NW = 0



! This was not working, I was getting dpar = 0. 
! Suspect that increases to sig were making sum totals very 
! large so that minor increments were having no effect on summations. 
! Reverted to pure form until this gets sorted out. 

      ! Sharpen up mask...
      ! Compute total derivative at each wavelength and
      ! multiply sigma by value ->6 for small derivs and ->1 for large derivs.
      ! Note that these are only activated on the "next" optimization loop. 
  
!      FSYN(1:NO,NFMAX) = 0
!      DO NP = 1,NPAR-1
!          IF ( SYN__DABU(NP) > 0.0 ) THEN
!             FSYN(1:NO,NFMAX) = FSYN(1:NO,NFMAX) + ABS(FSYN(1:NO,NP+1))
!          ENDIF
!      ENDDO
!      DFMAX = MAXVAL ( FSYN(1:NO,NFMAX) ) - 0.001
!      IF ( DFMAX > 0. ) THEN
!      WHERE ( FSYN(1:NO,NFMAX) < SOBS(1:NO) ) 
!      !  refined 2018.x.08
!      !   SOBS(1:NO) =  SOBS(1:NO) * (1.0 - ALOG10( 0.00001 + FSYN(1:NO,NFMAX) / DFMAX) ) 
!      ENDWHERE
!      ENDIF


  ENDIF

  IF (NW == 0)  write (*,*) 'spec_synth:  --------------------------'  


  !   Identify NW from wavelength
  NW = NW + 1

  
  !   Output results
  F = FSYN(NW,1)
  IF (LDERIVS) DERIVS(1:NPAR) = FSYN(NW,2:NPAR+1) 


  !   Save local values of parameters to check on next call
  PARS0(1:NPAR) = PARS(1:NPAR)
  PARS0(NPAR+1) = WOBS(1)




END SUBROUTINE SPEC_SYNTH



