SUBROUTINE VSINI_CONV (W,F,NW,VSINI,B)

  !+
  !
  != Rotationally broaden a spectrum by convolution
  !  with the Unsold (1955) function given in Dufton (A+A,16,p301,1972).
  !  Data assumed to be at wavelength intervals.
  !     W(*)      array of wavelength points unchanged on exit
  !     F(*)      array of flux points contains convoluted flux
  !               on exit
  !     NPTS      number of points considered in above arrays
  !     VSINI     vsini in km/s (use 1.5 by default)
  !     B         limb darkening coefficient
  !
  !  Local array CONFLX(MXPTS) contains convoluted flux before
  !  it is written back into F(*).
  !
  !  History:
  !     Original version (ROTATE - written for DIPSO)
  !        C.S.Jeffery, University of St Andrews, 1988
  !
  !     This version:
  !        C.S.Jeffery, Armagh Observatory, 2000 February
  !
  !-


  ! Subroutine arguments
  REAL, DIMENSION(*) ::  W
  REAL, DIMENSION(*) ::  F


  ! Local variables
  REAL, DIMENSION(:), ALLOCATABLE, SAVE ::CONW
  INTEGER, SAVE ::  MXPTS=0 
  REAL  :: W0


  ! Local constants
  REAL, PARAMETER :: C = 2.99792458E5 


  !.

  !  Check limits on number of points and allocated convoution array to suit
  IF (NW.GT.MXPTS) THEN
     IF ( ALLOCATED (CONW) ) DEALLOCATE ( CONW )
     ALLOCATE ( CONW (NW) )
     MXPTS = NW
  ENDIF


  !  Check VSINI is non-zero
  IF (ABS(VSINI).LT.1.E-3) RETURN

  !  Call convolution routine and cycle over NPTS points
  DO I=1,NW,1
     W0=W(I)
     FAC=C/VSINI/W0
     XLO=W0-(3./FAC)
     XUP=W0+(3./FAC)
     CONW(I)=VSINI_TRAP(W,F,NW,XLO,XUP,W0,FAC,B)
  END DO

  !  Copy convoluted flux back into F(*)
  DO I=1,NW,1
     F(I)=CONW(I)
  END DO

END SUBROUTINE VSINI_CONV
