SUBROUTINE XFT_TGHV_IN1 ( TEFF, DT, LOGG, DG, ANHY, DH, VSINI, DV, &
     SINST, DVSYS, DELTAV, NWBIN, NTMOD, NGMOD, NHMOD, &
     WBIN, TMOD, GMOD, HMOD, FBIN, FINT )

  !+
  !
  !  Programme:
  !     xft_tghv_in1
  !
  !  Purpose:
  !=    Returns interpolated spectra ( NO  derivatives )
  !
  !  Method: 
  !     Interpolate synthetic spectra, as follows
  !
  !     1  Interpolation in tables for {TEFF,LOGG,ANHE}
  !
  !     2  Convolution of each spectrum with VSINI
  !
  !   [ 3  Convolution of {TEFF,LOGG} spectrum with VSINI+DV ]
  !
  !     4  Apply redshift DELTAV
  !
  !     5  Convolution of each spectrum with systemic change in velocity 
  !             shift
  !
  !     6  Convolution of each spectrum with Instrumental profile 
  !
  !   [ 7  Computation of derivatives wrt TEFF, LOGG, ANHE and VSINI ]
  !
  !
  !  Notes:
  !
  !  
  !  Language:
  !     Fortran 90
  ! 
  !  Authors:
  !     C.S.Jeffery (CSJ) (Armagh Observatory)
  !
  !  History:
  !     13-OCT-2004: CSJ
  !        corrected order of broadening functions
  !     29-AUG-2002: CSJ
  !        original version adaptedfrom xft_tghv_int
  !
  !-


  ! Global constants:
  USE XFIT_DIMS, ONLY : NWMAX

  USE mod_model_list
  USE mod_tetra_list


  ! Global typing
  IMPLICIT NONE


  !  Speed of light in a vacuum: c = 2.99792458E+08 m/s (exact).
  DOUBLE PRECISION, PARAMETER :: TAPP__C = 2.99792458D+08


  !  Subroutine arguments (given):

  !    Model grid parameters:
  INTEGER, INTENT(IN) ::  NWBIN, NTMOD, NGMOD, NHMOD   ! Grid dimensions

  !    Model grid: (assumed shape)
  REAL, INTENT(IN), DIMENSION(:)     :: WBIN
  REAL, INTENT(IN), DIMENSION(:)     :: TMOD
  REAL, INTENT(IN), DIMENSION(:)     :: GMOD
  REAL, INTENT(IN), DIMENSION(:)     :: HMOD
  REAL, INTENT(IN), DIMENSION(:,:,:,:) :: FBIN

  !    Interpolation parameters:
  REAL, INTENT(IN) ::  TEFF, DT    ! Effective temperature (K) and increments
  REAL, INTENT(IN) ::  LOGG, DG    ! Surface gravity (cgs)
  REAL, INTENT(IN) ::  ANHY, DH    ! Surface helium abundance (fractional)
  REAL, INTENT(IN) ::  VSINI       ! Projected rotation velocity (km/s)
  REAL, INTENT(INOUT) ::  DV       ! Projected rotation velocity increment

  !    Other arguments:
  REAL, INTENT(IN) ::  SINST       ! Instrumental profile (Gaussian FWHM)
  REAL, INTENT(IN) ::  DVSYS       ! velocity broadening function (boxcar)
  REAL, INTENT(IN) ::  DELTAV      ! Radial velcoity shift

  !  Subroutine arguments (returned): (assumed shape)
  REAL, INTENT(OUT), DIMENSION(:,:) :: FINT ! Interpolated fluxes and derivatives


  !  Local variables:
  !  Interpolation done in double precision 
  DOUBLE PRECISION  :: TG__T(NTMOD)        ! T vals for interpolation (automatic arrays)
  DOUBLE PRECISION  :: TG__G(NGMOD)        ! g values for interpolation
  DOUBLE PRECISION  :: TG_HE(NHMOD)        ! He values for interpolation 
  DOUBLE PRECISION  :: TG__X(NHMOD)        ! Flux interpolation table
  DOUBLE PRECISION  :: TG_XX(1,1,1,NHMOD)  ! Flux interpolation table
  DOUBLE PRECISION  :: TG__F(NTMOD,NGMOD)  ! Flux interpolation table
  DOUBLE PRECISION  :: TG_DF(NTMOD,NGMOD)  ! Flux interpolation table
  DOUBLE PRECISION  :: T__D                ! temperature
  DOUBLE PRECISION  :: G__D                ! gravity
  DOUBLE PRECISION  :: H__D, H_DY          ! hydrogen or helium abundance
  DOUBLE PRECISION  :: F__D = 0.           ! flux
  DOUBLE PRECISION  :: S__D = 0.           ! flux error

  !  Temporary store for redshifts
  REAL :: BETA = 1.5    ! limb darkening - assumed
  REAL :: REDSHIFT      ! reshift
  REAL :: WRED(NWMAX)
  REAL :: TEMP(NWMAX)

  !  Index counters
  INTEGER :: NW, NT, NG, NO, I

  DOUBLE PRECISION, DIMENSION(3) :: POINT

  !.  ! this is only here because there is a SEG fault if it is not.
  ! WRITE (*,*) 'XFT_TGHV_IN1: '

  ! 1  On first call with a new set of parameters PARS
  !         interpolate FOUR new synthetic spectra, as follows:

  TG__T(1:NTMOD) = TMOD(1:NTMOD)
  TG__G(1:NGMOD) = GMOD(1:NGMOD)
  TG_HE(1:NHMOD) = HMOD(1:NHMOD)
  TG__F = 0.

  H__D = ANHY


  !!! Decide whether or not we're using CWR's extensions
  IF ( ASSOCIATED(MODSLIST) .AND. ASSOCIATED(TETRALIST) ) THEN
     !!! Setup the search point
     POINT(1) = TEFF; POINT(2) = LOGG; POINT(3) = H__D

     !!! Interpolate to the point of interest
     CALL TETRALIST_INTERPOLATE(POINT, TETRALIST, MODSLIST, FBIN(1,1,1,1), FINT(1:NWBIN, 1))

  ELSE

    ! Loop over wavelength
    DO NW = 1,NWBIN

      ! Interpolate fluxes onto local H abundance and create 2d interpolation array
      DO NG = 1,NGMOD
        DO NT = 1,NTMOD
          TG__X(1:NHMOD) = FBIN(NW,NT,NG,1:NHMOD)
!          CALL NR_POLINT (TG_HE,TG__X,H__D,TG__F(NT,NG),S__D)
!         write(*,*) tg_he
!         write(*,*) tg__x
!         write(*,*) nhmod
!         write(*,*) h__d
!         stop
          CALL MTH_LININT (TG_HE,TG__X,NHMOD,H__D,TG__F(NT,NG),S__D)
!         if (nt == 1 .and. ng == 1) write(*,*) 'xft_tghv_in1: ',wbin(nw), tg__f(nt,ng)
!         write(*,*) s__d 
        ENDDO
      ENDDO


      ! Loop over TEFF, LOGG and increments usin 2d polynomial interpolation
      T__D = TEFF; G__D = LOGG
!      CALL NR_POLIN2 (TG__T,TG__G, TG__F,NTMOD,NGMOD,T__D,G__D,F__D,S__D)
      CALL MTH_LININ2 (TG__T,TG__G, TG__F,NTMOD,NGMOD,T__D,G__D,F__D,S__D)
      FINT(NW,1) = F__D 
 !     write(*,*) 'xft_tghv_in1: ',wbin(nw), f__d

    ENDDO
  ENDIF

  NO = NWBIN             ! abbreviation for NWOBS


 !  write(*,*)  'xft_tghv_in1: ',vsini,beta,deltav,dvsys,sinst

  ! 2  Convolution of each spectrum with VSINI
  CALL VSINI_CONV (WBIN,FINT(1:NO,1),NO,VSINI,BETA)
 !  write(*,*)  'xft_tghv_in1: ',fint(1,1),fint(no,1)

  ! 4  Apply redshift 
  REDSHIFT =  DELTAV * 1.0E+03 /  TAPP__C 
  WRED(1:NO) = WBIN(1:NO) * ( 1. + REDSHIFT )
  IF (ABS(WBIN(NO)*REDSHIFT)>SINST/20.) THEN 
    CALL RESAMPLE (WRED(1:NO),FINT(1:NO,1),NO, &
         WBIN(1:NO),TEMP(1:NO),NO, 0.)
    FINT(1:NO,1) = TEMP(1:NO)
  ENDIF
 !  write(*,*)  'xft_tghv_in1: ',fint(1,1),fint(no,1)

  ! 5  Convolution of each spectrum with systemic change in velocity shift
  IF (DVSYS.GT.SINST) THEN
    CALL BOXCAR (WBIN,FINT(1:NO,1),FINT(1:NO,7),NO,DVSYS)
    FINT(1:NO,1) = FINT(1:NO,7)   ! use spare slot as work area
  ENDIF
 !  write(*,*)  'xft_tghv_in1: ',fint(1,1),fint(no,1)
   
  ! 6  Convolution of each spectrum with instrumental profile.
  IF ( SINST < 1.E2 ) THEN  
    CALL QSMOOTH (WBIN,FINT(1:NO,1),FINT(1:NO,7),NO,SINST)
  ELSE
    WRED(1:NO) = ALOG( WBIN(1:NO) ) 
    CALL QSMOOTH (WRED,FINT(1:NO,1),FINT(1:NO,7),NO,1./(2.354*SINST))
  ENDIF
  FINT(1:NO,1) = FINT(1:NO,7)   ! use spare slot as work area
 ! write(*,*)  'xft_tghv_in1: ',fint(1,1),fint(no,1)  
  
 

!  write (99,*) teff, logg, anhy
!  write (99,*) fint(:,1)
!  write (99,*) dt
!  write (99,*) fint(:,2)
!  write (99,*) dg
!  write (99,*) fint(:,3)
!  write (99,*) dh
!  write (99,*) fint(:,4)
!  write (99,*) dv
!  write (99,*) fint(:,5)
!  stop

END SUBROUTINE XFT_TGHV_IN1



